﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#pragma once

#include <nex.h>
#include "AvatarDDL.h"

#define AVATAR_SIZE 10

/*!
    @brief アバターを表現するクラス

    プレイヤーキャラのイメージです
    これは各ステーションで作成されます
 */
class Avatar : public NAMESPACE_NEX DOCLASS(Avatar)
{
public:
    Avatar();
    Avatar(qReal x, qReal y);
    ~Avatar();

    virtual void InitDO();

    void SetPosition(qReal x, qReal y);
    void MovePosition(qReal x, qReal y);

    void DoPhysic();
    void Render();

private:
    void UpdateMoveDir();
    void UpdateCollisionWithWorld();
    void UpdateCollisionWithWorldSub(qReal& x, qReal& y);

private:
    // 定数定義
    enum
    {
        MOVE_TIME = 3000,                   // 移動時間(経過で方向を変更)
        MOVE_POWER = 2,                     // 移動量
        EXTRAPOLATION_DELAY_TIME = 500,     // 推定遅延時間
        LOOPBACK_DELAY_TIME = 500,          // ループバック遅延時間設定
        SUBSTREAM_ID = nn::nex::SubStreamIDDefine::SYSTEM // 使用する SubStream ID。
    };

private:
    // 速度が変化した時間
    nn::nex::Time m_movedTime;
    // 速度ベクトル
    qReal m_velocity[2];
};

