#!/usr/bin/perl
#
# CTR-NEX
#   独自サーバ用認証トークン生成プログラムサンプル Ver 1.1
#
# ※利用方法
#   tokenenc_ctr.pl key pid friendcode servertype [timestamp]
#      key        : 認証トークン鍵（文字表記）
#      pid        : PID（先行0なし数値最大10桁）
#      friendcode : フレンドコード（先行0あり数値12桁固定）
#      servertype : 英数2桁（開発環境は"D1"）
#      timestamp  : UNIXタイムスタンプ値
#                   未指定時は現在時間
#

use strict;
use warnings;
use Crypt::CBC;
use Digest::SHA1 qw(sha1_hex);
use MIME::Base64 qw(encode_base64);

# 埋め込み値セット
my ($key, $pid, $friendcode, $servertype, $timestamp);
if ($#ARGV < 3) {
  die "usage: tokenenc_ctr.pl key pid friendcode servertype [timestamp]\n";
}
if ($ARGV[0] !~ m/^\w{16}$/xms) {
  die "invalid parameter (key)\n";
}
if ($ARGV[1] !~ m/^\d{1,10}$/xms) {
  die "invalid parameter (pid)\n";
}
if ($ARGV[2] !~ m/^\d{12}$/xms) {
  die "invalid parameter (friendcode)\n";
}
if ($ARGV[3] !~ m/^\w{2}$/xms) {
  die "invalid parameter (servertype)\n";
}
if (defined $ARGV[4] && $ARGV[4] !~ m/^\d+$/xms) {
  die "invalid parameter (timestamp)\n";
}
$key        = $ARGV[0];
$pid        = $ARGV[1];
$friendcode = $ARGV[2];
$servertype = $ARGV[3];
if (defined $ARGV[4]) {
  $timestamp = $ARGV[4];
} else {
  $timestamp = time;
}

# 各値表示
print "Token Key   : $key\n";
print "PID         : $pid\n";
print "Friend Code : $friendcode\n";
print "Server Type : $servertype\n";
print "Timestamp   : $timestamp\n";


# ハッシュ値取得後連結
my $token = "\\u\\$pid\\f\\$friendcode\\s\\$servertype\\t\\$timestamp";
my $hash = substr sha1_hex($token), 0, 8;
print "SHA-1 Hash  : $hash\n";
$token .= "\\h\\$hash";
print "Plain Token : $token\n";


# AES暗号化
# CBCモード、鍵=128bit、IV=オール\0、パディング=PKCS#5
my $iv  = pack 'H32', '0' x 32; # IV
my $cipher = Crypt::CBC->new(
  -key         => $key,
  -keysize     => 16,
  -literal_key => 1,
  -cipher      => 'Crypt::Rijndael',
  -iv          => $iv,
  -padding     => 'standard',
  -prepend_iv  => 0,
);
my $enctoken = $cipher->encrypt($token);

# Base64+エンコード
my $tmptoken = encode_base64($enctoken, q{});
$tmptoken =~ tr/\/\+\=/\-\.\*/;
print "Token       : $tmptoken\n";

exit;

