﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Platform/Core/qSortedVector.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>

BEGIN_NAMESPACE_NEX

class ConfirmFaultyStationTable: public RootObject
{
public:
    enum VoteState
    {
        VOTE_DECLINED = (qByte)0, //落ちていないので否定
        VOTE_CONFIRMED = (qByte)1, //落ちていたので肯定
        VOTE_CALLING = (qByte)0xf0, //呼び出し状態
        VOTE_DEAD = (qByte)0xf1, //死んだ状態
        VOTE_INVALID = (qByte)0xff
    };

    class Entry: public RootObject
    {
    public:
        Entry(TimeInterval deadLine, qInt maxStations);
        //maxStationsには、落ちたといわれたStationを含まない。自分は含む
        virtual ~Entry() {}


        enum State
        {
            INVALID,
            WAITING,
            CONFIRMED,
            DECLINED
        };

        State GetState()
        {
            UpdateState();
            return m_state;
        }

        String GetStateString()
        {
            return GetStateString(m_state);
        }
        static String GetStateString(State state);

        State SetVote(DOHandle fromStation, VoteState vote);

        qBool IsTimeout();

        qBool HasExpired();

        String ToString();

        void StationDeadCheck();

        TimeInterval GetElapsedTime();

    private:
        qMap<DOHandle, VoteState> m_voteStations;

        Time m_tStartTime;
        Time m_tConfirmDeadline;
        Time m_tExpiration;
        State m_state;
        qInt m_confirmedStations;
        qInt m_declinedStations;
        qInt m_maxStations;

        void UpdateState();
    };

    ConfirmFaultyStationTable();
    virtual ~ConfirmFaultyStationTable();

    Entry::State SetVote(DOHandle faultyStation, DOHandle fromStation, VoteState vote);

    Entry* GetEntryAndCreate(DOHandle faultyStation);

    String ToString(DOHandle faultyStation);

    static String GetVoteStateString(VoteState state);

    void DeleteExpiredEntries();

    static qUnsignedInt32 GetEntryTimeout()
    {
        return Stream::GetSettings(Stream::DO).GetMaxSilenceTime() + 2066;
    }

private:
    qMap<DOHandle, Entry*> m_faultyStationEntries;
};

END_NAMESPACE_NEX
