﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

////////////////////////////////////////////////////////////////////////////////
// Macros to ease the generation of the code for a specific DuplicateObject...
////////////////////////////////////////////////////////////////////////////////

//Update Macros
//{secret}

#define _DO_DatasetUpdate(DS,INDEX) \
    if (pDataset==NULL || pDataset==&DS) {  \
    qResult _result = GetDOClass()->GetUpdatePolicy(INDEX)->Update(this,static_cast<void*>(&DS),INDEX,tUpdateTime, DS.GetSubStreamID());    \
    if (!_result) { return _result;}    \
    if (pDataset==&DS) {return QRESULT_SUCCESS(Core, Success);} \
    }


//Refresh Macros
#define _DO_DataSetRefresh(DS)          \
    if (pDataset==NULL) {DS.Refresh(tRefreshTime);}         \
    if (pDataset==&DS) {return DS.Refresh(tRefreshTime);}

////////////////////////////////////////////////////////////////////////////////
// Information:
// DOClassCG::GenerateDOImplementation in DOClassCG.cpp writes this macro in the
// implementation output files.
////////////////////////////////////////////////////////////////////////////////
#define _DO_DSExtract(DS,INDEX)     \
    case INDEX: GetDOClass()->GetUpdatePolicy(INDEX)->ExtractFromUpdateMessage(this, static_cast<void*>(&DS), INDEX, pMsg); return true;
#define _DO_DSExtractFromDiscovery(DS,INDEX)    \
    GetDOClass()->GetUpdatePolicy(INDEX)->ExtractFromDiscoveryMessage(this, static_cast<void*>(&DS), INDEX, pMsg);

#define _DO_DSAddToDiscovery(DS,STATION,INDEX)  \
    GetDOClass()->GetUpdatePolicy(INDEX)->AddToDiscoveryMessage(this,static_cast<void*>(&DS),INDEX,STATION,pMsg)

#define _DO_DeclareGeneralMethods(DO)   \
    _DO_##DO(); virtual ~_DO_##DO() {};                                                     \
    qResult SpecificUpdate(DataSet *pDataset, const Time& tUpdateTime);             \
    qBool SpecificRefresh(DataSet *pDataset, const Time& tRefreshTime);             \
    void SpecificAddDSToDiscoveryMessage(Message* pMsg, Station* pNewStation);      \
    void SpecificExtractDSFromDiscoveryMessage(Message* pMsg);                      \
    qBool SpecificExtractADataset(Message* pMsg, qByte byDatasetIndex);             \
    static qBool InitDOClass(qUnsignedInt32 uiClassID);                             \
    virtual void CallOperationOnDatasets(DOOperation* pOperation, Operation::Event eEvent); \
    void RegisterDatasets();                                            \
    static DO* Create(qUnsignedInt32 uiTimeout = DuplicatedObject::DEFAULT_CREATE_TIMEOUT);        \
    static DO* Create(DOID idPreDefinedID);                             \
    private:                                                            \
    static DO* Create(DOHandle &dohHandle);                             \
    static DO* Create(DOClassID ciID, DOID uiUserDefinedID);            \
    public:                                                             \
    static DO* CreateWellKnown(WKHandle& hWK);                          \
    static DOHandle GetIDGenerator();                                   \
    typedef RefTemplate<DO> Ref;                                        \
    typedef SelectionIteratorTemplate<DO> SelectionIterator;            \
    static qUnsignedInt32 s_uiClassID;                                  \
    static DO* DynamicCast(DuplicatedObject* pDO)                       \
    {if (pDO!=NULL && pDO->IsAKindOf(s_uiClassID)) {return reinterpret_cast<DO*>(pDO);} else {return NULL;}}        \
    static const DO* DynamicCast(const DuplicatedObject* pDO)           \
    {if (pDO!=NULL && pDO->IsAKindOf(s_uiClassID)) {return reinterpret_cast<const DO*>(pDO);} else {return NULL;}}  \
    static qUnsignedInt32 GetStaticClassID() {return s_uiClassID;}      \
    static qUnsignedInt32 GetRealDOClassID() {return DOCLASSID(DO);}




#ifdef NEX_DEBUG_VERSION
#define _DO_RegisterDataset(DS) DS.RegisterDOContainer(this);
#else
#define _DO_RegisterDataset(DS)
#endif

#define _DO_InitDO(DO)  SetDOClassID(DOCLASSID(DO)); RegisterDatasets();

#define _DO_InitAutoID(DO) \
    qBool _DO_##DO::InitDOClass(qUnsignedInt32 uiClassID) {s_uiClassID=uiClassID;\
        QLOG(EventLog::Info,  NEX_T( #DO ) NEX_T("'s DOClassID is ") << dec << uiClassID << NEX_T("(") << hex << uiClassID << NEX_T(") [auto generate]"));\
        DOClass* pNewClass=qNew _DOC_##DO(uiClassID); pNewClass->CompleteInitialisation();\
        return true;}

#define _DO_DeclareAutoID() static const qBool HAS_STATIC_ID = false;


#define _DO_InitSetID(DO, ID) \
    qBool _DO_##DO::InitDOClass(qUnsignedInt32 uiClassID) {DUMMY_PARAM(uiClassID); s_uiClassID=ID; \
        QLOG(EventLog::Info, NEX_T( #DO ) NEX_T("'s DOClassID is ") << dec << ID << NEX_T("(") << hex << ID << NEX_T(") [static setting]"));\
        DOClass* pNewClass=qNew _DOC_##DO(ID); pNewClass->CompleteInitialisation();\
        return false;}

#define _DO_DeclareSetID() static const qBool HAS_STATIC_ID = true;

#define _DO_ImplementSomeGeneralMethods(DO)     \
    DO* _DO_##DO::Create( qUnsignedInt32 uiTimeout ) {return (DO*) DuplicatedObject::Create(GetRealDOClassID(), uiTimeout);}    \
    DO* _DO_##DO::Create( DOID idPreDefinedID ) {return (DO*) DuplicatedObject::Create(GetRealDOClassID(), idPreDefinedID);} \
    DO* _DO_##DO::CreateWellKnown(WKHandle& hWK) {if (!hWK.IsA(DO::GetRealDOClassID())) {SIGNAL_SYSTEMERROR(SYSTEMERROR_OBJDUP_INVALID_CONSTRUCTION,0); return NULL;} \
    return (DO*)DuplicatedObject::CreateWellKnown(hWK);}    \
    DOHandle _DO_##DO::GetIDGenerator() {return GetDOClass(GetRealDOClassID())->GetIDGenerator();}      \
    qUnsignedInt32 _DO_##DO::s_uiClassID=0;

#define _DO_DeclareActionStub(ACTION,ARGS)      \
    qBool ACTION##_OnMaster ARGS;               \
    qBool ACTION##_OnDuplicas ARGS;             \
    void Callee##ACTION##Stub(Message* pMsg);

#define _DO_DeclareLocationSet(SPACE)   SPACE::LocationSet  m_ls##SPACE;

#define _DOC_DuplicatedObject   DOClass

