﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

// A few macros used to ease the generation of the specific DOClass code.

#define _DOC_DeclareCallback(CLASS)                                                                         \
                                                                                                            virtual qBool ApproveFaultRecovery(DuplicatedObject* pDO);                                                  \
                                                                                                            virtual qBool ApproveEmigration(DuplicatedObject* pDO, qUnsignedInt32 uiReason);                            \
                                                                                                            virtual void Trace(DuplicatedObject* pDO, qUnsignedInt64 uiTraceFlag)

#define _DOC_ImplementCallback(CLASS)                                   \
                                                                        qBool _DOC_##CLASS::ApproveFaultRecovery(DuplicatedObject* pDO) {       \
                                                                        return ((CLASS*)pDO)->ApproveFaultRecovery();}                  \
                                                                        qBool _DOC_##CLASS::ApproveEmigration(DuplicatedObject* pDO, qUnsignedInt32 uiReason) {         \
                                                                        return ((CLASS*)pDO)->ApproveEmigration(uiReason);}                     \
                                                                        void _DOC_##CLASS::Trace(DuplicatedObject* pDO, qUnsignedInt64 uiTraceFlag) {((CLASS*)pDO)->Trace(uiTraceFlag);}


#define _DOC_DeclareGeneralMethods(CLASS)                   \
                                                            virtual void DataSetsOperation(qUnsignedInt32 uiOp);                \
                                                            virtual const qChar* GetClassNameString() const;                \
                                                            virtual const qChar* GetDatasetNameString(qByte byDatasetIndex) const;         \
                                                            virtual DuplicatedObject* Create();                     \
                                                            virtual void Delete(DuplicatedObject* pDO);             \
                                                            _DOC_##CLASS(qUnsignedInt32 uiClassID);         \
                                                            virtual qBool IsAKindOf(DOClassID idDOCID);             \
                                                            _DOC_DeclareCallback(CLASS);                                \
                                                            virtual qBool DispatchAction(DuplicatedObject* pDO, MethodID idMethod, Message* pMsg);      \
                                                            virtual qBool FormatVariableValue(const DuplicatedObject* pDO, Variable *pDataset, Variable *pVariable, String *pstrResult) const; \
                                                            virtual void DispatchRMCCall(const CallMethodOperation& refOp);                                         \
                                                            virtual qBool DispatchRMCResult(RMCContext* pContext);                                          \
                                                            virtual void FillDupSpacesInfo(DupSpace::Role eRole, DupSpace::ID* pArray, qUnsignedInt* puiIndex);

#define _DOC_ImplementSomeGeneralMethods(CLASS)                         \
                                                                        DuplicatedObject* _DOC_##CLASS::Create() {return qNew CLASS();}     \
                                                                        void _DOC_##CLASS::Delete(DuplicatedObject* pDO) {qDelete ((CLASS*)pDO);} \
                                                                        const qChar* _DOC_##CLASS::GetClassNameString() const {return NEX_T(#CLASS);}      \
                                                                        _DOC_ImplementCallback(CLASS)


#define _DOC_InitMethod(CLASS, METHOD) m_mid##METHOD=MethodIDGenerator::AssignID(#CLASS "::" #METHOD, (qByte)DOCLASS(CLASS)::GetStaticClassID());
#define _DOC_DeclareMethod(METHOD) MethodID m_mid##METHOD;

#define _DOC_DispatchAction(ACTION, DOCLASS)    \
                                                if (idMethod==m_mid##ACTION) {              \
                                                ((DOCLASS*)pDO)->Callee##ACTION##Stub(pMsg);            \
                                                return true;\
                                                }

#define _DOC_FormatMatchingVariableValue(CLASS, DSVAR)      \
                                                if (pDataset->GetName()==NEX_T(#DSVAR)) {return ((CLASS*)pDO)->DSVAR.FormatVariableValue(pVariable,pstrResult);}

#define _DOC_DispatchRMCCall(RMC, DOCLASS)  \
    if (refOp.GetTargetMethodID()==m_mid##RMC) {                        \
        static_cast<DOCLASS*>(refOp.GetAssociatedDO())->RMC##Wrapper(refOp);    \
        refOp.SetOutcome(QRESULT_SUCCESS(DOCore,Success));              \
        return;                                                     \
    }

#define _DOC_DispatchRMCResult(RMC, DOCLASS)                                    \
                                                                                if (pContext->GetTargetMethodID()==m_mid##RMC) {                            \
                                                                                DOCLASS::RMC##ReturnStub(pContext);                                 \
                                                                                return true;                                                        \
                                                                                }

#define _DOC_DeclareAction      _DOC_DeclareMethod
#define _DOC_DeclareRMC         _DOC_DeclareMethod
#define _DOC_InitAction         _DOC_InitMethod
#define _DOC_InitRMC            _DOC_InitMethod

#define _DOC_DupSpaceInfo(R,DS)     if ((eRole & DupSpace::R)==DupSpace::R) {                       \
                                    if (pArray) {pArray[*puiIndex]=DUPSPACEID(DS);}     \
                                    (*puiIndex)++;                                          \
}

#define _DOC_LocationSetUpdate(CLASS,SPACE)                     \
                                                                if (idDupSpace==DUPSPACEID(SPACE)) {((CLASS*)pDO)->m_ls##SPACE.Update(hStation, bAdd); return;}

#define _DOC_LocationSet(CLASS,SPACE)                       \
                                                            if (idDupSpace==DUPSPACEID(SPACE)) {return &(((CLASS*)pDO)->m_ls##SPACE);}

#define _DOC_RegisterProtocol(INDEX,PROTOCOL) if (uiOp==DATASET_OP_REGISTER) {CreateUpdatePolicy(INDEX); GetUpdatePolicy(INDEX)->RegisterProtocol(qNew PROTOCOL);}

#define _DOC_RegisterFilter(INDEX,FILTER) if (uiOp==DATASET_OP_REGISTER) {GetUpdatePolicy(INDEX)->AddFilter(qNew FILTER);}

#define _DOC_DSGetDatasetName(DS,INDEX)     \
    case INDEX: return DS::GetDatasetNameString()


