﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/StepSequenceJob.h>
#include <NetZ/src/Core/DOCore/Station.h>
#include <OnlineCore/src/Core/SystemComponent.h>

BEGIN_NAMESPACE_NEX

class JobChangeConnection: public StepSequenceJob
{
    // Group=
public:
    JobChangeConnection(const String& strJobDescription, DOHandle hStation);
    virtual ~JobChangeConnection();

    typedef enum _Type
    {
        DisconnectionJob,
        ConnectionJob
    } Type;

    virtual Type GetType() const =0;

#if     NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB
    virtual void CancelJob()
    {
        m_bCancelled=true;
    }
#else   // NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB
    void CancelJob()
    {
        m_bCancelled=true;
    }
#endif
    qBool HasBeenCancelled() const
    {
        return m_bCancelled;
    }

    qBool HoldsStationReference() const
    {
        return m_refStation.IsValid();
    }
    DOHandle GetAssociatedStationHandle() const
    {
        return m_refStation.GetHandle();
    }
    const Station* GetAssociatedStation() const
    {
        return *m_refStation;
    }
    Station* GetAssociatedStation()
    {
        return *m_refStation;
    }

    virtual Station::ConnectionState GetTargetConnectionState() const=0;

    void ReportJobCompletion();
    qBool ReportedComplete() const
    {
        return m_bCompletionReported;
    }

    virtual void Trace(qUnsignedInt64 uiTraceFlags);

private:
    /* Invalid operations */
    JobChangeConnection& operator=(const JobChangeConnection&);
    JobChangeConnection(const JobChangeConnection&);

protected:
    qBool m_bCancelled;
    Station::Ref m_refStation;
    qBool m_bCompletionReported;
};

END_NAMESPACE_NEX

