﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/Job.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <OnlineCore/src/Core/PeriodicJob.h>


#ifdef NEX_CLIENT_PBPOOL // クライアントのみ

BEGIN_NAMESPACE_NEX

/*! @internal
 @brief MessageをListに蓄えて処理するジョブ。
        バンドリングメッセージ展開中かどうかで呼び出す処理を変える。
 */
class Message;

class JobProcessMessageList: public PeriodicJob
{
private:
    JobProcessMessageList();
public:
    virtual ~JobProcessMessageList();
    /*! @internal
     @brief Listに蓄えたMessageに対し、ObjDupProtocol::Dispatch()/ObjDupProtocol::ProcessBundleMessage()を呼び出す
     */
    virtual void Execute();
    /*! @internal
     @brief ジョブにメッセージを追加。
     */
    void AddMessage(Message* pMessage, qBool processBundle);
private:
    /*! @internal
     @brief リストの先頭メッセージの削除。
     @param[in] toRelease Messageに対してReleaseMessage()するか
     */
    void RemoveFrontMessage(qBool toRelease);

public:
    /*! @internal
     @brief インスタンスの取得、無ければ作ってSchdulerに登録。
     */
    static JobProcessMessageList* GetInstance();
    /*! @internal
     @brief 通常時/バッファ枯渇時のウェイト時間を設定。
            default 通常時:0ms 枯渇時:5ms
            waitNormal <= 10, waitNoBuffer <= 100でないとfalse
     */
    static qBool SetWaitInterval(qUnsignedInt32 waitNormal, qUnsignedInt32 waitNoBuffer);

    virtual void SetDefaultPostExecutionState()
    {
        // PeriodicJobの実装のままだと、Periodを0msにするとExecute()後にJobCompleteになってしまう。
        // 枯渇解消時にMessageを0msウェイトでどんどん処理できるように仮想関数を別途実装する。
        // Execute()毎に0ms待ちのWaitingステートとなる。

        SetToWaiting(GetPeriod()); // s_waitNormalもしくはs_waitNoBufferの値で待つ
    }

#ifdef NEX_TRACE_ENABLED
    virtual void Trace(qUnsignedInt64 traceFlag)
    {
        // トレースとして、メッセージリストの長さを追加で出すようにする。
        PeriodicJob::Trace(traceFlag);
        QTRACE(traceFlag, NEX_T("MessageListSize:") << m_messageList.size());
    }
#endif // NEX_TRACE_ENABLED     

private:
    qList< std::pair<Message*, qBool> > m_messageList;
    static PseudoGlobalVariable<JobProcessMessageList*> s_pInstance;
    static qUnsignedInt32 s_waitNormal; // 通常ループでのウェイト
    static qUnsignedInt32 s_waitNoBuffer; // バッファ枯渇時でのウェイト
    // デフォルト値
    static const qUnsignedInt32 WAIT_NORMAL_DEFAULT = 0;
    static const qUnsignedInt32 WAIT_NORMAL_MAX = 10;
    static const qUnsignedInt32 WAIT_NOBUFFER_DEFAULT = 10;
    static const qUnsignedInt32 WAIT_NOBUFFER_MAX = 100;
};

END_NAMESPACE_NEX

#endif // NEX_CLIENT_PBPOOL

