﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/StationOperation.h>
#include <NetZ/src/Core/DOCore/Station.h>

BEGIN_NAMESPACE_NEX

/*!
 * @brief   ステーションがセッションから離脱するときに実行される @ref StationOperation です。
 *
 *          @ref Station::RegisterOperationBeginCallback 、@ref Station::RegisterOperationEndCallback で
 *          登録したコールバックで得られる @ref StationOperation の、@ref StationOperation::GetType が
 *          @ref Operation::LeaveStation だった場合に、LeaveStationOperation::DynamicCast を行うことで、
 *          本クラスへのポインタを取得できます。
 *
 *          <br>
 *          【 Station::RegisterOperationBeginCallback で登録したコールバックでの LeaveStationOperation の用途】<br>
 *          ・ステーションの離脱処理の開始を示します。<br>
 *          ・LeaveStationOperation::GetLeavingStation で取得できるステーションに関連する、アプリケーション独自データの削除を行う場合は、これ以降に行うと良いでしょう。<br>
 *          <br>
 *          <br>
 *          【 Station::RegisterOperationEndCallback で登録したコールバックでの LeaveStationOperation の用途】<br>
 *          ・ステーションの離脱処理完了を示します。<br>
 *          ・アプリケーション側での使用推奨例は特にありません。<br>
 */
class LeaveStationOperation: public StationOperation
{
public:

    LeaveStationOperation();
    virtual ~LeaveStationOperation();

    NEX_IMPL_OPERATION_COMMON_METHODS(LeaveStation);

private:
#ifdef NEX_TRACE_ENABLED
    virtual void TraceImpl(Event oEvent, qUnsignedInt64 uiTraceFlags)const;
#endif
};

END_NAMESPACE_NEX

