﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/EndPointEventHandler.h>
#include <OnlineCore/src/Transport/Interface/EndPoint.h>
#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DuplicatedObject.h>
#include <NetZ/src/Core/DOCore/CallRegister.h>
#include <NetZ/src/Core/DOCore/RMCContext.h>
#include <NetZ/src/Core/DOCore/ParticipationManager.h>
#include <NetZ/src/Core/DOCore/CodeGenerationExports.h>

BEGIN_NAMESPACE_NEX

class Message;
class FetchContext;
class MigrationContext;
class CallMethodOperation;
class ProcessAuthentication;
class StationIdentification;
class JobJoinSession;
class JobProcessMessage;

// Join responses.
#define SESSION_JOIN_INCOMPLETE                             0
#define WELCOME_TO_THE_SESSION                              1
#define SESSION_JOIN_DENIED_NOT_PRIMARY                     2
#define SESSION_JOIN_DENIED_INVALID_AUTHENTICATION          3
#define SESSION_JOIN_TEST_CONNECTIONS                       4
#define SESSION_JOIN_NOT_ALLOWED                            5

class ObjDupProtocol: public EndPointEventHandler
{
public:

    ObjDupProtocol();
    virtual ~ObjDupProtocol();

    enum Flag
    {
        // Global flags.
        IgnoreUnknownClasses        =1<<0,
        // Join flags.
        SkipTitleValidation         =1<<1,
        TransmitParseTree           =1<<2
    };

    // These two methods are responsible for handling conversions
    // between buffers and messages.
    virtual void Receive(EndPoint *pEndPoint, Buffer *pBuffer, qByte subStreamID);
private:
    // ObjDupProtocol内からの呼び出しのみ
    void Receive(EndPoint *pEndPoint, Buffer *pBuffer, qByte subStreamID, Message::TargetType type, qBool isLocalMessage);
public:
    qResult Send(EndPoint *pEndPoint, Message *pMessage, qUnsignedInt32 uiFlags);
    void QueueMessageFromLocalStation(Message *pMessage);

    // End point event handler virtual methods
    virtual void FaultDetection(EndPoint *pEndPoint, qUnsignedInt32 uiFault);
    virtual void PeerDisconnected(EndPoint *pEndPoint);

    Message* CreateMessage(qByte byMessageType);

    // Dispatch helpers
private:
    qBool PreDispatch(Message *pMessage);
    qByte DispatchExtractMessageType(Message *pMessage);
    void  DispatchTrace(Message *pMessage, qByte byType);
    qBool DispatchPreParse(JobProcessMessage* pJob, Message *pMessage, qByte byType, qBool* pIsJobProcessMessageCreated);
public:
    qBool Dispatch(JobProcessMessage* pJob, Message *pMessage, qBool* pIsJobProcessMessageCreated=NULL);

    Message* CreateGetParticipantsRequest();
    qBool ParseGetParticipantsRequestMessage(Message *pMessage, qBool bProcessIt, String *pStrTrace);
    void ProcessGetParticipantsRequest(EndPoint *pSourceEndPoint, Message *pMessage);

    Message* CreateGetParticipantsResponse();
    qBool ParseGetParticipantsResponseMessage(Message *pMessage, qBool bProcessIt, String *pStrTrace);
    void ProcessGetParticipantsResponse(Message *pMessage);

    Message* CreateJoinRequest();
    qBool ParseJoinRequestMessage(Message *pMessage, qBool bProcessIt, String *pStrTrace);
    void ProcessJoinRequest(EndPoint *pSourceEndPoint, ProcessAuthentication* pAuthentication, StationIdentification* pStationIdentification);

    Message* CreateJoinResponse(qByte byDetails);
    qBool ParseJoinResponseMessage(Message *pMessage, qBool bProcessIt, String *pStrTrace);
    void ProcessJoinResponse(Message* pMessage, qByte &byJoinResponse);

    Message* CreateUpdateMessage(qUnsignedInt32* puiID, qByte* pbyIndex);
    qBool ParseUpdateMessage(Message *pMessage, qBool bProcessIt, String *pStrTrace);

    Message* CreateDeleteMessage(DOHandle hDO);
    qBool ParseDeleteMessage(Message *pMessage, qBool bProcessIt, String *pStrTrace);
    void ProcessDeleteMessage(DOHandle hSender, DOHandle hDO);

    Message* CreateActionMessage(DOHandle* pidDO, MethodID* pidMethod);
    qBool ParseActionMessage(Message *pMessage, qBool bProcessIt, String *pStrTrace);
    void ProcessActionMessage(Message* pMessage, DuplicatedObject::Ref &ppDO, MethodID &idMethod);

    Message* CreateRMCCallMessage(RMCContext* pContext);
    qBool ParseRMCCallMessage(Message *pMessage, qBool bProcessIt, String *pStrTrace);
    qBool ProcessRMCCallMessage(Message* pMessage, RMCContext::ID &idRMC, DOHandle &hSource, qUnsignedInt32 &uiFlags, DOHandle &hTargetObject, MethodID &idMethod);

    Message* CreateRMCResponseMessage(CallMethodOperation* pContext);
    qBool ParseRMCResponseMessage(Message *pMessage, qBool bProcessIt, String *pStrTrace);
    void ProcessRMCResponseMessage(Message* pMessage, RMCContext::ID &idRMC);

    Message* CreateFetchRequestMessage(FetchContext* pFetchContext);
    qBool ParseFetchRequestMessage(Message *pMessage, qBool bProcessIt, String *pStrTrace);
    void ProcessFetchRequestMessage(DOHandle &hFetchedObject, DOHandle &hSourceStation, DOCallContext::ID &idCall);

    Message* CreateMigrationMessage(MigrationContext* pFetchContext);

    qBool ParseMigrationMessage(Message *pMessage, qBool bProcessIt, String *pStrTrace);


    Message* CreateCreateDuplicaMessage(const DOHandle& hObject, const DOHandle& hMaster, const qUnsignedInt8& byMasterVersion, const Buffer& bufObjectContent);

    qBool ParseCreateDuplicaMessage(Message *pMessage, qBool bProcessIt, String *pStrTrace);

    Message* CreateCreateAndPromoteDuplicaMessage(const RMCContext::ID& uiCallID, const DOHandle& hObject, const DOHandle& hMaster, const qUnsignedInt8& byMasterVersion, const Buffer& bufObjectContent, const qList<DOHandle>& lstDupSet);

    qBool ParseCreateAndPromoteDuplicaMessage(Message *pMessage, qBool bProcessIt, String *pStrTrace);

    void MigrateObject(const DDLTYPE(uint16)::CPPType& uiCallID,
                       const DDLTYPE(dohandle)::CPPType& hSender,
                       const DDLTYPE(dohandle)::CPPType& hObject,
                       const DDLTYPE(dohandle)::CPPType& hNewMaster,
                       const DDLTYPE(byte)::CPPType& byNewMasterVersion,
                       const DDLTYPE(qlist)< DDLTYPE(dohandle) >::CPPType& lstDupSet);

    void CreateDuplica(const DDLTYPE(dohandle)::CPPType& hObject,
                       const DDLTYPE(dohandle)::CPPType& hMaster,
                       const DDLTYPE(byte)::CPPType& byMasterVersion,
                       const DDLTYPE(buffer)::CPPType& bufObjectContent,
                       DOHandle hSender);
    void CreateAndPromoteDuplica(const DDLTYPE(uint16)::CPPType& uiCallID,
                                 const DDLTYPE(dohandle)::CPPType& hObject,
                                 const DDLTYPE(dohandle)::CPPType& hMaster,
                                 const DDLTYPE(byte)::CPPType& byMasterVersion,
                                 const DDLTYPE(buffer)::CPPType& bufObjectContent,
                                 const DDLTYPE(qlist)< DDLTYPE(dohandle) >::CPPType& lstDupSet,
                                 DOHandle hSender);

    Message* CreateCallOutcomeMessage(DOCallContext::ID idCall, DOCallContext::Outcome uOutcome);
    qBool ParseCallOutcomeMessage(Message *pMessage, qBool bProcessIt, String *pStrTrace);
    void ProcessCallOutcomeMessage(DOHandle hSender, DOCallContext::ID idCall, DOCallContext::Outcome uOutcome);

    qBool ProcessBundleMessage(Message *pMessageBundle, qBool bProcessIt, String *pStrTrace);

    Message* CreateEOSMessage(DOHandle hStation);
    void QueueEOS(DOHandle hStation);
    qBool ParseEOSMessage(Message *pMessage, qBool bProcessIt, String *pStrTrace);
    void ProcessEOSMessage(DOHandle &hStation);

    qBool AddLocalURLs(Station *pStation);

    CallRegister& CallRegisterRef()
    {
        return m_oCallRegister;
    }

    qBool ListenOnAnyPort();
    qBool ListenOnArbitrary();
    qBool ListenOnWellKnown();

    // Used as a callback.
    void StopToListen(qInt /*iDummy*/)
    {
        StopToListen();
    }
    void StopToListen();

    qBool IsListening(qUnsignedInt16 *pListenPort = NULL) const;
    qBool IsListeningOnWellKnown() const;
    qBool IsListeningOnArbitrary() const;

    //n1589:memo:bQueueFrontはScheduler::Queueのpushで使用される
    void QueueMessage(Message *pMessage, ConnectionID cidSource, EndPoint *pEndPoint, qBool bQueueFront = false, qBool isLocalMessage = false);
    ICContext GetInstanceContext() const
    {
        return m_icContext;
    }

#ifdef NEX_TRACE_ENABLED    // n2370
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
#else
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS)
    {
        (void)uiTraceFlags;
    }
#endif  // NEX_TRACE_ENABLED

    String GetMessageTypeString(Message *pMessage);
    String GetMessageTypeString(qUnsignedInt32 uiType);
    qBool ParseMessage(Message *pMessage, qBool bProcessIt, String *pStrTrace);
private:
    qBool ParseSpecificMessage(Message *pMessage, qUnsignedInt32 uiType, qBool bProcessIt, String *pStrTrace);
public:

    friend class JobListenOnWellKnown;

    // {secret}
    void SetFlag(qUnsignedInt32 uiFlag);
    // {secret}
    void ClearFlag(qUnsignedInt32 uiFlag);
    // {secret}
    qBool FlagIsSet(qUnsignedInt32 uiFlag) const
    {
        return (m_uiFlags & uiFlag)==uiFlag;
    }
    qUnsignedInt32 GetProtocolFlags() const
    {
        return m_uiFlags;
    }

    void SetPermittedProtocolFlags(qUnsignedInt32 uiValidFlags)
    {
        m_uiValidFlags=uiValidFlags;
    }
    qUnsignedInt32 GetPermittedProtocolFlags() const
    {
        return m_uiValidFlags;
    }
    qBool ProtocolFlagsArePermitted(qUnsignedInt32 uiNewStationFlags) const
    {
        return (m_uiValidFlags & uiNewStationFlags)==uiNewStationFlags;
    }

private:
    qBool SetPluginObject(PluginObject **pPluginObject, qChar *szGUID, qUnsignedInt32 uiPluginType);
    void ReleasePluginObject(PluginObject **pPluginObject);

public:
    static void AddMessageType(Message* pMsg, qByte byType);
private:
    static void ExtractMessageType(Message* pMsg, qByte* pbyType);

private:
    qBool ShouldGrabWellKnown();
    qBool CreateDuplicaImpl(const DDLTYPE(dohandle)::CPPType& hObject,
                            const DDLTYPE(dohandle)::CPPType& hMaster,
                            const DDLTYPE(byte)::CPPType& byMasterVersion,
                            const DDLTYPE(buffer)::CPPType& bufObjectContent,
                            DOHandle hSender);
    qBool DecodeBuffer(Buffer* pBuffer) const;
    void EncodeBuffer(Buffer* pBuffer) const;
    void SetMessageTargetType(Message *pMessage, DOHandle hDO);

private:

    qBool m_bListeningOnArbitrary;
    qBool m_bListeningOnWellKnown;

    CallRegister m_oCallRegister;

public:

    static qByte GetBundleMessageType();

    void ReleaseMessage(Message* pMessage);

    static ObjDupProtocol* GetInstance();

    void RegisterJoinJob(JobJoinSession* pJob)
    {
        m_pPendingJoin = pJob;
    }
    void UnregisterJoinJob()
    {
        m_pPendingJoin = NULL;
    }

#ifdef NEX_CLIENT_PBPOOL // クライアントのみ
    qBool IsEnoughPacketSpace()
    {
        size_t requiredPacketSpace = PacketBufferManager::GetInstance()->GetRequiredPacketSpace();

        // ObjDupProtocol::Send()時にパケットバッファプールへのコピーが発生する可能性があるので、
        // パケットバッファプールのみでMaxUnitSizeを所定の個数、事前に取れることを確認する。
        return PacketBufferManager::GetInstance() == NULL || PacketBufferManager::GetInstance()->IsAllocatable(RootTransport::DEFAULT_MTU, requiredPacketSpace);
    }
#endif // NEX_CLIENT_PBPOOL

private:
    JobJoinSession* m_pPendingJoin;
    ParticipationManager m_oParticipationManager;

    ICContext           m_icContext;

    qUnsignedInt32  m_uiFlags; // My own flags.
    qUnsignedInt32  m_uiValidFlags; // The flags that are valid for the processes that joins.

    qUnsignedInt16  m_uiWellKnownPort;
    qUnsignedInt16  m_uiArbitraryPort;
    qBool m_isLocalMessage; // SYSTEMCHECK用
};


END_NAMESPACE_NEX

