﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/SessionDiscoveryDecl.h>
#include <NetZ/src/Core/DOCore/SessionDiscoveryProtocol.h>
#include <NetZ/src/Core/DOCore/SessionDescription.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>

#include <OnlineCore/src/Platform/Core/qList.h>

BEGIN_NAMESPACE_NEX

class CallContext;

class SessionDiscoveryTable: public RootObject
{
public:
    SessionDiscoveryTable();
    virtual ~SessionDiscoveryTable();

    qBool FindSessions(CallContext* pCallContext, qList<SessionDescription>* pOutputList,
                       qBool (*pfSessionFilter)(const SessionDescription* pSessionDescription),
                       qUnsignedInt32 uiTimeout);
    void CompleteFindSession(qUnsignedInt32 uiCallContextID);
    void CancelFindSession(qInt iDummy);

    void Create(); // Calls Create session on the primary SessionDiscovery.
    void Delete(); // Calls Delete session on the primary SessionDiscovery.
    void UpdateDescription(); // Calls UpdateDescription on the primary SessionDiscovery.

    // TODO - Decide if we register the SessionDiscoveryProtocol, or a
    // SessionDiscoveryPlugin.
    qBool RegisterSessionDiscovery(SessionDiscoveryProtocol* pSessionDiscovery, qBool bSelectAsPrimarySessionDiscovery);
    qBool UnregisterSessionDiscovery(SessionDiscoveryProtocol* pSessionDiscovery);

    qBool SelectPrimarySessionDiscovery(SessionDiscoveryProtocol* pSessionDiscovery);
    qBool SelectPrimarySessionDiscovery(const qChar *szType);

    SessionDiscoveryProtocol* GetSessionDiscovery(const qChar *szType);

    // Eventually private...
    void PrepareRequest(qList<SessionDescription> *pSessionDescriptionList,
                        qBool (*pfSessionFilter)(const SessionDescription* pSessionDescription));
    void ClearRequest();
    void ProbeSessions();
    void GatherProbeResults();

    void ReceiveMessage(ConnectionOrientedStream *pStream, Buffer *pBuffer, const StationURL *pStationURL);

    // Services to the SessionDiscoveryProtocol.
    void AddSessionDescription(const SessionDescription& oNewSessionDescription);

    SessionDescription* GetLocalSessionDescription();
    void SetSessionMasterFlag()
    {
        m_bSessionMaster=true;
    }
    void ClearSessionMasterFlag()
    {
        m_bSessionMaster=false;
    }
    qBool IsSessionMaster() const
    {
        return m_bSessionMaster;
    }

    void SetReadyFlag()
    {
        m_bReady=true;
    }
    void ClearReadyFlag()
    {
        m_bReady=false;
    }
    qBool IsReady() const
    {
        return m_bReady;
    }

    CriticalSection& GetSessionDiscoveryListCS()
    {
        return m_csSessionDiscovery;
    }

private:
    CriticalSection m_csSessionDiscovery;
    qList<SessionDiscoveryProtocol*> m_lstSessionDiscovery;
    SessionDiscoveryProtocol* m_pPrimarySessionDiscovery;

    // Structure to receive session information, when probing.
    CriticalSection                 m_csSessionDescriptionList;
    qList<SessionDescription>   *m_pSessionDescriptionList;
    qBool (*m_pfSessionFilter)(const SessionDescription* pSessionDescription);

    // Data to keep local session information.
    SessionDescription m_oLocalSession;
    qBool m_bSessionMaster; // Authoritative???
    qBool m_bReady;
};

END_NAMESPACE_NEX





