﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/SystemError.h>

// group = System Errors

// Description: A method which is only valid on a Duplication master was called on a Duplica.
#define SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA   SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,0)

// Description: A method which is only valid on a Duplica was called on a Duplication master.
#define SYSTEMERROR_OBJDUP_INVALID_ON_MASTER    SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,1)

/*
Description: An operation was called that would cause an invalid object property transition.
Remarks: An example of such an operation is to call DuplicatedObject::DeleteMainRef
more than once on a duplicated object.
*/
#define SYSTEMERROR_OBJDUP_INVALID_PROPERTY_TRANSITION  SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,2)

/*
Description: An attempt was made to call a non-existent duplicated object method.
Remarks: Check the DDL file and ensure that the function is declared.

Note: Callbacks are now obsolete, please see Iterators Overview for more details.
{secret}
*/
#define SYSTEMERROR_OBJDUP_UNDEFINED_CALLBACK   SET_ERROR_CODE(SEV_WARNING,SUBSYSTEM_OBJDUP,3)

/*
Description: The duplicated object was not found.
Remarks: This is usually because the DOHandle passed to
a function is invalid or the object represented by the DOHandle no longer exists.
*/
#define SYSTEMERROR_OBJDUP_DO_NOT_FOUND     SET_ERROR_CODE(SEV_WARNING,SUBSYSTEM_OBJDUP,4)

// Description: One or more arguments to the function are invalid.
#define SYSTEMERROR_OBJDUP_INVALID_PARAMETER    SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,5)

/*
Description: An attempt was made to migrate an object to an invalid location.
Remarks: Examples of invalid locations are the migration of an object to the local
station and the migration of an object to a StationID that does not exist or
that has earlier failed.
See Also: DuplicatedObject::AttemptEmigration
*/
#define SYSTEMERROR_OBJDUP_CANNOT_EMIGRATE  SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,6)

/*
Description: An attempt was made at publishing a duplicated object using a pointer that
was obtained by manually instantiating the DuplicatedObject.
Remarks: The correct way to create a duplicated object is via the method DuplicatedObject::Create.
See Also: DuplicatedObject::Create, DuplicatedObject::Publish
*/
#define SYSTEMERROR_OBJDUP_INVALID_CONSTRUCTION  SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,7)

/*
Description: A duplicated object cannot be published more than once.
See Also: DuplicatedObject::Publish
*/
#define SYSTEMERROR_OBJDUP_ALREADY_PUBLISHED    SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,8)

/*
Description: A dataset specified a custom update policy but the corresponding callback was not defined.
See Also: DataSet::ReliableUpdate
*/
#define SYSTEMERROR_MISSING_DATASET_CALLBACK    SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,9)

// Description: The duplicated object corresponding to a declared well-known handle has not been initialized yet.
#define SYSTEMERROR_OBJDUP_WKHANDLE_NOT_INITIALIZED SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,10)

/*
Description: A duplicated object corresponding to a well-known handle has been
created on a node that joins a session.
*/
#define SYSTEMERROR_OBJDUP_WKHANDLE_WERE_CREATED SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,11)

/*
Description: The duplicated object that a DOHandle refers to does not match the
type of the reference being constructed.
*/
#define SYSTEMERROR_OBJDUP_MISMATCHED_DOCLASS SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,12)

// Description: A duplicated object corresponding to a well-known handle has been created twice.
#define SYSTEMERROR_OBJDUP_WKHANDLE_EXISTS      SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,13)

/*
Description: A generic duplicated object cannot be published.
See Also: DuplicatedObject::Publish
*/
#define SYSTEMERROR_OBJDUP_CANNOT_PUBLISH   SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,14)

/*
Description: The station was not found.  This is usually because the StationID passed to a
function is invalid or represents a station that has left the session.
*/
#define SYSTEMERROR_OBJDUP_STATION_NOT_FOUND        SET_ERROR_CODE(SEV_WARNING,SUBSYSTEM_OBJDUP,15)

/*
Description: An attempt was made to create or join the session more than once.
See Also: Session::CreateSession, Session::JoinSession.
*/
#define SYSTEMERROR_DC_CLUSTER_REDEFINITION SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,16)

/*
Description: An attempt was made to join a session and the station hosting the session
denied the join request.
See Also: Session::JoinSession.
*/
#define SYSTEMERROR_DC_CLUSTER_JOIN_DENIED  SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,17)

/*
Description: The join operation failed due to a transient network failure.
See Also: Session::JoinSession.
*/
#define SYSTEMERROR_DC_CLUSTER_JOIN_FAILED  SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,18)

// {Secret}
#define SYSTEMERROR_DC_MISMATCHED_BARRIER_ID    SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,19)

// {Secret}
#define SYSTEMERROR_DC_BARRIER_TIMEOUT      SET_ERROR_CODE(SEV_INFORMATIONAL,SUBSYSTEM_OBJDUP,20)

/*
Description: An operation that requires the session to be initialized was called, but
the session had not been initialized beforehand.
See Also: Session::CreateSession, Session::JoinSession
*/
#define SYSTEMERROR_DC_CLUSTER_NOT_INITIALIZED  SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,21)

// Description: An operation that must be called on the session master was called on a secondary node.
#define SYSTEMERROR_DC_NOT_CLUSTER_MASTER   SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,22)

// Description: An invalid system state transition occurred.
#define SYSTEMERROR_DC_INVALID_STATE_TRANSITION SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,23)

/*
Description: A method that must be called prior to calling Session::CreateSession, Session::JoinSession, or
Session::CreateOrJoinSession was called after any one of these three methods.
See Also: Session::CreateSession, Session::JoinSession, Session::CreateOrJoinSession
*/
#define SYSTEMERROR_DC_CLUSTER_ALREADY_INITIALIZED SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,24)

/*
Description: No responsible transport found for the specified URL.  The
transport type was not recognized by the loaded transports.
*/
#define SYSTEMERROR_OBJDUP_TRANSPORT_NOT_FOUND SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,25)

//n1589:エラーとする必要があるか調査中、それまでは下のワーニングを使用
//#define SYSTEMERROR_OBJDUP_RELIABLE_SEND_FAILURE SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_OBJDUP,26)  //n1769:未使用のため、除去。

//n1769
#define SYSTEMERROR_OBJDUP_RELIABLE_SEND_WARNING SET_ERROR_CODE(SEV_WARNING,SUBSYSTEM_OBJDUP,27)


