﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/UpdateContext.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <OnlineCore/src/Platform/Core/qMap.h>


BEGIN_NAMESPACE_NEX
// An UpdateContextMap maps a <DOID, dataset index> to an UpdateContext.
// Every Station contains an update context map.

class DOOperation;
class DuplicatedObject;

class UpdateContextMap: public RootObject
{
public:
    UpdateContextMap();
    virtual ~UpdateContextMap();

    void Add(DOHandle idDO, qUnsignedInt32 uiDataSetIndex, UpdateContext* pNewContext);
    void Remove(DOHandle idDO, qUnsignedInt32 uiDataSetIndex);
    void RemoveAll(DOHandle idDO);
    UpdateContext* Find(DOHandle idDO, qUnsignedInt32 uiDataSetIndex);

    size_t GetNbUpdateContextes() const
    {
        return m_mapUC.size();
    }
    static void RemoveCorrespondingContextes(DOOperation* pOperation);

private:

    static void RemoveCorrespondingContextes(DuplicatedObject* pDO);

private:

    qInt64 CreateKey(DOHandle idDO, qUnsignedInt32 uiDataSetIndex);

private:
    qMap<qInt64, UpdateContext*> m_mapUC;

};

END_NAMESPACE_NEX



