﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>

BEGIN_NAMESPACE_NEX


class DOSelections;

// Group=ID Classes

/*
Summary: Refers to a well-known duplicated object.
Remarks: A WKHandle refers to a well-known duplicated object.  Whenever a well-known duplicated object
is declared in the DDL, a global variable of the WKHandle type is generated by the DDL compiler.  The WKHandle
class instances inherit from the DOHandle class.
See Also: DOHandle
*/

/*!
 * @brief   既知複製オブジェクトへの参照ハンドルです。
 *
 *          WKHandleクラスは既知複製オブジェクトへの参照です。
 *          既知オブジェクトがDDL(データ定義言語)ファイルで宣言されるたびに、
 *          WKHandle型のグローバル変数がDDLコンパイラによって生成されます。
 *
 *          このWKHandleクラスは、DOHandleクラスから派生します。
 */
class WKHandle: public DOHandle
{
    // Group=Methods
public:

    /*
    Remarks: A WKHandle object should never be directly instantiated. Instead, it is created by $PRODUCTNAME$
    prior to session creation as a result of a DDL well-known delaration.
    See Also:  DDL File Syntax, DOHandle
    */

    /*!
     * @brief   このコンストラクタはアプリケーション側から呼び出さないでください。セッション作成時にシステムが自動的に作成します。
     */
    WKHandle();

    /*
    Remarks: A WKHandle object should never be directly destroyed. It is destroyed by
    $PRODUCTNAME$ when the session terminates.
    See Also: DDL File Syntax, DOHandle
    */

    /*!
     * @brief   このデストラクタはアプリケーション側から呼び出さないでください。セッション終了時にシステムが自動的に破棄します。
     */
    virtual ~WKHandle();

    // {Secret}
    void Init(DOClassID idDOClass);

    qBool InitCancel();
    // {Secret}
    static void Cleanup();

    // Error checking methods...
    // {Secret}
    static qBool AllInDOS();
    // {Secret}
    static qBool AtLeastOneCreated();
    // {secret}
    static void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
    // {Secret}
    void Instanciated()
    {
        m_bInstanciated.SetValue(PseudoSingleton::GetCurrentContext(),true);
    }
    // {Secret}
    qBool IsInstanciated()
    {
        return m_bInstanciated.GetValue();
    }

    // {secret}
    static qBool Initialized()
    {
        return (s_phFirst.GetValue()!=NULL);
    }

    qBool IsInitialized()
    {
        return m_bInitialized;
    }

    // {Secret}
    static qBool IsAWKHandle(DOHandle hDO);

private:
    OSDK_DISABLE_EXPORT_WARNING(PseudoGlobalVariable<qBool> m_bInitialized);
    OSDK_DISABLE_EXPORT_WARNING(PseudoGlobalVariable<qBool> m_bInstanciated);

    // The list of all WKHandles could be implemented as a qChain -
    // the only reason it was not done like this is that qChain
    // didn't exist when this was created.
    OSDK_DISABLE_EXPORT_WARNING(PseudoGlobalVariable<WKHandle*> m_phNext);
    OSDK_DISABLE_EXPORT_WARNING(static PseudoGlobalVariable<WKHandle*> s_phFirst);

    OSDK_DISABLE_EXPORT_WARNING(static PseudoGlobalVariable<qUnsignedInt32> s_iCurrentID);
};

END_NAMESPACE_NEX

