﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DupSpace/DupSpaceOperation.h>
#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <OnlineCore/src/Platform/Core/Time.h>

BEGIN_NAMESPACE_NEX
class DuplicatedObject;
#undef GetClassName

//group=DDL Output

/*
Description: $PRODUCTNAME$ calls a MatchOperation to perform the calls DuplicationSpace::PSMatch
or DuplicationSpace::CellMatch.
Remarks:
When a MatchOperation is called on a Duplication space, the following sequence of events occurs:

    o   The method DuplicationSpace::OperationBegin is called,
    o   The appropriate DuplicationSpace::PSMatch or DuplicationSpace::CellMatch sequence is called,
    o   According to the return value of the DuplicationSpace::PSMatch or DuplicationSpace::CellMatch function the AddToStoreOperation or RemoveFromStoreOperation may be triggered,
    o   The method DuplicationSpace::OperationEnd is called.

*/
class MatchOperation: public DupSpaceOperation
{

    //Group=Methods
public:

    // This macro is never defined.  It is used only to get around a compilation problem with
    // DocJet, as _MatchTypes cannot be made secret.
#ifdef FOR_DOC_PURPOSE

#else

    typedef enum _MatchType
    {
        CheckRelevance,
        MatchSingleObject,
        MatchAcrossPublishers, MatchAcrossSubscribers,
        MatchAcrossCells, MatchAcrossPS,
        Dummy
    } MatchType;

#endif

    //{secret}
    MatchOperation(DuplicationSpace* pDupSpace, MatchType eMatchType,
                   DOHandle hDOArgument=INVALID_DOHANDLE);
    //{secret}
    virtual ~MatchOperation();

    //{secret}
    const qChar* GetClassName(DuplicatedObject* pDO) const;

    //{secret}
    static void Queue(DuplicationSpace* pDupSpace, MatchType eMatchType,
                      DOHandle hDOArgument=INVALID_DOHANDLE, TimeInterval tiWhenFromNow=0);
    //{secret}
    static void Execute(DuplicationSpace* pDupSpace, MatchType eMatchType,
                        DOHandle hDOArgument=INVALID_DOHANDLE);

    //{secret}
    void ExecuteOperation();
    //{secret}
    void ExecuteQueuedOperation(qInt iDummy);
    //{secret}
    MatchType GetMatchType()
    {
        return m_eMatchType;
    }

    NEX_IMPL_OPERATION_COMMON_METHODS( Match );

private:

    virtual void TraceImpl(Event eEvent, qUnsignedInt64 uiTraceFlags) const;

private:
    MatchType m_eMatchType;
    DOHandle m_hDOArgument;
};

END_NAMESPACE_NEX

