﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>

#include <NetZ/src/Extensions/Adapter/AdapterEvent.h>
#include <NetZ/src/Core/DOCore/DORef.h>

BEGIN_NAMESPACE_NEX

//group=Adapters

class Adapter;

/*
Description: This class
Remarks:
*/
class AdapterDOEvent: public AdapterEvent
{

    // group=Methods

public:
    AdapterDOEvent(Type eEventType,
                   DORef refAssociatedDO,
                   Adapter** ppAssociatedAdapterPtr,
                   qUnsignedInt uiAdapterIndex,
                   qBool bDuplicationMaster);
    virtual ~AdapterDOEvent();

    DuplicatedObject* GetAssociatedDO();
    Adapter* GetAssociatedAdapter()
    {
        return *m_ppAssociatedAdapterPtr;
    }
    Adapter** GetAssociatedAdapterPtr()
    {
        return m_ppAssociatedAdapterPtr;
    }
    qUnsignedInt GetAdapterIndex()
    {
        return m_uiAdapterIndex;
    }

    /*
    Description: Returns true if the DuplicatedObject associated with this Adapter is a Duplication master.
    Returns: true if the duplicated object associated with this Adapter is a Duplication master, false if the
    object is a duplica.
    */
    qBool IsADuplicationMaster() const
    {
        return m_bDuplicationMaster;
    }

    /*
    Description: Returns true if the DuplicatedObject associated with this Adapter is a Duplica.
    Returns: true if the duplicated object associated with this Adapter is a Duplica, false if the
    object is a Duplication master.
    */
    qBool IsADuplica() const
    {
        return !m_bDuplicationMaster;
    }

    virtual void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);

private:
    qBool m_bDuplicationMaster;
    DORef m_refAssociatedDO;
    Adapter** m_ppAssociatedAdapterPtr;
    qUnsignedInt m_uiAdapterIndex;
};

END_NAMESPACE_NEX

