﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/PseudoSingleton.h>
#include <OnlineCore/src/Core/SystemComponent.h>
#include <NetZ/src/Extensions/Adapter/AdapterEventDispatcher.h>

BEGIN_NAMESPACE_NEX

//Group=Adapters

/*
Description: This extension has been deprecated.
Remarks: This extension has been deprecated. See System Operations and information on Duplicated Objects instead.
This class indicates to $PRODUCTNAME$ that an adapter will be used. If you wish to use an Adapter, the AdapterExtension must be registered to $PRODUCTNAME$ via Register to
ensure that the system knows that it should use an Adapter. If not the appropriate files will not be loaded
and you will end up with compiler errors.
See Also: Net-Z Integration, System Operations, Duplicated Object Overview
*/
// {noAutoLink}
class AdapterExtension: public SystemComponent
{
public:
    NEX_DECLARE_SYSCOMPONENT_METHODS(AdapterExtension,SystemComponent);

    //group=Methods

    AdapterExtension();
    virtual ~AdapterExtension();

    //{secret}
    static AdapterExtension* GetInstance()
    {
        return (AdapterExtension*) InstanceControl::FindDelegatorInstance(IC_TYPE_ADAPTER_EXT, PseudoSingleton::GetCurrentContext());
    }

    //{secret}
    virtual qBool BeginTermination();

    /*
    Description: Registers an AdapterExtension to $PRODUCTNAME$.
    Returns: true if the AdapterExtension was successfully registered to $PRODUCTNAME$, false if not
    or an error occurred.
    Remarks: This method must be called after the $PRODUCTNAME$ object is created and before a Session
    is either created or joined. For example,

            // Create $PRODUCTNAME$ object
            $PRODUCTNAME$ *g_p$PRODUCTNAME$ = qNew $PRODUCTNAME$;

            // Register the AdapterExtension
            AdapterExtension::Register

            // Create the Session
            if(!(Session::CreateSession())){
                // Process the session creation error
                // SystemError
            }

    See Also: Session::CreateSession, Session::JoinSession
    */
    static qBool Register();

    // {secret}
    AdapterEventDispatcher* GetEventDispatcher()
    {
        return &m_oDispatcher;
    }

private:
    AdapterEventDispatcher m_oDispatcher;
    PseudoSingleton m_oInstanceControl;
};

END_NAMESPACE_NEX

