﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/Holder.h>
#include <OnlineCore/src/Transport/Interface/Message.h>
#include <NetZ/src/Extensions/Any/AnyObjectAdapter.h>

BEGIN_NAMESPACE_NEX

template <typename ROOT, typename TYPE>
class AnyObjectHolder: public Holder<ROOT>
{
    //Group=Methods
public:
    AnyObjectHolder(ROOT *pObject=NULL): Holder<ROOT>(pObject) {}
    virtual ~AnyObjectHolder() {}

    // The following operations are implemented to allow
    // a AnyObjectHolder to be placed in a list.  This is the
    // equivalent of the operations performed by a Trule - to
    // transfer ownership from a holder to another one (see C++
    // templates, the complete guide, p. 377).

    AnyObjectHolder(AnyObjectHolder<ROOT,TYPE> const& oOther):
        Holder<ROOT>(const_cast<AnyObjectHolder<ROOT,TYPE>&>(oOther).Release())
    {
    }
    AnyObjectHolder<ROOT,TYPE>& operator= (AnyObjectHolder<ROOT,TYPE> const& oOther)
    {
        Holder<ROOT>::operator=(const_cast<AnyObjectHolder<ROOT,TYPE>& >(oOther).Release());
        return *this;
    }

    // By definition, two holders cannot refer to the same object.
    qBool operator==(AnyObjectHolder<ROOT,TYPE> const&) const
    {
        return false;
    }

    AnyObjectHolder<ROOT,TYPE>& operator=(ROOT* p)
    {
        Holder<ROOT>::operator=(p);
        return *this;
    }
    void StreamIn(Message* pMessageToSend) const
    {
        SYSTEMCHECK(s_pAnyAdapter!=NULL);
        s_pAnyAdapter->StreamIn(pMessageToSend, *Holder<ROOT>::Get());
    }


    void StreamOut(Message* pReceivedMessage)
    {
        ROOT* pResult;
        s_pAnyAdapter->StreamOut(pReceivedMessage, &pResult);
        operator=(pResult);
    }

public:
    static AnyObjectAdapter<ROOT,TYPE>* s_pAnyAdapter;

};

template <typename ROOT, typename TYPE> AnyObjectAdapter<ROOT,TYPE>* AnyObjectHolder<ROOT,TYPE>::s_pAnyAdapter = NULL;

#ifdef TEST_ENABLED
extern qBool AnyObjectHolderTest();
#endif


END_NAMESPACE_NEX

