﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/UpdateContext.h>
#include <NetZ/src/Extensions/PHBDR/PHBDRVar.h>
#include <OnlineCore/src/Platform/Core/Time.h>

BEGIN_NAMESPACE_NEX


class Message;


class PHBDRModel: public UpdateContext
{
public:

    PHBDRModel()
    {
        m_byMessagesReliability=0;
    }
    virtual ~PHBDRModel() {}

    void SetModel(qInt& v, qReal dValue, qReal dRateOfChange, qReal dAcceleration, Time tTimestamp=SessionClock::GetTime());

    qInt32 GetValue(const PHBDRParameters& oParams, PHBDRVAR(qInt32)& v);
    qInt16 GetValue(const PHBDRParameters& oParams, PHBDRVAR(qInt16)& v);
    qInt8 GetValue(const PHBDRParameters& oParams, PHBDRVAR(qInt8)& v);
    qDouble GetValue(const PHBDRParameters& oParams, PHBDRVAR(qDouble)& v);
    qFloat GetValue(const PHBDRParameters& oParams, PHBDRVAR(qFloat)& v);

    qInt32 PredictValue(const PHBDRParameters& oParams, PHBDRVAR(qInt32)& v, const Time& tPredictionTime)
    {
        return v.PredictValue(oParams, m_vectorTime, tPredictionTime);
    }
    qInt16 PredictValue(const PHBDRParameters& oParams, PHBDRVAR(qInt16)& v, const Time& tPredictionTime)
    {
        return v.PredictValue(oParams, m_vectorTime, tPredictionTime);
    }
    qInt8 PredictValue(const PHBDRParameters& oParams, PHBDRVAR(qInt8)& v, const Time& tPredictionTime)
    {
        return v.PredictValue(oParams, m_vectorTime, tPredictionTime);
    }
    qDouble PredictValue(const PHBDRParameters& oParams, PHBDRVAR(qDouble)& v, const Time& tPredictionTime)
    {
        return v.PredictValue(oParams, m_vectorTime, tPredictionTime);
    }
    qFloat PredictValue(const PHBDRParameters& oParams, PHBDRVAR(qFloat)& v, const Time& tPredictionTime)
    {
        return v.PredictValue(oParams, m_vectorTime, tPredictionTime);
    }

    qFloat ComputeESE(const PHBDRParameters& oParams, qInt rv, PHBDRVAR(qInt32)& ev, const Time& tPredictionTime);
    qFloat ComputeESE(const PHBDRParameters& oParams, qInt rv, PHBDRVAR(qInt16)& ev, const Time& tPredictionTime);
    qFloat ComputeESE(const PHBDRParameters& oParams, qInt rv, PHBDRVAR(qInt8)& ev, const Time& tPredictionTime);
    qDouble ComputeESE(const PHBDRParameters& oParams, qDouble rv, PHBDRVAR(qDouble)& ev, const Time& tPredictionTime);
    qFloat ComputeESE(const PHBDRParameters& oParams, qFloat rv, PHBDRVAR(qFloat)& ev, const Time& tPredictionTime);

    void AddSourceTo(Message* pMsg) const;

    TimeInterval CurrentUpdateDelay(Time tNow=SessionClock::GetTime()) const;
    qBool ModelIsReliable(qByte byRequiredPointsMask=0x7) const;
    void AddPointReliabilityData(qBool bReliable);

    const PHBDRTimeVector& GetTimeVector() const;

protected:

    PHBDRTimeVector m_vectorTime;
    qByte m_byMessagesReliability;
};

END_NAMESPACE_NEX

