﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#pragma once

#include <cstring>
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/StringStream.h>



BEGIN_NAMESPACE_NEX

/**
 * @internal
 * @brief PRUDPメッセージの署名データを管理するクラス
 */
class SignatureBytes: public RootObject
{
    friend StringStream& operator<<(StringStream& oStr, const SignatureBytes& oSign);
public:
    static const size_t SIGNATURE_SIZE = 16;

    SignatureBytes()
    {
        mSize = 0;
    }

    SignatureBytes(const SignatureBytes& obj)
    {
        Set(obj.mSign, obj.mSize);
    }

    void Set(const qByte sign[], size_t size)
    {
        SYSTEMCHECK(size <=  SIGNATURE_SIZE);
        memcpy(mSign, sign, size);

        mSize = size;
    }

    const qByte* Get() const
    {
        return mSign;
    }

    size_t GetSize() const
    {
        return mSize;
    }

    qByte* GetHead()
    {
        return mSign;
    }

    void SetSize(size_t size)
    {
        SYSTEMCHECK(size <=  SIGNATURE_SIZE);
        mSize = size;
    }


    qBool Equal(const SignatureBytes& obj) const
    {
        if (obj.GetSize() != GetSize())
        {
            return false;
        }

        return memcmp(obj.Get(), Get(), GetSize()) == 0;
    }

    qBool operator==(const SignatureBytes &obj) const
    {
        return Equal(obj);
    }
    
    qBool operator!=(const SignatureBytes &obj) const
    {
        return !Equal(obj);
    }


    SignatureBytes &operator =(const SignatureBytes& obj)
    {
        Set(obj.mSign, obj.mSize);
        return *this;
    }

    String ToString();

private:
    qByte mSign[SIGNATURE_SIZE];
    size_t mSize;
};

StringStream& operator<<(StringStream& oStr, const SignatureBytes& oSign);


END_NAMESPACE_NEX

