﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
//n1769


#pragma once

#include <OnlineCore/src/Core/Checksum/Interface/ChecksumDecl.h>
#include <OnlineCore/src/Core/Checksum/Interface/KeyedChecksumAlgorithm.h>
#include <OnlineCore/src/Core/Checksum/MD5/MD5Checksum.h>


BEGIN_NAMESPACE_NEX


class Buffer;

///< これは実データの末尾に鍵データをくっつけて作成したデータをMD5で署名計算するためのクラスです。
///< 鍵データを与えなかった場合（あるいは鍵長がゼロ）の場合の挙動は、MD5Checksum
///< クラスと等価となるように設計しています。
class MD5ChecksumWithKey : public KeyedChecksumAlgorithm
{
public:
    MD5ChecksumWithKey(void);

    virtual ~MD5ChecksumWithKey(void);

    virtual qBool ComputeChecksum(const qByte** pBuffer, const size_t nSize[], size_t length, SignatureBytes &signature);

    virtual qBool ComputeChecksum(const Buffer &oBuffer, Buffer *pChecksum);

    virtual qByte GetChecksumLength(void)
    {
        return 16;    // MD5なので、ダイジェスト長は128bit（＝16byte）
    }

    // {secret}
    virtual qUnsignedInt32 ComputeChecksumForTransportArray( const qByte** pBuffer, const size_t nSize[], size_t length );

    static qBool Test(void);

protected:
    //n1769: このクラスは、鍵がセットされていてもいなくても署名計算をして欲しいので、
    //       常にtrueを返すようにした。
    virtual qBool IsReady(void) const
    {
        return true;
    }


private:
    ///< コピーコンストラクタ、代入演算子は封印します。
    MD5ChecksumWithKey           (const MD5ChecksumWithKey&);
    MD5ChecksumWithKey& operator=(const MD5ChecksumWithKey&);
    void ChecksumComputeHelper( const qByte** pBuffer, const size_t nSize[], size_t length, MD5 &oOuter);

};


END_NAMESPACE_NEX


