﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/RootObject.h>
#include <OnlineCore/src/Platform/Core/qVector.h>

// This class is used to make it easier to parse the command line for
// the various tools and samples of the system.

BEGIN_NAMESPACE_NEX

class OSDK_API CommandLineParser : public RootObject
{
public:
    ~CommandLineParser();

    void Release();

    // Generic syntax:
    // -<option1> <option_arg1> <option_arg2> ... -option2 -option3 ...

    void Parse( qChar** szArgv, qInt iArgc );
    void Parse( const qChar *szCmdLine );

    size_t GetNbOptions();
    qChar *GetOptionName( size_t iIndex );

    qBool IsSet( const qChar *szOption );
    qVector<qChar*> *GetOptionArguments(const qChar *szOption);
    qVector<qChar*> *GetOptionArguments(size_t iIndex);


    // Parsing functions.
    static void CmdLineToArgV( qChar *szCmdLine, qChar **&szArgv, qInt *iArgc );
    static void FreeArgV( qChar **szArgv, qInt iArgc );
    static qInt CountArguments( qChar *szCmdLine, size_t iLineLength );
    static void ExtractArguments( qChar *szCmdLine, size_t iLineLength, qChar **szArgv );

private:
    struct Option
    {
        qChar *m_szName;
        qVector<qChar*> m_vArguments;
    };

    Option *FindOption( const qChar *szOption );

    OSDK_DISABLE_EXPORT_WARNING(qVector<Option> m_vOptions);
};

END_NAMESPACE_NEX

