﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/Compression/Interface/CompressionDecl.h>
#include <OnlineCore/src/Core/Compression/Interface/CompressionAlgorithm.h>
#include <OnlineCore/src/Platform/Core/Type.h>

BEGIN_NAMESPACE_NEX

class Buffer;

//group=Compression

/*
Description: This class implements Net-Z's ZLib compression algorithm.
Remarks: For further details about the ZLib compression algorithm refer to
{hyperlink:http://www.zlib.net/|http://www.zlib.net/}.
*/
class ZLibCompression : public CompressionAlgorithm
{
public:
    //group=Methods

    ZLibCompression();
    ZLibCompression(qInt windowBits, qInt memLevel);
    virtual ~ZLibCompression();

    // n2208
    static const qInt DEFAULT_WINDOW_BITS = -1;
    static const qInt DEFAULT_MEM_LEVEL   = -1;

private:
    void Initialize(qInt windowBits, qInt memLevel);

    //{secret}
    virtual CompressStatus CompressImpl(const Buffer &oInBuffer, Buffer *pOutBuffer);

    //{secret}
    virtual DecompressStatus DecompressImpl(const Buffer &oInBuffer, Buffer *pOutBuffer);

    qUnsignedInt16 Compression(qByte *pbSource, qUnsignedInt16 wSourceSize, Buffer *pOutBuffer);
    qUnsignedInt16 Decompression(qByte* pbSource, qUnsignedInt16 wSourceSize, Buffer *pOutBuffer);

    // We use the pimpl idiom in order to remove the dependency
    // to zlib.h header file. We want to avoid the distribution
    // of zlib with our plugin.
    //{secret}
    struct Impl;
    Impl* m_pImpl;
};

END_NAMESPACE_NEX

