﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Platform/Core/SystemError.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Core/InstantiationContext.h>

BEGIN_NAMESPACE_NEX

typedef qVector<InstantiationContext*> ContextVector;
const ICContext DEFAULT_CONTEXT = 0;
const ICContext INVALID_CONTEXT = 0xFFFF;

class InstanceTable: public RootObject
{
public:
    InstanceTable();
    virtual ~InstanceTable();

    ICContext CreateContext();
    qBool DeleteContext(ICContext icInstanceContext);

    ICContext GetHighestID() const;

    qBool AddInstance(InstanceControl* icElement, ICType icInstanceType, ICContext icInstanceContext);
    qBool AddInstanceToVector(InstanceControl* icElement, ICType icInstanceType, ICContext icInstanceContext);
    void DelInstance(InstanceControl* icElement, ICType icInstanceType, ICContext icInstanceContext);
    qBool DelInstanceFromVector(InstanceControl* icElement, ICType icInstanceType, ICContext icInstanceContext);

    qBool ContextIsValid(ICContext icInstanceContext)
    {
        if (icInstanceContext == DEFAULT_CONTEXT)
        {
            return true;
        }
        if (icInstanceContext < m_pvContextVector->size() &&
                (*m_pvContextVector)[icInstanceContext] != NULL)
        {
            return true;
        }
        return false;
    }

    InstanceControl* FindInstance(ICType icInstanceType, ICContext icInstanceContext)
    {
        if(icInstanceContext == DEFAULT_CONTEXT)
        {
            return m_oDefaultContext.GetInstance(icInstanceType);
        }

        SYSTEMCHECK(m_pvContextVector != NULL);
        SYSTEMCHECK(icInstanceContext < m_pvContextVector->size());
        if(icInstanceContext >= m_pvContextVector->size())
        {
            SIGNAL_SYSTEMERROR(SYSTEMERROR_GEN_INDEX_OOB,0);
            return NULL;
        }

        // This systemcheck will crash if the current context has
        // been deleted.
        SYSTEMCHECK((*m_pvContextVector)[icInstanceContext] != NULL);
        return  (*m_pvContextVector)[icInstanceContext]->GetInstance(icInstanceType);
    }

    ICContext   FindInstanceContext(InstanceControl* icElement, ICType icInstanceType);

    void AllocateExtraContexts(size_t uiNewSize);
    void FreeExtraContexts();

private:
    InstantiationContext m_oDefaultContext;
    ContextVector* m_pvContextVector;
};


END_NAMESPACE_NEX



