﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
// n2973
#include <OnlineCore/src/Platform/Core/SystemError.h>

BEGIN_NAMESPACE_NEX

class InstanceControl;

// For NEX Developer <wiki/Pseudo Singleton#IDs>
enum
{
    IC_TYPE_FACADE=0,
    IC_TYPE_NETWORK=1,
    IC_TYPE_DUPLICATION_SPACE_TABLE=2,
    IC_TYPE_CORE=3,
    IC_TYPE_DOCORE=4,
    IC_TYPE_ADAPTER_EXT=5,
    IC_TYPE_RV_FRAMEWORK=6,
    IC_TYPE_SERVICECORE=7,
    IC_TYPE_SPARK_UIFLOW=8,
    IC_TYPE_SPARK_LOBBY_BINDING=9,

    IC_TYPE_CONNECTIVITY_MANAGER=11,
    IC_TYPE_RVCLIENTCORE=12,
    IC_TYPE_PRUDPUSERPROTOCOL=13,
    IC_TYPE_PACKET_BUFFER_MANAGER=14,
    IC_TYPE_MEMBERSHIP_POLICY_MANAGER=15,
    NB_INSTANCE_TYPES=16
};

typedef qUnsignedInt ICType;
typedef qUnsignedInt ICContext;


class InstantiationContext: public RootObject
{
public:
    InstantiationContext();
    InstantiationContext(InstanceControl* icElement, ICType icInstanceType);
    virtual ~InstantiationContext();

    qBool AddInstance(InstanceControl* icElement, ICType icInstanceType);
    qBool DelInstance(InstanceControl* icElement, ICType icInstanceType);
    // For NEX Developer <wiki/Pseudo Singleton#GetPseudoSingleton>
    // n2973 高速化のためinline化
    inline InstanceControl* GetInstance(ICType icInstanceType);

    void InitContext();
    qBool IsValid()
    {
        return m_IsValid;
    }

private:
    // For NEX Developer <wiki/Pseudo Singleton#PseudoSingletonArray>
    InstanceControl* m_icInstanceContext[NB_INSTANCE_TYPES];
    qBool m_IsValid;
};

// n2973 高速化のためInstantiationContext.cppから移動
// Return the instance if we found it. NULL if a problem occurred. Set error flag.
InstanceControl* InstantiationContext::GetInstance(ICType icInstanceType)
{
    if(icInstanceType >= NB_INSTANCE_TYPES)
    {
        SIGNAL_SYSTEMERROR(SYSTEMERROR_GEN_INDEX_OOB,0);
        return NULL;
    }

    return m_icInstanceContext[icInstanceType];
}


END_NAMESPACE_NEX



