﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/NonCopyable.h>
#include <OnlineCore/src/Core/StepSequenceJob.h>

BEGIN_NAMESPACE_NEX

class CallContext;

class JobCallContextCallback : public StepSequenceJob, public NonCopyable
{
public:

    JobCallContextCallback(JobType type, const qChar* szDescription, CallContext* associatedCallContext);
    virtual ~JobCallContextCallback();

    //Jobをセットしてスタートさせる。falseの場合は、RELEASEREFでこのオブジェクトを削除 trueの場合は、Schedulerが削除するので、削除は不要
    //スタート前にGetCallContext()で取得できるCallContextがすでに初期化されてCallContextRegisterに登録されている必要がある。
    qBool StartJob();

    void CompleteJob(const qResult& result);

    void StepCallStart();
    void StepCallCompletion() { StepCallCompletionImp(); }
    void StepFinish();

    //newしたあと、このGetCallContext()を使って、CallContextを初期化する必要がある
    CallContext* GetCallContext()
    {
        if (m_pCallContext == NULL)
        {
            m_pCallContext = CreateCallContext();
        }
        return m_pCallContext;
    }

protected:
    virtual CallContext* CreateCallContext();

    //CallCompletion時に呼び出されるステップの実装 StepCallCompletion自体は、virtual化しても派生クラスのが呼ばれないため
    //こちらに実装
    virtual void StepCallCompletionImp() { StepFinish(); }
    //成功時に呼び出される pCCは、コンストラクタのassociatedCallContextと同じ
    virtual void ProccessSuccess(CallContext* pCC) { DUMMY_PARAM(pCC); }

    //失敗時に呼び出される  pCCは、コンストラクタのassociatedCallContextと同じ
    virtual void ProccessFailure(CallContext* pCC) { DUMMY_PARAM(pCC); }
    virtual void ProccessCancel() {}
    qUnsignedInt32 m_AssociatedCallID;

private:

    CallContext* m_AssociatedCallContext;
    CallContext* m_pCallContext;
};

END_NAMESPACE_NEX

