﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformImpl.h>
#include <OnlineCore/src/Platform/Core/SystemClock.h>


BEGIN_NAMESPACE_NEX


// Group=Time Management

/*
Description:  This class has been deprecated. Do not use. Use SystemClock instead.

Remarks:  Provides the time, in milliseconds, on the local station since the creation of the process.
Provides the time, in milliseconds, on the local station since the creation of the $PRODUCTNAME$ object.
The time provided by the local clock is not synchronized across
the stations that belong to the session. This class should not be
used anymore, the SystemClock should be used instead.

See also: SessionClock, Time, SystemClock
*/
class LocalClock: public RootObject
{

    // Group=Methods
public:
    // {Secret}
    static LocalClock* Instance()
    {
        if (!s_pInstance)
        {
            s_pInstance=qNew LocalClock();
        }
        return s_pInstance;
    }
    // {Secret}
    static void DeleteInstance();
    // {Secret}
    static void CreateInstance()
    {
        Instance();
    }

#ifdef WIN_FAMILY
    friend class Chrono;
private:
#endif

    /*
    Remarks: A LocalClock object should never be directly instantiated. Instead, it is created by
    $PRODUCTNAME$ when a station joins or creates a session and initialized to 0 ms.
    */
    LocalClock();

    /*
    Remarks: A LocalClock object should never be directly destroyed. It is destroyed by $PRODUCTNAME$
    when the local station leaves the session or the session terminates.
    */
    virtual ~LocalClock();

public:

    //Summary: Returns the current local time in milliseconds.
    //Returns: The current local time in milliseconds.
    static Time GetTime()
    {
        return Instance()->Now();
    }

    // {Secret}
    Time Now() const
    {
        return Time(SystemClock::GetTime() - m_oBaseClockTime);
    }

private:
    static LocalClock* s_pInstance;

#ifdef WIN_FAMILY
    Time m_tiRatio;
#endif

    Time    m_oBaseClockTime;
};

END_NAMESPACE_NEX


