﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <stdlib.h>  /* For abs() */

BEGIN_NAMESPACE_NEX

template<class Type, class SignedType>
class LogicalClockTmpl: public RootObject
{
public:
    LogicalClockTmpl()
    {
        m_Value=0;
    }
    LogicalClockTmpl(const LogicalClockTmpl& lcClock):RootObject()
    {
        m_Value=lcClock.m_Value;
    }
    LogicalClockTmpl(Type Value)
    {
        m_Value=Value;
    }
    LogicalClockTmpl(qInt iValue)
    {
        m_Value=static_cast<Type>(iValue);
    }
    virtual ~LogicalClockTmpl() {};

    // TODO - The signature/implementation of these methods is not
    // correct

    void operator++()
    {
        m_Value++;
    }
    void operator++(qInt iDummy)
    {
        DUMMY_PARAM(iDummy);
        m_Value++;
    }
    void operator--()
    {
        m_Value--;
    }
    void operator--(qInt iDummy)
    {
        DUMMY_PARAM(iDummy);
        m_Value--;
    }


    LogicalClockTmpl operator+(Type Increment) const
    {
        return LogicalClockTmpl(Type(m_Value+Increment));
    }

    LogicalClockTmpl operator-(Type Decrement) const
    {
        return LogicalClockTmpl(Type(m_Value-Decrement));
    }

    SignedType operator-(const LogicalClockTmpl &lcClock) const
    {
        return static_cast<SignedType>(m_Value - lcClock.m_Value);
    }

    LogicalClockTmpl& operator=(const LogicalClockTmpl& lcClock)
    {
        m_Value=lcClock.m_Value;
        return *this;
    }
    
    static Type ClockMaximumNbValues()
    {
        return (static_cast<Type>(1) << (8 *sizeof(Type) - 1)) - 1 ;
    }
    
    static Type ClockMaximumValues()
    {
        return static_cast<Type>(static_cast<SignedType>(-1)) ;
    }

    static LogicalClockTmpl Max(const LogicalClockTmpl& oA, const LogicalClockTmpl& oB)
    {
        if (oA>oB)
        {
            return oA;
        }
        else
        {
            return oB;
        }
    }

    qBool operator>(const LogicalClockTmpl& lcClock) const
    {
        return static_cast<SignedType>(*this - lcClock) > 0;
    }
    qBool operator==(const LogicalClockTmpl& lcClock) const
    {
        return m_Value==lcClock.m_Value;
    }
    qBool operator>=(const LogicalClockTmpl& lcClock) const
    {
        return (*this==lcClock) || (*this>lcClock);
    }
    qBool operator<=(const LogicalClockTmpl& lcClock) const
    {
        return lcClock>=*this;
    }
    qBool operator<(const LogicalClockTmpl& lcClock) const
    {
        return lcClock>*this;
    }
    qBool operator!=(const LogicalClockTmpl& lcClock) const
    {
        return !(lcClock==*this);
    }

    Type GetValue() const
    {
        return m_Value;
    }

private:
    Type m_Value;
};

typedef LogicalClockTmpl<qUnsignedInt8, qInt8> LogicalClock;
typedef LogicalClockTmpl<qUnsignedInt8, qInt8> LogicalClock8;
typedef LogicalClockTmpl<qUnsignedInt16, qInt16> LogicalClock16;
typedef LogicalClockTmpl<qUnsignedInt32, qInt32> LogicalClock32;

#ifdef OSDK_EXPORT_TEMPLATE
#define OSDK_EXPORT_LOGICALCLOCK(Type)                            \
    OSDK_DISABLE_EXPORT_WARNING(template class OSDK_API LogicalClockTmpl<Type >);
#else
#define OSDK_EXPORT_LOGICALCLOCK(Type)
#endif


END_NAMESPACE_NEX

