﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/LogDevice.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/qSet.h>

BEGIN_NAMESPACE_NEX

//group= Log Devices

/*
Description: This class implements logging to multiple devices.
*/
class MultipleLogDevice : public LogDevice
{
public:
    //group=Methods

    /*
    Parameters:
    bFreePointersOnDestruction: If true, frees the pointers to the associated LogDevices
    when the MultipleLogDevice is destroyed. The default is false.
    */
    MultipleLogDevice(qBool bFreePointersOnDestruction=false);
    virtual ~MultipleLogDevice();

    virtual void Output(const qChar *pLog);

    /*
    Description: Adds the specified LogDevice to this MultipleLogDevice.
    Parameters:
    pLogDevice: A pointer to the LogDevice that will be added to this MultipleLogDevice.
    See Also: RemoveLogDevice, RemoveAll
    */
    qBool AddLogDevice(LogDevice* pLogDevice);

    /*
    Description: Removes the specified LogDevice from this MultipleLogDevice.
    Parameters:
    pLogDevice: A pointer to the LogDevice that will be removed from this MultipleLogDevice.
    bFreePointer: A Boolean that when true frees the pointer to the specified pLogDevice when
    the LogDevice is removed from this MultipleLogDevice. The default is false.
    See Also: AddLogDevice, RemoveAll
    */
    qBool RemoveLogDevice(LogDevice* pLogDevice, qBool bFreePointer=false);

    /*
    Description: Returns the size of this MultipleLogDevice.
    Returns: The size in number of bytes of this MultipleLogDevice.
    */
    size_t GetSize()
    {
        return m_setLogDevices.size();
    };

    /*
    Description: Removes all LogDevice s that are currently associated with this MultipleLogDevice.
    Parameters:
    bFreePointers: If true, frees the pointers to the LogDevice s being removed.
    The default is false.
    See Also: RemoveLogDevice, AddLogDevice
    */
    void RemoveAll(qBool bFreePointers=false);

private:
    void OutputImpl(const qChar *pLog);

    CriticalSection m_csSetDevices;
    qSet<LogDevice*> m_setLogDevices;
    qBool m_bFreePointersOnDestruction;
};

END_NAMESPACE_NEX


