﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Core/CallPolicy.h>
#include <OnlineCore/src/Platform/Core/ObjectThreadRoot.h>

BEGIN_NAMESPACE_NEX

//==============================================================================
// This CallPolicy enforces that only one thread calls a method, at a given
// time. It is also possible to enforce that only one specific thread calls it
// (AcceptOneThread), as well as to restrict the maximum recursion level for a
// method. See the tests for examples.
//
// NOTE: Using the class in release mode as no effect (only ASSERT)
//
// SingleThreadCallPolicyは同時に一つのスレッドからコールされていることを
// 保証するポリシー。複数のスレッドから同時に呼び出された場合はSYSTEMCHECKに引っかかる。
// SYSTEMCHECKのみを行うためリリースビルドでは何もチェックしない
//==============================================================================
class SingleThreadCallPolicy : public CallPolicy
{
public:
    SingleThreadCallPolicy()
        : m_uiCurrentRecursionLevel(0)
        , m_uiCurrentThread(NEX_INVALID_THREAD_ID)
        , m_uiMaximumRecursionLevel(1)
        , m_uiValidThread(NEX_INVALID_THREAD_ID)
    {
    }
    virtual ~SingleThreadCallPolicy() {}

    void AcceptOneThread(ThreadID uiThreadID);
    void AcceptAnyThread();

    void SetMaximumRecursionLevel(qUnsignedInt32 uiMaximumRecursionLevel);
    qUnsignedInt32 GetCurrentRecursionLevel() const
    {
        return m_uiCurrentRecursionLevel;
    }

    virtual qBool CurrentThreadIsCalling() const;
    virtual qBool OtherThreadIsCalling() const;

private:
    // Not implemented
    SingleThreadCallPolicy(const SingleThreadCallPolicy&);
    SingleThreadCallPolicy& operator=(const SingleThreadCallPolicy&);

    // SingleThreadCallPolicyではBeginCallImpl()は必ずtrueを返すそのため常に続行可能
    virtual qBool BeginCallImpl();
    virtual void EndCallImpl();

private:
    // Current call sequence information
    qUnsignedInt32 m_uiCurrentRecursionLevel;
    ThreadID m_uiCurrentThread;

    // Restrictions
    qUnsignedInt32 m_uiMaximumRecursionLevel;
    ThreadID m_uiValidThread;
};

END_NAMESPACE_NEX

