﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Platform/Core/ObjectThread.h>
#include <OnlineCore/src/Platform/Core/qVector.h>

BEGIN_NAMESPACE_NEX

class WorkerThreads: public RootObject
{
public:
    typedef enum _State {Running, Stopping, Stopped} State;

    WorkerThreads();
    virtual ~WorkerThreads();

    qBool Start(qUnsignedInt uiNbThreads);

    qBool Stop();

    size_t GetNbWorkers() const;
    qBool IsRunning() const;

    qBool IsAWorker(ThreadID uiThreadID) const;
    qBool CurrentThreadIsAWorker() const;

    State GetState();

    static void SetThreadPriority(qUnsignedInt priority)
    {
        s_threadPriority = priority;
    }
    static qUnsignedInt GetThreadPriority()
    {
        return s_threadPriority;
    }

    static void SetThreadCoreNo(qInt32 no)
    {
        s_threadCoreNo = no;
    }

    static qInt32 GetThreadCoreNo()
    {
        return s_threadCoreNo;
    }
protected:

    virtual void Initialize() {}
    virtual void Teardown() {}

private:

    void Run(qInt iDummy);

    virtual void Work()=0;

    static qUnsignedInt s_threadPriority;
    static qInt32 s_threadCoreNo;
    typedef ObjectThread<WorkerThreads, qInt> WorkerThread;

    CriticalSection m_csState;
    State m_eState;

    qVector<WorkerThread*> m_vecThreads;
};

END_NAMESPACE_NEX

