﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

//==============================================================================
// Marshalling code generation macros.
//
// The src/ddl/BackEnd/DatasetCG.cpp source inserts those macros into the
// generated output code (*DDL.cpp) files.
//
//==============================================================================

BEGIN_NAMESPACE_NEX

#define _DS_ARRAY_OP(OP, SIZE)              {qInt i; for (i=0; i<SIZE; i++) {OP;}}

#define _DS_ARRAY_OP_BEGIN(SIZE)            {qInt i; for (i=0; i<SIZE; i++)
#define _DS_ARRAY_OP_END()                  }

// Generic marshalling macros.

#define _MSG_ADD_VARIABLE(METATYPE,VAR)                             METATYPE::Add(pMsg, VAR);
#define _MSG_ADD_VARIABLE_ARRAY(METATYPE,VAR,SIZE)                  _DS_ARRAY_OP(_MSG_ADD_VARIABLE(METATYPE, VAR[i]), SIZE);
#define _MSG_ADD_VARIABLE_WITH_ERROR_CHECK_BEGIN(METATYPE,VAR)      _MSG_ADD_VARIABLE(METATYPE, VAR); if(pMsg->ErrorHasOccured()) {QLOG(EventLog::Error, NEX_T("Encode Error:") NEX_T(#METATYPE) NEX_T(" ") NEX_T(#VAR));
#define _MSG_ADD_VARIABLE_WITH_ERROR_CHECK_END()                    };
#define _MSG_ADD_VARIABLE_ARRAY_WITH_ERROR_CHECK_BEGIN(METATYPE,VAR,SIZE)      _MSG_ADD_VARIABLE_ARRAY(METATYPE, VAR, SIZE); if(pMsg->ErrorHasOccured()) {QLOG(EventLog::Error, NEX_T("Encode Error:") NEX_T(#METATYPE) NEX_T(" ") NEX_T(#VAR));
#define _MSG_ADD_VARIABLE_ARRAY_WITH_ERROR_CHECK_END()                         };

#define _MSG_EXTRACT_VARIABLE(METATYPE,VAR)                     METATYPE::Extract(pMsg, &VAR); if(pMsg->ErrorHasOccured()) {QLOG(EventLog::Error, NEX_T("Parse Error:") NEX_T(#METATYPE) NEX_T(" ") NEX_T(#VAR)); return;};
#define _MSG_EXTRACT_VARIABLE_ARRAY(METATYPE,VAR,SIZE)          _DS_ARRAY_OP(_MSG_EXTRACT_VARIABLE(METATYPE, VAR[i]), SIZE);

// Containers special cases.

#define _MSG_ADD_CONTAINER(METATYPE,VAR,UPDATE_ALL)             METATYPE::Add(pMsg, VAR, UPDATE_ALL);
#define _MSG_ADD_CONTAINER_ARRAY(METATYPE,VAR,SIZE,UPDATE_ALL)  _DS_ARRAY_OP(_MSG_ADD_CONTAINER(METATYPE, VAR[i], UPDATE_ALL), SIZE);

#define _MSG_EXTRACT_CONTAINER(METATYPE,VAR)                    METATYPE::Extract(pMsg, &VAR); if(pMsg->ErrorHasOccured()) {QLOG(EventLog::Error, NEX_T("Parse Error:") NEX_T(#METATYPE) NEX_T(" ") NEX_T(#VAR)); return;};
#define _MSG_EXTRACT_CONTAINER_ARRAY(METATYPE,VAR,SIZE)         _DS_ARRAY_OP(_MSG_EXTRACT_CONTAINER(METATYPE, VAR[i]), SIZE);

// Particular marshalling macros.
// TODO: Should eventually be cleaned up, and use the generic ones.

#define _DS_ADD_TIMESTAMP()         *pMsg << tTimestamp;
#define _DS_ADD_BREAK()             m_oBreak.AddSourceTo(pMsg, bDiscovery);

#define _DS_EXTRACT_TIMESTAMP()     Time tTimeStamp; *pMsg >> tTimeStamp; PHBDRParameters::ApplyExtrapolationDelay(&tTimeStamp);
#define _DS_EXTRACT_BREAK()         ContinuityBreak oContinuityBreak(pMsg);

// The following macro allows to notify each member attribute of a dataset of a
// DO operation event (beginning/ending). The OperationBegin and OperationEnd
// referred here are implemented in "MarshallingCGMacros.h" and "DOCoreTypes.h".

#define _MSG_OPERATION_EVENT(METATYPE, VAR)             \
    if (eEvent == Operation::Begin) {                   \
        METATYPE::OperationBegin(&VAR, pDOOperation);   \
    }                                                   \
    else if (eEvent == Operation::End) {                \
        METATYPE::OperationEnd(&VAR, pDOOperation);     \
    }

END_NAMESPACE_NEX

