﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/ParseTree/Declaration.h>
#include <OnlineCore/src/ParseTree/NameSpaceRef.h>
#include <OnlineCore/src/ParseTree/NameSpace.h>
#include <OnlineCore/src/ParseTree/DOClassID.h>

BEGIN_NAMESPACE_NEX

class OSDK_API DOClassDeclaration: public Declaration
{
public:
    DOClassDeclaration(const String& strName, DOClassDeclaration* pParent=NULL);
    virtual ~DOClassDeclaration();

    DECLARE_NAMESPACE_ITEM(DOClassDeclaration, Declaration);

    virtual void SetNameSpace(NameSpace *pNS);

#ifdef NEX_TRACE_ENABLED
    virtual void Trace(qUnsignedInt64 uiTraceFlag) const;
#endif

    qResult StreamIn(BinaryRepresentation *pStream) const;
    qResult StreamOut(BinaryRepresentation *pStream);

    virtual qBool IsEqual(const ParseTreeItem* pOther) const;

    DOClassDeclaration* GetParent() const;
    void AssignParent(DOClassDeclaration* pParent);

    virtual qBool ResolveReferences();

    virtual qBool MayHaveProperties() const
    {
        return true;
    }
    virtual Declaration* GetParentDeclaration() const
    {
        return GetParent();
    }

    NameSpace& NS()
    {
        return m_nsDOClass;
    }
    const NameSpace& NS() const
    {
        return m_nsDOClass;
    }

    void SetID(DOClassID uiID)
    {
        m_uiID=uiID;
    }
    DOClassID GetID() const
    {
        return m_uiID;
    }

    virtual void ForEachSubItem(void (*pf)(ParseTreeItem*))
    {
        Declaration::ForEachSubItem(pf);
        m_nsDOClass.ForEachItem(pf);
    }

    static void AddDefaultProperty(PropertyDeclaration* pDefaultProperty);

private:
    OSDK_DISABLE_EXPORT_WARNING(static qVector<PropertyDeclaration*> s_vecDefaultProperties);

    NameSpaceRef m_refParent;
    NameSpace m_nsDOClass;

    DOClassID m_uiID;
};

END_NAMESPACE_NEX

