﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

BEGIN_NAMESPACE_NEX
// System reserved DOClassIDs:

enum
{
    INVALID_CLASSID            =0,
    DuplicatedObject_ClassID   =1,
    FIRST_COREDOCLASSID        =2,
    FIRST_USERDOCLASSID            =100,
    MAX_DOCLASSID        =255
};

/*
    \ingroup group_onlinecore_core
    \brief A DOClassID is a unique identifier for a given DOClass.

    A DOClassID is a 8-bit value automatically generated by the DDL compiler for each duplicated
    object class. As this value is generated, it may change when you recompile your project, alter
    your DDL file, or upgrade to a new version of this framework.

    For example, if the following is defined in the DDL file:

         doclass CustomSessionObject;

    The DDL compiler will generate the appropriate DOClassID that can be accessed via the DOCLASSID
    macro as follows:

         DOCLASSID(CustomSessionObject)

    The DOHandle of a duplicated object is made up of the object's DOClassID and its DOID.
    \sa DDL File Syntax, Duplicated Object Overview, DOCLASSID
*/

/*!
 * @brief   複製オブジェクトクラスを識別するためのユニークなIDです。
 *
 *          DOClassIDは便宜上 符号なし32bit値(@ref qUnsignedInt32)のtypedefになっていますが、
 *          実際はそのうちの8bitを利用します。
 *
 *          値はすべて自動的にDDL(データ定義言語)コンパイラによって、
 *          すべての複製オブジェクト(@ref DuplicatedObject)に対して割り当てられます。
 *          この値はNet-Zによって自動的に生成されるため、複製オブジェクトクラスのIDは
 *          DDLファイルを変更した場合に、DOClassIDが異なる場合が生じます。
 *          互換性を保つためには、複製オブジェクトについての互換性の項を参考にしてください。
 *
 *          複製オブジェクトハンドル(@ref DOHandle)は、複製オブジェクトクラスIDと
 *          その複製オブジェクト自体のID(@ref DOID)によって構成されています。
 *
 *          より詳しくはNEXプログラミングマニュアルに掲載されている、
 *          DDLファイル文法や、複製オブジェクトの拡張機能の「DOの上位互換について」の項を参照してください。
 */
typedef qUnsignedInt32 DOClassID;

END_NAMESPACE_NEX

