﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/ParseTree/NameSpaceItem.h>
#include <OnlineCore/src/ParseTree/NameSpace.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/ParseTree/NameSpaceRef.h>

BEGIN_NAMESPACE_NEX

class PropertyDeclaration;
class NameSpaceRef;
class DDLUnitDeclaration;

class OSDK_API Declaration: public NameSpaceItem
{
public:
    Declaration(const String& strName): NameSpaceItem(strName) {}
    virtual ~Declaration() {}

    String GetSourceDDL() const;
    String GetGeneratedCodeDir() const;

    DDLUnitDeclaration* GetDDLUnit() const;
    void SetDDLUnit(DDLUnitDeclaration* pDDLUnit);
    void SetDDLUnit(const String& strUnitName);

    qResult StreamIn(BinaryRepresentation *pStream) const;
    qResult StreamOut(BinaryRepresentation *pStream);

    qBool StreamInDeclarationUse(BinaryRepresentation *pStream) const;
    static qBool StreamOutDeclarationUse(BinaryRepresentation *pStream, NameSpaceRef* prefDeclaration);

    void AddProperty(PropertyDeclaration* pNewDeclaration);
    void SetProperties(qVector<PropertyDeclaration*> vecProperties);
    qBool HasProperty(const String& strPropertyName) const;
    qBool HasInheritableProperty(const String& strPropertyName) const;
    PropertyDeclaration* GetProperty(const String& strPropertyName);
    PropertyDeclaration* GetFirstPropertyByCategory(qUnsignedInt32 uiCategory) const;

    qBool IsConcrete() const;
    qBool IsExportable() const;

    // Redefined by declarations that have properties...
    virtual qBool MayHaveProperties() const
    {
        return false;
    }
    void FillPropertyList(qList<PropertyDeclaration*>* pList, qBool bMustBeInheritable=false);

    // Default - no parent.  Will be redefined by declaration that
    // implement inheritance.
    virtual Declaration* GetParentDeclaration() const
    {
        return NULL;
    }
    virtual String GetDeclarationString() const
    {
        return GetName();
    }

    virtual void ForEachSubItem(void (*pf)(ParseTreeItem*))
    {
        m_nsProperties.ForEachItem(pf);
    }

    virtual qBool ResolveReferences();
    virtual qBool ReleaseReferences();

#ifdef NEX_TRACE_ENABLED
    virtual void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS) const;
#endif

    NameSpace& GetProperties()
    {
        return m_nsProperties;
    }
    const NameSpace& GetProperties() const
    {
        return m_nsProperties;
    }

    DECLARE_NAMESPACE_ITEM(Declaration, NameSpaceItem);
private:

    NameSpaceRef m_refDDLUnit;
    NameSpace m_nsProperties;
};

END_NAMESPACE_NEX

