﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Platform/Core/String.h>

BEGIN_NAMESPACE_NEX

class BinaryRepresentation;
class NameSpace;

#define DECLARE_PARSETREE_ITEM(CLASS, PARENTCLASS)          \
    CLASS(BinaryRepresentation* pStream): PARENTCLASS(pStream) {StreamOut(pStream);}    \
    virtual Type GetType() const {return T##CLASS;}                                 \
    virtual qBool IsAKindOf(Type eType) const {return eType==T##CLASS || PARENTCLASS::IsAKindOf(eType);} \
    virtual String GetClass() const {return NEX_T(#CLASS);}                            \
    virtual qResult VStreamIn(BinaryRepresentation *pStream) const {qResult rParent = PARENTCLASS::VStreamIn(pStream); if (!rParent) {return rParent;} else {return StreamIn(pStream);};} \
    virtual qResult VStreamOut(BinaryRepresentation *pStream) {qResult rParent = PARENTCLASS::VStreamOut(pStream); if (!rParent) {return rParent;} else {return StreamOut(pStream);};} \
    static CLASS* DynamicCast(ParseTreeItem* pItem) {if (pItem!=NULL && pItem->IsAKindOf(T##CLASS)) {return static_cast<CLASS*>(pItem);} else {return NULL;}}   \
    static const CLASS* DynamicCast(const ParseTreeItem* pItem) {if (pItem!=NULL && pItem->IsAKindOf(T##CLASS)) {return static_cast<const CLASS*>(pItem);} else {return NULL;}}

class OSDK_API ParseTreeItem: public RefCountedObject
{
public:
    typedef enum _Type
    {
        TParseTreeItem=0,
        TNameSpaceItem=1,
        TDeclaration=2,
        TDOClassDeclaration=3,
        TDatasetDeclaration=4,
        TTypeDeclaration=5,
        TVariable=6,
        TMethodDeclaration=7,
        TRMC=8,
        TAction=9,
        TAdapterDeclaration=10,
        TPropertyDeclaration=11,
        TProtocolDeclaration=12,
        TParameter=13,
        TReturnValue=14,
        TClassDeclaration=15,
        TTemplateDeclaration=16,
        TSimpleTypeDeclaration=17,
        TTemplateInstance=18,
        TDDLUnitDeclaration=19,
        TDupSpaceDeclaration=20,
        TValueDeclaration=21
    } Type;

    ParseTreeItem(const String& strName);
    virtual ~ParseTreeItem();

    // Methods corresponding to a parse tree item declaration
    ParseTreeItem(BinaryRepresentation* pStream)
    {
        m_pBackEndAnnotation=NULL;
        StreamOut(pStream);
    }
    virtual Type GetType() const
    {
        return TParseTreeItem;
    }
    virtual String GetClass() const
    {
        return NEX_T("ParseTreeItem");
    }
    virtual qResult VStreamIn(BinaryRepresentation *pStream) const
    {
        return StreamIn(pStream);
    }
    virtual qResult VStreamOut(BinaryRepresentation *pStream)
    {
        return StreamOut(pStream);
    }
    virtual qBool IsAKindOf(Type eType) const
    {
        return eType==TParseTreeItem;
    }

    void ForEachItem(void (*pf)(ParseTreeItem*))
    {
        pf(this);
        ForEachSubItem(pf);
    }
    virtual void ForEachSubItem(void (*pf)(ParseTreeItem*))
    {
        DUMMY_PARAM(pf);
    }

    const String& GetName() const
    {
        return m_strName;
    }
    void SetName(const String& strName)
    {
        m_strName=strName;
    }

    virtual qBool IsEqual(const ParseTreeItem* pOther) const
    {
        if (pOther==NULL)
        {
            return false;
        }
        return GetClass()==pOther->GetClass() && m_strName==pOther->m_strName;
    }


#if defined(NEX_TRACE_ENABLED)
    virtual void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS) const
    {
        DUMMY_PARAM(uiTraceFlag);
        QTRACE(uiTraceFlag,(GetClass().CStr()) << NEX_T(" ") << ( GetName().CStr()));
    }
#endif

    virtual void CheckCoherence() {};

    qResult StreamIn(BinaryRepresentation *pStream) const;
    qResult StreamOut(BinaryRepresentation *pStream);

    // Operators...
    qBool operator==(const ParseTreeItem& oOther) const
    {
        return IsEqual(&oOther);
    }
    qBool operator!=(const ParseTreeItem& oOther) const
    {
        return !IsEqual(&oOther);
    }

    virtual qBool ResolveReferences()
    {
        return true;
    }
    virtual qBool ReleaseReferences()
    {
        return true;
    }

    void* GetBackEndAnnotation() const
    {
        return m_pBackEndAnnotation;
    }
    void SetBackEndAnnotation(void* pBackEndAnnotation)
    {
        m_pBackEndAnnotation=pBackEndAnnotation;
    }

private:
    String m_strName;

    void *m_pBackEndAnnotation;
};

END_NAMESPACE_NEX

