﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/ParseTree/TypeDeclaration.h>
#include <OnlineCore/src/ParseTree/NameSpaceRef.h>
#include <OnlineCore/src/Platform/Core/qVector.h>

BEGIN_NAMESPACE_NEX

class TemplateDeclaration;

class TemplateInstance: public TypeDeclaration
{
public:
    TemplateInstance(const String& strName, TemplateDeclaration* pTypeDeclaration);
    TemplateInstance(const String& strName, const String& strTemplateDeclarationName);
    virtual ~TemplateInstance();

    DECLARE_NAMESPACE_ITEM(TemplateInstance, TypeDeclaration);

#ifdef NEX_TRACE_ENABLED
    virtual void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS) const;
#endif

    qResult StreamIn(BinaryRepresentation *pStream) const;
    qResult StreamOut(BinaryRepresentation *pStream);

    void StreamInTemplateInstanceDeclarationUse(BinaryRepresentation *pStream) const;
    static void StreamOutTemplateInstanceDeclarationUse(BinaryRepresentation *pStream, NameSpaceRef* prefDeclUse);

    virtual qBool IsEqual(const ParseTreeItem* pOther) const;

    Declaration* GetTypeDeclaration() const;
    String GetTypeName() const;

    virtual qBool ResolveReferences();

    qBool IsATemplate() const
    {
        return !m_vTemplateTypes.empty();
    }
    const qVector<NameSpaceRef>& GetTemplateTypes() const
    {
        return m_vTemplateTypes;
    }
    void SetTemplateTypes(const qVector<Declaration*>& vTypes );
    String GetTemplateTypeNameList() const;

    virtual String GetDeclarationString() const;

private:
    qVector<NameSpaceRef> m_vTemplateTypes;
    NameSpaceRef m_refTypeDeclaration;
};

END_NAMESPACE_NEX

