﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

template <typename Values>
class OSDK_API CircularVector: public RootObject
{
    class Element: public RootObject
    {
    public:
        Element()
        {
            m_bIsSet = false;
        }
        void Set(Values v)
        {
            m_aValue = v;
            m_bIsSet = true;
        }
        Values& Get()
        {
            m_bIsSet = true;
            return m_aValue;
        }
        qBool IsSet()
        {
            return m_bIsSet;
        }
        void Clear()
        {
            m_bIsSet = false;
        }
    protected:
        qBool       m_bIsSet;
        Values      m_aValue;
    };
public:
    CircularVector(qUnsignedInt uiSize)
    {
        m_uiSize = uiSize;
        m_aElements = qNew Element[uiSize];
    }
    ~CircularVector()
    {
        qDelete[] m_aElements;
    }
    Values& operator[](qUnsignedInt32 uiIndex)
    {
        return m_aElements[uiIndex%m_uiSize].Get();
    }
    qBool ValueStored(qUnsignedInt32 uiIndex)
    {
        return m_aElements[uiIndex%m_uiSize].IsSet();
    }
    void Clear(qUnsignedInt32 uiIndex)
    {
        m_aElements[uiIndex%m_uiSize].Clear();
    }
private:
    qUnsignedInt m_uiSize;
    Element     *m_aElements;
};

END_NAMESPACE_NEX

