﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/CriticalSection.h>

BEGIN_NAMESPACE_NEX

/*!
@internal
@brief    NEXの内部状態にかかわらず排他制御を行わないダミーのクリティカルセクションクラスです。

NEXの内部状態にかかわらず排他制御を行わない @ref CriticalSection を継承したダミーのクリティカルセクションです。

@see     ScopedCS
*/
class DummyCriticalSection: public CriticalSection
{
public:

    DummyCriticalSection()
        : CriticalSection(LockMasks::Ignored, true)
    {
    }

    virtual void Enter()
    {
    }

    virtual void Leave()
    {
    }

    // ForcedCriticalSectionならtrueを返す。
    virtual qBool IsForce() const { return false; }

    // 再入可能なクリティカルセクションかどうかを返す。
    virtual qBool IsReentrantable() const { return true; }
};

END_NAMESPACE_NEX
