﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once
#ifdef NEX_SIGLO_FAMILY
#include <nn/err/err_Types.h>
#endif

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/Result.h>


BEGIN_NAMESPACE_NEX

/*!
 @brief    ネットワークエラーコードへの変換関数を提供するクラスです。

 RootObjectを継承していません。
*/
class ErrorCodeConverter
{
public:
#ifndef NEX_SIGLO_FAMILY
    /*!
    @brief      qResult 型の値からネットワークエラーコードに変換します。

       NEX ライブラリが返した qResult 型の値をネットワークエラーコードに変換します。
       変換後の値は、 @errorappletname に渡すことができます。

       SUCCESS の場合 0 を返します。NEX ライブラリで定義されていないリザルトが渡された場合
        60000 
       
       を返します（デバッグビルドではアサート停止します）。
       本関数は NEX ライブラリで定義されている Result を一意に識別するコードに変換しますが、
       全てのコードをユーザーにエラーとして表示する必要はありません。表示すべきエラーコードについては
       API リファレンス・プログラミングマニュアルを参照してください。

    @param[in]     qresult    qResultクラスオブジェクト

    @return     ネットワークエラーコード
    */
#else
    /*!
    @brief      qResult 型の値からエラーコードに変換します(デバッグ用)。

       NEX ライブラリが返した qResult 型の値をネットワークエラーコードに変換します。
       本関数はデバッグ用です。@errorappletname に渡すことができる形式は、 ErrorCodeConverter::ConvertToErrorCodeStruct() を利用してください。

       SUCCESS の場合 0 を返します。NEX ライブラリで定義されていないリザルトが渡された場合 23060000 を返します（デバッグビルドではアサート停止します）。

    @param[in]     qresult    qResultクラスオブジェクト

    @return     エラーコード
    */
#endif
    static qUnsignedInt32 ConvertToNetworkErrorCode(const qResult& qresult);

#ifndef NEX_SIGLO_FAMILY
    /*!
    @brief      SystemError::GetLast()で取得した値をネットワークエラーコードに変換します。

       SystemError::GetLast()で取得したエラー値をネットワークエラーコードに変換します。
       変換後の値は、 @errorappletname に渡すことができます。

       NEXライブラリで定義されていないエラー値が渡された場合は
        60000 
       
       が返ります。
       本関数は NEX ライブラリで定義されている SYSTEMERROR_XXX_XXXX 定数を一意に
       識別するコードに変換しますが、全てのコードをユーザーにエラーとして表示する必要は
       ありません。表示すべきエラーコードについては API リファレンス・プログラミングマニュアルを
       参照してください。

    @param[in]     last    SystemError::GetLast()で取得した値

    @return     ネットワークエラーコード
    */
#else
    /*!
    @brief      SystemError::GetLast()で取得した値をエラーコードに変換します(デバッグ用)。

       SystemError::GetLast()で取得したエラー値をネットワークエラーコードに変換します。
       本関数はデバッグ用です。@errorappletname に渡すことができる形式は、 ErrorCodeConverter::ConvertToErrorCodeStruct() を利用してください。

       SUCCESS の場合 0 を返します。NEX ライブラリで定義されていないリザルトが渡された場合 23060000 を返します（デバッグビルドではアサート停止します）。

    @param[in]     last    SystemError::GetLast()で取得した値

    @return     エラーコード
    */

#endif
    static qUnsignedInt32 ConvertToNetworkErrorCode(qUnsignedInt32 last);

#ifdef NEX_SIGLO_FAMILY
    /*!
    @brief      qResult 型の値からエラーコードに変換します。

       NEX ライブラリが返した qResult 型の値をエラーコードに変換します。
       変換後の値は、エラービューアアプレットに渡すことができます。

       SUCCESS の場合 0 を返します。NEX ライブラリで定義されていないリザルトが渡された場合
       2306-0000
       を返します（デバッグビルドではアサート停止します）。
       本関数は NEX ライブラリで定義されている Result を一意に識別するコードに変換しますが、
       全てのコードをユーザーにエラーとして表示する必要はありません。表示すべきエラーコードについては
       API リファレンス・プログラミングマニュアルを参照してください。

    @param[in]     qresult    qResultクラスオブジェクト

    @return     エラーコード
    */
    static nn::err::ErrorCode ConvertToErrorCodeStruct(const qResult& qresult);

    /*!
    @brief      SystemError::GetLast()で取得した値をエラーコードに変換します。

       SystemError::GetLast()で取得したエラー値をエラーコードに変換します。
       変換後の値は、エラービューアアプレットに渡すことができます。

       NEXライブラリで定義されていないエラー値が渡された場合は
       2306-0000
       が返ります。
       本関数は NEX ライブラリで定義されている SYSTEMERROR_XXX_XXXX 定数を一意に
       識別するコードに変換しますが、全てのコードをユーザーにエラーとして表示する必要は
       ありません。表示すべきエラーコードについては API リファレンス・プログラミングマニュアルを
       参照してください。

    @param[in]     last    SystemError::GetLast()で取得した値

    @return     エラーコード
    */
    static nn::err::ErrorCode ConvertToErrorCodeStruct(qUnsignedInt32 last);
#endif
};

class ErrorCodeConverterInner
{
public:
    static qInt32 ConvertToErrorCode(const qResult& qresult);
    static qInt32 ConvertToErrorCode(qUnsignedInt32 last);
#ifdef NEX_SIGLO_FAMILY
    static nn::err::ErrorCode ConvertToErrorCodeStruct(qUnsignedInt32 errorCode);
#endif
};

END_NAMESPACE_NEX

