﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/Event.h>

BEGIN_NAMESPACE_NEX

class EventHandler: public RootObject
{

    class Attributes;

public:
    EventHandler(qUnsignedInt16 ui16MaxNbEvents);

    Event *CreateEventObject(qUnsignedInt32 uiType, qUnsignedInt32 uiContext);
    void DeleteEventObject(Event *pEvent);
    void ResetEvent(Event *pEvent);
    void SetEvent(Event *pEvent);
    qBool EventSet(Event *pEvent) const;

    void *GetPlatformSpecificEvent(Event *pEvent);

    virtual ~EventHandler();

    qBool WaitForEvent(qUnsignedInt32 uiTimeout, Event** ppEvent) const;

    //n1769 イベントもしくはタイムアウトを期待する。タイムアウトになってもエラーや警告を出さない。
    qBool WaitForEventOrTimeOut(qUnsignedInt32 uiTimeout, Event** ppEvent) const;

    static  qBool Test();
    qUnsignedInt16  GetMaxEvents();

private:
    //n1769 第3引数を追加。タイムアウトのケースでエラーとする（従来）のであれば、bIgnoreTimeoutErrにfalseをセット。
#ifdef WIN_FAMILY
    qBool           WaitForWin32Event(qUnsignedInt32 uiTimeout, Event** ppEvent, qBool bIgnoreTimeoutErr = false) const;
#else
    qBool           WaitForBoolEvent(qUnsignedInt32 uiTimeout, Event** ppEvent, qBool bIgnoreTimeoutErr = false) const;
#endif
    qUnsignedInt16  GetEventIndex(Event *pEvent) const;

private:
    mutable CriticalSection m_csEventTable;

    Attributes      *m_pAttributes;

    qUnsignedInt16  GetSignaledEvent() const;

    Event           **m_apEvents;
    qUnsignedInt32  m_uiNbEventsCreated;
    qUnsignedInt16  m_ui16MaxNbEvents;
};

END_NAMESPACE_NEX


