﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDeclInternal.h>
#include <OnlineCore/src/Platform/Core/Log.h>

BEGIN_NAMESPACE_NEX

#ifdef NEX_PERFCOUNTER_ENABLED
// forward declare PerfCounter to avoid having to include PerfCounter.h
// at this point.
class PerfCounter;
#endif // NEX_PERFCOUNTER_ENABLED

class String;
class StringStream;
class qResult;

#ifdef NEX_LOG_ENABLED

#if defined(CTR_UNICODE)

#define QLOG(LEVEL,ARGS) \
    if ((LEVEL >= NAMESPACE_NEX EventLog::GetInstance()->GetLogLevel())) {\
        NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX QLOGSTR(LEVEL, __ss.CStr());} else {}

#else

#if defined(CAFE_UNICODE)

#define QLOG(LEVEL,ARGS) \
    if ((LEVEL >= NAMESPACE_NEX EventLog::GetInstance()->GetLogLevel())) {\
    NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX _QLOG(LEVEL, NEX_T("%ls"), __ss.CStr());} else {}

#else

#define QLOG(LEVEL,ARGS) \
    if ((LEVEL >= NAMESPACE_NEX EventLog::GetInstance()->GetLogLevel())) {\
        NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX _QLOG(LEVEL, NEX_T("%s"), __ss.CStr());} else {}

#endif

#endif


#if defined(CTR_UNICODE) || defined(CAFE_UNICODE)

#define QTLOG(LEVEL,ARGS)             \
      if ((LEVEL >= NAMESPACE_NEX EventLog::GetInstance()->GetLogLevel())) {\
              NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX _QTLOG(LEVEL,NEX_T("%ls"), __ss.CStr());} else {}

#else

#define QTLOG(LEVEL,ARGS)       \
    if ((LEVEL >= NAMESPACE_NEX EventLog::GetInstance()->GetLogLevel())) {\
        NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX _QTLOG(LEVEL,NEX_T("%s"), __ss.CStr());} else {}
#endif

// The StringStream is all done inline since we want to be able to get and use the qResult return code from the PrintErrorCode function.

#define _QLOG_ERRORCODE(LEVEL, FACILITY, ERRORCODE, TEXT) \
    PrintErrorCode(EventLog::LEVEL, QRESULT_ERROR(FACILITY, ERRORCODE), \
        StringStream() << NEX_T("[[Error Code == ") << NEX_T(#FACILITY) << NEX_T("::") << NEX_T(#ERRORCODE) \
                       << NEX_T(" (") << hex << QERROR(FACILITY, ERRORCODE) << dec << NEX_T(")]]   ") \
                       << TEXT)

#define QLOG_FATAL(FACILITY, ERRORCODE, TEXT)   _QLOG_ERRORCODE(Fatal, FACILITY, ERRORCODE, TEXT)
#define QLOG_ERROR(FACILITY, ERRORCODE, TEXT)   _QLOG_ERRORCODE(Error, FACILITY, ERRORCODE, TEXT)
#define QLOG_WARNING(FACILITY, ERRORCODE, TEXT) _QLOG_ERRORCODE(Warning, FACILITY, ERRORCODE, TEXT)
#define QLOG_INFO(FACILITY, ERRORCODE, TEXT)    _QLOG_ERRORCODE(Info, FACILITY, ERRORCODE, TEXT)
#define QLOG_DEBUG(FACILITY, ERRORCODE, TEXT)   _QLOG_ERRORCODE(Debug, FACILITY, ERRORCODE, TEXT)

#else //NEX_LOG_ENABLED

#define QLOG(LEVEL,ARGS) \
    if(false){ \
        if ((LEVEL >= NAMESPACE_NEX EventLog::GetInstance()->GetLogLevel())) {\
            NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX _QLOG(LEVEL, NEX_T("%s"), __ss.CStr());} } else {}
#define QTLOG(LEVEL,ARGS) \
    if(false){ \
        if ((LEVEL >= NAMESPACE_NEX EventLog::GetInstance()->GetLogLevel())) {\
            NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX _QTLOG(LEVEL,NEX_T("%s"), __ss.CStr());} } else {}

#define _QLOG_ERRORCODE(LEVEL, FACILITY, ERRORCODE, TEXT) \
    if(false){ \
        PrintErrorCode(EventLog::LEVEL, QRESULT_ERROR(FACILITY, ERRORCODE), \
            StringStream() << NEX_T("[[Error Code == ") << NEX_T(#FACILITY) << NEX_T("::") << NEX_T(#ERRORCODE) \
                           << NEX_T(" (") << hex << QERROR(FACILITY, ERRORCODE) << dec << NEX_T(")]]   ") \
                           << TEXT);} else {}

#define QLOG_FATAL(FACILITY, ERRORCODE, TEXT) QRESULT_ERROR(FACILITY, ERRORCODE)
#define QLOG_ERROR(FACILITY, ERRORCODE, TEXT) QRESULT_ERROR(FACILITY, ERRORCODE)
#define QLOG_WARNING(FACILITY, ERRORCODE, TEXT) QRESULT_ERROR(FACILITY, ERRORCODE)
#define QLOG_INFO(FACILITY, ERRORCODE, TEXT) QRESULT_ERROR(FACILITY, ERRORCODE)
#define QLOG_DEBUG(FACILITY, ERRORCODE, TEXT) QRESULT_ERROR(FACILITY, ERRORCODE)

#endif //NEX_LOG_ENABLED


// group=Logging Events

// This macro is never defined.  It is used only to get around a compilation problem with
// DocJet.
#ifdef FOR_DOC_PURPOSE
/*
Description: Logs a string using printf syntax.
Parameters:
lLevel: The EventLog::LogLevel of the event.
szFormat: The log message. The format is the same as a standard printf function.
Remarks: The EventLog::LogLevel specifies the severity of the message. A given message will not
be displayed if its level is lower than the EventLog::LogLevel set via EventLog::SetLogLevel.
For example, if the EventLog::LogLevel is set to Warning, Verbose and Info messages will not be logged.

For example, a message at the Info level can be printed using the following syntax:

        _QLOG(EventLog::Info, NEX_T("This uses an int: %d"), 5);

See Also: EventLog::LogLevel, EventLog::SetLogLevel, QLOG
*/
void _QLOG( EventLog::LogLevel lLevel, const qChar *szFormat, ... );

/*
Description: Logs a string using a stream-like syntax.
Parameters:
lLevel: The EventLog::LogLevel of the event.

szString: The log message in a string stream format
Remarks: The EventLog::LogLevel specifies the severity of the message. A given message will not
be displayed if its level is lower than the EventLog::LogLevel set via EventLog::SetLogLevel.
For example, if the EventLog::LogLevel is set to Warning, Verbose and Info messages will not be logged.

For example, a message at the Info level can be printed using the following syntax:

        QLOG(EventLog::Info, NEX_T("This uses an int")<<5);

See Also: EventLog::LogLevel, EventLog::SetLogLevel, _QLOG
*/
void QLOG( EventLog::LogLevel lLevel, const qChar *szString, ... );

/*
Description: Logs data to the trace log using printf syntax.
Parameters:
lLevel: The EventLog::LogLevel of the event.
szFormat: The log message. The format is the same as a standard printf function.
Remarks: This method is similar to _QLOG except that it will output data to the trace log.

The EventLog::LogLevel specifies the severity of the message. A given message will not
be displayed if its level is lower than the EventLog::LogLevel set via EventLog::SetLogLevel.
For example, if the EventLog::LogLevel is set to Warning, Verbose and Info messages will not be logged.

For example, a message at the Info level can be printed using the following syntax:

        _QTLOG(EventLog::Info, NEX_T("This uses an int: %d"), 5);

See Also: EventLog::LogLevel, EventLog::SetLogLevel, QTLOG
*/
void _QTLOG( EventLog::LogLevel lLevel, const qChar *szFormat, ... );

/*
Description: Logs data to the trace log and event log using a stream-like syntax.
Parameters:
lLevel: The EventLog::LogLevel of the event. See below.
szString: The log message.
Remarks: This method is similar to QLOG except that it will output data to the trace log and the event log.

The EventLog::LogLevel specifies the severity of the message. A given message will not
be displayed if its level is lower than the EventLog::LogLevel set via EventLog::SetLogLevel.
For example, if the EventLog::LogLevel is set to Warning, Verbose and Info messages will not be logged.

For example, a message at the Info level can be printed using the following syntax:

        QTLOG(EventLog::Info, NEX_T("This uses an int")<<5);

See Also: EventLog::LogLevel, EventLog::SetLogLevel
*/
void QTLOG( EventLog::LogLevel lLevel, const qChar *szString, ... );

#endif

// group=Logging Events

/*
Description: Logs events that occur in your $PRODUCTNAME$ application.
Remarks: When you instantiate an EventLog object you need to attach a LogDevice and a OutputFormat
object to the EventLog object to define to which output device the data is logged and how it
is outputted.

This class implements persistent logging capabilities via a LogLevel system.
When a log output request is made, it will only be outputted to the associated LogDevice if its
LogLevel is equal to or higher than the EventLog's minimum allowed LogLevel set via SetLogLevel.
The _QLOG and _QTLOG functions are used to define log messages.
See Also: LogDevice, OutputFormat, SetLogLevel, _QLOG, _QTLOG
*/

/*!
 @brief    NEX の内部ログ出力の管理クラスです。

 NEX の内部ログ出力の管理クラスです。 @ref EventLog::LogLevel を設定することで NEX のログ出力を
 コントロールすることができます。Release ビルドではログ出力は常に無効化されます。
 */
class EventLog : public Log
{
public:
    // group=Methods

    //{secret}
    virtual ~EventLog();

    /*
    {group: Type Definitions}
    Description: The valid LogLevel s that may be used with an EventLog.
    Remarks: The LogLevel specifies the severity of the message. A given message will not be
    displayed if its level is lower than the LogLevel set via SetLogLevel. For example, if the
    LogLevel is set to Error, then Warning, Info and Debug messages will not be logged.
    See Also: SetLogLevel, _QLOG, _QTLOG
    */

    /*!
     @brief    ログの重要度を表す列挙子です。
     */
    enum LogLevel
    {
        // Severity level = 1. Messages with this LogLevel provide
        // information that can be used to debug the system.  In a
        // debug version of the system, the TraceLog is generally more
        // appropriate.
        Debug = 1,    //!< NEX内部のデバッグ情報
        // Severity level = 2. Messages with this LogLevel provide information about the processes that
        // the system performs and are designed to let you know what the system is currently doing or the steps
        // that have been performed.
        Info = 2,    //!< NEX内部の処理の詳細情報(デフォルト)
        // Severity level = 3. Messages with this LogLevel indicate a possible problem that should be fixed,
        // although it is not essential to do so.
        Warning = 3,    //!< 修正すべきだが無視しても動作可能なワーニング情報
        // Severity level = 4. Messages with this LogLevel indicate an
        // error that must be fixed, but for which the system can
        // recover.
        Error = 4,    //!< 修正すべきだが復帰可能なエラー
        // Severity level = 5. Messages with this LogLevel indicate an
        // error for which the system can not recover.
        Fatal = 5,    //!< フェータルエラー。復帰不可能なエラー
        // Severity level = 6. Messages with this LogLevel are always logged.
        Always = 6    //!< 常に出力されるログ情報
    };

#ifdef NEX_LOG_ENABLED
    // {secret}
    friend void _QLOG( EventLog::LogLevel lLevel, const qChar *szFormat, ... );

    // {secret}
    friend void _QTLOG( EventLog::LogLevel lLevel, const qChar *szFormat, ... );

    // {secret}
    friend void QLOGSTR( EventLog::LogLevel lLevel, const qChar *szString, ... );
#endif

    /*
    Description: Sets the LogLevel of this EventLog.
    Parameters:
    lLevel: The LogLevel. This may be one of, in order of increasing severity, Debug,
    Info, Warning, Error, or Fatal.

    Remarks: The LogLevel specifies the severity of the message. A given message will not
    be displayed if its level is lower than the LogLevel set via SetLogLevel. For example,
    if the LogLevel is set to Warning, Verbose and Info messages will not be logged.
    See Also: GetLogLevel, LogLevel
    */

    /*!
     @brief    ログレベルを設定します

     ログレベルはログの重要度を表します。設定したログレベル以上の重要度のログのみ出力されます。

     @param[in]    lLevel    出力するログの重要度
     */
    void SetLogLevel( LogLevel lLevel );

    /*
    Description: Returns the LogLevel of this EventLog.
    Returns: The LogLevel set via SetLogLevel.
    See Also: SetLogLevel, LogLevelToString
    */

    /*!
     @brief    現在のログレベルを取得します。

     @return    現在のログレベル
     */
    LogLevel GetLogLevel();

    /*
    Description: Returns the LogLevel of this EventLog as a string.
    Returns: The LogLevel as a string or Unknown if no string is defined by the LogLevel.
    See Also: GetLogLevel
    */

    /*!
     @brief    ログレベルの文字列表現を得ます。

     @return   ログレベル文字列へのポインタ。未定義のログレベルを渡した場合は、Unknownとなります。
     */
    static const qChar *LogLevelToString( LogLevel lLevel );

    /*
    Description: Returns a pointer to the EventLog instance.
    Returns: A reference to the EventLog instance.
    Remarks: Use the reference returned by this function to control what information is
    to be traced and how. For example:

          EventLog::GetInstance()->SetLogLevel(EventLog::Warning);
    */

    /*!
     @brief    EventLog のインスタンスを取得します。

     @return     EventLog のインスタンス
     */
    static EventLog *GetInstance();

    // {secret}
    void TraceEvents()
    {
        m_bTraceEvents = true;
    }
    void TurnOffTraceEvents()
    {
        m_bTraceEvents = false;
    }

#ifdef NEX_PERFCOUNTER_ENABLED
    // {secret}
    static PerfCounter  s_ErrorCounter;
    // {secret}
    static PerfCounter  s_WarningCounter;
    // {secret}
    static PerfCounter  s_FatalCounter;
    // {secret}
    static PerfCounter * WarningCounterInstance();
    // {secret}
    static PerfCounter * FatalCounterInstance();
    // {secret}
    static PerfCounter * ErrorCounterInstance();
#endif // NEX_PERFCOUNTER_ENABLED

private:
    EventLog();

    void OutputImpl( LogLevel lLevel, const qChar *szFormat, va_list vlArgPtr );
    void TraceOutput( LogLevel lLevel, const qChar *szFormat, va_list vlArgPtr );

    virtual void AddCustomPrefix( qChar *szString, qInt iStrSize, void *pLogLevel );

    LogLevel m_lLevel;

    qBool m_bTraceEvents;
#ifdef CTR_TARGET
    qByte padding1[2];  // to avoid compiler warning.
#endif

    friend class GlobalVariables;

};

// {secret}
class ScopedLogLevel
{
public:
    ScopedLogLevel(EventLog::LogLevel level);
    ~ScopedLogLevel();

private:
    const EventLog::LogLevel m_llOld;

    ScopedLogLevel& operator=(ScopedLogLevel& oSLL); //not implemented, keeps compiler happy
};

// These used to not be declared here and it was working
// presumably because of the friends declarations above.
// When upgrading to gcc4.1.1 on NON_SUPPORT_C it broke and adding the
// prototypes here fixes it.
// {secret}
#ifdef NEX_LOG_ENABLED
void _QLOG( EventLog::LogLevel lLevel, const qChar *szFormat, ... );
// {secret}
void QLOGSTR( EventLog::LogLevel lLevel, const qChar *szString, ... );
// {secret}
void _QTLOG( EventLog::LogLevel lLevel, const qChar *szFormat, ... );
#else
inline void _QLOG( EventLog::LogLevel, const qChar*, ... ) {}
inline void _QTLOG( EventLog::LogLevel, const qChar*, ... ) {}
inline void QLOGSTR( EventLog::LogLevel, const qChar*, ... ) {}
#endif

#ifdef NEX_LOG_ENABLED
#if defined(CAFE_TARGET)
#define NEX_NO_ALLOC_PRINT( ... ) OSReport(__VA_ARGS__)
#elif defined(CAFEIOP_TARGET)
#define NEX_NO_ALLOC_PRINT( ... ) printf(__VA_ARGS__)
#elif defined(CTR_TARGET)
#define NEX_NO_ALLOC_PRINT( ... ) NN_LOG(__VA_ARGS__)
#else
#define NEX_NO_ALLOC_PRINT( ... ) printf(__VA_ARGS__)
#endif
#else // NEX_LOG_ENABLED
#define NEX_NO_ALLOC_PRINT( ... )
#endif

#ifdef NEX_LOG_ENABLED
#define NO_ALLOC_QLOG(logLevel, ...) if(EventLog::GetInstance()->GetLogLevel() <= logLevel) { NEX_NO_ALLOC_PRINT(__VA_ARGS__); }
#else
#define NO_ALLOC_QLOG(logLevel, ...) if(false && logLevel) { NEX_NO_ALLOC_PRINT(__VA_ARGS__); }
#endif


// {secret}
qResult PrintErrorCode(EventLog::LogLevel level, qResult errorCode, StringStream& ssText);

END_NAMESPACE_NEX


