﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Platform/Core/ThreadVariable.h>
#include <OnlineCore/src/Platform/Core/ObjectThreadRoot.h>
#include <OnlineCore/src/Platform/Core/EventLog.h>
#include <OnlineCore/src/Platform/Core/TraceLog.h>
#include <OnlineCore/src/Platform/Core/PacketBufferManager.h>
#include <OnlineCore/src/Platform/Core/ThreadSafeMemoryAllocator.h>
#include <OnlineCore/src/Core/CacheManager.h>

BEGIN_NAMESPACE_NEX

/*!
    @brief NEX ライブラリ内部で使用するグローバル変数を管理するクラスです。

    @deprecated 本クラスは過去との互換性のために残されています。アプリケーションから使用する必要はありません。
*/
class GlobalVariables : public RefCountedObject
{
public:

    /*!
        @brief NEX ライブラリ内部で使用するグローバル変数を初期化します。

        @deprecated 本関数は過去との互換性のために残されています。アプリケーションから使用する必要はありません。

        本関数を呼び出す場合は呼び出した回数分 @ref ReleaseInstance() を呼び出してください。
    */
    static void AcquireInstance();

    /*!
        @brief NEX ライブラリ内部で使用するグローバル変数を開放します。

        @deprecated 本関数は過去との互換性のために残されています。アプリケーションから使用する必要はありません。

        @ref AcquireInstance() を呼び出した回数だけ本関数を呼ぶ必要があります。
     */
    static void ReleaseInstance();

    /*!
        @brief ライブラリ解放時に @ref ReleaseInstance() を呼び出すべき回数を取得します。

        @deprecated 本関数は過去との互換性のために残されています。アプリケーションから使用する必要はありません。

        複数回 @ref AcquireInstance() を呼んだ場合、 @ref ReleaseInstance() も同じ回数呼ぶ必要があります。
        本関数は、 @ref AcquireInstance() を呼び出した回数から@ref ReleaseInstance() を呼び出した回数を引いた数を返します。
        NEX の Core オブジェクト解放後に利用可能です。 本関数が返した値だけ @ref ReleaseInstance() を呼び出してグローバル変数を解放するようにしてください。
        NEX の Core オブジェクトが解放されていない状態では、不正な値が返ることがあります。 Debug ビルドにおいては、 Assert による停止状態となります。

        @return @ref AcquireInstance() を呼び出した回数から @ref ReleaseInstance() を呼び出した回数を引いた数
     */
    static qInt32 GetCount();

    //{secret}
    static GlobalVariables* GetInstance()
    {
#ifdef NEX_DEBUG_VERSION
        if (!s_pGv)
        {
            NEX_PANIC("(FATAL) You should initialize NEX before using this API.\n");
        }
#endif
        return s_pGv;
    }

    //{secret}
    // キャッシュ性のデータを破棄する。通常呼ぶ必要はありません。Core::~Core()で呼ばれます。
    static void ClearCache();
private:
    //{secret}
    GlobalVariables();
    //{secret}
    virtual ~GlobalVariables();

    //{secret}
    GlobalVariables(const GlobalVariables &);
    //{secret}
    GlobalVariables& operator=(const GlobalVariables &);

public:
    static const size_t THREAD_SAFE_ALLOCATOR_MEMORY_SIZE = 256 * 1024;
    static GlobalVariables* s_pGv;
    static ThreadVariableList* GetThreadVariableListPtr();

    // for LANSessionDiscovery
    String s_broadcastAddress;

    // for SystemError
    ThreadVariable<qUnsignedInt32> s_uiLastError;
    ThreadVariable<qUnsignedInt32> s_uiLastExtError;

    // for ObjectThreadRoot
    ThreadVariable<ObjectThreadRoot*> t_pCurrentThreadRoot;

#if defined(CTR_TARGET) || defined(CAFE_TARGET)
    // for RootCaReader
    qMap<String, qVector<qByte> > s_RootCaFileCache;
#endif

#if defined(CAFE_TARGET)
    qMap<qUnsignedInt32, String> s_AuthTokenCache;
    qMap<qUnsignedInt32, String> s_PasswordCache;
    qMap<qUnsignedInt32, String> s_ServerIPCache;
    qMap<qUnsignedInt32, qUnsignedInt16> s_ServerPortCache;
#endif

    CacheManager s_oCacheManager;

    ThreadSafeMemoryAllocator s_threadSafeMemoryAllocator;
};

END_NAMESPACE_NEX

