﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#if (defined(WIN_FAMILY) && !defined(VS2008_COMPILER)) || defined(UNIX_FAMILY)

#include <cstring>
#include <functional>
#include <cstddef>
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
BEGIN_NAMESPACE_NEX

//Boost::hash_combineと互換API 中身は、FNV-1aアルゴリズムを参考。
//vのハッシュとして、std::hash<T>()(v)として取得していることに注意
template <typename T>
inline void HashCombine(std::size_t& seed, const T& v)
{
#ifdef NEX_64BIT_ARCH
    std::size_t FNV_prime = 1099511628211ULL;
    std::size_t offset_basis = 14695981039346656037ULL;
#else
    std::size_t FNV_prime = 16777619UL;
    std::size_t offset_basis = 2166136261UL;
#endif

    seed ^= offset_basis; //0だった場合を考慮して、offset_basisをxorしてしまう
    seed ^= std::hash<T>()(v);;
    seed *= FNV_prime;
}

END_NAMESPACE_NEX

#endif

