﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

// This class is part of the deadlock detection system.  See
// For NEX Developer http://wiki/Deadlock Detection for more details.

class LockChecker: public RootObject
{
public:
    LockChecker(qUnsignedInt32 uiLockMask);
    virtual ~LockChecker();

    // Copy constructing a critical section will create a new object.
    LockChecker(const LockChecker &);

    // Assigning a critical section alters neither object.
    LockChecker& operator=(const LockChecker &);

    qUnsignedInt32 GetCurrentLockRecursion() const
    {
        return m_uiRecursionLevel;
    }
    //{secret}
    qUnsignedInt32 GetMaximumLockRecursion() const
    {
        return m_uiMaximumLockRecursion;
    }
    //{secret}
    void SetMaximumLockRecursion(qUnsignedInt32 uiMaximumLockRecursion)
    {
        m_uiMaximumLockRecursion=uiMaximumLockRecursion;
    }
    //{secret}
    void ForceRecursionLevel(qUnsignedInt32 uiRecursionLevel)
    {
        m_uiRecursionLevel = uiRecursionLevel;
    }

#ifdef NEX_LOCK_RESTRICTIONS_ENABLED
    //n1589
    static const qChar* GetLockName(qUnsignedInt32 lockMask);

    static qUnsignedInt32& CurrentLockState();
    static qBool LockIsOwned(qUnsignedInt32 uiLockMask);

    void EndEnter();
    void BeginLeave();
#else
    void EndEnter() {}
    void BeginLeave() {}
#endif

private:
    qUnsignedInt32 GetLockMask() const
    {
        return m_uiLockMask;
    }

    qBool m_bLockRestrictionsInUse;
#ifdef CTR_TARGET
    qByte padding1[3];  // to avoid compiler warning.
#endif
    qUnsignedInt32 m_uiLockMask;
    qUnsignedInt32 m_uiRecursionLevel;
    qUnsignedInt32 m_uiMaximumLockRecursion;
};

END_NAMESPACE_NEX

