﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/LogDevice.h>

BEGIN_NAMESPACE_NEX

//group=Log Devices

/*
Description: This class implements logging to a file.
See Also: LogDeviceConsole, LogDeviceDebugOutput
*/
class LogDeviceFile : public LogDevice
{
public:
    //group=Methods

    LogDeviceFile();
    virtual ~LogDeviceFile();

    // {secret}
    // Outputs the LogEntry to the file opened via SetFile.
    // See Also: SetFile
    virtual void Output(const qChar *pLog);

    /*
    Description: Sets the file to which data will be logged.
    Remarks: Data will be logged to the file szFileName. This method opens the file
    szFileName and keeps it open until the object is destroyed.
    Returns: true if the file was opened correctly, false if not or an error occurred.
    Parameters:
    szFileName:  The name of the file to which data will be logged.
    */
    qBool SetFile(const qChar *szFileName );

protected:
    // This is equivalent to the file size in our case, but faster and
    // included unflush data.
    qUnsignedInt64 GetFilePosition() const;

private:
    void *m_fpFile;
};

END_NAMESPACE_NEX


