﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <limits>
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/AtomicValue.h>
#include <OnlineCore/src/Platform/Core/String.h>

#define NN_NEX_THREAD_STACK_CHECK_WORD      0xCC

BEGIN_NAMESPACE_NEX

#ifdef NEX_SIGLO_FAMILY
static const qUnsignedInt NEX_THREAD_PRIORITY_LOW           = 24;
static const qUnsignedInt NEX_THREAD_PRIORITY_BELOW_NORMAL  = 20;
static const qUnsignedInt NEX_THREAD_PRIORITY_NORMAL        = 16;   // DefaultThreadPriority 
static const qUnsignedInt NEX_THREAD_PRIORITY_ABOVE_NORMAL  = 12;
static const qUnsignedInt NEX_THREAD_PRIORITY_HIGH          = 8;

#elif defined(WII_TARGET) || defined(CAFE_TARGET)
static const qUnsignedInt NEX_THREAD_PRIORITY_LOW           = 22;
static const qUnsignedInt NEX_THREAD_PRIORITY_BELOW_NORMAL  = 19;
static const qUnsignedInt NEX_THREAD_PRIORITY_NORMAL        = 17;   // NEX3の初期バージョンから17になっている。
static const qUnsignedInt NEX_THREAD_PRIORITY_ABOVE_NORMAL  = 13;
static const qUnsignedInt NEX_THREAD_PRIORITY_HIGH          = 10;

#elif defined(CAFEIOP_TARGET)
static const qUnsignedInt NEX_THREAD_PRIORITY_LOW           = 0;
static const qUnsignedInt NEX_THREAD_PRIORITY_BELOW_NORMAL  = 25;
static const qUnsignedInt NEX_THREAD_PRIORITY_NORMAL        = 50;   // <priority.h>で定義されているFPDやBOSSといったBGプロセスの優先度
static const qUnsignedInt NEX_THREAD_PRIORITY_ABOVE_NORMAL  = 75;
static const qUnsignedInt NEX_THREAD_PRIORITY_HIGH          = 127;

#elif defined(DS_TARGET)
static const qUnsignedInt NEX_THREAD_PRIORITY_LOW           = 0;
static const qUnsignedInt NEX_THREAD_PRIORITY_BELOW_NORMAL  = 8;
static const qUnsignedInt NEX_THREAD_PRIORITY_NORMAL        = 16;
static const qUnsignedInt NEX_THREAD_PRIORITY_ABOVE_NORMAL  = 24;
static const qUnsignedInt NEX_THREAD_PRIORITY_HIGH          = 31;

#elif defined(CTR_TARGET)
static const qUnsignedInt NEX_THREAD_PRIORITY_LOW           = 22;
static const qUnsignedInt NEX_THREAD_PRIORITY_BELOW_NORMAL  = 19;
static const qUnsignedInt NEX_THREAD_PRIORITY_NORMAL        = 16;
static const qUnsignedInt NEX_THREAD_PRIORITY_ABOVE_NORMAL  = 13;
static const qUnsignedInt NEX_THREAD_PRIORITY_HIGH          = 10;

#elif defined(WIN_FAMILY)
// Windowsのスレッド優先度は本来intで指定する。実際のAPIに渡すときにintにキャストされる
// windows.hで定義されているTHREAD_PRIORITY_BELOW_NORMALなどと被らないようにNEX_をつけている。
static const qUnsignedInt NEX_THREAD_PRIORITY_LOW           = static_cast<qUnsignedInt>(-2);
static const qUnsignedInt NEX_THREAD_PRIORITY_BELOW_NORMAL  = static_cast<qUnsignedInt>(-1);
static const qUnsignedInt NEX_THREAD_PRIORITY_NORMAL        = 0;
static const qUnsignedInt NEX_THREAD_PRIORITY_ABOVE_NORMAL  = 1;
static const qUnsignedInt NEX_THREAD_PRIORITY_HIGH          = 2;

#elif defined(UNIX_FAMILY)
// Linuxではスレッド優先度の変更をしていない。
static const qUnsignedInt NEX_THREAD_PRIORITY_LOW           = 0;
static const qUnsignedInt NEX_THREAD_PRIORITY_BELOW_NORMAL  = 0;
static const qUnsignedInt NEX_THREAD_PRIORITY_NORMAL        = 0;
static const qUnsignedInt NEX_THREAD_PRIORITY_ABOVE_NORMAL  = 0;
static const qUnsignedInt NEX_THREAD_PRIORITY_HIGH          = 0;

#else
#error not supported platform
#endif

// For the TraceOutput.
extern const qChar* GetThreadName();

// NEX_INVALID_THREAD_IDはスタティックイニシャライザで参照されるので、
// 初期化順により正しく初期化されない不具合を防ぐためstatic constではなくdefineで定義する。
#define NEX_INVALID_THREAD_ID (std::numeric_limits<NAMESPACE_NEX ThreadID>::max)()

class ObjectThreadRoot: public RootObject
{

    class Handle;

public:
    static const qInt32 DEFAULT_CORE_NO = QINT32_MIN;

    ObjectThreadRoot(const String& strName, qUnsignedInt threadPrio);

    virtual ~ObjectThreadRoot();

    static ObjectThreadRoot* GetInstance();

    qBool Wait(qUnsignedInt32 uiTimeout=WAIT_INFINITE_TIMEOUT);

    static qUnsignedInt32 NbRunningThreads()
    {
        return s_avRunningThreads.Get();
    }
    ThreadID GetThreadId() const
    {
        return m_uiThreadID;
    }

    void Attach(ThreadID uiThreadID);
    void Detach();

    void SetPriority(qUnsignedInt32 uiThreadPrio);
    qUnsignedInt32 GetPriority() const
    {
        return m_uiThreadPrio;
    }
    static void SetDefaultPrio(qUnsignedInt32 uiDefaultPrio);
    static qUnsignedInt32 GetDefaultPrio();
    static void SetThreadStackSize(size_t uiThreadSize);
    static size_t GetThreadStackSize();

    static void UseAutoStack(qBool usingAutoStack = true);
    static qBool IsUsingAutoStack();

    qBool Launched() const
    {
        return m_bLaunched;
    }
    qBool Finished() const
    {
        return m_bFinished;
    }
    Handle *GetThreadHandle();

    static ThreadID GetCurrentThreadID();

    void SetThreadName(const String& strName);
    String m_strName;

#ifdef NEX_THREAD_AFFINITY_SUPPORT
    static void SetDefaultThreadAffinity(qUnsignedInt32 uiThreadAffinityMask);
    static qUnsignedInt32 GetDefaultThreadAffinity()
    {
        return s_uiDefaultThreadAffinityMask;
    }
    static void SetThreadAffinity(qUnsignedInt32 uiThreadAffinityMask);
#endif
    qBool CheckStack();

    //n2967
    void SetCoreNo(qInt32 no)
    {
        m_coreNo = no;    //n2967: Core Noの設定 DEFAULT_CORE_NOだとNEXデフォルトのCPUを使う
    }
    qInt32 GetCoreNo() const
    {
        return m_coreNo;    //n2967: Core Noの設定 DEFAULT_CORE_NOだとNEXデフォルトのCPUを使う
    }


#if defined(NEX_DEBUG_VERSION)
    static volatile qBool s_isObjectThreadProcNewSection;
#endif
protected:
    /*!
    @brief      オブジェクトスレッドを起動します。
    @param[in]  uiStackSize   スタックのサイズ(CTRのみ有効)。デフォルト0を指定すると ObjectThreadRoot::s_uiDefaultThreadStackSize が使用されます。
    */
    qBool Launch(size_t uiStackSize = 0);
    void ReadyToRun();

private:
    qBool LaunchImpl();

private:
#ifdef NEX_SIGLO_FAMILY
    static void ObjectThreadProc(void *lpParameter);
#elif defined(UNIX_FAMILY) || defined(WIN32_TARGET)
    static qUnsignedInt STDCALL ObjectThreadProc(void *lpParameter);
#elif defined(WII_TARGET)
    static void* ObjectThreadProc(void *lpParameter);
#elif defined(CAFE_TARGET)
    static int ObjectThreadProc(int intArg, void *lpParameter);
#elif defined(CAFEIOP_TARGET)
    static void ObjectThreadProc(u32 lpParameter);
#elif defined(DS_TARGET) || defined(CTR_TARGET)
    static void ObjectThreadProc(void *lpParameter);
#else
#error not supported platform
#endif

    void CallObjectMethodWrapper();
    virtual void CallObjectMethod()=0;
    void WrapExceptionHandling();

private:
    Handle       *m_pHandle;
    ThreadID m_uiThreadID;
    qUnsignedInt m_uiThreadPrio;
    size_t m_uiThreadStackSize;

    volatile qBool m_bLaunched;
    volatile qBool m_bRunning;
    volatile qBool m_bFinished;
#ifdef CTR_TARGET
    qByte padding1;  // to avoid compiler warning.
#endif

    static size_t s_uiDefaultThreadStackSize;

    static AtomicValue s_avRunningThreads;
    static qUnsignedInt32 s_uiDefaultPrio;

#ifdef NEX_THREAD_AFFINITY_SUPPORT
    static qUnsignedInt32 s_uiDefaultThreadAffinityMask;
#endif

    qInt32 m_coreNo; //n2967

#if defined(CAFE_TARGET) && defined(NEX_UNICODE_VERSION)
    static const qUnsignedInt OS_THREAD_NAME_LEN = 256;
    qChar8 m_OSThreadNameChar8[OS_THREAD_NAME_LEN];
#endif
};

END_NAMESPACE_NEX


