﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <vector>
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Platform/Core/qChain.h>
#include <OnlineCore/src/Platform/Core/String.h>

BEGIN_NAMESPACE_NEX


// {Secret}
//class used to keep track of the stress on an object (mesured in the number of events it had to deal with in a second)
//Useful for sandbox nodes
class PeriodicCounter: public RootObject
{
protected:
    PeriodicCounter(const String& name);
    
public:
    virtual ~PeriodicCounter();

    // Finds PeriodicCounter. if does not found, creates object and returns it.
    static PeriodicCounter* FindOrCreate(const String& name);

    // Resets all the stastics to 0 
    void Reset();
    // Counts up
    void Inc(qUnsignedInt32 num=1);

    void Trace(qUnsignedInt64 uiTraceFlags);

    //Logs the information about the PeriodicCounter object (statistics and name)
    void Log();
    static void LogAll(qBool bSkipUnused = true);
    //Calls the Reset(...) function on all the global PeriodicCounter objects
    static void ResetAll();

    //Returns the total number of events that occured 
    qUnsignedInt32 GetTotal() { return m_uiTotal;}

    //Returns the total number of events that occured and reset it
    qUnsignedInt32 GetAndReset();

    typedef qChain<PeriodicCounter*>  List;
    //Gets a list containing all global PeriodicCounter objects
    static List& GetList();

    //Delete all elements
    static void DeleteAll();

    //Gets the name of the PeriodicCounter object
    const String& GetName() const;
    // Finds a PeriodicCounter named strName in the list and return it.  This is not meant to be efficient, and will iterate accross the entire
    //list of watermarks.
    static PeriodicCounter* Find(const String& strName);


public:
    CHAINABLE(PeriodicCounter);
    //Returns the synchronization variable (s_oCS) that indicates if another PeriodicCounter object is doing operations on the
    //list of global PeriodicCounter objects
    static CriticalSection& GetCriticalSection();

protected:    
    //Name of the PeriodicCounter object
    String m_strName;
    qUnsignedInt32      m_uiTotal;

    static CriticalSection s_oCS;
    CriticalSection m_oCS;
};

END_NAMESPACE_NEX

