﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/RandomNumberGenerator.h>
#include <OnlineCore/src/Platform/Core/SystemError.h>
#include <OnlineCore/src/Platform/Core/String.h>


#if defined(UNIX_FAMILY) && defined(UNICODE)
#error NO SUPPORT FOR UNICODE UNDER LINUX
#endif

#ifdef MSC_COMPILER
#pragma warning(disable : 4786)
#pragma warning(disable : 4291)
#endif

#ifdef APPLY_PATCHES
#include <OnlineCore/src/Platform/Core/bugs.patches>
#endif

BEGIN_NAMESPACE_NEX

class ErrorDescriptionTable;

//group=Miscellaneous Classes

/* Description: Encapsulates the methods used by $PRODUCTNAME$ to monitor itself.
     A Platform object should never be directly instantiated. It is created
     by $PRODUCTNAME$ when required.
   Remarks: Encapsulates the methods used by $PRODUCTNAME$ to monitor itself.
*/

/*!
    @brief プラットフォーム固有の機能をラップしたクラスです。
*/
class Platform: public RootObject
{
public:
    Platform();
    virtual ~Platform();

    // Description: Returns a reference to the Platform instance.
    static Platform *Instance();

    //{secret}
    static void CreateInstance();

    //{secret}
    static void DeleteInstance();

    static qInt64 GetTick();

    /*!
        @brief カレントスレッドを指定時間だけ休止状態にします。

        @param[in] uiTime 休止する時間（単位：ms）。
    */
    static void Sleep(qUnsignedInt32 uiTime);

    //{secret}
    static void YieldThread();

    /*!
        @brief 乱数のシード値を設定します。

        @param[in] uiSeed シード値。
    */
    static void SetRandomNumberSeed(qUnsignedInt32 uiSeed);

    /*!
        @brief 乱数を実数で取得します。

        本関数は、 @ref SetRandomNumberSeed() によって乱数のシード値が設定されていない場合、シード値に現在の時間を設定します。

        @param[in] fMaxValue 最大値。

        @return 乱数。乱数の範囲は [0, fMaxValue) です。
    */
    static qReal GetRealRandomNumber(qReal fMaxValue);

    /*!
        @brief 乱数を整数で取得します。

        本関数は、 @ref SetRandomNumberSeed() によって乱数のシード値が設定されていない場合、シード値に現在の時間を設定します。

        @param[in] uiMaxValue 最大値。

        @return 乱数。乱数の範囲は [0, uiMaxValue) です。
    */
    static qUnsignedInt32 GetRandomNumber(qUnsignedInt32 uiMaxValue);


    static qUnsignedInt32 GetRandomSeed();

    static qUnsignedInt32 GetProcessID();

    //{secret}
    static void Breakpoint();

    //{secret}
    static void NetworkToHost(const qByte *ucNet, qUnsignedInt16 *ucHost);

    //{secret}
    static void NetworkToHost(const qByte *ucNet, qUnsignedInt32 *ucHost);

    //{secret}
    static void NetworkToHost(const qByte *ucNet, qUnsignedInt64 *ucHost);

    //{secret}
    static void HostToNetwork(const qUnsignedInt16 *ucHost, qByte *ucNet);

    //{secret}
    static void HostToNetwork(const qUnsignedInt32 *ucHost, qByte *ucNet);

    //{secret}
    static void HostToNetwork(const qUnsignedInt64 *ucHost, qByte *ucNet);

    //{secret}
    static void WarnObsoleteMethod(const qChar* strOldMethod, const qChar* strNewMethod);

#if defined(NEX_FILE_SUPPORT) && (defined(WIN32_TARGET) || defined(LINUX_TARGET))

    static qBool ChangeCurrentDirectory(const qChar* szNewDir);

    //  Build a file path with the appropriate delimiter for the OS (\ or /)
    static String JoinPath(const qChar* szFirstPart, const qChar* szSecondPart);

    static qBool GetCWD(qChar* szNewDir, qUnsignedInt32 szlen);
#endif



private:
    static qInt AssertTranslator( qInt ReportType, qChar8 *UserMsg, qInt *RetVal );

private:
    static  Platform        *_Instance;
    static ErrorDescriptionTable m_oErrorTable;
    static RandomNumberGenerator    s_oRNG;
    static qBool s_bSeedIsInitialized;
    static qUnsignedInt64 GetPlatformID();
};

END_NAMESPACE_NEX

