﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/Recognition.h>
#include <OnlineCore/src/Platform/Core/Capabilities.h>
#include <OnlineCore/src/Platform/Core/TargetDefinition.h>
#include <OnlineCore/src/Platform/Core/CustomBuildDefs.h>
#include <OnlineCore/src/Platform/Core/NameSpaceDefinition.h>
#include <OnlineCore/src/Platform/Core/OptimizationDefs.h>

#ifdef WIN32_TARGET
// Some implicit additional libraries to be linked with
#pragma comment( lib, "ws2_32" )
#pragma comment( lib, "advapi32" )
#pragma comment( lib, "wldap32" )   // libcurlのLDAP機能が要求する
#endif

#ifdef LINUX_TARGET
#include <cstring>
#include <cstdio>
#endif

#ifdef WIN_FAMILY
#ifdef NEX_RELEASE_VERSION
// In the release version, we disable some warnings.

// We don't care if a dummy parameter or variable is found.  Since
// these where checked in debug, then the ones that show up is release
// must be caused by macros that have no effect in releae (e.g.
// disabled SYSTEMCHECKs).  We don't care.

#pragma warning(disable : 4189 4100 4710)

#endif

#if defined(VS2005_COMPILER) || defined(VS2008_COMPILER) || defined(VS2010_COMPILER)
// Visual Studio 2005 has extended the C Run-Time Library by including "secure"
// runtime functions and deprecating the previous function prototypes.  Since
// we need to use the previous prototypes to maintain compatibility with other
// platforms we are going to disable the deprecation warnings if we are compiling
// on Visual Studio 2005
#pragma warning(disable: 4996) // function deprecation
#endif



// Disable exception handling related warnings.
#pragma warning(disable : 4530)

#pragma warning(disable : 4786 4710 4702)

// Due to the way that the trace macro is implemented, constant
// expressions are used in the code.
#pragma warning(disable : 4127)

#endif


#if defined(CAFE_TARGET)
#pragma ghs nowarning 1795  //C++ symbols with the sequence '__' in their names might not be debuggable
#pragma ghs nowarning 381   //extra ";" ignored
#pragma ghs nowarning 550   //variable "iArgIndex" was set but never used
#pragma ghs nowarning 1895  //implicit conversion from bool to pointer type

/*
Typical offending line:
ASSERT_EQ(0, accounts2.size());

What it is expanded to:
switch (0) case 0: if (const ::testing::AssertionResult gtest_ar = (::testing::internal:: EqHelper<(sizeof(::testing::internal::IsNullLiteralHelper(0)) == 1)> ::Compare("0", "accounts2.size()", 0, accounts2 . size()))) ; else return ::testing::internal::AssertHelper(::testing::TestPartResult::kFatalFailure, "GuestAccountTest.cpp", 555, gtest_ar . failure_message()) = ::testing::Message();

The IsNullLiteralHelper overload picked in this case is a function definition.
*/
#pragma ghs nowarning 1931 //operand of sizeof is not a type, variable, or dereferenced pointer expression
#endif

#ifdef WIN_FAMILY
#ifndef _INC_TCHAR
#include <tchar.h>
#endif
#endif

#include <OnlineCore/src/Platform/Core/Type.h>
#include <OnlineCore/src/Platform/Core/Defs.h>
#include <OnlineCore/src/Platform/Core/ConnectionID.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <OnlineCore/src/Platform/Core/RootObject.h>

// [NEX1][LINUS] Stub out DLL support.
#define OSDK_DISABLE_EXPORT_WARNING(...) __VA_ARGS__


