﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/RapidJsonNexWrapper.h>

BEGIN_NAMESPACE_NEX

/*!
    @internal
    @brief      rapidjson用のユーティリティーです。
*/
class RapidJsonUtil: public RootObject
{
public:
    /*!
        @brief      JSON文字列を受け取り検証し、JSONドキュメントを初期化します。
        @param[in,out]  doc 初期化したJSONドキュメント。
        @param[in]      pIn JSON文字列。
        @return 検証に成功したかどうか。
    */
    static qBool ParseDocument(RapidJsonUtf8NexDocument &doc, const qChar8* pIn);

    /*!
        @brief      JSONドキュメントからqBoolを読み取ります。。
        @param[in]  doc JSONドキュメント。
        @param[in]  o 検索する要素の親オブジェクト。
        @param[in]  key 検索するキー名。
        @param[in]  pOut パースした値。
        @retval true パースに成功。pOutに値が書き込まれます。
        @retval false パースに失敗。pOutに値は書き込まれません。指定したキーが見つからないか、値がqBool互換ではない。
    */
    static qBool ParseBool(const RapidJsonUtf8NexDocument &doc, const RapidJsonUtf8NexValue& o, const qChar8* key, qBool *pOut);

    /*!
        @brief      JSONドキュメントからqIntを読み取ります。。
        @param[in]  doc JSONドキュメント。
        @param[in]  o 検索する要素の親オブジェクト。
        @param[in]  key 検索するキー名。
        @param[in]  pOut パースした値。
        @retval true パースに成功。pOutに値が書き込まれます。
        @retval false パースに失敗。pOutに値は書き込まれません。指定したキーが見つからないか、値がqInt互換ではない。
    */
    static qBool ParseInt(const RapidJsonUtf8NexDocument &doc, const RapidJsonUtf8NexValue& o, const qChar8* key, qInt *pOut);

    /*!
        @brief      JSONドキュメントからqInt64を読み取ります。。
        @param[in]  doc JSONドキュメント。
        @param[in]  o 検索する要素の親オブジェクト。
        @param[in]  key 検索するキー名。
        @param[in]  pOut パースした値。
        @retval true パースに成功。pOutに値が書き込まれます。
        @retval false パースに失敗。pOutに値は書き込まれません。指定したキーが見つからないか、値がqInt64互換ではない。
    */
    static qBool ParseInt64(const RapidJsonUtf8NexDocument &doc, const RapidJsonUtf8NexValue& o, const qChar8* key, qInt64 *pOut);

    /*!
        @brief      JSONドキュメントからqUnsignedIntを読み取ります。。
        @param[in]  doc JSONドキュメント。
        @param[in]  o 検索する要素の親オブジェクト。
        @param[in]  key 検索するキー名。
        @param[in]  pOut パースした値。
        @retval true パースに成功。pOutに値が書き込まれます。
        @retval false パースに失敗。pOutに値は書き込まれません。指定したキーが見つからないか、値がqUnsignedInt互換ではない。
    */
    static qBool ParseUint(const RapidJsonUtf8NexDocument &doc, const RapidJsonUtf8NexValue& o, const qChar8* key, qUnsignedInt *pOut);

    /*!
        @brief      JSONドキュメントからqUnsignedInt64を読み取ります。。
        @param[in]  doc JSONドキュメント。
        @param[in]  o 検索する要素の親オブジェクト。
        @param[in]  key 検索するキー名。
        @param[in]  pOut パースした値。
        @retval true パースに成功。pOutに値が書き込まれます。
        @retval false パースに失敗。pOutに値は書き込まれません。指定したキーが見つからないか、値がqUnsignedInt64互換ではない。
    */
    static qBool ParseUint64(const RapidJsonUtf8NexDocument &doc, const RapidJsonUtf8NexValue& o, const qChar8* key, qUnsignedInt64 *pOut);

    /*!
        @brief      JSONドキュメントからStringを読み取ります。。
        @param[in]  doc JSONドキュメント。
        @param[in]  o 検索する要素の親オブジェクト。
        @param[in]  key 検索するキー名。
        @param[in]  pOut パースした値。
        @retval true パースに成功。pOutに値が書き込まれます。
        @retval false パースに失敗。pOutに値は書き込まれません。指定したキーが見つからないか、値がString互換ではない。
    */
    static qBool ParseString(const RapidJsonUtf8NexDocument &doc, const RapidJsonUtf8NexValue& o, const qChar8* key, String *pOut);
};

END_NAMESPACE_NEX
