﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/LockMasks.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>
#include <OnlineCore/src/Platform/Core/Condition.h>


BEGIN_NAMESPACE_NEX

/*!
 * @internal
 * @brief ReadLockとWriteLockを提供するクラスです。
 *
 * 各種処理系の排他制御機構の違い考慮して、自前で実装しています。
 *
 */
class ReadWriteLock: public RootObject
{
public:
    ReadWriteLock(qUnsignedInt32 uiLockGroup);
    virtual ~ReadWriteLock();

    void BeginRead();
    void EndRead();
    void BeginWrite();
    void EndWrite();

    class ScopedWriteLock: public RootObject
    {
    public:
        ScopedWriteLock(ReadWriteLock *lock);
        ~ScopedWriteLock();
    private:
        ReadWriteLock* m_pReadWriteLock;
    };

    class ScopedReadLock : public RootObject
    {
    public:
        ScopedReadLock(ReadWriteLock *lock);
        ~ScopedReadLock();
    private:
        ReadWriteLock* m_pReadWriteLock;
    };


private:

    volatile qInt m_readerCounts;
    volatile qInt m_waitingWriterCounts;
    volatile qBool m_isWriting;

    Condition m_condition;
    NonReentrantableForcedCriticalSection m_lock;
};

END_NAMESPACE_NEX

