﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

// マクロ定義の意味を変えていろいろ定義するのに利用
// pragma onceは指定しないこと

/*! @addtogroup nnResult nn::Result
 *  @headerfile ResultDefine.h <RendezVous/Services/Jugem/ErrorCodeConverter/ErrorCodeConverter.h>
 *  @brief nn::Result() 型のエラー定義のグループです。
 *
 *  @see qResult::GetNnResult(), NEX_NNRESULT()
 */

#ifdef FOR_DOC_PURPOSE
BEGIN_NAMESPACE_NEX
#define NEX_ERROR_RESULT_DEFINE(FACILITY, CODE)  class  Result##FACILITY##CODE {}
#define NEX_ERROR_RESULT_DEFINE_WITHINDEX(FACILITY, CODE, INDEX) class  Result##FACILITY##CODE {}
#define NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(FACILITY, CODE) class Result##FACILITY##CODE {}
#endif

/*! @defgroup nnResultCoreFacility nn::Result(Core)
 *  @ingroup nnResult
 *  @headerfile ResultDefine.h <RendezVous/Services/Jugem/ErrorCodeConverter/ErrorCodeConverter.h>
 *  Core モジュールのエラーの nn::Result 型の定義です。
 *  @see CoreErrorCode::ErrorCode
 */
/*! @{ */
NEX_ERROR_RESULT_DEFINE(Core,      Unknown                      ); //!<    定義されていないエラー
NEX_ERROR_RESULT_DEFINE(Core,      NotImplemented               ); //!<    未実装
NEX_ERROR_RESULT_DEFINE(Core,      InvalidPointer               ); //!<    不正なポインタ
NEX_ERROR_RESULT_DEFINE(Core,      OperationAborted             ); //!<    処理が中断された
NEX_ERROR_RESULT_DEFINE(Core,      Exception                    ); //!<    例外発生
NEX_ERROR_RESULT_DEFINE(Core,      AccessDenied                 ); //!<    アクセスが許可されていない
NEX_ERROR_RESULT_DEFINE(Core,      InvalidHandle                ); //!<    不正な DOHandle
NEX_ERROR_RESULT_DEFINE(Core,      InvalidIndex                 ); //!<    不正な index
NEX_ERROR_RESULT_DEFINE(Core,      OutOfMemory                  ); //!<    メモリ不足
NEX_ERROR_RESULT_DEFINE(Core,      InvalidArgument              ); //!<    不正な引数
NEX_ERROR_RESULT_DEFINE(Core,      Timeout                      ); //!<    タイムアウトで終了
NEX_ERROR_RESULT_DEFINE(Core,      InitializationFailure        ); //!<    初期化に失敗
NEX_ERROR_RESULT_DEFINE(Core,      CallInitiationFailure        ); //!<    非同期処理の初期化に失敗
NEX_ERROR_RESULT_DEFINE(Core,      RegistrationError            );
NEX_ERROR_RESULT_DEFINE(Core,      BufferOverflow               ); //!<    送信できないサイズのバッファ
NEX_ERROR_RESULT_DEFINE(Core,      InvalidLockState             ); //!<    不正なロック状態（ロックが必要な状態で別のスレッドがロックしているなど）
NEX_ERROR_RESULT_DEFINE(Core,      InvalidSequence              ); //!<    不正なシーケンス
NEX_ERROR_RESULT_DEFINE(Core,      SystemError                  ); //!<    システムエラー
NEX_ERROR_RESULT_DEFINE(Core,      Cancelled                    ); //!<    キャンセルされた
/*! @} */

NEX_ERROR_RESULT_DEFINE(DDL,       InvalidSignature             );
NEX_ERROR_RESULT_DEFINE(DDL,       IncorrectVersion             );

/*! @defgroup nnResultTransportFacility nn::Result(Transport)
 *  @ingroup nnResult
 *  Transport モジュールの nn::Result 型のエラー定義です。
 *  @see TransportErrorCode::ErrorCode
 */
/*! @{ */
NEX_ERROR_RESULT_DEFINE(Transport, Unknown                      ); //!<    NGS との接続に失敗
NEX_ERROR_RESULT_DEFINE(Transport, ConnectionFailure            ); //!<    NGS 認証からの認証処理が完了していない
NEX_ERROR_RESULT_DEFINE(Transport, InvalidUrl                   ); //!<    不正なユーザー名
NEX_ERROR_RESULT_DEFINE(Transport, InvalidKey                   ); //!<    不正なパスワード
NEX_ERROR_RESULT_DEFINE(Transport, InvalidURLType               ); //!<    同じユーザー名が登録されている
NEX_ERROR_RESULT_DEFINE(Transport, DuplicateEndpoint            ); //!<    アカウントが使用できない
NEX_ERROR_RESULT_DEFINE(Transport, IOError                      ); //!<    ネットワークの向こう側にデータが送られませんでした。
NEX_ERROR_RESULT_DEFINE(Transport, Timeout                      ); //!<    タイムアウトが発生しました。
NEX_ERROR_RESULT_DEFINE(Transport, ConnectionReset              ); //!<    ネットワーク接続がリセットされました。
NEX_ERROR_RESULT_DEFINE(Transport, IncorrectRemoteAuthentication); //!<    接続先のステーションが正しく認証しませんでした。
NEX_ERROR_RESULT_DEFINE(Transport, ServerRequestError           ); //!<    サーバからエラーが返りました。
NEX_ERROR_RESULT_DEFINE(Transport, DecompressionFailure         ); //!<    受信データの伸張時にエラーが発生しました。
NEX_ERROR_RESULT_DEFINE(Transport, ReliableSendBufferFullFatal  ); //!<    送信バッファが満杯で、それ以上データを送ることができません。
NEX_ERROR_RESULT_DEFINE(Transport, UPnPCannotInit               );
NEX_ERROR_RESULT_DEFINE(Transport, UPnPCannotAddMapping         );
NEX_ERROR_RESULT_DEFINE(Transport, NatPMPCannotInit             );
NEX_ERROR_RESULT_DEFINE(Transport, NatPMPCannotAddMapping       );
NEX_ERROR_RESULT_DEFINE(Transport, UnsupportedNAT               ); //!<    サポートされていない NAT です。
NEX_ERROR_RESULT_DEFINE(Transport, DnsError                     ); //!<    DNS による名前解決に失敗しました。
NEX_ERROR_RESULT_DEFINE(Transport, ProxyError                   ); //!<    プロキシエラーが発生しました。
NEX_ERROR_RESULT_DEFINE(Transport, DataRemaining                ); //!<    受信バッファにデータが残っています。使用されません。
NEX_ERROR_RESULT_DEFINE(Transport, NoBuffer                     ); //!<    受信バッファに空きがありません。
NEX_ERROR_RESULT_DEFINE(Transport, NotFound                     ); //!<    要求されたリソースが見つかりません。
NEX_ERROR_RESULT_DEFINE(Transport, TemporaryServerError         ); //!<    一時的なサーバエラーが発生しました。あとでもう一度やり直してください。
NEX_ERROR_RESULT_DEFINE(Transport, PermanentServerError         ); //!<    恒久的なサーバエラーが発生しました。
NEX_ERROR_RESULT_DEFINE(Transport, ServiceUnavailable           ); //!<    サーバは一時的な過負荷もしくはメンテナンスのためリクエストを扱うことができません。
NEX_ERROR_RESULT_DEFINE(Transport, ReliableSendBufferFull       ); //!<    送信バッファが満杯で、一時的にそれ以上データを送ることができません
NEX_ERROR_RESULT_DEFINE(Transport, InvalidStation               ); //!<    指定されたステーションが無効なステーションです。
NEX_ERROR_RESULT_DEFINE(Transport, InvalidSubStreamID           ); //!<    指定されたSubStreamIDは無効なIDです。相手のステーションが受け付けることができないSubStreamIDの場合も発生します。
NEX_ERROR_RESULT_DEFINE(Transport, PacketBufferFull             ); //!<    パケットバッファが不足し、一時的にそれ以上データを送ることができません。 パケットバッファの設定は@ref PacketBufferManager を参照してください。
NEX_ERROR_RESULT_DEFINE(Transport, NatTraversalError            ); //!<    NATトラバーサル処理に失敗しました。
NEX_ERROR_RESULT_DEFINE(Transport, NatCheckError                ); //!<      NATチェック処理に失敗しました。
/*! @} */

NEX_ERROR_RESULT_DEFINE(DOCore,    StationNotReached            );
NEX_ERROR_RESULT_DEFINE(DOCore,    TargetStationDisconnect      );
NEX_ERROR_RESULT_DEFINE(DOCore,    LocalStationLeaving          );
NEX_ERROR_RESULT_DEFINE(DOCore,    ObjectNotFound               );
NEX_ERROR_RESULT_DEFINE(DOCore,    InvalidRole                  );
NEX_ERROR_RESULT_DEFINE(DOCore,    CallTimeout                  );
NEX_ERROR_RESULT_DEFINE(DOCore,    RMCDispatchFailed            );
NEX_ERROR_RESULT_DEFINE(DOCore,    MigrationInProgress          );
NEX_ERROR_RESULT_DEFINE(DOCore,    NoAuthority                  );
NEX_ERROR_RESULT_DEFINE(DOCore,    NoTargetStationSpecified     );
NEX_ERROR_RESULT_DEFINE(DOCore,    JoinFailed                   );
NEX_ERROR_RESULT_DEFINE(DOCore,    JoinDenied                   );
NEX_ERROR_RESULT_DEFINE(DOCore,    ConnectivityTestFailed       );
NEX_ERROR_RESULT_DEFINE(DOCore,    Unknown                      );
NEX_ERROR_RESULT_DEFINE(DOCore,    UnfreedReferences            );
NEX_ERROR_RESULT_DEFINE(DOCore,    JobTerminationFailed         );
NEX_ERROR_RESULT_DEFINE(DOCore,    InvalidState                 );
NEX_ERROR_RESULT_DEFINE(DOCore,    FaultRecoveryFatal           );
NEX_ERROR_RESULT_DEFINE(DOCore,    FaultRecoveryJobProcessFailed);
NEX_ERROR_RESULT_DEFINE(DOCore,    StationInconsistency         );
NEX_ERROR_RESULT_DEFINE(DOCore,    AbnormalMasterState          );
NEX_ERROR_RESULT_DEFINE(DOCore,    VersionMismatch              );

// RendezVousのコード定義が飛び飛びなので、nn::Resultの値を詰めるためにindexも定義

/*! @defgroup nnResultRendezVousFacility nn::Result(RendezVous)
 *  @ingroup nnResult
 *  RendezVous モジュールの nn::Result 型のエラー定義です。
 *  @see RendezVousErrorCode::ErrorCode
 */
/*! @{ */
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, ConnectionFailure                    ,  0); //!<    NGS との接続に失敗
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, NotAuthenticated                     ,  1); //!<    NGS 認証からの認証処理が完了していない
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, InvalidUsername                      ,  2); //!<    不正なユーザー名
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, InvalidPassword                      ,  3); //!<    不正なパスワード
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, UsernameAlreadyExists                ,  4); //!<    同じユーザー名が登録されている
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, AccountDisabled                      ,  5); //!<    アカウントが使用できない
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, AccountExpired                       ,  6); //!<    アカウントの有効期限が切れている
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, ConcurrentLoginDenied                ,  7); //!<    すでに同じユーザーがログインしている。
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, EncryptionFailure                    ,  8); //!<    暗号化エラー
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, InvalidPID                           ,  9); //!<    不正な PrincipalID
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, MaxConnectionsReached                , 10); //!<    NGS 側のコネクション数に達した
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, InvalidGID                           , 11); //!<    不正な GatheringID（存在しないなど）。
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, InvalidControlScriptID               , 12); //!<    不正なスレッド ID
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, InvalidOperationInLiveEnvironment    , 13); //!<    許可されていない関数を実行しようとした
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, DuplicateEntry                       , 14); //!<    同一のエントリを登録しようとした
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, ControlScriptFailure                 , 15);
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, ClassNotFound                        , 16); //!<    クライアントとサーバーのプロトコルが一致していない
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, SessionVoid                          , 17); //!<    ギャザリングが存在しない。
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, DDLMismatch                          , 19); //!<    クライアントとサーバーのプロトコルが一致していない
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, InvalidConfiguration                 , 20); //!<    ゲームサーバの設定に不備がある。製品環境では発生しません。
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, SessionFull                          , 21); //!<    ギャザリングの最大人数に達している
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, InvalidGatheringPassword             , 22); //!<    不正なギャザリングのパスワード
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, WithoutParticipationPeriod           , 23); //!<    参加可能期間外
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, PersistentGatheringCreationMax       , 24); //!<    永続ギャザリング登録数の上限に達した
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, PersistentGatheringParticipationMax  , 25); //!<    永続ギャザリング参加数の上限に達した
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, DeniedByParticipants                 , 26); //!<    参加者に拒否された(ブロックリストに登録されている)
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, ParticipantInBlockList               , 27); //!<    参加者の中に自分がブロックリストに登録したユーザーが存在する
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, GameServerMaintenance                , 28); //!<    ゲームサーバーのメンテナンスが開始される
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, OperationPostpone                    , 29);
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, OutOfRatingRange                     , 30); //!< レーティングマッチの範囲外となるレーティング値が指定された
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, ConnectionDisconnected               , 31); //!< ゲームサーバーとの接続が切れた
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, InvalidOperation                     , 32); //!<    不正な操作をしようとした
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, NotParticipatedGathering             , 33); //!< 指定したユーザーがギャザリングに参加していない
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, MatchmakeSessionUserPasswordUnmatch  , 34); //!< マッチメイクセッションユーザーパスワードが一致しない
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, MatchmakeSessionSystemPasswordUnmatch, 35); //!< マッチメイクセッションシステムパスワードが一致しない
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, UserIsOffline                        , 36); //!< 指定したユーザーがオフライン
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, AlreadyParticipatedGathering         , 37); //!< ギャザリングにすでに参加している
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, PermissionDenied                     , 38); //!< 権限がない操作をしようとした
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, NotFriend                            , 39); //!< フレンド関係がない
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, SessionClosed                        , 40); //!< ギャザリングが締め切られている
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, DatabaseTemporarilyUnavailable       , 41); //!< 一時的にデータベースにアクセスできない
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, InvalidUniqueId                      , 42); //!< 不正な NEX ユニーク ID
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, MatchmakingWithdrawn                 , 43); //!< マッチメイクリクエストが取り下げられた
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, LimitExceeded                        , 44); //!< 制限を超過した
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, AccountTemporarilyDisabled           , 45); //!< 一時的にアカウントが使用できない
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, PartiallyServiceClosed               , 46); //!< 一部サービスが終了して利用できない
NEX_ERROR_RESULT_DEFINE_WITHINDEX(RendezVous, ConnectionDisconnectedForConcurrentLogin, 47); //!< 同一のプリンシパル ID を持つユーザーがログインしたためゲームサーバーから切断された
/*! @} */

/*! @defgroup nnResultPythonCoreFacility nn::Result(PythonCore)
 *  @ingroup nnResult
 *  PythonCore モジュールの nn::Result 型のエラー定義です。
 *  @see PythonCoreErrorCode::ErrorCode
 */
/*! @{ */
NEX_ERROR_RESULT_DEFINE(PythonCore, Exception            ); //!<    例外発生
NEX_ERROR_RESULT_DEFINE(PythonCore, TypeError            ); //!<    型変換エラー
NEX_ERROR_RESULT_DEFINE(PythonCore, IndexError           ); //!<    範囲外のindex
NEX_ERROR_RESULT_DEFINE(PythonCore, InvalidReference    );  //!<    不正な参照
NEX_ERROR_RESULT_DEFINE(PythonCore, CallFailure         );  //!<    処理を中断した
NEX_ERROR_RESULT_DEFINE(PythonCore, MemoryError         );  //!<    メモリ不足
NEX_ERROR_RESULT_DEFINE(PythonCore, KeyError         );     //!<    キーが存在しない
NEX_ERROR_RESULT_DEFINE(PythonCore, OperationError      );  //!<    処理を中断した
NEX_ERROR_RESULT_DEFINE(PythonCore, ConversionError      ); //!<    python から 他の言語への変換でエラーが発生
NEX_ERROR_RESULT_DEFINE(PythonCore, ValidationError      ); //!<    許可されていないオペレーション
/*! @} */

NEX_ERROR_RESULT_DEFINE(FPD, NotInitialized);
NEX_ERROR_RESULT_DEFINE(FPD, AlreadyInitialized);
NEX_ERROR_RESULT_DEFINE(FPD, NotConnected);
NEX_ERROR_RESULT_DEFINE(FPD, Connected);
NEX_ERROR_RESULT_DEFINE(FPD, InitializationFailure);
NEX_ERROR_RESULT_DEFINE(FPD, OutOfMemory);
NEX_ERROR_RESULT_DEFINE(FPD, RmcFailed);
NEX_ERROR_RESULT_DEFINE(FPD, InvalidArgument);
NEX_ERROR_RESULT_DEFINE(FPD, InvalidLocalAccountID);
NEX_ERROR_RESULT_DEFINE(FPD, InvalidPrincipalID);
NEX_ERROR_RESULT_DEFINE(FPD, InvalidLocalFriendCode);
NEX_ERROR_RESULT_DEFINE(FPD, LocalAccountNotExists);
NEX_ERROR_RESULT_DEFINE(FPD, LocalAccountNotLoaded);
NEX_ERROR_RESULT_DEFINE(FPD, LocalAccountAlreadyLoaded);
NEX_ERROR_RESULT_DEFINE(FPD, FriendAlreadyExists);
NEX_ERROR_RESULT_DEFINE(FPD, FriendNotExists);
NEX_ERROR_RESULT_DEFINE(FPD, FriendNumMax);
NEX_ERROR_RESULT_DEFINE(FPD, NotFriend);
NEX_ERROR_RESULT_DEFINE(FPD, FileIO);
NEX_ERROR_RESULT_DEFINE(FPD, P2PInternetProhibited);
NEX_ERROR_RESULT_DEFINE(FPD, Unknown    );

/*! @defgroup nnResultAuthenticationFacility nn::Result(Authentication)
 *  @ingroup nnResult
 *  Authentication モジュールの nn::Result 型のエラー定義です。
 *  @see AuthenticationErrorCode::ErrorCode
 */
/*! @{ */
NEX_ERROR_RESULT_DEFINE(Authentication, NASAuthenticateError            );
NEX_ERROR_RESULT_DEFINE(Authentication, TokenParseError                 ); //!< 認証エラー
NEX_ERROR_RESULT_DEFINE(Authentication, HttpConnectionError             ); //!< HTTP 接続に失敗
NEX_ERROR_RESULT_DEFINE(Authentication, HttpDNSError                    ); //!< DNS エラー
NEX_ERROR_RESULT_DEFINE(Authentication, HttpGetProxySetting             ); //!< プロキシ設定の取得に失敗
NEX_ERROR_RESULT_DEFINE(Authentication, TokenExpired                    ); //!< トークンの有効期限切れ
NEX_ERROR_RESULT_DEFINE(Authentication, ValidationFailed                ); //!< 検証に失敗
NEX_ERROR_RESULT_DEFINE(Authentication, InvalidParam                    ); //!< 不正なパラメータ
NEX_ERROR_RESULT_DEFINE(Authentication, PrincipalIdUnmatched            ); //!< PID が一致しない
NEX_ERROR_RESULT_DEFINE(Authentication, MoveCountUnmatch                );
NEX_ERROR_RESULT_DEFINE(Authentication, UnderMaintenance                ); //!< サーバは現在メンテナンス中である
NEX_ERROR_RESULT_DEFINE(Authentication, UnsupportedVersion              ); //!< サポートされていないバージョンのクライアントを使用している
NEX_ERROR_RESULT_DEFINE(Authentication, ServerVersionIsOld              ); //!< NGSのバージョンがクライアントの要求バージョン未満のために使用できない。NGSの更新が必要。
NEX_ERROR_RESULT_DEFINE(Authentication, Unknown                         ); //!< 不明なエラー
NEX_ERROR_RESULT_DEFINE(Authentication, ClientVersionIsOld              ); //!< NEXのバージョンがサーバーの要求バージョン未満のために使用できない。NEXの更新が必要。
NEX_ERROR_RESULT_DEFINE(Authentication, AccountLibraryError             ); //!< アカウントライブラリのエラー
NEX_ERROR_RESULT_DEFINE(Authentication, ServiceNoLongerAvailable        ); //!< サービス終了
NEX_ERROR_RESULT_DEFINE(Authentication, UnknownApplication              ); //!< アプリケーションが登録されていないため、NGS にアプリケーションの登録が必要。
NEX_ERROR_RESULT_DEFINE(Authentication, ApplicationVersionIsOld         ); //!< アプリケーションバージョンが NGS に登録されている必須バージョン未満のため、アプリケーションの更新が必要。
NEX_ERROR_RESULT_DEFINE(Authentication, OutOfService                    ); //!< サービス休止中
NEX_ERROR_RESULT_DEFINE(Authentication, NetworkServiceLicenseRequired   ); //!< ネットワークサービスを利用する権利がない。
NEX_ERROR_RESULT_DEFINE(Authentication, NetworkServiceLicenseSystemError); //!< ネットワークサービスの利用権に関するシステムエラー
NEX_ERROR_RESULT_DEFINE(Authentication, NetworkServiceLicenseError3     ); //!< ネットワークサービスを利用する権利がない。(予約コード)
NEX_ERROR_RESULT_DEFINE(Authentication, NetworkServiceLicenseError4     ); //!< ネットワークサービスを利用する権利がない。(予約コード)
/*! @} */

/*! @defgroup nnResultRankingFacility nn::Result(Ranking)
 *  @ingroup nnResult
 *  Ranking モジュールの nn::Result 型のエラー定義です。
 *  @see RankingErrorCode::ErrorCode
 */
/*! @{ */
NEX_ERROR_RESULT_DEFINE(Ranking, NotInitialized                 ); //!<    初期化されていません。
NEX_ERROR_RESULT_DEFINE(Ranking, InvalidArgument                ); //!<    引数が不正です。
NEX_ERROR_RESULT_DEFINE(Ranking, RegistrationError              ); //!<    登録処理が失敗しました。
NEX_ERROR_RESULT_DEFINE(Ranking, NotFound                       ); //!<    データは存在しません。
NEX_ERROR_RESULT_DEFINE(Ranking, InvalidScore                   ); //!<    スコアが不正です。
NEX_ERROR_RESULT_DEFINE(Ranking, InvalidDataSize                ); //!<    データサイズが不正です。
NEX_ERROR_RESULT_DEFINE(Ranking, PermissionDenied               ); //!<    アクセス権がありません。
NEX_ERROR_RESULT_DEFINE(Ranking, Unknown                        ); //!<    不明なエラーです。
NEX_ERROR_RESULT_DEFINE(Ranking, NotImplemented                 ); //!<    実装されていない機能を呼び出そうとしました。
/*! @} */

/*! @defgroup nnResultDataStoreFacility nn::Result(DataStore)
 *  @ingroup nnResult
 *  DataStore モジュールの nn::Result 型のエラー定義です。
 *  @see DataStoreErrorCode::ErrorCode
 */
/*! @{ */
NEX_ERROR_RESULT_DEFINE(DataStore, Unknown                      ); //!<    不明なエラーです。
NEX_ERROR_RESULT_DEFINE(DataStore, InvalidArgument              ); //!<    引数が不正です。
NEX_ERROR_RESULT_DEFINE(DataStore, PermissionDenied             ); //!<    アクセス権限がありません。
NEX_ERROR_RESULT_DEFINE(DataStore, NotFound                     ); //!<    指定したデータIDが見つかりません。
NEX_ERROR_RESULT_DEFINE(DataStore, AlreadyLocked                ); //!<    すでにリードロックされているため参照できません。
NEX_ERROR_RESULT_DEFINE(DataStore, UnderReviewing               ); //!<    審査中のためまだ参照できません。
NEX_ERROR_RESULT_DEFINE(DataStore, Expired                      ); //!<    期限切れです。
NEX_ERROR_RESULT_DEFINE(DataStore, InvalidCheckToken            ); //!<    指定したチェックトークンが不正です。
NEX_ERROR_RESULT_DEFINE(DataStore, SystemFileError              ); //!<    システムファイルの読み込みに失敗しました。
NEX_ERROR_RESULT_DEFINE(DataStore, OverCapacity                 ); //!<    一括処理できる数を超えています。
NEX_ERROR_RESULT_DEFINE(DataStore, OperationNotAllowed          ); //!<    要求された操作は許可されていません。
NEX_ERROR_RESULT_DEFINE(DataStore, InvalidPassword              ); //!<    パスワードが間違っています。
NEX_ERROR_RESULT_DEFINE(DataStore, ValueNotEqual                ); //!<    値が等しくありません。
/*! @} */

/*! @defgroup nnResultServiceItemFacility nn::Result(ServiceItem)
 *  @ingroup nnResult
 *  ServiceItem モジュールの nn::Result 型のエラー定義です。
 *  @see ServiceItemErrorCode::ErrorCode
 */
/*! @{ */
NEX_ERROR_RESULT_DEFINE(ServiceItem, Unknown                     ); //!<    不明なエラーです。
NEX_ERROR_RESULT_DEFINE(ServiceItem, InvalidArgument             ); //!<    引数が不正です。
NEX_ERROR_RESULT_DEFINE(ServiceItem, EShopUnknownHttpError       ); //!<    eShop サーバー へのHTTPアクセスで不明なエラーが発生しました。
NEX_ERROR_RESULT_DEFINE(ServiceItem, EShopResponseParseError     ); //!<    eShop サーバー からのレスポンスのパースに失敗しました。
NEX_ERROR_RESULT_DEFINE(ServiceItem, NotOwned                    ); //!<    権利を所持していません。
NEX_ERROR_RESULT_DEFINE(ServiceItem, InvalidLimitationType       ); //!<    権利種別が不正です。
NEX_ERROR_RESULT_DEFINE(ServiceItem, ConsumptionRightShortage    ); //!<    消費型サービス利用券の残り個数が足りません。
/*! @} */

/*! @defgroup nnResultMatchmakeRefereeFacility nn::Result(MatchmakeReferee)
 *  @ingroup nnResult
 *  MatchmakeReferee モジュールの nn::Result 型のエラー定義です。
 *  @see MatchmakeRefereeErrorCode::ErrorCode
 */
/*! @{ */
NEX_ERROR_RESULT_DEFINE(MatchmakeReferee, Unknown                 ); //!<   不明なエラーです。
NEX_ERROR_RESULT_DEFINE(MatchmakeReferee, InvalidArgument         ); //!<   引数が不正です。
NEX_ERROR_RESULT_DEFINE(MatchmakeReferee, AlreadyExists           ); //!<   すでに存在します。
NEX_ERROR_RESULT_DEFINE(MatchmakeReferee, NotParticipatedGathering); //!<   ギャザリングに参加していません。
NEX_ERROR_RESULT_DEFINE(MatchmakeReferee, NotParticipatedRound    ); //!<   ラウンドに参加していません。
NEX_ERROR_RESULT_DEFINE(MatchmakeReferee, StatsNotFound           ); //!<   統計情報が見つかりません。
NEX_ERROR_RESULT_DEFINE(MatchmakeReferee, RoundNotFound           ); //!<   ラウンドが見つかりません。
NEX_ERROR_RESULT_DEFINE(MatchmakeReferee, RoundArbitrated         ); //!<   ラウンド集計処理が完了しています。
NEX_ERROR_RESULT_DEFINE(MatchmakeReferee, RoundNotArbitrated      ); //!<   ラウンド集計処理が完了していません。
/*! @} */

/*! @defgroup nnResultSubscriberFacility nn::Result(Subscriber)
 *  @ingroup nnResult
 *  Subscriber モジュールの nn::Result 型のエラー定義です。
 *  @see SubscriberErrorCode::ErrorCode
 */
/*! @{ */
NEX_ERROR_RESULT_DEFINE(Subscriber, Unknown          ); //!<   不明なエラーです。
NEX_ERROR_RESULT_DEFINE(Subscriber, InvalidArgument  ); //!<   引数が不正です。
NEX_ERROR_RESULT_DEFINE(Subscriber, OverLimit        ); //!<   限界数を超えた。
NEX_ERROR_RESULT_DEFINE(Subscriber, PermissionDenied ); //!<   権限がない。
/*! @} */

/*! @defgroup nnResultRanking2Facility nn::Result(Ranking2)
 *  @ingroup nnResult
 *  Ranking2 モジュールの nn::Result 型のエラー定義です。
 *  @see Ranking2ErrorCode::ErrorCode
 */
/*! @{ */
NEX_ERROR_RESULT_DEFINE(Ranking2, Unknown            ); //!<   不明なエラーです。
NEX_ERROR_RESULT_DEFINE(Ranking2, InvalidArgument    ); //!<   引数が不正です。
NEX_ERROR_RESULT_DEFINE(Ranking2, InvalidScore       ); //!<   スコアが不正です。
/*! @} */

#ifdef NEX_ONLINE_LOUNGE_SUPPORT
/*! @defgroup nnResultOnlineLoungeFacility nn::Result(OnlineLounge)
 *  @ingroup nnResult
 *  OnlineLounge モジュールの nn::Result 型のエラー定義です。
 *  @see OnlineLoungeErrorCode::ErrorCode
 */
/*! @{ */
NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(OnlineLounge, Unknown                ); //!<    不明なエラーです。
NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(OnlineLounge, InvalidArgument        ); //!<    引数が不正です。
NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(OnlineLounge, InvalidResponse        ); //!<    サーバーからのレスポンスが不正です。
NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(OnlineLounge, InvalidAccessToken     ); //!<    アクセストークンが不正もしくは期限切れです。
NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(OnlineLounge, Unauthorized           ); //!<    認証エラーです。
NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(OnlineLounge, AccessError            ); //!<    サーバーにアクセスを拒否されたエラーです。通常発生しません。
NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(OnlineLounge, UserNotFound           ); //!<    ユーザーが見つからないエラーです。
NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(OnlineLounge, RoomNotFound           ); //!<    ルームが見つからないエラーです。
NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(OnlineLounge, RoomNotActivated       ); //!<    対象のルームが活性化していません。
NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(OnlineLounge, ApplicationNotSupported); //!<    非対応のアプリケーションです。アプリケーションの申請が必要です。
NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(OnlineLounge, InternalServerError    ); //!<    サーバーの処理中にエラーが発生した場合のエラーです。
NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(OnlineLounge, ServiceUnavailable     ); //!<    サービスが利用できません。
NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(OnlineLounge, UnexpectedError        ); //!<    サーバーからの不明なエラーです。
NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(OnlineLounge, UnderMaintenance       ); //!<    サーバーはメンテナンス中です。
NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(OnlineLounge, ServiceNoLongerAvailable ); //!<    サービス終了のエラーです。
NEX_ONLINE_LOUNGE_ERROR_RESULT_DEFINE(OnlineLounge, AccountTemporarilyDisabled); //!<    このアカウントは一時的に無効化されています。
/*! @} */
#endif

/*! @defgroup nnResultScreeningFacility nn::Result(Screening)
 *  @ingroup nnResult
 *  Screening モジュールの nn::Result 型のエラー定義です。
 *  @see ScreeningErrorCode::ErrorCode
 */
/*! @{ */
NEX_ERROR_RESULT_DEFINE(Screening, Unknown        ); //!<   不明なエラーです。
NEX_ERROR_RESULT_DEFINE(Screening, InvalidArgument); //!<   引数が不正です。
NEX_ERROR_RESULT_DEFINE(Screening, NotFound       ); //!<   該当のデータが存在しません。
/*! @} */

#ifdef FOR_DOC_PURPOSE
END_NAMESPACE_NEX
#endif
