﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDeclInternal.h>

BEGIN_NAMESPACE_NEX

/*!
 @brief    NEX ライブラリで定義されるクラスの基底クラスです。

 NEX ライブラリで定義される公開クラスは本クラスを基本的に継承しています。
 継承されていない場合には、クラス説明に特記されています。
 本クラスを継承したクラスは、@ref MemoryManager::SetBasicMemoryFunctions() で設定された
 メモリ確保・解放関数を使って new/delete されます。

 コンストラクト時にメモリ開放関数へのポインタをセットしています。
 したがって、@ref MemoryManager::SetBasicMemoryFunctions() でメモリ解放関数を上書きしても
 その前に確保されたオブジェクトに対するメモリ解放関数は上書きされません。
 */
class RootObject
{
public:
    enum TargetPool
    {
        // メモリ確保する際の対象プール(将来増えることを考慮してenum定義)
        POOL_PB_MANAGEMENT // PacketBufferManagerによる管理オブジェクト用プール
    };
public:
    RootObject() {}
    ~RootObject() {}

    void *operator new(size_t size, void* pPtr) throw()
    {
        DUMMY_PARAM(size);
        return pPtr;
    }
    //n2775 CTRビルドの時には使用できずワーニングになるので除外
#ifndef CTR_TARGET
    void operator delete(void* pPtr1, void* pPtr2)
    {
        DUMMY_PARAM(pPtr1);
        DUMMY_PARAM(pPtr2);
    }
#endif

    void *operator new(size_t size, const qChar* file, qUnsignedInt32 line) throw();
    void *operator new(size_t size) throw();

    void *operator new[](size_t size, const qChar* file, qUnsignedInt32 line) throw();
    void *operator new[](size_t size) throw();

    void operator delete(void *uptr);
    void operator delete[](void *uptr);

    /* !@internal
     @brief メモリ確保するプールを指定するnewです。
            メモリ確保失敗(返り値 NULL)を考慮したエラーハンドリングが必要です。
     */
    void *operator new(size_t size, TargetPool pool) throw();
    void *operator new(size_t size, TargetPool pool, const qChar *file, qUnsignedInt32 line) throw();
private:// 配列確保は未サポートです。
    void *operator new[](size_t size, TargetPool pool) throw();
public:

    //n2775 CTRビルドの時には使用できずワーニングになるので除外
#ifndef CTR_TARGET
    // The two following methods must be called explicitely to be used.
    // NOTE: Those method are currently not called or exercised through any test.
    void operator delete(void *uptr, const qChar *file, qUnsignedInt32 line);
    void operator delete[](void *uptr, const qChar *file, qUnsignedInt32 line);
#endif
};

END_NAMESPACE_NEX

