﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

// A ScopedCounter can be used to more easily test the variation of a value, in
// a test.    This class it meant to be derived from.

class ScopedCounter: public RootObject
{
public:
    ScopedCounter(const qChar* szName): m_bInitialized(false), m_szName(szName) {}
    virtual ~ScopedCounter() {};

    qUnsignedInt32 ReadValue() const
    {
        SYSTEMCHECK(m_bInitialized);
        return GetValue();
    }

    void Reset()
    {
        m_uiBaseValue = ReadValue();
    }
    qBool HasIncreased() const
    {
        return ReadValue()>m_uiBaseValue;
    }
    qBool HasDecreased() const
    {
        return ReadValue()<m_uiBaseValue;
    }
    qBool IsEqual() const
    {
        return ReadValue()==m_uiBaseValue;
    }
    void Trace(qUnsignedInt64 uiTraceFlag) const
    {
#ifndef NEX_TRACE_ENABLED
        DUMMY_PARAM(uiTraceFlag); // Dummy if QTRACE is turned off.
#endif
        QTRACE(uiTraceFlag,m_szName << NEX_T("  base value: ") << m_uiBaseValue <<NEX_T(" current value: ") << GetValue());
    }

protected:
    virtual qUnsignedInt32 GetValue() const =0;

    void Init()
    {
        // To prevent calls to pure virtual method, this HAS to be
        // called by derived class.
        m_uiBaseValue = GetValue();
        m_bInitialized = true;
    }

private:
    qBool m_bInitialized;
#ifdef CTR_TARGET
    qByte padding1[3];  // to avoid compiler warning.
#endif
    // Note that this name could be done using a
    // DebugString, but this introduces circular
    // dependencies (at compile time) between the
    // MemoryManager.h and ScopedCounter.h.
    const qChar* m_szName;
    qUnsignedInt32 m_uiBaseValue;
};

END_NAMESPACE_NEX

