﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/Time.h>

// Group=Platform Subsystem

BEGIN_NAMESPACE_NEX

class SpinTest: public RootObject
{
    // Group=
public:
    static const qUnsignedInt32 INFINITE_SPIN=0xFFFFFFFF;

    SpinTest(qUnsignedInt32 uiSpinPeriod, qUnsignedInt32 uiTimeout = INFINITE_SPIN);
    virtual ~SpinTest();

    void SetTrace(qUnsignedInt32 uiSpinPeriod, qUnsignedInt64 uiTraceFlag = D_TRACE_THREAD);
    qUnsignedInt64 GetTraceFlag();

    qBool SpinOnce(const qChar *strFileName, qUnsignedInt32 uiLine, const qChar *strCondition);
    void AssertOnTimeout();
    void LeaveOnTimeout();

    qUnsignedInt32 GetElapsed();
    qUnsignedInt32 GetTimeout();

    qUnsignedInt32 GetRemainingTime();

    qBool HasTimedOut()
    {
        return m_bTimedOut;
    }
    void Reset()
    {
        m_bTimedOut=false;
    }

private:
    qUnsignedInt16 m_ui16TimeoutConsequence;
    qUnsignedInt32  m_uiTimeout;
    Time           m_tBegin;
    qUnsignedInt32  m_uiElapsed;
    qUnsignedInt32  m_uiSpinPeriod;
    qUnsignedInt32  m_uiTracePeriod;
    qUnsignedInt64  m_uiTraceFlag;
    qBool           m_bTimedOut;
};

/*
   @brief Condがtrueの間Condとスリープを実行する。

   @param oSpinTest スリープ時間やタイムアウトが定義されたSpinTestオブジェクト。
   @param Cond 条件。
*/
#define SpinOn(oSpinTest,Cond)  \
    { while ((Cond) && oSpinTest.SpinOnce(NEX_T(__FILE__),__LINE__,NEX_T(#Cond))) {}; }

/*
   @brief Condがtrueの間CondとSpinActionとスリープを実行する。

   @param oSpinTest スリープ時間やタイムアウトが定義されたSpinTestオブジェクト。
   @param Cond 条件。
   @param SpinAction Condがtrueとなった場合に実行される文。
*/
#define SpinOnAction(oSpinTest,Cond,SpinAction) \
    { \
        while (Cond) { \
            SpinAction; \
            if (!oSpinTest.SpinOnce(NEX_T(__FILE__),__LINE__,NEX_T(#Cond))) { \
                break; \
            }\
        } \
    }

/*
@brief Condがtrueの間CondとSpinPreActionとスリープを実行する。
       ただし、SpinPreActionは最初にCondを評価する前に一度は実行される。

@param oSpinTest スリープ時間やタイムアウトが定義されたSpinTestオブジェクト。
@param Cond 条件。
@param SpinPreAction Condがtrueとなった場合と最初に一度実行される文。
*/
#define SpinOnPreAction(oSpinTest,Cond,SpinPreAction) \
    { \
        do \
        { \
            SpinPreAction; \
        } \
        while (Cond && oSpinTest.SpinOnce(NEX_T(__FILE__),__LINE__,NEX_T(#Cond))); \
    }

END_NAMESPACE_NEX

