﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

// Utility functions

/*!
   @internal
   @brief 文字列を変換してコピーする。

   pSourceの型からpDestinationの型へ適宜変換されます。

   @param[out] pDestination コピー先文字列
   @param[in] pSource コピー元文字列
   @param[in] size pDestinationに確保されている、終端を含めた文字の個数(文字列長 + 1)。
   @return コピーした終端を含めた文字の個数(文字列長 + 1)
*/
extern OSDK_API size_t CopyStringHelper(qChar8 *pDestination, const qChar8 *pSource, size_t size);
extern OSDK_API size_t CopyStringHelper(qWideChar *pDestination, const qWideChar *pSource, size_t size);
extern OSDK_API size_t CopyStringHelper(qWideChar *pDestination, const qChar8 *pSource, size_t size);
extern OSDK_API size_t CopyStringHelper(qChar8 *pDestination, const qWideChar *pSource, size_t size);

#ifdef NEX_QCHAR16_IS_NOT_WCHAR_T
extern OSDK_API size_t CopyStringHelper(qChar16 *pDestination, const qChar8 *pSource, size_t size);
extern OSDK_API size_t CopyStringHelper(qChar8 *pDestination, const qChar16 *pSource, size_t size);
extern OSDK_API size_t CopyStringHelper(qChar16 *pDestination, const qWideChar *pSource, size_t size);
extern OSDK_API size_t CopyStringHelper(qWideChar *pDestination, const qChar16 *pSource, size_t size);
#endif

/*!
   @internal
   @brief TConvertFrom型からTConvertTo型に変換した際に必要な書き込み先の個数を返す。

   返された個数 * sizeof(TConvertTo) が必要なバイト数となる。

   @param[in] szSource コピー元文字列
   @return コピーするのに必要な文字の個数(文字列長 + 1)
*/
template<typename TConvertTo, typename TConvertFrom>
size_t GetRequiredDestSize(const TConvertFrom* szSource);

/*!
   @internal
   @brief pszDestinationに必要なメモリを確保した後、TConvertFromからTConvertToへ文字列を変換しpszDestinationに書き込む。

   返された個数 * sizeof(TConvertTo) が必要なバイト数となる。

   @param[out] pszDestination コピー先文字列へのポインタ
   @param[in] szSource コピー元文字列
   @return コピーした終端を含めた文字の個数(文字列長 + 1)
*/
template<typename TConvertTo, typename TConvertFrom>
size_t CopyStringWithAlloc(TConvertTo** pszDestination, const TConvertFrom* szSource);

END_NAMESPACE_NEX
