﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

//
//  Values are 32 bit values layed out as follows:
//
//   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
//   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
//  +---+-+-+-----------------------+-------------------------------+
//  |Sev|1|R|     Subsystem         |               Code            |
//  +---+-+-+-----------------------+-------------------------------+
//
//  where
//
//      Sev - is the severity code
//
//          00 - Success
//          01 - Informational
//          10 - Warning
//          11 - Error
//
//      R - is a reserved bit
//
//      Subsystem - is the Subsystem code
//
//      Code - is the Subsystem's status code
//
//
// Define the facility codes
//

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#ifdef CTR_TARGET
#include <nn/Result.h>
#endif

//n1589:SEVフラグは30 - 32bitの2bit = 4個
#define SEV_SUCCESS                         0x0
#define SEV_INFORMATIONAL                   0x1u
#define SEV_WARNING                         0x2u
#define SEV_ERROR                           0x3u

//n1589:SUBSYSTEMフラグは16 - 28bitの13 bit
enum SubSystemType
{
    SUBSYSTEM_GENERAL = 0,
    // SUBSYSTEM_NETWORK, // not used
    // SUBSYSTEM_DC, // not used
    SUBSYSTEM_OBJDUP,
    // SUBSYSTEM_TRANSPORT_GENERAL, // not used
    // SUBSYSTEM_TRANSPORT_UDS, // not used
    // SUBSYSTEM_TRANSPORT_SOCKET, // not used
    NUMBER_OF_SUBSYSTEMS,
    SUBSYSTEM_NONE = 0x10
};

BEGIN_NAMESPACE_NEX

//n1589:?:ERROR_IS_WIN32 flag is always ON ? or reserved ?
#define SET_ERROR_CODE(SEV,SUBSYS,STATUS)   (NAMESPACE_NEX qUnsignedInt32)((SEV<<30)|(1<<29)|(SUBSYS<<16)|STATUS)
#define ERROR_IS_WIN32(ERRCODE)             ((ERRCODE&(0x1<<29))==0)
#define GET_SUBSYSTEM(ERRCODE)              ((ERRCODE>>16)&0xFFF)
#define GET_STATUS(ERRCODE)                 (ERRCODE&0xFFFF)
#define GET_SEVERITY(ERRCODE)               ((ERRCODE>>30)&0x3)

// {secret}
// SUBSYSTEM_NONE Error Codes
#define SYSTEMERROR_FACADE_NOT_INITIALIZED  SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_NONE,0)

#define SIGNAL_SYSTEMERROR(Code,ExtCode)            SystemError::SignalError(NEX_T(__FILE__),__LINE__,Code,ExtCode);
// #define SIGNAL_CTRSDK_RESULT(result)                SystemError::SignalError(NEX_T(__FILE__),__LINE__,result);
// #define CALL_ERROR_HANDLING_FUNCTION(result)        SystemError::CallErrorHandlingFunction(result,NEX_T(__FILE__),__LINE__);

// group = System Errors

/*
Description: There was not enough memory to complete the operation.
*/
#define SYSTEMERROR_GEN_NOT_ENOUGH_MEM  SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,0)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_ILLEGAL_ACCESS  SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,1)

/*
Description: An underlying WIN_FAMILY operation has failed.  The WIN_FAMILY error code can be
retrieved with SystemError::GetLastExt
*/
#define SYSTEMERROR_GEN_WIN32_ERROR     SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,2)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_INDEX_OOB           SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,3)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_DUPLICATED_KEY  SET_ERROR_CODE(SEV_WARNING, SUBSYSTEM_GENERAL,4)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_ITEM_NOT_DELETED    SET_ERROR_CODE(SEV_WARNING, SUBSYSTEM_GENERAL,5)
/*
{Secret}
*/
#define SYSTEMERROR_GEN_ITEM_NOT_FOUND  SET_ERROR_CODE(SEV_WARNING, SUBSYSTEM_GENERAL,6)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_END_OF_COLLECTION   SET_ERROR_CODE(SEV_WARNING, SUBSYSTEM_GENERAL,7)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_OBJ_NOT_INIT        SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,8)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_NOT_IMPLEMENTED SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,9)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_OUT_OF_ID           SET_ERROR_CODE(SEV_WARNING, SUBSYSTEM_GENERAL, 10)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_INTERNAL            SET_ERROR_CODE(SEV_WARNING, SUBSYSTEM_GENERAL, 11)

/*
Description: The delay allowed for an operation to occur has expired.
*/
#define SYSTEMERROR_GEN_TIMEOUT         SET_ERROR_CODE(SEV_ERROR, SUBSYSTEM_GENERAL, 12)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_CHILDPROCESS_FAILED SET_ERROR_CODE(SEV_WARNING, SUBSYSTEM_GENERAL, 13)

/*
Description: An operation was called in an invalid context.
*/
#define SYSTEMERROR_GEN_INVALID_OPERATION   SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,14)

/*
Description: An attempt was made at instantiating more than one $PRODUCTNAME$ object in the same process.
*/
#define SYSTEMERROR_GEN_MULTIPLE_INSTANCES SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,15)

/*
{secret}
Description: An attempt to register more than one ExceptionHandler was made.
*/
#define SYSTEMERROR_GEN_MULTIPLE_EXCEPTION_HANDLER SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,16)

/*
Description: An attempt to use an expired version of $PRODUCTNAME$ was made.
*/
#define SYSTEMERROR_GEN_EXPIRED SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,17)  // プロダクトキーのチェックはしていないので現状発生しない

/*
Description: A method was called with an out-of-range parameter.
*/
#define SYSTEMERROR_GEN_OUT_OF_RANGE SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,18)

//{secret}
#define SYSTEMERROR_GEN_ASSERTION_FAILED SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,19)

//{secret}
#define SYSTEMERROR_GEN_DEADLOCK_DETECTED SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,20)

/*
Description: $PRODUCTNAME$ has not been initialized.
*/
#define SYSTEMERROR_GEN_NOT_INITIALIZED SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,21)

/*
Description: An invalid parameter, which is typically NULL, was passed to a method.
*/
#define SYSTEMERROR_GEN_INVALID_PARAMETER SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,22)

//Description: An invalid input was passed to a method.
#define SYSTEMERROR_GEN_IO_ERROR SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,23)

/*
Description: The product key used for $PRODUCTNAME$ does not exist or is invalid.
*/
#define SYSTEMERROR_GEN_INVALID_PRODUCT_KEY   SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,24)

/*
Description: A buffer extraction tried to go beyond the end of the buffer.
*/
#define SYSTEMERROR_GEN_BUFFER_EXTRACTION_OVERFLOW   SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,25)

/*
Description: The timeout value set is invalid.
Remarks: This error is returned if DuplicatedObject::Create or DuplicatedObject::Publish are called
in a singlethread context with a timeout value other than 0.
See Also: DuplicatedObject::Create, DuplicatedObject::Publish
*/
#define SYSTEMERROR_GEN_INVALID_WAIT   SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,26)

// Description: Access to the requested resource was refused.
#define SYSTEMERROR_GEN_ACCESS_DENIED  SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,27)

// Description: The operation was denied.
#define SYSTEMERROR_GEN_OP_DENIED  SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,28)

//n1769
#define SYSTEMERROR_GEN_INVALID_OPERATION_WARNING SET_ERROR_CODE(SEV_WARNING,SUBSYSTEM_GENERAL,29)

//n1769
#ifdef CTR_TARGET
#define SYSTEMERROR_GEN_CTR_ERROR   SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,30)
#endif

//n1589
//#define SYSTEMERROR_TRANSPORT_GENERAL_RELIABLE SET_ERROR_CODE(SEV_ERROR, SUBSYSTEM_TRANSPORT_GENERAL, 0)  //n1769:未使用のため、除去。

//n1589
//#define SYSTEMERROR_TRANSPORT_UDS SET_ERROR_CODE(SEV_ERROR, SUBSYSTEM_TRANSPORT_UDS, 0)                   //n1769:未使用のため、除去。

//n1769
//#define SYSTEMERROR_TRANSPORT_UDS_INVALID_STATE   SET_ERROR_CODE(SEV_ERROR, SUBSYSTEM_TRANSPORT_UDS, 1)   //n1769:未使用のため、除去。

class UserContext;
class qResult;

// {Secret}
class ErrorDescriptionTable: public RootObject
{

public:

    ErrorDescriptionTable(const qChar *aszStatusMessage[], SubSystemType uiSubsystem);
    virtual ~ErrorDescriptionTable();

    SubSystemType       m_uiSubsystem;
    const qChar         **m_aszStatusMessage;
};


// Group=Results Classes

/*
Description: This class enables the retrieval of error information.
Remarks: $PRODUCTNAME$ uses error codes to signal errors.  The error codes are used to
obtain more information about the error.  If an error occurs
while calling a method from any $PRODUCTNAME$ interface, information about the error
is stored within this class.  If a method causes a $PRODUCTNAME$ error, it is the return value
of that method that will signal the error.  For example, Session::CreateSession will
return true if the call did not signal an error, and false if it did.  In the case of a method that
returns a value such as DuplicatedObject::Create, NULL is returned in case of error.
To retrieve the error information, use the methods of this class.  For example:

        Avatar* pNewAvatar= Avatar::Create();
        if (pNewAvatar==NULL) {
            Display(NEX_T("ERROR: Could not create Avatar."));
            qChar strError[256] = NEX_T("");
            SystemError::GetErrorString(SystemError::GetLast(), strError, 256);
            Display(NEX_T("%s!\n"), strError);
            return 0;
        }

*/

/*!
  @brief 主にNEXライブラリのエラーを扱うクラスです。

  NEX APIを呼び出した後に、明示的に @ref GetLast() を呼び出すことでエラーの内容が
  取得できるように設計された NEX API は、通常3種類に分類されます。返値がbool型の関数、
  返値がポインタ型の関数、 IsValid() の3種類です。

  IsValid() は、ユーザーによって直接呼び出され、障害が発生する可能性のあるコンストラクタが
  関連クラスを正常にインスタンス化したかどうかを確認するために使用されます。

  エラーが発生すると、 IsValid() と bool型の関数はfalse、つまり 0 を返し、ポインタ型の関数は
  NULL ポインタを返します。

  このような NEX API を呼び出している間にエラーが発生すると、そのエラー情報がSystemError
  クラスに保管されます。@ref GetLast() は、保管された最新のエラーを返します。

  システムエラーなど、エラーが NEXの外で発生した場合、@ref GetLastExt() は
  発生した最新エラーのシステムエラーコードを返します。

  @ref ClearLast() は、保管されているエラーを削除します。NEX APIを呼び出す前に
  @ref ClearLast() を呼び出しておくことで、返されたエラーが最後に呼び出した
  NEX APIで生成されたものであることを保証できます。

  また、@ref GetErrorString() を使用してエラーの文字列表現を取得することもできます。
   */
class SystemError: public RootObject
{
    //{secret}
    class ImplAttributes;
    //Group=Methods
public:

    // {secret}
    SystemError(const qChar *szFilename, qUnsignedInt LineNo);
    // {Secret}
    virtual ~SystemError() {}

    // {secret}
    SystemError SetErrorInfo(qUnsignedInt32 uiErrorCode, qUnsignedInt32 uiExtErrorCode);
    // {Secret}
    void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS);

    /*
    Description: Returns a description of the error as a string.
    Parameters:
    uiErrorCode: The $PRODUCTNAME$ error code that caused the error.
    szStringBuf: A string buffer that will hold the result.
    uiBufSize: Specifies the maximum number of characters that can be copied into
    the buffer pointed to by szStringBuf, including a terminating null character.
    Returns: A description of the error as a string.
    Remarks: This method fills a user supplied buffer with the description of an error.
    See Also: GetLast, GetLastExt
    */

    /*!
    @brief エラーの文字列表現を得ます。

    @param[in]  uiErrorCode GetLast() で取得できるエラー。
    @param[out] szStringBuf エラーの文字列表現（null終端）が格納されます。
    @param[in]  uiBufSize   szStringBufにコピーされるキャラクタの最大値（NUL文字含む）を与えます。

    @see GetLast, GetLastExt
    */
    static void GetErrorString(qUnsignedInt32 uiErrorCode, qChar *szStringBuf, qUnsignedInt uiBufSize);

    // {Secret}
    static void SignalError(const qChar *szFilename, qUnsignedInt uiLineNo, qUnsignedInt32 uiErrorCode, qUnsignedInt32 uiExtErrorCode);

    /*
    Summary: ClearLast will remove the stored error.
    Remarks: This method may be called before GetLast or GetLastExt to ensure that the
    returned error is generated by the most recently called method.
    See Also: GetLast, GetLastExt
    */

    /*!
    @brief SystemErrorクラスに保管されていたエラーを削除します。

    NEX APIを呼び出す前にこの関数を呼び出しておくことで、返されたエラーが
    最後に呼び出されたNEX APIで生成されていることを保証できます。

    エラーはスレッド毎に保管されており、この関数呼び出しによって
    削除できるエラーは、同じスレッドのものに限定されます。
    */
    static void ClearLast();

    //{secret}
    static void ResetSystemError();

    /*
    Summary: Returns the $PRODUCTNAME$ error code of the most recent error.
    Returns: The specific error code of the most recent $PRODUCTNAME$
    error that occurred. 0 is returned if no error is encountered.
    See Also: GetLastExt
    */

    /*!
    @brief  NEX API呼び出しによって発生したエラーを取得します。

    SystemErrorクラスが保管している、最新のエラーを返します。
    エラーはスレッド毎に保管されており、この関数呼び出しによって
    取得できるエラーは、同じスレッドのものに限定されます。

    @see    GetLastExt, ClearLast

    @return 最新のエラー。
    */
    static qUnsignedInt32 GetLast();

    /*
    Summary: Returns the system error code of the most recent error.
    Returns: The system error code of the most recent error that occurred.
    See Also: GetLast
    */

    /*!
    @brief  NEX API呼び出しによって発生したシステムエラーを取得します。

    SystemErrorクラスが保管している、最新のシステムエラーを返します。
    システムエラーはスレッド毎に保管されており、この関数呼び出しによって
    取得できるエラーは、同じスレッドのものに限定されます。

    現状、このAPIを利用する機会はありません。

    @see    GetLast, ClearLast
    @return 最新のシステムエラー。
    */
    static qUnsignedInt32 GetLastExt();

#ifdef CTR_TARGET
    //n1769
    /*!
     * @internal
    @brief  CTR固有のエラーを取得します。

    アプリケーションプログラムが直接呼び出すことのできるNEX APIがfalseやNULLポインタを
    返した場合、まず @ref GetLast() によって要因を探ってください。
    もし @ref GetLast() が SYSTEMERROR_GEN_CTR_ERROR を返してきていたら、
    この関数を呼び出して、nn::Result値を取得し、適切なエラーハンドリングを行って下さい。

    エラーはスレッド毎に保管されており、この関数呼び出しによって
    取得できるエラーは、同じスレッドのものに限定されます。

    @see    GetLast

    @return 最新のCTR固有エラー。
    */
    static nn::Result GetLastCtr();
#endif

    //n1589
    // {Secret}
    //  最新のエラーの深刻度がErrorレベルであるかどうかを判定します。
    //  最新のエラーの深刻度がSEV_ERRORであればtrue、そうでなければfalseが返ります。
    static qBool IsError();

    //n1589
    // 渡されたエラーの深刻度がErrorレベルであるかどうかを判定します。
    // uiCodeには、 GetLast() で取得できるエラーを渡します。
    // エラーの深刻度がSEV_ERRORであればtrue、そうでなければfalseが返ります。
    static qBool IsError(qUnsignedInt32 uiCode);

    //n1769
    // {Secret}
    // 渡されたエラーの深刻度がWarningレベルであるかどうかを判定します。
    // uiCodeには、 GetLast() で取得できるエラーを渡します。
    // エラーの深刻度がSEV_WARNINGであればtrue、そうでなければfalseが返ります。
    static qBool IsWarning(qUnsignedInt32 uiCode);

    //n1769
    // {Secret}
    // 渡されたエラーの深刻度がInformationalレベルであるかどうかを判定します。
    // uiCodeには、 GetLast() で取得できるエラーを渡します。
    // エラーの深刻度がSEV_INFORMATIONALであればtrue、そうでなければfalseが返ります。
    static qBool IsInformational(qUnsignedInt32 uiCode);

    //n1769
    // {Secret}
    // 渡されたエラーの深刻度がSuccessレベルであるかどうかを判定します。
    // uiCodeには、 GetLast() で取得できるエラーを渡します。
    // エラーの深刻度がSEV_SUCCESSであればtrue、そうでなければfalseが返ります。
    static qBool IsSuccess(qUnsignedInt32 uiCode);

    /*
    Summary: Traces the most recent error.
    Parameters:
    uiTraceFlag: A flag that determines what data is traced. Refer to DataLogging for a list of valid trace flags.
    Remarks: This method outputs a string describing the most recent error to the display, the debugger, or a file.
    See Also: Data Logging
    */

    /*!
    @internal
    @brief  直近のエラーのトレースを実行します。

    @param[in] uiTraceFlag トレースフラグを指定します。
    */
    static void TraceLast(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS);

    // {Secret}
    qUnsignedInt32 GetErrorCode()
    {
        return m_uiErrorCode;
    }
    // {Secret}
    qUnsignedInt32 GetExtErrorCode()
    {
        return m_uiExtErrorCode;
    }

    //n1769
    // {Secret}
    // 指定したスレッドIDに対応するエラーコード関係の要素を除去します。
    // このAPIは、スレッドの生成や破棄を繰り返すような状況で、メモリリークを防止するために
    // 用意されています。
    // このAPIはNEXライブラリ側から呼び出すことを想定しているため、
    // アプリケーションプログラムがこのAPIを直接呼び出すのは避けて下さい。
    static void EraseErrorElements(ThreadID threadID);

    //n1769 ↑と異なり、こちらは{Secret}なAPIではないことに注意。
    /*!
    @brief このAPIを呼び出したスレッドに対応するエラー情報をSystemErrorクラスから除去します。

    アプリケーションプログラム側でスレッドを作成し、そのスレッドが
    NEX APIを呼び出している場合は、そのスレッドが終了する直前の
    タイミングで、そのスレッドの中からこのAPIを呼び出して下さい。

    NEXでは、エラー情報はスレッドごと（より正確には、スレッドIDごと）に
    記録されます。そのため、NEX APIを呼び出すスレッドを頻繁に生成／破棄
    するような状況では、作成したスレッドの数だけエラー情報を記録するメモリ領域が
    確保されていくため、最終的にはメモリリークによるメモリ枯渇でクラッシュします。
    本APIは、このような事態を避けるために用意されています。

    もし nn::nex::ObjectThread を用いてスレッドを実行しているのであれば、
    このAPIを呼び出す必要はありません。同等の処理が nn::nex::ObjectThreadRoot で
    行われるようになっているためです。

    @see   nn::nex::ObjectThread
    */
    static void EraseErrorElements(void);

    //n1769
    // {Secret}
    // これまでに記録されていたエラー情報の全てをコンテナから除去します。
    // この関数はNEXライブラリ内部から呼び出されることを想定しているので、
    // アプリケーションプログラムはこの関数を直接呼び出さないで下さい。
    static void EraseAllErrorElements(void);

private:
    qUnsignedInt32      m_uiErrorCode;
    qUnsignedInt32      m_uiExtErrorCode;
    const qChar         *m_szFilename;
    qUnsignedInt        m_uiLineNo;
};

END_NAMESPACE_NEX


