﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

// Group = Time Management

/*
Description: This class provides the actual time used by $PRODUCTNAME$ in various time-related
computations.
Remarks: A custom TimeProvider may be used to provide the time that $PRODUCTNAME$ uses for
time related computations, such as those associated with the SessionClock and LocalClock.
In order to implement your own TimeProvider you must sub class this class and implement
the GetTime method.
One situation where you may want to implement your own TimeProvider is where you require time
related computations within your game to be identical to those performed by $PRODUCTNAME$.
*/

/*!
 * @brief   タイムプロバイダののクラスです。
 *
 * タイムプロバイダとは、時間値を提供するクラスオブジェクトです。
 * このクラスが提供する値によって、@ref SystemClock クラスや、
 * @ref SessionClock クラスが時間計算を行うことが出来るようになります。
 * アプリケーションでカスタマイズされたタイムプロバイダを設定するためには、
 * クラスから派生したクラスを作成し、@ref GetTime 関数をオーバーライドして実装する必要があります。
 *
 * このクラスを使用しない場合でもシステムはデフォルトのタイムプロバイダを用意しています。
 * よりパフォーマンスのよい、あるいはより高精度のタイムプロバイダを作成する場合は、
 * このクラスによりカスタマイズすることが出来ます。
 *
 * @see     SystemClock::RegisterTimeProvider
 * @see     UpdatableTimeProvider
 */
class TimeProvider: public RootObject
{
public:
    //group=Methods
    TimeProvider() {}
    virtual ~TimeProvider() {}

    /*
    Description: Returns the Time as defined by this TimeProvider.
    Returns: The current Time as defined by this TimeProvider.
    Remarks: If you sub class the TimeProvider class you must implement this method.
    */

    /*!
     * @brief   タイムプロバイダによって定義される時間を返します。
     *
     * この関数は純粋仮想関数なので派生クラスを作る場合に実装する必要があります。
     *
     * @return  タイムプロバイダが提供する時間値をミリ秒単位で返します。
     */
    virtual Time GetTime() = 0;
};

END_NAMESPACE_NEX


