﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once


#if defined(NINTENDO_FAMILY)
#include <cstddef>
using std::size_t;
#endif

#if defined(NEX_NX_TARGET)
#include<limits.h>
#include<uchar.h>
#endif

// STL not optimized for level 4 warnings
#ifdef WIN_FAMILY
#pragma warning( push, 1 )
#pragma warning(disable : 4786)
#endif // WIN_FAMILY
#if defined(NEX_NX_TARGET)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wbitwise-op-parentheses" //suppress endian.h warning
#pragma clang diagnostic ignored "-Wshift-op-parentheses" // suppress endian.h warning
#endif // NEX_NX_TARGET

#ifndef CAFEIOP_TARGET
#include <string>
#endif
#if defined(NEX_NX_TARGET)
#pragma clang diagnostic pop
#endif // NEX_NX_TARGET

#ifdef WIN_FAMILY
#pragma warning( pop ) // restore previous warning
#endif // WIN_FAMILY

#if defined(LINUX_TARGET) || defined(NON_SUPPORT_A_TARGET) || defined(NON_SUPPORT_C_TARGET) || defined(MACOSX_TARGET) || defined(CAFEIOP_TARGET)
// On Linux, uint64 constants have to be postfixed with ULL
#define _ULL(CONSTANT)  CONSTANT##ULL
#define _LL(CONSTANT)   CONSTANT##LL
#else
// On Windows, prior to .NET03, that was not a valid syntax
#define _ULL(CONSTANT)  CONSTANT
#define _LL(CONSTANT)   CONSTANT
#endif

#ifdef NEX_STDINT_SUPPORT
#include <stdint.h>
#define __STDC_FORMAT_MACROS
#include <inttypes.h> // %PRId64などの書式指定子を使えるようにインクルードする
#endif

BEGIN_NAMESPACE_NEX

/*!
    @brief 論理型です。
*/
typedef bool                    qBool;

/*!
    @brief 符号なし8ビットの整数型です。
*/
typedef unsigned char           qByte;

/*!
    @brief 符号あり8ビットの整数型です。s8型と等価です。
*/
typedef signed char             qInt8;

/*!
    @brief 符号なし8ビットの整数型です。u8型と等価です。
*/
typedef unsigned char           qUnsignedInt8;

/*!
    @brief 符号あり16ビットの整数型です。s16型と等価です。
*/
typedef short                   qInt16;

/*!
    @brief 符号なし16ビットの整数型です。u16型と等価です。
*/
typedef unsigned short          qUnsignedInt16;

/*!
    @brief 符号あり32ビットの整数型です。s32型と等価です。
*/
typedef int                     qInt32;

/*!
    @brief 符号なし32ビットの整数型です。u32型と等価です。
*/
typedef unsigned int            qUnsignedInt32;

/*!
    @brief 符号ありの整数型です。
*/
typedef int                     qInt;

/*!
    @brief 符号なしの整数型です。
*/
typedef unsigned int            qUnsignedInt;


/*!
    @brief 単精度浮動小数点型です。
*/
typedef float                   qFloat;

/*!
    @brief 倍精度浮動小数点型です。
*/
typedef double                  qDouble;

/*!
    @brief 浮動小数点型です。
*/
typedef float                   qReal;
#define QREAL(_num) (_num##f)

/*!
    @brief 符号あり64ビットの整数型です。s64型と等価です。
*/
#ifdef NEX_STDINT_SUPPORT
typedef int64_t                 qInt64;
#else
typedef long long               qInt64;
#endif

/*!
    @brief 符号なし64ビットの整数型です。u64型と等価です。
*/
#ifdef NEX_STDINT_SUPPORT
typedef uint64_t                qUnsignedInt64;
#else
typedef unsigned long long      qUnsignedInt64;
#endif

/*!
    @brief 8ビットの文字型です。
*/
typedef char                    qChar8;


#if defined(NEX_QCHAR16_IS_NOT_UINT16_T)
/*!
    @brief 16ビットの文字型です。
 */
#if defined(NEX_SIGLO_FAMILY)  || (defined(_MSC_VER) && _MSC_VER >= 1900)
typedef char16_t                qChar16; // VS2015およびNX
#else
typedef wchar_t                 qChar16;
#endif
#else

/*!
    @brief 16ビットの文字型です。
*/
#if defined(NEX_SIGLO_FAMILY)
typedef char16_t                qChar16; // VS2015未満の場合。qUnsignedInt16と等しい。
#else
typedef qUnsignedInt16          qChar16;
#endif
#endif

/*!
    @brief ワイド文字を扱う文字型です。
*/
/* With Linux, wchar_t is 32-bit wide, probably to accomodate UTF-32. */
#if defined(UNIX_FAMILY) || defined(NEX_SIGLO_FAMILY) || defined(WIN_FAMILY)
typedef wchar_t                 qWideChar;
#else
typedef qChar16                 qWideChar;
#endif

#ifdef NEX_UNICODE_VERSION
#define USE_UNICODECHAR
#endif

#ifdef USE_UNICODECHAR
#ifdef NEX_QCHAR16_IS_NOT_WCHAR_T
typedef qWideChar               qChar;
#else
/*!
    @brief NEXで通常使用される文字型。 qChar16 のtypedefです。
*/
typedef qChar16                 qChar;
#endif
#else
/*!
    @brief 8ビットの文字型です。
*/
typedef qChar8                  qChar;
#endif

//Description: An ID generated by an IDGenerator object that can be assigned to a given entity.
//See Also: IDGenerator, DuplicatedObject::Create
/*!
    @brief IDGenerator で生成されるエンティティごとに一意なIDです。
*/
typedef qUnsignedInt32          ID;

#define QBYTE_BIT             8                     /* number of bits in a qByte */
#define QBYTE_MAX             0xff                  /* maximum qByte value */
#define QBYTE_MIN             0                     /* minimum qByte value */

#define QINT8_BIT             8                     /* number of bits in a qInt8 */
#define QINT8_MAX             127                   /* maximum qChar value */
#define QINT8_MIN           (-QINT8_MAX - 1)        /* minimum qChar value */

#define QUNSIGNEDINT8_BIT     QBYTE_BIT             /* number of bits in a qUnsignedInt8 */
#define QUNSIGNEDINT8_MAX     QBYTE_MAX
#define QUNSIGNEDINT8_MIN     QBYTE_MIN

#define QINT16_BIT            16                    /* number of bits in a qInt16 */
#define QINT16_MAX            32767                 /* maximum qInt16 value */
#define QINT16_MIN          (-QINT16_MAX - 1)       /* minimum qInt16 value */

#define QUNSIGNEDINT16_BIT    QINT16_BIT            /* number of bits in a qInt16 */
#define QUNSIGNEDINT16_MAX    0xffff                /* maximum qUnsignedInt16 value */
#define QUNSIGNEDINT16_MIN    0                     /* minimum qUnsignedInt16 value */

#define QINT32_BIT            32                    /* number of bits in a qInt32 */
#define QINT32_MAX            2147483647            /* maximum qInt32 value */
#define QINT32_MIN          (-QINT32_MAX - 1)       /* minimum qInt32 value */

#define QUNSIGNEDINT32_BIT    QINT32_BIT            /* number of bits in a qInt16 */
#define QUNSIGNEDINT32_MAX    0xffffffff            /* maximum qUnsignedInt32 value */
#define QUNSIGNEDINT32_MIN    0                     /* minimum qUnsignedInt32 value */

#define QINT_BIT              QINT32_BIT
#define QINT_MAX              QINT32_MAX            /* maximum qInt value */
#define QINT_MIN              QINT32_MIN            /* minimum qInt value */

#define QUNSIGNEDINT_BIT      QUNSIGNEDINT32_BIT
#define QUNSIGNEDINT_MAX      QUNSIGNEDINT32_MAX    /* maximum qUnsignedInt value */
#define QUNSIGNEDINT_MIN      QUNSIGNEDINT32_MIN    /* minimum qUnsignedInt value */

#define QINT64_BIT            64                    /* number of bits in a qInt64 */
#define QINT64_MAX            _ULL(9223372036854775807)   /* maximum qInt64 value */
#define QINT64_MIN            (-QINT64_MAX - 1)     /* minimum qInt64 value */

#define QUNSIGNEDINT64_BIT    QINT64_BIT            /* number of bits in a qInt16 */
#define QUNSIGNEDINT64_MAX    _LL(0xffffffffffffffff)    /* maximum qUnsignedInt64 value */
#define QUNSIGNEDINT64_MIN    0                     /* minimum qUnsignedInt64 value */

#define QCHAR8_BIT         QINT8_BIT             /* number of bits in a Unicode qChar */

// NXターゲットの場合は、charが unsigned char
#if defined(NEX_NX_TARGET)
#define QCHAR8_MAX         CHAR_MAX             /* maximum Unicode qChar value */
#define QCHAR8_MIN         CHAR_MIN             /* minimum Unicode qChar value */
#else
#define QCHAR8_MAX         QINT8_MAX             /* maximum Unicode qChar value */
#define QCHAR8_MIN         QINT8_MIN             /* minimum Unicode qChar value */
#endif

#define QCHAR16_BIT      QUNSIGNEDINT16_BIT    /* number of bits in a Unicode qChar */
#define QCHAR16_MAX      QUNSIGNEDINT16_MAX    /* maximum Unicode qChar value */
#define QCHAR16_MIN      QUNSIGNEDINT16_MIN    /* minimum Unicode qChar value */

#ifdef NEX_UNICODE_VERSION
#define QCHAR_BIT             QCHAR16_BIT      /* number of bits in a Unicode qChar */
#define QCHAR_MAX             QCHAR16_MAX      /* maximum Unicode qChar value */
#define QCHAR_MIN             QCHAR16_MIN      /* minimum Unicode qChar value */
#else
#define QCHAR_BIT             QCHAR8_BIT         /* number of bits in an ANSI qChar */
#define QCHAR_MAX             QCHAR8_MAX         /* maximum ANSI qChar value */
#define QCHAR_MIN             QCHAR8_MIN         /* minimum ANSI qChar value */
#endif // NEX_UNICODE_VERSION

#if defined(NEX_SIGLO_FAMILY)
typedef uintptr_t ThreadID;
#else
typedef qUnsignedInt32 ThreadID;
#endif

END_NAMESPACE_NEX

