﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

// For NEX Developer See jwiki:MemoryAllocation for an overview of memory allocation in
// the system.

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/MemAllocator.h>

// STL compiles with level 4 warnings...
#ifdef WIN_FAMILY
#pragma warning( push, 1 )
#endif

#include <list>

#ifdef WIN_FAMILY
#pragma warning( pop ) /* restore previous warning */
#endif

BEGIN_NAMESPACE_NEX

//$ifdef SPARK$
//Group=Lists
//$else$
//Group=Data Types
//$endif$

/*
Description: A list container template.
Remarks: This list inherits all operations from the STL list container template,
std::list, the only difference being that qList allocates memory using
the Quazal memory allocator. Standard operations such as size(), empty(),
push_back(), pop_front(), and insert() may be used with this template.
*/

/*!
    @brief  List 型コンテナのテンプレートクラスです

    std::listを継承していますが qList はアロケータに MemoryManager で指定したメモリ確保・開放関数を用いる点が異なります。

    @tparam T List 型コンテナにするオブジェクト。
*/
template <typename T> class qList
#ifdef NEX_ALLOCATOR_SUPPORT
    : public std::list<T, MemAllocator<T> >, public RootObject
#else
    : public std::list<T>, public RootObject
#endif
{
};

END_NAMESPACE_NEX

