﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/MemAllocator.h>

// STL compiles with level 4 warnings...
#ifdef WIN_FAMILY
#pragma warning( push, 1 )
#include <set>
#pragma warning( pop ) /* restore previous warning */
#else
#include <set>
#endif

BEGIN_NAMESPACE_NEX

//==============================================================================
// Inherits all operations from the std::set using the appropriate allocator.
//==============================================================================
/*!
    @brief  Set 型コンテナのテンプレートクラスです

    std::setを継承していますが qSet はアロケータに MemoryManager で指定したメモリ確保・開放関数を用いる点が異なります。

    @tparam Key Set 型コンテナのKeyとなるオブジェクト。
*/
template <typename Key> class qSet
#if defined(NEX_ALLOCATOR_SUPPORT)
    : public std::set<Key, std::less<Key>, MemAllocator<Key> >, public RootObject
#else
    : public std::set<Key>, public RootObject
#endif
{
};

END_NAMESPACE_NEX

