﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <algorithm>

BEGIN_NAMESPACE_NEX

template<typename K, typename V>
class qSortedVector : public RootObject
{
public:
    typedef K key_type;
    typedef std::pair<K,V> value_type;
    typedef typename qVector<value_type>::size_type size_type;
    typedef typename qVector<value_type>::iterator iterator;
    typedef typename qVector<value_type>::const_iterator const_iterator;

    // From Effective STL item 23
    class KeyPredicate
    {
    public:
        KeyPredicate() {}
        qBool operator()(const value_type& lhs,
                         const value_type& rhs) const
        {
            return keyLess(lhs.first,rhs.first);
        }
        qBool operator()(const value_type& lhs,
                         const typename value_type::first_type& k) const
        {
            return keyLess(lhs.first,k);
        }
        qBool operator()(const typename value_type::first_type &k,
                         const value_type &rhs) const
        {
            return keyLess(k,rhs.first);
        }
    private:
        qBool keyLess(const typename value_type::first_type &k1,
                      const typename value_type::first_type &k2) const
        {
            return k1 < k2;
        }
    };
    qSortedVector() {}
    explicit qSortedVector(size_t vecSize)
    {
        m_v.reserve(vecSize);
    }
    iterator begin()
    {
        return m_v.begin();
    }
    iterator end()
    {
        return m_v.end();
    }
    const_iterator begin() const
    {
        return m_v.begin();
    }
    const_iterator end() const
    {
        return m_v.end();
    }

    qBool empty()
    {
        return m_v.empty();
    }
    size_type size()
    {
        return m_v.size();
    }

    iterator find(const key_type& k)
    {
        KeyPredicate oKeyPredicate;
        iterator it = std::lower_bound(m_v.begin(),m_v.end(),k,oKeyPredicate);
        if( it != m_v.end() && oKeyPredicate(k,*it) )
        {
            it = m_v.end();
        }
        return it;
    }
    const_iterator find(const key_type& k) const
    {
        KeyPredicate oKeyPredicate;
        const_iterator it = std::lower_bound(m_v.begin(),m_v.end(),k,oKeyPredicate);
        if( it != m_v.end() && oKeyPredicate(k,*it) )
        {
            it = m_v.end();
        }
        return it;
    }

    std::pair<iterator, qBool> insert(const value_type& x);
    size_type erase(const key_type& k);
private:
    qVector<value_type> m_v;
};

template<typename K, typename V>
std::pair<typename qSortedVector<K,V>::iterator, qBool> qSortedVector<K,V>::insert(const value_type& x)
{
    qBool bNotFound = false;
    KeyPredicate oKeyPredicate;
    iterator it = std::lower_bound(m_v.begin(), m_v.end(), x, oKeyPredicate);
    if( it == m_v.end() || oKeyPredicate(x,*it) )
    {
        it = m_v.insert(it,x);
        bNotFound = true;
    }
    return std::make_pair(it,bNotFound);
}

template<typename K, typename V>
typename qSortedVector<K,V>::size_type qSortedVector<K,V>::erase(const key_type& k)
{
    iterator it = find(k);
    if( it != m_v.end() )
    {
        m_v.erase(it);
        return 1;
    }
    else
    {
        return 0;
    }
}

END_NAMESPACE_NEX

