﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

#ifdef NEX_UNORDERED_MAP_SUPPORT

#include <OnlineCore/src/Platform/Core/MemAllocator.h>
#include <unordered_map> 

BEGIN_NAMESPACE_NEX

/*!
    @brief  unordered_multimap 型コンテナのテンプレートクラスです

    std::unordered_multimapを継承していますが qUnorderedMultiMap はアロケータに MemoryManager で指定したメモリ確保・開放関数を用いる点が異なります。

    @tparam Key unordered_multimap 型コンテナのKeyとなるクラス
    @tparam T unordered_multimap 型コンテナのValueとなるクラス
    @tparam Hash unordered_multimap 型のハッシュを生成するクラス
    @tparam Pred unordered_multimap　型の比較するクラス
*/
template <class Key, class T, class Hash = std::hash<Key>, class Pred = std::equal_to<Key> > class qUnorderedMultiMap
#if defined(NEX_STD_TREE_PAIR_ALLOCATOR_SUPPORT)
    : public std::unordered_multimap<Key, T, Hash, Pred, MemAllocator<std::pair<const Key,T> > >, public RootObject

#elif defined(NEX_ALLOCATOR_SUPPORT)
    : public std::unordered_multimap<Key, T, Hash, Pred, MemAllocator<T> >, public RootObject

#else
    : public std::unordered_multimap<Key, T, Hash, Pred>, public RootObject

#endif
{
public:
};


END_NAMESPACE_NEX

#endif
