﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/MemAllocator.h>

// STL compiles with level 4 warnings...
#ifdef WIN_FAMILY
#pragma warning( push, 1 )
#endif

#include <vector>

#ifdef WIN_FAMILY
#pragma warning( pop ) // Restore previous warning
#endif

BEGIN_NAMESPACE_NEX

//==============================================================================
// Inherits all operations from the std::vector using the appropriate allocator.
//==============================================================================
/*!
    @brief  Vector 型コンテナのテンプレートクラスです

    std::vector を継承していますが qVector はアロケータに MemoryManager で指定したメモリ確保・開放関数を用いる点が異なります。

    @tparam T Vector 型コンテナにするオブジェクト。
*/
template <typename T> class qVector
#ifdef NEX_ALLOCATOR_SUPPORT
    : public std::vector<T, MemAllocator<T> >, public RootObject
#else
    : public std::vector<T>, public RootObject
#endif
{
public:
    qVector()
#ifdef NEX_ALLOCATOR_SUPPORT
        : std::vector<T, MemAllocator<T> >()
#else
        : std::vector<T>()
#endif
    {
    }
#ifdef NEX_ALLOCATOR_SUPPORT
    explicit qVector(typename std::vector<T, MemAllocator<T> >::size_type size)
        : std::vector<T, MemAllocator<T> >(size)
#else
    explicit qVector(typename std::vector<T>::size_type size)
        : std::vector<T>(size)
#endif
    {
    }
};

END_NAMESPACE_NEX

