﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once


#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Stack/Core/Socket.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>

//n1589
#define BERKELEY_SOCKET_DRIVER_PEER_ADDRESS_MAX 15 // 自分を除くStationのアドレスを保存する最大数。SendMulti用のアドレスとして使用。
//n1589:todo2:ポート0がSendMulti用であることを内部仕様ドキュメントに掲載しておきたい
#define BERKLEY_SOCKET_DRIVER_SEND_MULTI_PORT 0

BEGIN_NAMESPACE_NEX

#if NEX_ENABLE_BERKLEY_SOCKET_PROFILING
typedef struct _BerkeleySocketAnalyzeInfo
{
    qUnsignedInt32 uiSendToCount;
    qUnsignedInt32 uiSendToMultiCount;
    qUnsignedInt32 uiRecvFromCount;
    qUnsignedInt32 uiRecvFromTryCount;
} BerkeleySocketAnalyzeInfo;
#endif

class BerkeleySocketDriver: public SocketDriver
{
public:
    class BerkeleySocket;

#if NEX_ENABLE_BERKLEY_SOCKET_PROFILING
    static void ResetAnalyzeInfo();
    static void LogAnalyzeInfo(EventLog::LogLevel logLevel);
    static BerkeleySocketAnalyzeInfo* GetAnalyzeInfo();
#endif

#if NEX_ENABLE_UNRELIABLE_BC
    static qBool IsSendMultiAddress(const InetAddress* address);
    static qUnsignedInt16 GetSendMultiPort();

    static void SetPeerAddress(const NAMESPACE_NEX InetAddress* addresses, qUnsignedInt32 num);
    static void GetPeerAddress(NAMESPACE_NEX InetAddress* addresses, qUnsignedInt32 numIn, qUnsignedInt32* numOut);

#endif // NEX_ENABLE_UNRELIABLE_BC

    BerkeleySocketDriver() {}
    virtual ~BerkeleySocketDriver() {}

    virtual Socket *Create();
    virtual void Delete(SocketDriver::Socket *pSocket);

    virtual qBool Poll(PollInfo aPollInfo[], qUnsignedInt uiPollInfoSize, qUnsignedInt32 uiTimeoutInMs);
    static qUnsignedInt16 GetBoundPort(SocketHandle fd);
    static qBool StaticPoll(PollInfo aPollInfo[], qUnsignedInt uiPollInfoSize, qUnsignedInt32 uiTimeoutInMs);
    static SocketDriver::Result CheckOrWaitForTransmission(SocketDriver::Socket* pDriverSocket, qBool bNonBlock, qBool isSend);
    static SocketDriver::Result StaticShutdown(SocketHandle socketHandle);
    static SocketDriver::Result StaticGetPeerName(SocketHandle fd, SocketDriver::InetAddress* addr);
    static qBool StaticSetAsync(SocketHandle socket, qBool bAsyncMode);
};

END_NAMESPACE_NEX

