﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once


#include <OnlineCore/src/Platform/Stack/Core/InetDecl.h>
#include <OnlineCore/src/Platform/Stack/Core/SocketDriver.h>

BEGIN_NAMESPACE_NEX

class ClientWebSocketDriver: public WebSocketDriver
{
public:
    class ClientWebSocket;
    ClientWebSocketDriver()
#if defined(NEX_FORCE_USE_NATIVE_WINSOCK)
        : m_sslAllowSelfSignedCerts(true)
#else
        : m_sslAllowSelfSignedCerts(false)
#endif
        , m_sslRootCaCertPath()
    {}
    virtual ~ClientWebSocketDriver() {}

    virtual Socket *Create();
    virtual void Delete(SocketDriver::Socket *pSocket);

    virtual qBool Poll(PollInfo aPollInfo[], qUnsignedInt uiPollInfoSize, qUnsignedInt32 uiTimeoutInMs);

    // 自己署名された証明書を許容するかどうかをセットする。Connect側のみ影響する。
    virtual void SetSslAllowSelfSignedCerts(qBool allow)
    {
        m_sslAllowSelfSignedCerts = allow;
    }

    // 自己署名された証明書を許容するかどうかを取得する。
    virtual qBool GetSslAllowSelfSignedCerts()
    {
        return m_sslAllowSelfSignedCerts;
    }

    // 使用するルートCA証明書のパスをセットする。Connect側のみ影響する。
    virtual void SetSslRootCaCertPath(const String path)
    {
        m_sslRootCaCertPath = path;
    }

    // 使用するルートCA証明書のパスを取得する。
    virtual String GetSslRootCaCertPath() const
    {
        return m_sslRootCaCertPath;
    }
private:
    qBool m_sslAllowSelfSignedCerts;
    String m_sslRootCaCertPath;
};

END_NAMESPACE_NEX
