﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Stack/Core/InetDecl.h>
#include <OnlineCore/src/Platform/Stack/Core/IOCompletionContext.h>


BEGIN_NAMESPACE_NEX

//n1589:todo2:MAX_NB_CONTEXT 8, 8もいらないのでは
#define MAX_NB_CONTEXT 8

class IOCompletionNotifier: public RootObject
{
public:
    IOCompletionNotifier();
    ~IOCompletionNotifier();

    IOCompletionContext*    CreateIOCompletionContext();
    void                    DeleteIOCompletionContext(IOCompletionContext* pIOCompletionContext);
    IOCompletionContext*    CreateIOCompletionContextSimple();
    void                    DeleteIOCompletionContextSimple(IOCompletionContext* pIOCompletionContext);
    qBool                   WaitForIOCompletion(qUnsignedInt uiTimeout);

    //n1589:memo:only called by QueuingSocket::ClearCompletionContext
    //  No-op for non WINSOCK2_STACK_SUPPORT implementation. Was this done on purpose?
    qBool                   WaitForIOCompletion(IOCompletionContext* pIOCompletionContext, qUnsignedInt uiTimeout);

#ifdef WINSOCK2_STACK_SUPPORT
    //  For monitoring send/receive calls
    //  If it returns true, the IO is completed
    qBool                   CheckForIOCompletion(IOCompletionContext* pIOCompletionContext, qBool& bPending, qUnsignedInt32& uiErrCode);
#endif

    qBool WaitForPollIOCompletion(qUnsignedInt uiTimeout);

private:
    CriticalSection         m_csTable;
    IOCompletionContext     *m_pIOCompletionContextTable[MAX_NB_CONTEXT];
#ifdef WINSOCK2_STACK_SUPPORT
    WSAEVENT*               m_pWSAEventTable;
#endif

protected:

#ifdef WINSOCK2_STACK_SUPPORT
    static qUnsignedInt32 WaitForWin32Event(WSAEVENT *pEventTable, qUnsignedInt uiNbEvents, qUnsignedInt uiTimeout);
#endif

};

END_NAMESPACE_NEX

