﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once


#include <OnlineCore/src/Platform/Stack/Core/InetDecl.h>
#include <OnlineCore/src/Platform/Core/Hash.h>

#if !defined(NEX_SIGLO_FAMILY) && !defined(WIN_FAMILY)
// TODO: 単にinlineする最適化。else節の方は本当に必要なものか怪しい。
#define NEX_INET_ADDRESS_GETKEY_OPTIMIZATION
#endif

BEGIN_NAMESPACE_NEX

// IPアドレスの最大文字列サイズ。IPv6非対応。
#define NEX_INET_ADDRESS_STRING_SIZE_MAX sizeof(NEX_T("255.255.255.255"))

/*!
    @brief アドレス情報を表します。
 */
class InetAddress: public RootObject
{
public:
    //n1589
    static void EnableAutoLookup(qBool o);

    InetAddress();
    InetAddress(const qChar *szMachine, const qUnsignedInt16 uiPort);
    InetAddress(const qUnsignedInt32 uiAddress, const qUnsignedInt16 uiPort);
    InetAddress(void *pAddr, qUnsignedInt uiSizeOfType);
    InetAddress(const InetAddress &oInetAddress);
    virtual ~InetAddress();

    void Init();

    qBool SetAddress(const qChar *szAddress);
    void SetAddress(const qUnsignedInt32 uiAddress);
    void SetNetworkAddress(const qUnsignedInt32 uiAddress);
    qBool GetAddress(qChar *szAddress, size_t uiAddressLen) const;

    /*!
        @brief IP アドレスを取得します。

        返値は常にホストバイトオーダーです。
        @return IP アドレス。
     */
    qUnsignedInt32 GetAddress() const;
    String GetAddressStr() const;
    qUnsignedInt32 GetProxyAddress() const { return m_oProxyAddress;}
    void SetProxyAddress(qUnsignedInt32 address) { m_oProxyAddress = address;}
    void SetPortNumber(const qUnsignedInt16 ui16Port);
    void SetNetworkPortNumber(const qUnsignedInt16 ui16Port);

    /*!
        @brief ポート番号を取得します。

        返値は常にホストバイトオーダーです。
        @return ポート番号。
     */
    qUnsignedInt16 GetPortNumber() const;
    String GetPortNumberStr() const;
    qBool GetPortNumber(qChar *szPort, qUnsignedInt32 uiPortLen) const;

    qBool SetLocalHost();
    qBool IsLocalHost() const;

#if !defined(NEX_INET_ADDRESS_GETKEY_OPTIMIZATION)
    //n2967:Windowsでは、Winsockをincludeするとmaxなどがコンフリクトするので、最適化しない
    qBool operator==(const InetAddress &oInetAddress) const
    {
        return (GetKey()==oInetAddress.GetKey() && GetProxyAddress() == oInetAddress.GetProxyAddress());
    }
#else
    inline qBool operator==(const InetAddress &oInetAddress) const
    {
        return (GetSoackAddrInAddress(GetSockAddrIn()) == GetSoackAddrInAddress(oInetAddress.GetSockAddrIn()))
               && (GetSoackAddrInPort(GetSockAddrIn()) == GetSoackAddrInPort(oInetAddress.GetSockAddrIn())
               && (GetProxyAddress() == oInetAddress.GetProxyAddress()));
    }
#endif

    qBool operator!=(const InetAddress &oInetAddress) const
    {
        return !operator==(oInetAddress);
    }
    qBool operator<(const InetAddress &oInetAddress) const
    {
        return (GetKey() < oInetAddress.GetKey()) || (GetKey() == oInetAddress.GetKey() && GetProxyAddress() < oInetAddress.GetProxyAddress());
    }

    InetAddress &operator=(const InetAddress &oInetAddress);

    void Trace(qUnsignedInt64 uiTraceFlag) const;

    void ToStr(qChar *tszStr) const;
    String ToStr() const;


    void* GetAddressPtr()
    {
        return (void*)&m_oAddressBytes;
    }

    static qUnsignedInt32 String2Address(const qChar8 *cp);

private:
    //n1589
    static qBool s_bAutoLookup;

#ifndef NEX_INET_ADDRESS_GETKEY_OPTIMIZATION
    qInt64 GetKey() const;
    qUnsignedInt32 m_oAddressBytes[32];
#else
    inline qInt64 GetKey() const
    {
        return
            ( static_cast<qInt64>(GetSoackAddrInAddress(GetSockAddrIn())) << 32 ) |
            ( static_cast<qInt64>(GetSoackAddrInPort(GetSockAddrIn())) ); //n2343:高速化のためバイトオーダーを気にせず使います
    }
    SockAddrStorage m_oAddressBytes;

    inline SockAddrIn &GetSockAddrIn() const
    {
        return *((NAMESPACE_NEX SockAddrIn*)&m_oAddressBytes);
    }

    static inline qUnsignedInt32 GetSoackAddrInAddress(const SockAddrIn& x);
    static inline qUnsignedInt16 GetSoackAddrInPort(const SockAddrIn& x);
#endif
    qUnsignedInt32 m_oProxyAddress; // Proxyのアドレスを保持
};

StringStream& operator<<(StringStream& oStr, const InetAddress& oAddress);

#ifdef NEX_INET_ADDRESS_GETKEY_OPTIMIZATION

#if defined(WII_STACK) || defined(CTR_TARGET)
qUnsignedInt32 InetAddress::GetSoackAddrInAddress(const SockAddrIn& x)
{
    return x.addr.addr;
}
qUnsignedInt16 InetAddress::GetSoackAddrInPort(const SockAddrIn& x)
{
    return x.port;
}
#elif defined(BERKELEY_STACK_SUPPORT)
qUnsignedInt32 InetAddress::GetSoackAddrInAddress(const SockAddrIn& x)
{
    return x.sin_addr.s_addr;
}
qUnsignedInt16 InetAddress::GetSoackAddrInPort(const SockAddrIn& x)
{
    return x.sin_port;
}
#else
qUnsignedInt32 InetAddress::GetSoackAddrInAddress(const SockAddrIn& x)
{
    return x.sin_addr.S_un.S_addr;
}
qUnsignedInt16 InetAddress::GetSoackAddrInPort(const SockAddrIn& x)
{
    return x.sin_port;
}
#endif
#endif

#if (defined(WIN_FAMILY) && !defined(VS2008_COMPILER)) || defined(UNIX_FAMILY)


inline std::size_t hash_value (const InetAddress& d)
{
    std::size_t hash = 0;
    HashCombine(hash, d.GetAddress());
    HashCombine(hash, d.GetPortNumber());
    HashCombine(hash, d.GetProxyAddress());

    return hash;
}

#endif

END_NAMESPACE_NEX

