﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

#include <OnlineCore/src/Transport/Interface/TransportEventHandler.h>
#include <OnlineCore/src/Transport/Interface/StreamTable.h>
#include <OnlineCore/src/Core/InstanceControl.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Core/ComponentState.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>

#include <OnlineCore/src/Platform/Core/qList.h>

BEGIN_NAMESPACE_NEX

#if     NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB
class JobConnectEndPoint;
#endif  // NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB

//==============================================================================
// Manages a set of transport, sharing connections to a set of other stations. A
// connection to a station, through which ever transport, is considered
// equivalent to one via any other transport. All incoming and outcoming
// connections will share the same end point event dispatcher.
//==============================================================================
class ConnectionManager: public TransportEventHandler
{
public:
    typedef qBool (*ConnectionFilter)(const UserContext &oContext,
                                      Buffer* pInBuffer, Buffer* pOutBuffer, EndPoint* pEndPoint);

    ConnectionManager(
        EndPointEventHandler *pEndPointEventHandler,
        Job::JobType jobTypeToUse,
        qBool bDirectConnection = true,
        qBool bRoutedConnection = false
    );
    virtual ~ConnectionManager();

    void SetInstanceContext(ICContext icContext)
    {
        m_icContext=icContext;
    }

    ComponentState* GetComponentState()
    {
        return &m_oConnectionManagerState;
    }
    const ComponentState* GetComponentState() const
    {
        return &m_oConnectionManagerState;
    }

    void Terminate();
    qBool IsTerminated() const;
    qBool IsTerminating() const;

    void AttachStream(ConnectionOrientedStream *pStream, qBool bOwner);
    //n1589:memo:not used
    StreamTable& GetStreamTable()
    {
        return m_oStreams;
    }

    void RegisterConnectionFilter(ConnectionFilter pfConnectionFilter, UserContext oConnectionFilterContext);

    void AcceptIncomingConnections()
    {
        m_bAcceptIncomingConnections=true;
    }
    void DenyIncomingConnections()
    {
        m_bAcceptIncomingConnections=false;
    }

    virtual qBool ConnectionRequest(ConnectionOrientedStream *pStream,
                                    const StationURL *pStationURL, Buffer *pBuffer, EndPoint *pEndPoint);

    virtual qBool FilterConnectionRequest(ConnectionOrientedStream *pStream,
                                          const StationURL *pStationURL, Buffer *pInOutBuffer, EndPoint *pEndPoint);

    EndPoint* OpenEndPoint(const StationURL& urlStation);

    qBool Connect(
        CallContext* pCallContext,
        const qChar* szURL,
        EndPoint** ppEndPoint
        ,qUnsignedInt32 uiTimeout

#if     NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB
        , JobConnectEndPoint** ppJob = NULL
#endif  // NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB

    )
    {
        qList<StationURL> lstURLs;
        lstURLs.push_back(StationURL(szURL));
        return ConnectImpl(
                   pCallContext,
                   NULL,
                   NULL,
                   lstURLs,
                   ppEndPoint,
                   uiTimeout,
                   false

#if     NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB
                   , ppJob
#endif  // NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB

               );
    }

    qBool Connect(
        CallContext* pCallContext,
        const StationURL& urlStation,
        EndPoint** ppEndPoint
        ,qUnsignedInt32 uiTimeout

#if     NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB
        , JobConnectEndPoint** ppJob = NULL
#endif  // NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB

    )
    {
        qList<StationURL> lstURLs;
        lstURLs.push_back(urlStation);
        return ConnectImpl(
                   pCallContext,
                   NULL,
                   NULL,
                   lstURLs,
                   ppEndPoint,
                   uiTimeout,
                   false

#if     NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB
                   , ppJob
#endif  // NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB

               );
    }

    qBool Connect(
        CallContext* pCallContext,
        Buffer *pRequestBuffer,
        Buffer *pResponseBuffer,
        const StationURL& urlStation,
        EndPoint** ppEndPoint
        ,qUnsignedInt32 uiTimeout

#if     NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB
        , JobConnectEndPoint** ppJob = NULL
#endif  // NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB

    )
    {
        qList<StationURL> lstURLs;
        lstURLs.push_back(urlStation);
        return ConnectImpl(
                   pCallContext,
                   pRequestBuffer,
                   pResponseBuffer,
                   lstURLs,
                   ppEndPoint,
                   uiTimeout,
                   false

#if     NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB
                   , ppJob
#endif  // NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB

               );
    }

    qBool Connect(
        CallContext* pCallContext,
        Buffer *pRequestBuffer,
        Buffer *pResponseBuffer,
        const qList<StationURL>& lstURLs,
        EndPoint** ppEndPoint,
        qUnsignedInt32 uiTimeout,
        qBool bAvoidPrepareNATTraversal = false

#if     NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB
                                          , JobConnectEndPoint** ppJob = NULL
#endif  // NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB

    )
    {
        return ConnectImpl(
                   pCallContext,
                   pRequestBuffer,
                   pResponseBuffer,
                   lstURLs,
                   ppEndPoint,
                   uiTimeout,
                   bAvoidPrepareNATTraversal

#if     NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB
                   , ppJob
#endif  // NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB

               );
    }

    qBool Disconnect(CallContext* pCallContext, EndPoint* pEndPoint);
    //{secret}
    qBool Disconnect(EndPoint* pEndPoint);

    qBool DirectConnectionEnabled()
    {
        return m_bDirectConnection;
    }
#ifdef NEX_ROUTING_SUPPORT
    qBool RoutedConnectionEnabled()
    {
        return m_bRoutedConnection;
    }

    InetAddress *GetRouterAddress()
    {
        return &m_oRouterAddress;
    }
    void SetRouterAddress(const InetAddress &routerAddr);

    void ConfigureEndPointForRouting(EndPoint* pEndPoint);
    void ConstructAndAssociateRoutingURL(qUnsignedInt16 id, const StationURL refStationURL, StationURL *pRoutingURL);
#endif

protected:
    ICContext m_icContext;

private:
    qBool ConnectImpl(
        CallContext* pCallContext,
        Buffer *pRequestBuffer,
        Buffer *pResponseBuffer,
        const qList<StationURL>& lstURLs,
        EndPoint** ppEndPoint,
        qUnsignedInt32 uiTimeout,
        qBool bAvoidPrepareNATTraversal

#if     NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB
        , JobConnectEndPoint** ppJob = NULL
#endif  // NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB

    );

    ComponentState m_oConnectionManagerState;
#ifdef NEX_ROUTING_SUPPORT
    InetAddress m_oRouterAddress;
#endif

    ConnectionFilter m_pfConnectionFilter;
    UserContext m_oConnectionFilterContext;

    qBool m_bAcceptIncomingConnections;
    EndPointEventHandler *m_pEndPointEventHandler;
    StreamTable m_oStreams;
    qBool m_bDirectConnection;
    qBool m_bRoutedConnection;

    Job::JobType m_jobTypeToUse;
};

END_NAMESPACE_NEX

