﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <OnlineCore/src/Transport/Interface/StationProbeList.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Core/Job.h>

BEGIN_NAMESPACE_NEX

class JobJoinSession;
class CallContext;

// Group=Session Management

/*
Description: Performs pre-connection tests to ensure that any Station that attempts to join a Session can
communicate with all current participants in the Session.
Remarks: The essence of a peer-to-peer architecture such as that used by $PRODUCTNAME$ is that each participant in a game
Session can communicate with each other participant. However, when games are played over the Internet, due to players
being located behind incompatible NAT devices it is possible that not all players can establish connection with every other
player, which leads to inconsistencies in object duplication. Therefore, to avoid this highly undesirable situation a
connectivity test should be performed via this class before a player is permitted to join any given game session.

This is the base class for all ConnectivityTester objects. In order to implement pre-connection tests you need
to sub class this class, redefine the virtual methods of the class, namely TestConnectivity, and then register
the class to a given Session via Session::RegisterConnectivityTester. Then when you join a game session the following
sequence should be followed:

    1) Register your custom ConnectivityTester to the appropriate Session via Session::RegisterConnectivityTester,
    2) Call Session::JoinSession or Session::CreateOrJoinSession,
    3) $PRODUCTNAME$ will retrieve the list of participants' StationURL s (GetParticipantsURLs) from the Session master and call TestConnectivity with this list,
    4) Perform your connectivity test,
    5) Call PostponeDecision until the result of your connectivity test is returned. True indicates that the player can connect to all current participants and false that they cannot. If true is returned then the join call will continue, if false is returned then the join call will fail with a Session::JoinOutcome of /JoinFailed/.

*/
class ConnectivityTester: public RootObject
{
    //Group=Methods
public:
    ConnectivityTester() {}
    virtual ~ConnectivityTester() {}

    //{secret}
    virtual qBool ProbeStations(CallContext *pContext, StationProbeList* preflstStationProbe, qUnsignedInt32 tiTimeout = 5000, qBool bWaitForAllResult = false, Job::JobType jobTypeToUse = Job::RV) = 0;
};


END_NAMESPACE_NEX

