﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/ConnectionID.h>
#include <OnlineCore/src/Transport/Interface/NetFaults.h>
BEGIN_NAMESPACE_NEX
/*!
 * @brief   EndPointのパケット解析情報を扱う構造体です。
 *
 * サイズの単位はバイトです。
 *
 * @see EndPointInfoInterface::GetEndPointInfo(), Station::GetEndPointInfoInterface()
 */
struct EndPointInfo : public RootObject
{
    Time resetTime; //!< リセットした時刻

    //connectionIDをプリミティブ型の最初としている See Reset
    ConnectionID   connectionID;
    qByte          streamID; //!< ストリームID
    qByte          sessionID;
    qByte          peerSessionID;
    NetFaultReason netFaultReason; //!< ネットワークでフォルトが発生した場合の理由
    qUnsignedInt32 rtt; //!< 往復時間 [msec]
    qUnsignedInt32 rttAverage; //!< 平滑化された往復時間 [msec]

    size_t sendCount; //!< パケット送信回数
    size_t sendUnreliableCount; //!< アンリライアブルパケット送信回数
    size_t sendUnreliableDataCount; //!< アンリライアブルのデータパケット送信回数
    size_t sendUnreliableBroadcastCount; //!< アンリライアブルのブロードキャストでのパケット送信回数
    size_t sendUnreliableSendMultiCount; //!< アンリライアブルのセンドマルチ機能を使ったパケット送信回数
    size_t sendPingCount; //!< Pingの送信回数 ACKはカウントしません
    size_t sendTimeoutCount; //!< タイムアウトしたパケット数
    size_t sendDisconnectWaitCount; //!< ディスコネクトパケットを送信するために待った回数
    size_t sendUnreliableSizeOverCount; //!< アンリライアブルで許容しているパケットサイズを超えて送信しようとした回数

    size_t sendTotalSize; //!< パケット送信合計サイズ
    size_t sendUnreliableTotalSize; //!< アンリライアブルパケット送信合計サイズ
    size_t sendUnreliableDataTotalSize; //!< アンリライアブルのデータパケット送信合計サイズ
    size_t sendUnreliableBroadcastTotalSize; //!< アンリライアブルのブロードキャストでのパケット送信合計サイズ
    size_t sendUnreliableSendMultiTotalSize; //!< アンリライアブルのセンドマルチ機能を使ったパケット送信合計サイズ

    size_t receiveUnreliableCount; //!< アンリライアブルパケットを受信した回数
    size_t receiveUnreliableDataCount; //!< アンリライアブルなデータパケット受信回数
    size_t receiveUnreliableBroadcastCount; //!< アンリライアブルなブロードキャストパケットの受信回数
    size_t receiveUnreliableSendMultiCount; //!< アンリライアブルのセンドマルチパケットの受信回数

    size_t receiveUnreliableTotalSize; //!< アンリライアブルパケットを受信した合計サイズ
    size_t receiveUnreliableDataTotalSize; //!< アンリライアブルなデータパケット合計サイズ
    size_t receiveUnreliableBroadcastTotalSize; //!< アンリライアブルなブロードキャストパケットの合計サイズ
    size_t receiveUnreliableSendMultiTotalSize; //!< アンリライアブルのセンドマルチパケットの合計サイズ

    size_t receivePingCount; //!< Pingの受信回数
    size_t receiveAckWantedCount; //!< Ack要求するパケットの受信回数
    size_t receiveUselessAckCount; //!< 無駄なAckを受信した回数
    size_t receiveUnreliableDropCount; //!< アンリライアブルなパケットのロス回数
    size_t receiveUnreliableNeglectCount; //!< @ref Scheduler::Dispatch のタイムアウトにより処理できなかったアンリライアブルのドロップ回数
    size_t receiveUnreliableDuplicateReorderingCount; //!< 重複発生もしくはリオーダリングによるシーケンス番号巻き戻りによってドロップしたアンリライアブルの回数

    size_t sendReliableCount; //!< リライアブル通信送信回数
    size_t sendReliableTotalSize; //!< リライアブル送信合計サイズ
    size_t sendReliableFragCount; //!< リライアブル通信でパケットフラグメンテーションが発生した回数
    size_t sendReliableFailureCount; //!< リライアブルパケットの送信ができなかった回数
    size_t sendReliableRetransmitCount; //!< リライアブルパケットの再送回数
    size_t sendReliableRetransmitTotalSize; //!< リライアブルパケットの再送回数
    size_t sendReliableRetransmitMax; //!< リライアブルパケットの最大再送回数

    size_t pushReliableCount; //!< リライアブルパケットの新規追加回数
    size_t pushReliableFailureCount; //!< リライアブルパケットの新規追加に失敗した回数

    size_t receiveReliableCount; //!< リライアブルパケットの受信回数
    size_t receiveReliableTotalSize; //!< リライアブルパケットの受信サイズ

    qUnsignedInt32 rtoMax; //!< リライアブル通信の再送タイムアウトの最大値
    qUnsignedInt32 rtoMin; //!< リライアブル通信の再送タイムアウトの最小値

    // デバッグ用
    qUnsignedInt32 sendPingLastSeqNo; // 最後に送信したPingのSeqNo
    qUnsignedInt32 receivePingLastSeqNo; // 最後に受信したPingのSeqNo
    //receivePingLastSeqNoをプリミティブ型の最後としている See Reset



    EndPointInfo();

    /*!
     * @brief 構造体の値をリセットします。
     */
    void Reset();

    /*!
     * @brief       EndPointInfo のパラメータをログ出力します。
     *
     * @param[in]   logLevel ログ出力するEventLogのログレベル
     * @see         EndPointInfo , ResetEndPointInfo() , GetEndPointInfo()
     */
    void Trace(EventLog::LogLevel logLevel = EventLog::Debug) const;
};

END_NAMESPACE_NEX
