﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/EndPointInfo.h>
#include <OnlineCore/src/Transport/Interface/SubStreamID.h>


BEGIN_NAMESPACE_NEX

/*!
 * @brief EndPointInfoを取得、操作するためのインタフェースです。
 *
 * RootObjectを継承していません。
 *
 */
class EndPointInfoInterface
{
public:
    virtual ~EndPointInfoInterface() {}
    /*!
     * @brief       EndPointInfo のパラメータをリセットします。
     *
     * @see         EndPointInfo ,  GetEndPointInfo()
     */
    virtual void ResetEndPointInfo() = 0;

    /*!
     * @brief       EndPointInfo のパラメータを取得します。
     * @param[out]   info コピー先のEndPointInfo構造体を設定します
     * @retval true 取得が成功した場合
     * @retval false 取得が失敗した場合 (infoがnull)
     * @see         EndPointInfo , ResetEndPointInfo() , GetEndPointInfo()
     */
    virtual qBool GetEndPointInfo(EndPointInfo* info) const = 0;

    /*!
     * @brief EndPointのシステムと共用のreliable送信バッファの現在のパケット数を取得します。
     *
     * reliable送信バッファは通信経路の混雑時などに徐々に増加していきます。
     * reliable送信バッファのパケット数がStream::DOのMaxWindowMapSize(SetMaxWindowMapSizeで指定したサイズ)を超えた場合、送信バッファあふれによるエラーとなります。
     *
     * @return reliable送信バッファのパケット数。
     */
    virtual size_t GetReliableBufferPacketCount() const = 0;

    /*!
     * @brief EndPointのすべてのreliable送信バッファの現在のパケット数を取得します。
     *
     * reliable送信バッファは通信経路の混雑時などに徐々に増加していきます。
     * reliable送信バッファのパケット数がStream::DOのMaxWindowMapSize(SetMaxWindowMapSizeで指定したサイズ)を超えた場合、送信バッファあふれによるエラーとなります。
     *
     * @param[out] counts パケット数を返すqVectorを指定します。添え字0は、システムと共用で、ユーザー用は添え字1から続きます。
     */
    virtual void GetReliableBufferPacketCounts(qVector<size_t>& counts) const = 0;


    /*!
     * @brief 最大SubStreamIDを取得します。
     *
     * 通信相手と StreamSettings::SetMaxUserReliableSubStreams()の値が異なった場合には、ネゴシエーションにより小さい方に最大サブストリームIDが決定されます。
     *
     * @return 最大SubStreamID
     */

    virtual SubStreamID GetMaxSubStreamID() const = 0;
};

END_NAMESPACE_NEX


