﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/EmulationDevice.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>

BEGIN_NAMESPACE_NEX

//Group = Network Emulation

/*
Description: This class defines an input EmulationDevice.
Remarks: An input emulation device is referenced via RootTransport::GetInstance()->GetInputEmulationDevice() .
EmulationDevice::Enable must be called before an emulation device can be used.
See Also: EmulationDevice, OutputEmulationDevice, RootTransport
*/

/*!
 * @brief   入力ネットワークデバイスを定義しているクラスです。デバッグ用機能です。
 *
 * 入力ネットワークデバイスは、RootTransport::GetInputEmulationDeviceで参照可能で、
 * RootTransport::GetInstance()->GetInputEmulationDevice()とすることで取得可能です。
 *
 * エミュレーションデバイスを使う前には、
 * RootTransport::GetInstance()->GetInputEmulationDevice()->Enable()
 * を呼び出す必要があります。
 *
 * このクラスは、EmulationDeviceの派生クラスで、特別なメソッドは定義されておりません。
 *
 * @see EmulationDevice, OutputEmulationDevice
 * @see RootTransport::GetInputEmulationDevice
 */
class InputEmulationDevice: public EmulationDevice
{
};


END_NAMESPACE_NEX

