﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Core/JobCallContextCallback.h>
#include <OnlineCore/src/Transport/Interface/RelaySignatureKey.h>
#include <OnlineCore/src/Core/CallContext.h>

BEGIN_NAMESPACE_NEX

class JobResolveRelayInfo : public JobCallContextCallback
{
public:
    virtual ~JobResolveRelayInfo() {}

    static JobResolveRelayInfo* CreateAndStartJob(CallContext* pContext);
    void ResolvedRelayInfo();


    void StepDNSCallCompletion();


    String& GetRelayAddressName()
    {
        return m_RelaySignatureKey.relayAddress;
    }

    RelayAddressType GetRelayAddressType()
    {
        return m_RelaySignatureKey.GetRelayAddressType();
    }


protected:
    virtual void ProccessSuccess(CallContext* pCC)
    {
        DUMMY_PARAM(pCC);
        //ResolvedRelayInfoは、StartNATSessio処理完了後に呼び出す
    }
    virtual void ProccessFailure(CallContext* pCC)
    {
        DUMMY_PARAM(pCC);
        FailResolvedRelayInfo();
    }
    virtual void ProccessCancel()
    {
        FailResolvedRelayInfo();
    }

private:
    JobResolveRelayInfo(CallContext* associatedCallContext)
        : JobCallContextCallback(Job::Core, NEX_T("StartResolveRelayInfoCallback"), associatedCallContext){}
    static void FailResolvedRelayInfo();

    qBool SetRelayAddress();





    RelaySignatureKey m_RelaySignatureKey;
};

END_NAMESPACE_NEX


