﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/ConnectionID.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <OnlineCore/src/Transport/Interface/RelaySignatureKey.h>
#include <OnlineCore/src/Transport/Interface/NATTraversalResult.h>

BEGIN_NAMESPACE_NEX

// Forward declaration
class Buffer;
class CallContext;
class NATTraversalEngine;
class SignatureKey;
class RelayAddress;
class CallContext;
class NATTraversalRelayClient;

class NATRelayInterface: public RootObject
{
public:
    NATRelayInterface();
    virtual     ~NATRelayInterface();
    
    // NATTraversalRelayClient と NATRelayInterface の間を抽象化するために登録用メソッドを追加しました。
    virtual void RegisterRelayClient(NATTraversalRelayClient *pClient) = 0;
    virtual void UnregisterRelayClient() = 0;

    virtual void RelayProbeRequest(const qList<StationURL> &lstURLRequestTargets) = 0;
    virtual void RelayProbeRequest(const qList<StationURL> &lstURLRequestTargets, const StationURL &oURLRequester) = 0;
    virtual void CheckCurrentPublicPort(CallContext *pCallContext, const InetAddress &rInterfaceAddress, qBool bPublicPortIncremental) = 0;
    virtual void DetectSimplifiedNATProperties(CallContext *pCallContext, const InetAddress &rInterfaceAddress) = 0;
    virtual void RequestProbe(const StationURL &oURLStationToProbe) = 0;

    virtual void ReportNATTraversalResult(const ConnectionID& cid, const qBool &result, const qUnsignedInt32& rtt) = 0;
    virtual void ReportNATProperties(qUnsignedInt32 natmapping, qUnsignedInt32 natfiltering, qUnsignedInt32 rtt) = 0;

    virtual qBool GetRelaySignatureKey(CallContext* pCallcontext, RelaySignatureKey* pRelaySignatureKey) = 0;

    virtual void ReportNATTraversalResultDetail(const ConnectionID& cid, const qBool &result, const NATTraversalResult &detail, qUnsignedInt32& rtt) = 0;

    // RVNATRelay や RVEcho に渡すためのインターフェース
    virtual void UpdateConnectionState(const ConnectionID cid, const StationURL& oURL) = 0;
};

END_NAMESPACE_NEX

