﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
//n1589

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Platform/Stack/Core/InterfaceTable.h>
#include <OnlineCore/src/Platform/Stack/Core/IOCompletionNotifier.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/ObjectThread.h>
#include <OnlineCore/src/Platform/Core/ProfilingUnit.h>
#include <OnlineCore/src/Platform/Core/qSortedVector.h>
#include <OnlineCore/src/Core/InstantiationContext.h>
#include <OnlineCore/src/Core/SystemPorts.h>
#include <OnlineCore/src/Transport/Interface/Network.h>
#include <OnlineCore/src/Transport/Interface/RootTransport.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>
#include <OnlineCore/src/Transport/Interface/TransportPerfCounters.h>
#include <OnlineCore/src/Transport/Interface/VirtualPort.h>
#include <OnlineCore/src/Transport/Routing/Router.h>
#include <OnlineCore/src/Transport/Relay/Relay.h>
#include <OnlineCore/src/Transport/UDP/PacketQueue.h>
#include <OnlineCore/src/Transport/UDP/PacketHistoryChainDefine.h>

BEGIN_NAMESPACE_NEX

#if NEX_ENABLE_SOCKET_TRANSPORT_PROFILING
typedef struct _SocketTransportAnalyzeInfo
{
    qUnsignedInt32 uiSendCount;
    qUnsignedInt32 uiReceiveCount;
} SocketTransportAnalyzeInfo;
#endif

class QueuingSocket;
class ICMPSocket;
class QueuingDevice;
class Event;
class Timeout;
class Buffer;
class PacketOut;
class PacketIn;
class Packet;
class Job;
class TransportBufferThreadInterface;
class TransportBuffer;

/*!
 * @brief  トランスポート層におけるソケットを表現するクラスです。
 *
 */
class SocketTransport: public RootTransport
{
public:
    friend class Network;
    friend class PRUDPStream;
    friend class QueuingSocketTest;
    class TransportJob;

    SocketTransport();
    virtual ~SocketTransport();

    /*
     * Inherited from RootTransport
     */
    virtual qResult Initialize();

    qBool TerminateTransport();

    virtual qBool Connect(const StationURL *pRemoteStationURL, TransportProtocol::Type transportProtocolType, qUnsignedInt16 uiOutgoingPort);
    virtual qBool OpenSocket(
        qUnsignedInt16 ui16Port,
        qUnsignedInt16 *pui16ListeningPort,
        qBool bPreferred,
        TransportProtocol::Type transportProtocolType,
        qBool usePacketBufferPool,
        qBool acceptIncomingConnections,
        qBool bReuse,
        qBool bForceUseExistenceSocket,
        qBool bUseExistenceConnectSocketIfPossible
    );

    // テスト用のショートカット関数
    virtual qBool OpenUdpSocket(
        qUnsignedInt16 ui16Port,
        qUnsignedInt16 *pui16ListeningPort
    );

    virtual qBool NotifyCreateEndPoint(TransportProtocol::Type transportProtocolType, qUnsignedInt16 localPort, const InetAddress& remoteAddr);
    virtual qBool NotifyDeleteEndPoint(TransportProtocol::Type transportProtocolType, qUnsignedInt16 localPort, const InetAddress& remoteAddr);
    virtual qBool CloseSocket(TransportProtocol::Type transportProtocolType, qUnsignedInt16 ui16Port);
    // テスト用のショートカット関数
    virtual qBool CloseUdpSocket(qUnsignedInt16 ui16Port);
    virtual qBool CloseSocket();
    virtual size_t GetNbListeningPorts();

    virtual TransportStatus GetTransportStatus(TransportProtocol::Type transportProtocolType, qUnsignedInt16 localPort);
    virtual qBool Send(
        TransportProtocol::Type transportProtocolType,
        qUnsignedInt16 ui16Port,
        Stream::Type oType,
        Stream::ID idSource,
        Stream::ID idDestination,
        PacketOut *pPacket,
        //n1589:memo:falseの場合、直接送信処理
        //n1589:重要なフラグなので明示的指定に変更
#if NEX_ENABLE_EXPLICIT_ARGUMENT_TRANSPORT_SEND_QUEUE
        qBool bQueue
#else
        qBool bQueue = false
#endif
    );

    virtual qBool Receive(
        TransportProtocol::Type transportProtocolType,
        qUnsignedInt16 uiPort,
        Buffer *pBuffer,
        const InetAddress *pSourceAddress
    );

#ifdef NEX_ROUTING_SUPPORT
    virtual Router* GetRouter();
#endif
    virtual Relay* GetRelay() { return &m_oRelay; }

    virtual qResult Send(TransportProtocol::Type transportProtocolType, StationURL *pStationURL, Buffer *pBuffer);

    void TransportJobMethod(Time currentTime);

    virtual qUnsignedInt64 GetPacketQueueSize() const;
    virtual qUnsignedInt64 GetEstimatedPacketQueueMemoryUsage() const;

    TransportBufferThreadInterface *GetTransportBufferThread()
    {
        return m_pTransportBufferThread;
    }

    static const qUnsignedInt INVALID_THREAD_PRIORITY = 0xffffffff; //!< 無効な優先度を表現するシンボルです。

    void GetBufferSetEmptyDropCount(qUnsignedInt32& uiSendDropCount, qUnsignedInt32& uiRecvDropCount) const ;

    void GetListenPortLocalUrls(qList<StationURL>* pUrls);

#if NEX_ENABLE_SOCKET_TRANSPORT_PROFILING
    void ResetAnalyzeInfo();
    void LogAnalyzeInfo(EventLog::LogLevel logLevel = EventLog::Debug);

    SocketTransportAnalyzeInfo* GetAnalyzeInfo()
    {
        return &m_AnalyzeInfo;
    }
#endif

#if NEX_ENABLE_BERKLEY_SOCKET_PROFILING
    qUnsignedInt32 GetBerkeleySocketSendToMultiCount();
    qUnsignedInt32 GetBerkeleySocketSendToCount();
    qUnsignedInt32 GetBerkeleySocketRecvFromCount();
    qUnsignedInt32 GetBerkeleySocketRecvFromTryCount();
#endif

#ifdef NEX_CLIENT_PBPOOL
    /*! @internal
     @brief パケットバッファプールに確保されたパケットをキューから削除する。
            パケットバッファプールを開放前に呼び、不正参照させないようにする。
     */
    virtual void ClearPacketInPacketBufferPool();
#endif // NEX_CLIENT_PBPOOL

protected:
    virtual void    StopTransportThreadImpl()
    {
        m_bShutdown = true;
    }
    void            StartTransport();

    qBool           BindSocket(
        qUnsignedInt16 ui16DesiredPort,
        qUnsignedInt16 *pui16ListeningPort,
        TransportProtocol::Type transportProtocolType,
        qBool usePacketBufferPool,
        qBool acceptIncomingConnections,
        qBool bReuse
    );

    void            DeliverOutgoing(const Time& currentTime);
    void            DeliverOutgoingBuffer(QueuingSocket *pSocket, Buffer* pBuffer, const InetAddress& destInetAddress, qUnsignedInt16 srcPort);
    void            DeliverIncoming(const Time& currentTime);
    void            DeliverIncomingBuffer();

    //n1589
#ifdef NEX_ROUTING_SUPPORT
#ifdef NEX_CLIENT_OPTIMIZATION
    qBool RouteRelaySendMultiMessage(Buffer *pBuffer, qUnsignedInt16 uiPort);
#endif // NEX_CLIENT_OPTIMIZATION
#endif

    struct ProtocolAndPort
    {
        ProtocolAndPort()
            : transportProtocolType(TransportProtocol::TYPE_UNKNOWN)
            , localPort(0)
        {}
        ProtocolAndPort(TransportProtocol::Type transportProtocolType_, qUnsignedInt16 localPort_)
            : transportProtocolType(transportProtocolType_)
            , localPort(localPort_)
        {}
        // qSortedVector::find用の比較関数
        bool operator<(const ProtocolAndPort& rhs) const
        {
            // ポートで比較するが、ポートが同じならプロトコルで比較
            if (this->localPort == rhs.localPort)
            {
                return this->transportProtocolType < rhs.transportProtocolType;
            }
            return this->localPort < rhs.localPort;
        }
        TransportProtocol::Type transportProtocolType;
        qUnsignedInt16 localPort;
    };
    QueuingSocket   *GetSocket(const ProtocolAndPort& protocolAndPort);

    qBool           FillPacketQueueFromBuffer(
        QueuingSocket *pSocket,
        Buffer *pBuffer,
        const InetAddress *pSourceAddress
    );

    TransportBufferThreadInterface *m_pTransportBufferThread;
    Job                                 *m_pTransportJob;

    typedef qSortedVector<ProtocolAndPort, QueuingSocket*> SocketPortMap;
    SocketPortMap               m_mapSocketPort;

    volatile qBool                      m_bShutdown;
    InterfaceTable              m_oInterfaceTable;

    PacketQueue                 m_oIncomingPacketQueue;

    PacketQueue* mp_oOutgoingPacketQueue;

    PacketHistoryChain          m_oOutgoingHistoryPacketChain;

    ProfilingUnit               m_puTransportJob;
    ICContext                   m_icPseudoSingletonContext;
#ifdef NEX_ROUTING_SUPPORT
    Router                      m_oRouter;
#endif
    Relay                       m_oRelay;
    
    Time                        m_ForcedRetransmitCheckTimeout;

    TransportBuffer*            m_pTransportBuffer;

    Buffer                      m_tempRecvBuffer;   // DeliverIncomingBuffer()で使う一時バッファ
#if NEX_ENABLE_SOCKET_TRANSPORT_PROFILING
    SocketTransportAnalyzeInfo m_AnalyzeInfo;
#endif
};

END_NAMESPACE_NEX

