﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/ConnectionOrientedStream.h>

#include <OnlineCore/src/Platform/Core/qList.h>

BEGIN_NAMESPACE_NEX

//==============================================================================
// Memory Table managing a list of connection-oriented stream instances allowing
// to find a specific stream object from a stream type, station URL or index.
//==============================================================================
class StreamTable: public RootObject
{
public:
    StreamTable();
    virtual ~StreamTable();

    qBool RegisterTransportPlugin(Plugin *pTransportPlugin);

    size_t GetSize()
    {
        return m_oStreamList.size();
    }
    ConnectionOrientedStream * GetItem(qUnsignedInt iTransportIndex);
    ConnectionOrientedStream * FindStream(const StationURL *pStationURL);
    ConnectionOrientedStream * FindStream(const qChar *szType);
    qBool RemoveStream(ConnectionOrientedStream *pStream);
    void AddStream(ConnectionOrientedStream *pStream, qBool /*bOwner*/)
    {
        //SYSTEMCHECK(pStream != NULL);
        m_oStreamList.push_back(pStream);
    }

private:
    qList< ConnectionOrientedStream * > m_oStreamList;
};


END_NAMESPACE_NEX

