﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
//n2342

#pragma once
#include <OnlineCore/src/Transport/Interface/TransportDecl.h>


#include <OnlineCore/src/Platform/Core/qReservedQueue.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>
#include <OnlineCore/src/Transport/Interface/RootTransport.h>

BEGIN_NAMESPACE_NEX

class Buffer;

// トランスポート層用のバッファプール
class TransportBuffer: public RootObject
{
    friend void OnlineCore_TransportBuffer_Test();
public:
    TransportBuffer(size_t queueSize, size_t bufferSize);
    ~TransportBuffer();

    //バッファの確保
    Buffer* AllocateBuffer( size_t uiSize = 0 );
    //バッファの解放
    void FreeBuffer( Buffer* pBuffer );

    //n2850 BufferListが枯渇してパケットがドロップされた回数を返す
    size_t GetBufferListEmptyDropCount()
    {
        return m_uiBufferListEmptyDropCount;
    }


private:
    //バッファをあらかじめ予約
    void ReserveBuffer( size_t uiBufferSize, size_t uiSize = 0 );

private:
    typedef qReservedQueue<Buffer*> BufferList;
    //解放済みバッファリスト
    BufferList* m_pFreeBufferList;
#ifdef NEX_SYSTEMCHECK_ENABLED
    // プールとして用意されたもの以外を返却されたらAssertするため
    BufferList* m_pFreeBufferListForReleaseCheck;
#endif

    ForcedCriticalSection       m_BufferOperationLock;
    qUnsignedInt32 m_uiBufferListEmptyDropCount;
};

END_NAMESPACE_NEX


