﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Platform/Core/MemoryManager.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>
#include <OnlineCore/src/Platform/Core/qReservedQueue.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Transport/Interface/TransportBuffer.h>

BEGIN_NAMESPACE_NEX

class Buffer;
class RootTransport;
class IOCompletionContext;

// Bufferとアドレスのセットを要素に持つキュー
// BufferはTransportBufferを使って確保する。
class TransportBufferSet : public RootObject
{
private:
    struct TransportBufferInfo : public RootObject
    {
        Buffer*         pBuffer;
        InetAddress     targetAddress;
    };

public:
    //コンストラクタ
    // n2343:TransportBufferInfoはTransportBufferPoolNumと同じ数だけあれば絶対に足ります
    TransportBufferSet(TransportBuffer *pTransportBuffer, qInt reserveSize = RootTransport::GetTransportBufferPoolNum() );

    ~TransportBufferSet();

    qBool Pop(Buffer* receivedBuffer, InetAddress* pInetAddress);

    qBool IsQueueEmpty() const
    {
        // volatileをつかってここではロックしない
        return m_isQueueEmpty;
    }

    qBool IsQueueFull() const
    {
        // volatileをつかってここではロックしない
        return m_isQueueFull;
    }

    // pBufferは破壊される(Swapされる)ためconstなし
    qBool Push(Buffer* pBuffer, const InetAddress& address);

    // m_UnusedTransportBufferInfoQueue枯渇によりDropした回数
    qUnsignedInt32 GetInfoPoolEmptyDropCount() const
    {
        return m_uiInfoPoolEmptyDropCount;
    }

private:
    typedef qReservedQueue<TransportBufferInfo*> TransportBufferInfoQueue;

    // リストを全破棄
    void ClearList_( TransportBufferInfoQueue& list );

    // 情報をひとつ開放
    void DeleteOneBuffer_( TransportBufferInfo* pInfo );

    // リスト
    volatile qBool m_isQueueEmpty;
    volatile qBool m_isQueueFull;

    // 処理中のTransportBufferInfoを保持するキュー
    TransportBufferInfoQueue        m_UsingTransportBufferInfoQueue;

    // 空きのTransportBufferInfoを保持するキュー
    TransportBufferInfoQueue        m_UnusedTransportBufferInfoQueue;

#ifdef NEX_SYSTEMCHECK_ENABLED
    // プールとして用意されたもの以外を返却されたらAssertするため
    TransportBufferInfoQueue        m_TransportBufferInfoPoolForReleaseCheck;
#endif

    qUnsignedInt32                  m_uiBufferAllocSize;

    // ロック
    mutable ForcedCriticalSection   m_Lock;

    qUnsignedInt32                  m_uiInfoPoolEmptyDropCount;

    TransportBuffer*                m_pTransportBuffer;
};


END_NAMESPACE_NEX



