﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>

BEGIN_NAMESPACE_NEX

//group=Error Code Enumerations

// Description: This enumeration defines the error codes that may be returned for the Transport facility.
/*!
 @brief    Transport モジュールのエラーコードを定義する名前空間です。
 */
#ifdef SWIG
class TransportErrorCode
{
public:
#else
namespace TransportErrorCode
{
#endif
//group= Type Definitions

/*
Description: Enumerates the error codes for the Transport facility that a call may return.
Remarks: Details as to the cause of each ErrorCode are detailed in the documentation
of each individual $PRODUCTNAME$ method that may return a result.
*/

/*!
 @brief    Transport モジュールのエラーコード
 */
enum ErrorCode
{
    START = 1,
    // The reason for the error is unknown.
    Unknown                     = 1,    //!<    不明なエラーです。
    // Network connection was unable to be established.
    ConnectionFailure           = 2,    //!<    ネットワーク接続が確立できませんでした。
    // The URL contained in the StationURL is invalid.  The syntax may be incorrect.
    InvalidUrl                  = 3,    //!<    StationURLに含まれている URL の構文が不正です。
    // The key used to authenticate a given station is invalid. The secure transport layer
    // uses secret-key based cryptography to ensure the integrity and confidentiality of data
    // sent across the network.
    InvalidKey                  = 4,    //!<    キーが不正です。
    // The specified transport type is invalid.
    InvalidURLType              = 5,    //!<    URLType が不正です。
    // The Station is already connected via another EndPoint.
    DuplicateEndpoint           = 6,    //!<    そのステーションにはすでに接続しています。
    // The data could not be sent across the network. This could be due to an invalid message, packet, or buffer.
    IOError                     = 7,    //!<    ネットワークの向こう側にデータが送られませんでした。
    // The operation did not complete within the specified timeout for that operation.
    Timeout                     = 8,    //!<    タイムアウトが発生しました。
    // The network connection was reset.
    ConnectionReset             = 9,    //!<    ネットワーク接続がリセットされました。
    // The destination Station did not authenticate itself properly.
    IncorrectRemoteAuthentication = 10, //!<    接続先のステーションが正しく認証しませんでした。
    // 3rd-party server or device answered with an error code
    // according to protocol used
    // e.g. HTTP error code
    ServerRequestError          = 11,   //!<    サーバーからエラーが返りました。
    // An error occured while decompressing received data. Make sure that the same compression algorithm is
    // used on both sides.
    DecompressionFailure        = 12,   //!<    受信データの伸張時にエラーが発生しました。
    // It is not possible anymore to send data to a given end point, since it is congested.
    ReliableSendBufferFullFatal = 13,   //!<    送信バッファが満杯で、それ以上データを送ることができません。
    // Impossible to retrieve necessary information to perform UPnP operation on router.
    UPnPCannotInit              = 14,   //!<    使用されません。
    // Failed to add port mapping to router through UPnP.
    UPnPCannotAddMapping        = 15,   //!<    使用されません。
    // Impossible to retrieve necessary information to perform NatPMP operation on router.
    NatPMPCannotInit            = 16,   //!<    使用されません。
    // Failed to add port mapping to router through NatPMP.
    NatPMPCannotAddMapping      = 17,   //!<    使用されません。

    // n2781: ???NAT
    UnsupportedNAT          = 19,       //!<    サポートされていない NAT です。

    // n2222: Failed to resolve DNS.
    DnsError                    = 20,   //!<    DNS による名前解決に失敗しました。
    // n2222: Proxy related error.
    ProxyError                  = 21,   //!<    プロキシエラーが発生しました。
    // n2222: The data is remaining to receive. Internal use only.
    DataRemaining               = 22,   //!<    受信バッファにデータが残っています。使用されません。
    // n2222: There is no buffer to receive data.
    NoBuffer                    = 23,   //!<    受信バッファに空きがありません。
    // n2222: The requested resource is not found.
    NotFound                    = 24,   //!<    要求されたリソースが見つかりません。
    // n2222: Temporary server error occurred. You can retry later.
    TemporaryServerError        = 25,   //!<    一時的なサーバーエラーが発生しました。あとでもう一度やり直してください。
    // n2222: Permanent server error occurred. You can not retry.
    PermanentServerError        = 26,   //!<    恒久的なサーバーエラーが発生しました。
    // n2222: Service unavailable error occurred. You can not retry.
    ServiceUnavailable          = 27,   //!<    サーバーは一時的な過負荷もしくはメンテナンスのためリクエストを扱うことができません。
    // It is not possible anymore to send data to a given end point temporary, since it is congested.
    ReliableSendBufferFull      = 28,   //!<    Reliable送信バッファが満杯で、一時的にそれ以上データを送ることができません。
    // It is not exist station.
    InvalidStation              = 29,   //!<    指定されたステーションが無効なステーションです。
    // It is not a valid substream ID
    InvalidSubStreamID          = 30,   //!<    指定されたSubStreamIDは無効なIDです。相手のステーションが受け付けることができないSubStreamIDの場合も発生します。
    // It is not possible anymore to send data to a given end point temporary, since it is congested. out of packetbuffer pool.
    PacketBufferFull            = 31,   //!<    パケットバッファが不足し、一時的にそれ以上データを送ることができません。 パケットバッファの設定は@ref PacketBufferManager を参照してください。
    //
    NatTraversalError           = 32,   //!<    NATトラバーサル処理に失敗しました。
    //
    NatCheckError               = 33,   //!<    NATチェック処理に失敗しました。
    // Last marker
    LastError                           // 最後尾
};
//note: 新規result追加時には、OnlineCore/src/Platform/Core/ResulDefine.hのNEX_ERROR_RESULT_DEFINEの変更も必要
//Wiki エラーコード追加時に行う作業 を参照
//アプリケーションがハンドリングするべきエラーを増やした場合には、commonmanualのoverall.rstの「アプリケーションがハンドリングするべきエラー一覧」に追加すること

#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX



