﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <OnlineCore/src/Platform/Core/CoreConstants.h>
#include <OnlineCore/src/Platform/Core/Time.h>

BEGIN_NAMESPACE_NEX

class URLProbe: public RootObject
{
public:

    URLProbe(const StationURL &oStationURL,
             const TimeInterval tiLifetime,
             const TimeInterval tiProbeKeepAlive = TimeInterval(90000),
             const qUnsignedInt uiMaxProbes = 12);
    URLProbe() {}
    virtual     ~URLProbe() {}


    const StationURL        &GetURL() const
    {
        return m_oURL;
    }

    qUnsignedInt32  GetPingTime() const
    {
        return m_ui32PingTime;
    }

    qBool           IsExpired(Time tCurrentTime = Time::GetTime()) const
    {
        return m_tExpiration < tCurrentTime;
    }
    void            ExtendLifetime(const TimeInterval tiLifetime)
    {
        m_tExpiration = Time::GetTime()+tiLifetime;
    }
    qBool           URLIsUpdated()  const;

    qBool           Update(const StationURL &refURL, const Time tiTime);
    qBool           UpdateIsNeeded(Time tCurrentTime = Time::GetTime())  const;

    qUnsignedInt    GetNbSentProbes() const
    {
        return m_uiNbSentProbes;
    }
    void            IncNbSentProbes()
    {
        m_uiNbSentProbes++;
    }

    Time GetLastUpdatedTime()
    {
        return m_tLastUpdatedTime;
    }

    void            Trace(qUnsignedInt64 uiTraceFlag = TRACE_NAT);

public:
    StationURL      m_oURL;
    Time            m_tExpiration;
    qUnsignedInt32  m_ui32PingTime;
    Time            m_tLastUpdatedTime;
    TimeInterval    m_tiProbeKeepAlive;
    qUnsignedInt    m_uiNbSentProbes;
    qUnsignedInt    m_uiMaxProbes;
};

END_NAMESPACE_NEX

