﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>

BEGIN_NAMESPACE_NEX

class VirtualPort: public RootObject
{
public:

    VirtualPort()
        : m_vPortUnion()
    {
    }

    VirtualPort(Stream::Type oType, Stream::ID idStream)
        : m_vPortUnion()
    {
        Set(oType, idStream);
    }
    // For PRUDPLiteV1 (idStream拡張)
    VirtualPort(qByte oType, qByte idStream)
        : m_vPortUnion()
    {
        Set(oType, idStream);
    }

    VirtualPort(const VirtualPort &obj)
    {
        *this = obj;
    }

    // qByteからVirtualPortを作成する。UDP向けのフォーマット用。
    static VirtualPort CreateVirtualPortFromByte(qByte byVPort)
    {
        VirtualPort vPort;
        vPort.SetVirtualPortFromByte(byVPort);
        return vPort;
    }

    // qByteをStreamTypeとStreamIDに割り当てる。UDP向けのフォーマット用。
    void SetVirtualPortFromByte(qByte byVPort)
    {
        m_vPortUnion.m_vPortIndividual.m_streamType = byVPort >> 4;
        m_vPortUnion.m_vPortIndividual.m_streamId = byVPort & 0xF;
    }

    VirtualPort &operator=(const VirtualPort &obj)
    {
        m_vPortUnion.m_vPortFull = obj.m_vPortUnion.m_vPortFull;
        return *this;
    }

    Stream::Type GetStreamType() const
    {
        SYSTEMCHECK(m_vPortUnion.m_vPortIndividual.m_streamType <= 0xF);
        return static_cast<Stream::Type>(m_vPortUnion.m_vPortIndividual.m_streamType);
    }

    Stream::ID GetStreamID() const
    {
        return static_cast<Stream::ID>(m_vPortUnion.m_vPortIndividual.m_streamId);
    }

    qByte GetStreamTypeByte() const
    {
        SYSTEMCHECK(m_vPortUnion.m_vPortIndividual.m_streamType <= 0xF);
        return m_vPortUnion.m_vPortIndividual.m_streamType;
    }

    qByte GetStreamIDByte() const
    {
        return m_vPortUnion.m_vPortIndividual.m_streamId;
    }

    // PRUDP用の従来のqByte型の形式
    qByte GetVirtualPortShortNumber() const
    {
        SYSTEMCHECK(m_vPortUnion.m_vPortIndividual.m_streamId <= 0xF);
        SYSTEMCHECK(m_vPortUnion.m_vPortIndividual.m_streamType <= 0xF);
        return static_cast<qByte>((m_vPortUnion.m_vPortIndividual.m_streamType << 4) | (m_vPortUnion.m_vPortIndividual.m_streamId & 0xF));
    }

    // m_streamTypeとm_streamIdを連結して一つの整数を返す
    qUnsignedInt16 GetVirtualPortFullNumber() const
    {
        return m_vPortUnion.m_vPortFull;
    }
    qBool operator ==(const VirtualPort &oVPort) const
    {
        return m_vPortUnion.m_vPortFull == oVPort.m_vPortUnion.m_vPortFull;
    }
    qBool operator <(const VirtualPort &oVPort) const
    {
        return m_vPortUnion.m_vPortFull < oVPort.m_vPortUnion.m_vPortFull;
    }

protected:
    void Set(Stream::Type oType, Stream::ID idStream)
    {
        SYSTEMCHECK(oType <= 0xF);
        SYSTEMCHECK(idStream <= 0xFF);
        m_vPortUnion.m_vPortIndividual.m_streamType = static_cast<qByte>(oType);
        m_vPortUnion.m_vPortIndividual.m_streamId = static_cast<qByte>(idStream);
    }

    void Set(qByte oType, qByte idStream)
    {
        SYSTEMCHECK(oType <= 0xF);
        m_vPortUnion.m_vPortIndividual.m_streamType = oType;
        m_vPortUnion.m_vPortIndividual.m_streamId = idStream;
    }

    // StreamIDとStreamTypeを表す値
    union VPortUnion
    {
        VPortUnion()
            : m_vPortFull(0)
        {}

        // StreamIDとStreamTypeを一緒にしたもの
        qUnsignedInt16 m_vPortFull;

        // StreamIDとStreamTypeを個別にしたもの
        struct VPortIndividual
        {
            qByte m_streamType;
            qByte m_streamId;
        } m_vPortIndividual;
    } m_vPortUnion;

};

END_NAMESPACE_NEX

