﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
//n1589

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Stack/Core/SocketDriver.h>
#include <OnlineCore/src/Platform/Core/ObjectThread.h>

#include <OnlineCore/src/Platform/Core/ScopedCS.h>
#include <OnlineCore/src/Core/qBuffer.h>
#include <OnlineCore/src/Platform/Core/qChain.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>

#include <OnlineCore/src/Core/Job.h>

BEGIN_NAMESPACE_NEX

//n1589
#define NEX_ENABLE_UDS_SOCKET_TRANSPORT_JOB 1

//class VirtualSocketDriver;
//n1589:memo:incomplete type is not allowed
//class VirtualSocketDriver::Socket;

// n2343:
// VirtualSocketDriver層でジョブやスレッドは必要なく、
// Send要求、Recv要求があったときにだけ
// 実際のDriverでSendとRecvを行えば良い。
// よって、以下のコードは削除しました。
//
// #define NEX_TEST_UDS_RECEIVE_THREAD
// class UdsSocketTransportThreadLauncher;
#if NEX_ENABLE_UDS_SOCKET_TRANSPORT_JOB
class UdsSocketTransportJob;
#endif

class UdsSocketTransportThreadLauncher;

class VirtualNetwork;

struct VirtualMessageIn : public RootObject
{
    VirtualMessageIn():RootObject(), m_oBuffer() {}
    virtual ~VirtualMessageIn() {}

    CHAINABLE(VirtualMessageIn);

    qBuffer m_oBuffer;
};

/*!
@brief  ローカル通信連携機能を使用するためのクラスです。

        プラットフォームに合わせて用意される「 @ref VirtualNetwork を継承した派生クラス」を
        このクラスに登録することで、ローカル通信モードになります。

        ローカル通信機能のプラットフォームによる違いを吸収して、インターネット通信用プログラムと
        連携させることが可能になります。
*/
class VirtualSocketDriver: public SocketDriver
{
    class TransportJob;

public:
    class Socket: public SocketDriver::Socket
    {
        friend class VirtualSocketDriver;

    public:
        // SocketDriverはこちらのインタフェースを要求する。
        // VirtualSocketDriverではTransportProtocol::Typeを引数に要求するため、
        // 互換性を維持するためVirtualSocketDriverではこのインタフェースではなくTransportProtocol::Typeを引数に取るOpen()を定義させる。
        // そのためTransportProtocol::Typeを引数に取るOpen()そのまま処理を渡す。
        virtual qBool Open(TransportProtocol::Type transportProtocolType)
        {
            return Open(static_cast<TransportProtocol::Type>(transportProtocolType));
        }

        // こちらのOpenがVirtualSocketDriverが要求するインタフェース。
        virtual qBool Open(SocketDriver::TrafficType eTrafficType = SocketDriver::UDP);
        virtual void Close();
        qBool TerminateTransport();

        virtual qBool Bind(qUnsignedInt16 & ui16Port);

        virtual Result RecvFrom(
            qByte* pBuffer,
            size_t uiBufferSize,
            SocketDriver::InetAddress* pSourceAddress,
            size_t* puiNbBytesRecvd,
            SocketDriver::SocketFlag flags = SocketDriver::NonBlock
        );

        // SocketDriverはこちらのインタフェースを要求する。
        // VirtualSocketDriverではpBufferにconstをつけていない問題があるため、
        // 互換性を維持するためVirtualSocketDriverではこのインタフェースではなく非const版を定義させる。
        // そのため非const版の関数へそのまま処理を渡す。
        virtual Result SendTo(
            const qByte *pBuffer,
            size_t uiBufferSize,
            const InetAddress &refDestinationAddress,
            size_t *puiNbBytesSent
        )
        {
            return SendTo(const_cast<qByte*>(pBuffer), uiBufferSize, refDestinationAddress, puiNbBytesSent);
        }

        // こちらのSendToがVirtualSocketDriverが要求するインタフェース。
        virtual Result SendTo(
            qByte* pBuffer,
            size_t uiBufferSize,
            const SocketDriver::InetAddress &refDestinationAddress,
            size_t* puiNbBytesSent
        );

        virtual Result Connect(const SocketDriver::InetAddress&)
        {
            return Error;
        }

        virtual qBool IsAcceptedSocket() const
        {
            return false;
        }

        // SocketDriverはこちらのインタフェースを要求する。
        // VirtualSocketDriverでは以前から引数が3つのRecvを要求しているため、
        // 互換性を維持するためVirtualSocketDriverではこのインタフェースではなく引数が3つのRecvを定義させる。
        // そのため引数が3つの関数へそのまま処理を渡す。
        virtual Result Recv(qByte *pBuffer, size_t uiBufferSize, size_t *puiNbBytesRecvd, SocketFlag flags)
        {
            (void)flags;
            return Recv(pBuffer, uiBufferSize, puiNbBytesRecvd);
        }

        // こちらのRecvがVirtualSocketDriverが要求するインタフェース。
        virtual Result Recv(qByte*, size_t , size_t*)
        {
            return Error;
        }

        // SocketDriverはこちらのインタフェースを要求する。
        // VirtualSocketDriverではpBufferにconstをつけていない問題があるため、
        // 互換性を維持するためVirtualSocketDriverではこのインタフェースではなく非const版を定義させる。
        // そのため非const版の関数へそのまま処理を渡す。
        virtual Result Send(const qByte *pBuffer, size_t uiBufferSize, size_t *puiNbBytesSent)
        {
            return Send(const_cast<qByte*>(pBuffer), uiBufferSize, puiNbBytesSent);
        }

        // こちらのSendがVirtualSocketDriverが要求するインタフェース。
        virtual Result Send(qByte*, size_t , size_t*)
        {
            return Error;
        }

        virtual qBool SetMulticastAddress(qUnsignedInt32 )
        {
            return false;
        }

        virtual Result Shutdown();

        virtual const qChar8* GetClassNameString() const
        {
            return "VirtualSocketDriver::Socket";
        }

        qBool IsReadyToRead();
        qBool ProcessReceive();
        void DeleteMessageChain();

    private:
        void Lock();
        void Unlock();
        Socket();
        virtual ~Socket();

#if NEX_ENABLE_UDS_SOCKET_TRANSPORT_JOB
        UdsSocketTransportJob* m_pTransportJob;
#endif

        qChain<VirtualMessageIn*> mChainMessageIn;

        mutable CriticalSection mCsReceivedMessage;
    };

    VirtualSocketDriver();
    virtual ~VirtualSocketDriver();

    virtual SocketDriver::Socket* Create();
    virtual void Delete(SocketDriver::Socket* pSocket);

    virtual qBool Poll(PollInfo aPollInfo[], qUnsignedInt uiPollInfoSize, qUnsignedInt32 uiTimeoutInMs);

    /*!
    @brief  @ref VirtualNetwork を継承した派生クラスのインスタンスを登録し、ローカル通信モードに変更します。

            「@ref VirtualNetwork 派生クラス」のインスタンス生成、およびこの API を呼び出しての
            インスタンス登録は、 NetZ インスタンスを生成する前に行ってください。

            ローカル通信モードに切り替わっている場合、インターネット通信を行うことはできません。

            インターネット通信モードに戻すには @ref UnregisterVirtualNetwork を呼び出した後、
            NetZ インスタンスを再生成してください。

    @param[in] pVirtualNetwork  登録する「 VirtualNetwork 派生クラス」のインスタンスへのポインタを指定します。

    @return 既にインスタンスが登録済みの場合は false 、それ以外の場合は true を返します。
    */
    static qBool RegisterVirtualNetwork(VirtualNetwork* pVirtualNetwork);

    /*!
    @brief  @ref VirtualNetwork を継承した派生クラスのインスタンス登録を解除し、インターネット通信モードに変更します。

            「@ref VirtualNetwork 派生クラス」のインスタンスを破棄する前に、この API を呼び出して
            インスタンス登録を解除してください。

            ローカル通信モードからインターネット通信モードに戻す場合、
            この API を呼び出した後に NetZ インスタンスを再生成してください。

    @return インスタンス登録が無い場合は false 、それ以外の場合は true が返ります。
    */
    static qBool UnregisterVirtualNetwork();

    static VirtualNetwork* GetVirtualNetwork();

private:
    static VirtualNetwork* s_pVirtualNetwork;
};

#if NEX_ENABLE_UDS_SOCKET_TRANSPORT_JOB

class UdsSocketTransportJob: public Job
{
public:
    UdsSocketTransportJob(
        const String& strJobDescription
    );

    virtual ~UdsSocketTransportJob();

    void Launch(void* pDummyParam);
    void Finish();

    virtual void Execute();

    qBool IsRunning()
    {
        return mb_Running;
    }

private:
    VirtualSocketDriver::Socket* socket;
    qBool mb_Running;
};

#endif

END_NAMESPACE_NEX

