﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#if (defined(WIN_FAMILY) && !defined(VS2008_COMPILER)) || defined(UNIX_FAMILY)
#include <OnlineCore/src/Transport/PRUDP/PRUDPEndPointMap.h>
#include <OnlineCore/src/Platform/Core/qUnorderedMap.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Core/Checksum/Interface/SignatureBytes.h>

BEGIN_NAMESPACE_NEX

class EndPointUniqueIDSignatureHolder  : public RootObject
{
public:
    static const qInt MAX_HOLDS = 3;
    static const qInt MAX_ENTRY_PER_MAP = 2000;

    EndPointUniqueIDSignatureHolder();

    virtual ~EndPointUniqueIDSignatureHolder() {}

    typedef qUnorderedMap<EndPointUniqueID, SignatureBytes> EndPointUniqueIDSignatureMap;

    qBool Add(const InetAddress& inetAddress, Stream::ID idStream, const SignatureBytes& signature);
    qBool GetAndUpdate(const InetAddress& inetAddress, Stream::ID idStream, SignatureBytes *signature);
    void TimeOut();

private:
    void Update(EndPointUniqueID& epui, const SignatureBytes& signature, qInt poi, EndPointUniqueIDSignatureMap::iterator &it);
    void Find(EndPointUniqueID& epui, qInt *poi, EndPointUniqueIDSignatureMap::iterator *it);

    inline qInt PositionFromLast(qInt poi)
    {
        return (mLastHolder + poi) % MAX_HOLDS;
    }


    qVector<EndPointUniqueIDSignatureMap> mHolders;
    qInt mLastHolder;

    CriticalSection mCs;
};

END_NAMESPACE_NEX

#endif



