﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/PRUDP/PRUDPDecl.h>
#include <OnlineCore/src/Core/Buffer.h>
#include <OnlineCore/src/Transport/Interface/EndPoint.h>

BEGIN_NAMESPACE_NEX

class EndPoint;

class IORequestContext: public RootObject
{

public:
    // moved here for inlining reasons
    void Clear()
    {
        if (m_pOutBuffer != NULL)
        {
            RELEASEREF(m_pOutBuffer);
        }
        m_pOutBuffer = NULL;
        m_pInBuffer = NULL;
        m_pCallback = NULL;
    }

    IORequestContext()
    {
        m_pOutBuffer = NULL;
        m_pInBuffer = NULL;
        m_pCallback = NULL;
    }
    virtual ~IORequestContext()
    {
        Clear();
    }

    void Set(Buffer *pOutBuffer, Buffer *pInBuffer, const EndPoint::Callback pCallback, const UserContext *pContext)
    {
        if (pOutBuffer != NULL)
        {
            m_pOutBuffer = (Buffer*)ACQUIREREF(pOutBuffer);
        }
        else
        {
            m_pOutBuffer = NULL;
        }
        m_pInBuffer = pInBuffer;
        m_pCallback = pCallback;
        m_oUserContext = *pContext;
    }


    qBool IsSet()
    {
        return m_pCallback != NULL;
    }

    Buffer *GetInBuffer()
    {
        return m_pInBuffer;
    }
    void SetInBuffer(Buffer *pInBuffer)
    {
        m_pInBuffer = pInBuffer;
    }
    Buffer *GetOutBuffer()
    {
        return m_pOutBuffer;
    }
    void SetOutBuffer(Buffer *pOutBuffer)
    {
        m_pOutBuffer = pOutBuffer;
    }

    void CompleteIO(EndPoint *pEndPoint, qResult rResult)
    {
        EndPoint::Callback pCallback = m_pCallback;
        Clear();
        pCallback(pEndPoint,rResult,&m_oUserContext);
    }

private:
    Buffer              *m_pInBuffer;
    Buffer              *m_pOutBuffer;
    EndPoint::Callback  m_pCallback;
    UserContext         m_oUserContext;
};

END_NAMESPACE_NEX

