﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/PRUDP/PRUDPDecl.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPEndPoint.h>
#include <OnlineCore/src/Transport/Interface/VirtualPort.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Platform/Core/Hash.h>
#include <OnlineCore/src/Platform/Core/qMap.h>

BEGIN_NAMESPACE_NEX

class EndPointUniqueID
{
public:
    EndPointUniqueID(const InetAddress& oInetAddress, Stream::ID idStream)
    {
        m_oInetAddress = oInetAddress;
        m_idStream = idStream;
    }
    EndPointUniqueID()
    {
    }
    qBool operator<(const EndPointUniqueID &oID) const
    {
        return (m_oInetAddress < oID.m_oInetAddress) ||
               ((m_oInetAddress == oID.m_oInetAddress) && (m_idStream < oID.m_idStream));
    }
    qBool operator==(const EndPointUniqueID &oID) const
    {
        return (m_oInetAddress == oID.m_oInetAddress) && (m_idStream == oID.m_idStream);
    }

    InetAddress m_oInetAddress;
    Stream::ID  m_idStream;
};


#if (defined(WIN_FAMILY) && !defined(VS2008_COMPILER)) || defined(UNIX_FAMILY)

inline std::size_t hash_value (const EndPointUniqueID& d)
{
    std::size_t hash = 0;
    HashCombine(hash, hash_value(d.m_oInetAddress));
    HashCombine(hash, d.m_idStream);

    return hash;
}

END_NAMESPACE_NEX

namespace std
{
template <>
struct hash<NAMESPACE_NEX EndPointUniqueID>
{
    std::size_t operator()(const NAMESPACE_NEX EndPointUniqueID& d) const
    {
        return NAMESPACE_NEX hash_value(d);
    }
};
}

BEGIN_NAMESPACE_NEX

#endif

class PRUDPEndPointMap: public qMap<EndPointUniqueID,PRUDPEndPoint*>
{
public:
    typedef qMap<EndPointUniqueID,PRUDPEndPoint*>::iterator Iterator;

    PRUDPEndPoint* Locate(const InetAddress &oInetAddress, Stream::ID streamId)
    {
        EndPointUniqueID oID(oInetAddress, streamId);
        Iterator it = find(oID);
        if (it == end())
        {
            return NULL;
        }
        else
        {
            return (*it).second;
        }
    }

    void Add(PRUDPEndPoint *pEndPoint, Stream::ID streamId)
    {
        insert(value_type(EndPointUniqueID(*pEndPoint->GetInetAddress(), streamId), pEndPoint));
    }

    void Remove(const InetAddress &oInetAddress, Stream::ID streamId)
    {
        EndPointUniqueID oID(oInetAddress, streamId);
        Iterator it = find(oID);
        if (it == end())
        {
            return;
        }
        else
        {
            erase(it);
        }
    }

#ifdef NEX_TRACE_ENABLED    // n2370
    void Trace(qUnsignedInt64 uiTraceFlag = TRACE_ALWAYS)
    {
        if (!(TraceLog::GetInstance()->GetFlags() & uiTraceFlag))
        {
            return;
        }

        Iterator it = begin();
        while (it != end())
        {
            it->second->Trace(uiTraceFlag);
            ++it;
        }
        return;
    }
#endif  // NEX_TRACE_ENABLED

};


END_NAMESPACE_NEX

