﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#pragma once

#include <OnlineCore/src/Transport/PRUDP/PRUDPDecl.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPMessageInterface.h>
#include <OnlineCore/src/Platform/Core/qVector.h>

#ifdef NEX_PRUDP_LITE_SUPPORT

BEGIN_NAMESPACE_NEX

class Packet;
class PacketIn;
class PacketOut;
class Key;
class ByteStream;
class TransportSignatureGenerator;

class PRUDPLiteMessageV1 : public PRUDPMessageInterface, public RootObject
{
    friend class  OnlineCore_Transport_PRUDPLiteMessageV1_OptionUnPackTest_Test; //For UnitTest
public:
    static const size_t SIGNATURE_SIZE = 16; /*! 署名サイズ */
    static const size_t HEADER_SIZE = 12; /*! ヘッダサイズ */
    static const size_t MAX_OPTION_SIZE = 40; /*! 最大オプションサイズ */
    static const qByte HEADER_VERSION = 0x80;

    enum OptionType
    {
        SUPPORTED_FUNCTIONS = (qByte)0x00,
        CONNECTION_SIGNATURE = (qByte)0x01,
        DISCONNECT_REASON = (qByte)0x05,
        ACCESS_KEY_SIGNATURE = (qByte)0x80,
        PADDING  = (qByte)0xff
    };
    
    static const qUnsignedInt32 SUPPORTED_FLAG_CONNECTION_SIGNATURE_MINOR_VERSION = 5;
    static const qUnsignedInt32 SUPPORTED_FLAG_DISCONNECT_REASON_MINOR_VERSION = 6;

    PRUDPLiteMessageV1();

    virtual ~PRUDPLiteMessageV1();

    virtual qBool Unpack(ByteStream *pByteStream, PacketIn *pPacketIn, size_t *pSizeUnpacked = NULL);

    virtual qBool FastUnpack(ByteStream *pByteStream, PacketIn *pPacketIn, size_t *pPayloadSize = NULL);

    virtual qBool Pack(PacketOut *pPacketOut, ByteStream *pByteStream);
    
    struct Header
    {
        qByte mVersion;
        qByte mOptionSize;
        qUnsignedInt16 mPayloadSize;
        qByte mStreamType;
        qByte mSrcStreamID;
        qByte mDstStreamID;
        qByte mPartNo;
        qUnsignedInt16 mPacketTypeFlags;
        qUnsignedInt16 mSeqId;
    };
    
    virtual qBool CalcSignature(PacketOut *pPacket, SignatureMethod oMethod, const Key *pKey, Stream::Type oType,
                                TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &calcSignature);

    virtual void CalcSignature(Packet *pPacket, SignatureMethod oMethod, const Key *pKey, Stream::Type oType,
                               TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &calcSignature);

    virtual SignatureMethod DecideSignatureMethod(const Packet *pPacket, Stream::Type type, qBool hasEndPoint);

    virtual SignatureMatchPattern CalcExpectedSignature(const Packet *, SignatureMethod , const Key *,
            TransportSignatureGenerator* , const SignatureBytes *, SignatureBytes &);

    

private:
    qBool OptionUnpack(ByteStream *pByteStream, PacketIn *pPacketIn, qUnsignedInt optionSize);
    qBool OptionBuilder(PacketOut *pPacketOut);
    qBool HeaderParse(ByteStream *pByteStream, PacketIn *pPacketIn, qUnsignedInt &optionSize, qUnsignedInt &payloadSize);
    void CalcSignatureHelper(const Packet *pPacket, SignatureMethod oMethod, Stream::Type oType,
                             const SignatureBytes *pConnectionSignature, SignatureBytes &calcSignature);

    RandomNumberGenerator mRng;
    CriticalSection mCsRng;
};


END_NAMESPACE_NEX

#endif
