﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#pragma once

#include <OnlineCore/src/Transport/PRUDP/PRUDPDecl.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPMessageInterface.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>

#if defined(LIVE_VERSION) && ( (defined(WIN_FAMILY) && !defined(VS2008_COMPILER)) || defined(UNIX_FAMILY) )
#define NEX_ENABLE_SYN_CACHE
#define NEX_ENABLE_SYN_CACHE_RVONLY
#endif

BEGIN_NAMESPACE_NEX
class Packet;
class TransportSignatureGenerator;
class SignatureBytes;
class InetAddress;
class EndPointUniqueIDSignatureHolder;
class PRUDPMessageSelector;
class TransportPerfCounters;

class PRUDPStreamTemplateBase : public PRUDPStreamTemplateInterface, public RootObject
{
public:
    PRUDPStreamTemplateBase(Stream::Type type, PRUDPMessageInterface* pPRUDPMessageInterface);

    virtual ~PRUDPStreamTemplateBase() {};
#ifdef NEX_PERFCOUNTER_ENABLED
    virtual void SetPerfCounters(TransportPerfCounters *pPerfCounters)
    {
        m_pPerfCounters = pPerfCounters;
    }
#endif

    Stream::Type GetStreamType()
    {
        return m_streamType;
    }


protected:
    Stream::Type m_streamType;
    PRUDPMessageInterface* m_pPRUDPMessageInterface;

#ifdef NEX_ENABLE_SYN_CACHE
    virtual void CreateSignatureForSynCache(Packet *pPacket, Stream::Type oType,
            TransportSignatureGenerator* pSignatureGenerator, SignatureBytes *pSendConnectionSignature) = 0;
    qBool CalcSendConnectionSignatureWithSynCache(Packet* pPacket, TransportSignatureGenerator* pSignatureGenerator,
            Stream::ID idSource, SignatureBytes* pSendConnectionSignature);

    EndPointUniqueIDSignatureHolder* m_pEpuisHolder;
    Time m_tFloodSynStart;
    Time m_tEpuisHolderTimeOutCheckTime;
    static const TimeInterval sCancelSynFloodWarningTimeInterval = 5000;
    static const TimeInterval sEpusiHolderTimeOutCheckTimeInterval = 2000;
#endif
    virtual void ServiceTimeouts(const Time& currentTime);

#ifdef NEX_PERFCOUNTER_ENABLED
    TransportPerfCounters *m_pPerfCounters;
#endif
};

END_NAMESPACE_NEX

