﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#pragma once

#include <OnlineCore/src/Transport/PRUDP/PRUDPDecl.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPMessageInterface.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPStreamTemplateInterface.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPStreamTemplateBase.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>



BEGIN_NAMESPACE_NEX
class Packet;
class TransportSignatureGenerator;
class SignatureBytes;
class InetAddress;
class EndPointUniqueIDSignatureHolder;
class PRUDPMessageSelector;
class TransportPerfCounters;

class PRUDPStreamTemplateImp : public PRUDPStreamTemplateBase
{
public:
    PRUDPStreamTemplateImp(Stream::Type type, PRUDPMessageInterface* pPRUDPMessageInterface);
    virtual ~PRUDPStreamTemplateImp();

    virtual qByte GetVersion();
    virtual qBool VersionNegotiation(Packet *pPacket, Packet *pAckPacket);
    virtual qBool CalcSendConnectionSignature(Packet* pPacket, TransportSignatureGenerator* pSignatureGenerator,
            Stream::ID idSource, SignatureBytes* pSendConnectionSignature);
    virtual PRUDPMessageInterface::SignatureMatchPattern
        CheckSignature(InetAddress* pInetAddress, qUnsignedInt16 uiPort,
                Stream::ID idSource, Packet *pPacket, Key *pKey, TransportSignatureGenerator* pSignatureGenerator, SignatureBytes *pSB,
                PRUDPMessageInterface::SignatureMethod oMethod);
    virtual qBool GetSendConnectionSignature(Packet* pPacket, TransportSignatureGenerator* pSignatureGenerator,
            Stream::ID idSource, SignatureBytes* pSendConnectionSignature);
    virtual Stream::Type GetStreamType();
#ifdef NEX_PERFCOUNTER_ENABLED
    virtual void SetPerfCounters(TransportPerfCounters *pPerfCounters)
    {
        m_pPerfCounters = pPerfCounters;
    }
#endif

    static void SetInitialVersion(qByte version)
    {
        s_bInitialVersion = version;
    }
    static qByte GetInitialVersion()
    {
        return s_bInitialVersion;
    }

    static void SetInitialMinorVersion(qByte version)
    {
        s_bInitialSupportedFunctionsMinorVersion = version;
    }
    static qByte GetInitialMinorVersion()
    {
        return s_bInitialSupportedFunctionsMinorVersion;
    }
    static void ResetInitialMinorVersion()
    {
        s_bInitialSupportedFunctionsMinorVersion.ResetContexts();
    }

    static void SetSupportedFunctionsOptionMask(qUnsignedInt32 optionMask)
    {
        s_uiInitialSupportedFunctionsOptionMask = optionMask;
    }
    static qUnsignedInt32 GetSupportedFunctionsOptionMask()
    {
        return s_uiInitialSupportedFunctionsOptionMask;
    }
protected:
    static PseudoGlobalVariable<qByte> s_bInitialSupportedFunctionsMinorVersion;
    static qByte s_bInitialVersion;
    static qUnsignedInt32 s_uiInitialSupportedFunctionsOptionMask;

#ifdef NEX_ENABLE_SYN_CACHE
    virtual void CreateSignatureForSynCache(Packet *pPacket, Stream::Type oType,
            TransportSignatureGenerator* pSignatureGenerator, SignatureBytes *pSendConnectionSignature);
#endif

#ifdef NEX_PERFCOUNTER_ENABLED
    TransportPerfCounters *m_pPerfCounters;
#endif

};

END_NAMESPACE_NEX

