﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <algorithm>
#include <OnlineCore/src/Transport/PRUDP/PRUDPDecl.h>

BEGIN_NAMESPACE_NEX

class RTT: public RootObject
{

public:
    static const qUnsignedInt32 AVG_GAIN_DIV       = 3; // (1/8)
    static const qUnsignedInt32 DEVIATION_GAIN_DIV = 2; // (1/4)
    static const qUnsignedInt32 DEVIATION_MULT     = 2; // 4
    static const qUnsignedInt32 MIN_DEVIATION_GAIN; // msec
    static const qUnsignedInt32 INITIALRTT_DEFAULT = 750; // msec
    static const LargeTimeInterval MAX_RTT_MSEC = 60 * 1000; //

    RTT(qUnsignedInt32 uiInitialRTT = INITIALRTT_DEFAULT);
    virtual ~RTT();

    void Adjust(LargeTimeInterval uiRTT);

    // RFC6298 RTO <- SRTT + max (G, K*RTTVAR) K=2^DEVIATION_MULT G=MIN_DEVIATION_GAIN
    qUnsignedInt32 GetRTO() const
    {
        return GetRTTSmoothedAvg()+(std::max)(GetRTTSmoothedDev()<<DEVIATION_MULT, MIN_DEVIATION_GAIN);
    }
    qUnsignedInt32 GetLastRTT() const
    {
        return m_uiLastRTT;
    }
    qUnsignedInt32 GetRTTSmoothedAvg() const
    {
        return m_uiRTTSmoothedAvg >> AVG_GAIN_DIV;
    }
    qUnsignedInt32 GetRTTSmoothedDev() const
    {
        return m_uiRTTSmoothedDev >> DEVIATION_GAIN_DIV;
    }

    void Trace(qUnsignedInt64 uiTraceFlags);
protected:
    qUnsignedInt32          m_uiRTTSmoothedAvg;
    qUnsignedInt32          m_uiRTTSmoothedDev;
    qUnsignedInt32          m_uiLastRTT;
    qBool                   m_isFirstAdjust;
};

END_NAMESPACE_NEX

