﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>


BEGIN_NAMESPACE_NEX

class RootTransport;
class InetAddress;
class Buffer;
class RoutingStream;
class RoutingAddressResolver;

class Router : public RootObject
{
public:
    Router();
    virtual ~Router();

    void SetTransport( RootTransport *pTransport );
    void EnableRouting(qBool bEnable = true);
    qUnsignedInt32 GetMTU();
#if NEX_ENABLE_HYBRID
    qBool HasRoute(const InetAddress *pAddress, const InetAddress *pRouteAddr = NULL);
#endif
    qBool ShouldRoute(const InetAddress *pAddress);
    qBool RouteMessage( Buffer *pBuffer, const InetAddress *pAddress, qUnsignedInt16 uiPort);

    //n1589:memo
    qBool AddRoute( const InetAddress &rDest, const InetAddress &rRouterAddr, qBool bOverride = false );

    RoutingStream &GetStream()
    {
        return *m_pRoutingStream;
    }

    void ConstructAndAssociateRoutingAddress(qUnsignedInt16 id, const InetAddress refRealAddress, InetAddress *pRoutingAddress);
    qUnsignedInt32 GetRoutingIPAddressTemplate();

    RoutingAddressResolver *GetResolver()
    {
        return m_pAddressResolver;
    }

    qBool IsEnabledRouting()
    {
        return m_bRoutingEnabled;
    }

    void Clean();

private:
    // Cannot copy
    Router( const Router & );
    Router &operator=(const Router &);

    void UnregisterStream();

    qBool m_bRoutingEnabled;
    qBool m_bStreamRegistered;
    RoutingAddressResolver* m_pAddressResolver;
    RoutingStream* m_pRoutingStream;
};

END_NAMESPACE_NEX

