﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Core/Buffer.h>
#include <OnlineCore/src/Transport/Interface/EmulationDevice.h>
#include <OnlineCore/src/Transport/UDP/VirtualModem.h>
#include <OnlineCore/src/Transport/UDP/VirtualInternet.h>

BEGIN_NAMESPACE_NEX

class NetworkEmulator: public RootObject
{
    class NetworkPacket
    {
    public:
        // Must appear at the top, for inlining to work
        // with SN.
        Buffer *GetBuffer() const
        {
            return m_pBuffer;
        }
        InetAddress& GetInetAddress() const
        {
            return m_oInetAddress;
        }

        NetworkPacket(Buffer *pBuffer, const InetAddress &oInetAddress)
        {
            m_pBuffer = pBuffer;
            ACQUIREREF(m_pBuffer);
            m_oInetAddress = oInetAddress;
        }
        NetworkPacket(const NetworkPacket &refNetworkPacket)
        {
            m_pBuffer = refNetworkPacket.GetBuffer();
            ACQUIREREF(m_pBuffer);
            m_oInetAddress = refNetworkPacket.GetInetAddress();
        }
        ~NetworkPacket()
        {
            RELEASEREF(m_pBuffer);
        }
        qBool operator ==(const NetworkPacket& refPacket) const
        {
            return m_pBuffer == refPacket.m_pBuffer;
        }
    public:
        Buffer      *m_pBuffer;
        mutable InetAddress m_oInetAddress;
    };
private:
    // 代入演算子は使えないことを明示
    NetworkEmulator& operator=(const NetworkEmulator&);
public:
    NetworkEmulator(EmulationDevice *pDevice);
    ~NetworkEmulator() {}

    void Queue(Buffer *pBuffer, const InetAddress &refInetAddress);
    qBool GetHead(Buffer **pBuffer, InetAddress *pInetAddress);
    void Dequeue(Buffer *pBuffer);
    qBool IsEnabled() const { return m_oDevice.IsEnabled(); }
    void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS);

protected:
    VirtualModem<NetworkPacket>     m_oModem;
    VirtualInternet<NetworkPacket>  m_oInternet;

    EmulationDevice&                m_oDevice;
};

END_NAMESPACE_NEX

