﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

// n2370
//#ifndef _PacketOut_H
#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/UDP/Packet.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Core/ByteStream.h>
#include <OnlineCore/src/Core/FixedSizeMemoryPool.h>

BEGIN_NAMESPACE_NEX

class PRUDPEndPoint;
class Timeout;

class PacketOut: public Packet
{
public:
    //n2967: 同じ宛先のパケットがない場合には、バンドリングによる送信が行われないので、個別に再送処理をするため、状態を管理します。
    enum PacketHistoryBundlingState
    {
        UnbundledAndUnretransmit, //バンドリングおよび再送信がされていない。初期状態。 次のStateは、Bundled or Retransmit
        Bundled, //バンドリングされた。次のStateは、OnceBundledOrRetransmited
        Retransmit, //バンドリングされないので再送信。次のStateはOnceBundledOrRetransmited
        OnceBundledOrRetransmited //一度でもバンドリングもしくは再送信された
    } ;


    PacketOut(
        PRUDPEndPoint *pEndPoint,
        PacketType byPacketType,
        qUnsignedInt16 uiFlags = FLAG_NONE,
        Buffer *pBuffer = NULL,
        qUnsignedInt32 uiCallID = 0
    );
    
    virtual ~PacketOut();
protected:
    // PacketBufferPacketOut向け
    PacketOut() {} // 何もしない。 派生クラスで使用。
    void Initialize(PRUDPEndPoint *pEndPoint,
                    PacketType byPacketType,
                    qUnsignedInt16 uiFlags,
                    Buffer *pBuffer,
                    qUnsignedInt32 uiCallID,
                    Timeout* pTimeout);
public:

    Timeout         *GetTimeout()
    {
        return m_pTimeout;
    }

    PRUDPEndPoint   *GetEndPoint()
    {
        return m_pEndPoint;
    }

    void            IncNbTransmission()
    {
        ++m_ui16NbTransmission;
    }
    qUnsignedInt16  GetNbTransmission() const
    {
        return m_ui16NbTransmission;
    }

    void            SetPackedSize(size_t size)
    {
        m_packedSize = size;
    }
    size_t  GetPackedSize() const
    {
        return m_packedSize;
    }

public:
    inline qBool    IsBundlable() const
    {
        return (GetType() != TYPE_RAW && GetType() != TYPE_DISCONNECT) &&
               (Stream::GetSettings(GetSourceVirtualPort().GetStreamType()).GetBundling().IsEnabled());
    }

    inline qBool    IsBundlableWith(PacketOut *pPacket)
    {
        return pPacket->IsBundlable() && IsBundlableWithPortTypeCheck(pPacket);
    }

    inline qBool    ShouldTestHistoryBundle() const
    {
        // このパケットを送信するときに履歴バンドリングを試みるか？
        return RootTransport::IsEnablePacketHistoryBundling()
               && CanHistoryBundle() && !IsAcked();
    }


    inline qBool    ShouldDuplicateForHistoryBundle() const
    {
        // このパケットを送信するときに履歴バンドリングによる重複処理を行うか
        return RootTransport::IsEnablePacketHistoryBundling()
               && ShouldDuplicateIfHistoryBundle() && !IsAcked();
    }


    inline qBool    IsHistoryBundlableWith(PacketOut *pPacket)
    {
        // pPacket 送信時にこのパケットを履歴バンドリング可能か
        return pPacket->ShouldTestHistoryBundle() && IsBundlableWithPortTypeCheck(pPacket);
    }
    void            SetExpress(qBool bExpress)
    {
        m_bExpress = bExpress;
    }
    qBool           IsExpress()
    {
        return m_bExpress;
    }

    qBool           ReadyToSend();
    /*! @internal
    @brief 指定Bufferを複製(qNew+copy)を作って、ペイロードとしてセットする。
    @return true:成功、false:メモリ確保できず設定でき無かった。ペイロードは以前のまま。
    */
    qBool           SetInternalPayload(Buffer * pBuffer);
#ifdef NEX_TRACE_ENABLED    // n2370
    virtual void Trace(qUnsignedInt64 uiTraceFlags = D_TRACE_TRANSPORT_PACKET)
    {
        TraceImpl(uiTraceFlags, false);
    }
#endif  // NEX_TRACE_ENABLED

    inline void SetPacketHistoryBundlingState(PacketHistoryBundlingState s)
    {
        m_packetHistoryBundlingState = s;
    }

    inline PacketHistoryBundlingState GetPacketHistoryBundlingState()
    {
        return m_packetHistoryBundlingState;
    }

    inline void ResetPacketHistoryForceRetransmitTimeout()
    {
        m_packetHistoryForceRetransmitTimeout = Time::GetTime()
                                                + RootTransport::GetPacketHistoryBundlingForcedRetransmitTimeout();
    }


    inline Time GetPacketHistoryForceRetransmitTimeout()
    {
        return m_packetHistoryForceRetransmitTimeout;
    }

    inline qBool IsAcked() const
    {
        return m_isAcked;
    }

    inline void SetAcked()
    {
        m_isAcked = true;
    }

    inline void IncPacketHistoryRetransmitCount()
    {
        m_packetHistoryRetransmitCount++;
    }

    qUnsignedInt16 GetPacketHistoryRetransmitCount() const
    {
        return m_packetHistoryRetransmitCount;
    }

    inline qUnsignedInt32 GetCallID() const
    {
        return m_callID;
    }
#ifdef NEX_CLIENT_PBPOOL
    /* @internal
     @brief Validでない場合は、このオブジェクトを使用してはいけない。
            現状メモリ確保できていない場合false。
     */
    qBool IsValid()
    {
        if (FlagSet(FLAG_ACK_WANTED) && GetTimeout()==NULL)
        {
            QTRACE(D_TRACE_MEMORY, NEX_T("PacketOut::IsValid() false"));
            // ACK待ち必要なのに、Timeoutがセットできていない。(メモリ枯渇でPacketBufferTimeoutが確保できていない)
            return false;
        }

        // GetPayload()はNULLがありうるので、チェックしない。

        return true;
    }
#endif // NEX_CLIENT_PBPOOL

private:
    qBool           IsBundlableWithPortTypeCheck(PacketOut *pPacket);

    Timeout             *m_pTimeout;
    PRUDPEndPoint       *m_pEndPoint;
    qUnsignedInt16      m_ui16NbTransmission;
    qUnsignedInt16      m_packetHistoryRetransmitCount; //n2967
    Time                m_oReadyToQueueTime;
    qBool               m_bExpress;
    qBool               m_isAcked; //n2967
    size_t              m_packedSize ; //n2967
    PacketHistoryBundlingState m_packetHistoryBundlingState; //n2967
    Time                m_packetHistoryForceRetransmitTimeout; //n2967
    qUnsignedInt32      m_callID;
};

END_NAMESPACE_NEX

