﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/UDP/QueuingSocket.h>
#include <OnlineCore/src/Transport/Interface/TransportBufferSet.h>

BEGIN_NAMESPACE_NEX

class Condition;
class TransportBuffer;

class QueuingSocketTransportBuffer: public QueuingSocket
{
public:
    QueuingSocketTransportBuffer(TransportProtocol::Type transportProtocolType, RootTransport *pTransport,
                                 qUnsignedInt32 uiMtu, Condition *pCond,
                                 TransportBuffer *pTransportBuffer);
    virtual ~QueuingSocketTransportBuffer();

    virtual qBool IsRecvDataExists() const;
    qBool SendBufferToSocket();
    qBool RecvBufferFromSocket(qBool isNonBlock);

    // Send RecvのTransportBufferSet枯渇時にドロップした回数
    qUnsignedInt GetSendBufferSetEmptyDropCount() const
    {
        return m_TransportSend.GetInfoPoolEmptyDropCount();
    }
    qUnsignedInt GetRecvBufferSetEmptyDropCount() const
    {
        return m_TransportRecv.GetInfoPoolEmptyDropCount();
    }

protected:
    qBool CanSendToSocket()
    {
        return !m_TransportSend.IsQueueEmpty();
    }
    qBool CanRecvFromSocket()
    {
        return !m_TransportRecv.IsQueueFull();
    }

    virtual qInt RecvBuffer(Buffer* receivedBuffer, InetAddress* pInetAddress);

    // 一時バッファ
    Buffer m_recvBufferFromSocketTmpBuffer;
    Buffer m_sendBufferTmpBuffer;
    Buffer m_sendBufferToSocketTmpBuffer;

    // 送信用キュー
    TransportBufferSet m_TransportRecv;

    // 受信用キュー
    TransportBufferSet m_TransportSend;

    // 送信スレッド起床用
    Condition* m_pCond;

    virtual qInt SendBuffer(const Buffer& sendBuffer, const InetAddress& inetAddress);

};

END_NAMESPACE_NEX

