﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Platform/Core/qList.h>

BEGIN_NAMESPACE_NEX

template<typename NodeType> class TimedQueue : public RootObject
{

    class TimedQueueNode
    {
    public:
        TimedQueueNode(const NodeType &refNode, Time tiTimestamp):m_oNode(refNode)
        {
            m_tiTimestamp = tiTimestamp;
        }
        qBool TimestampIsExpired(Time tiNow = Time::GetTime()) const
        {
            return m_tiTimestamp<=tiNow;
        }
        Time GetTimestamp() const
        {
            return m_tiTimestamp;
        }
        NodeType& GetNode() const
        {
            return m_oNode;
        }
    private:
        mutable NodeType    m_oNode;
        Time        m_tiTimestamp;
    };

public:
    TimedQueue()
    {
    }

    virtual ~TimedQueue()
    {
        while (!m_lstQueue.empty())
        {
            m_lstQueue.pop_front();
        }
    }

    virtual void Queue(const NodeType& refNode, Time tNow = Time::GetTime())
    {
        typename qList<TimedQueueNode>::reverse_iterator ritNode = m_lstQueue.rbegin();

        while ((ritNode != m_lstQueue.rend()) && (ritNode->GetTimestamp() > tNow))
        {
            ++ritNode;
        }
        m_lstQueue.insert(ritNode.base(),TimedQueueNode(refNode,tNow));
    }

    virtual qBool Ready(Time tiNow = Time::GetTime())
    {
        return (!m_lstQueue.empty() && m_lstQueue.front().TimestampIsExpired(tiNow));
    }

    virtual NodeType GetHead(Time tiNow = Time::GetTime())
    {
        (void)tiNow;
        SYSTEMCHECK(Ready(tiNow));
        return m_lstQueue.front().GetNode();
    }

    virtual void Dequeue(const NodeType& refNode)
    {
        (void)refNode;
        SYSTEMCHECK(!m_lstQueue.empty());
        SYSTEMCHECK(m_lstQueue.front().GetNode() == refNode);
        m_lstQueue.pop_front();
    }

    void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS)
    {
        (void)uiTraceFlags;
        typename qList<TimedQueueNode>::iterator itNode = m_lstQueue.begin();
        while (itNode != m_lstQueue.end())
        {
            QTRACE(uiTraceFlags,NEX_T("Node at ") << dec << (qUnsignedInt32(itNode->GetTimestamp())));
            itNode++;
        }
    }

protected:
    qList<TimedQueueNode>   m_lstQueue;
};

END_NAMESPACE_NEX

