﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Platform/Core/Time.h>

BEGIN_NAMESPACE_NEX

class Core;

class Timeout: public RootObject
{

public:

    Timeout();
    virtual ~Timeout();

    //  RTO = request timed out ???
    void SetRTO(qUnsignedInt32 uiRTO);
    void Start();
    qBool operator <(Timeout &pTimeout);
    qUnsignedInt32      GetRTO();
    Time        GetAwaitedTime();

    Time GetExpirationTime()
    {
        return m_oExpirationTime;
    }
    void SetExpirationTime(Time oExpirationTime);
    void SetRelativeExpirationTime(TimeInterval tiTimeFromNow);

    //n1589
    Time GetTimeoutValue()
    {
        return m_oTimeFromNow;
    }

    //  Note on packet 'expiration':
    //  Packets whose type is SYN, CONNECT or DISCONNECT are assigned an expiration delay,
    //  whereas this is not the case for DATA ones.
    //  See: PRUDPEndPoint::ServiceTimeout: the expiration of a SYN, CONNECT or DISCONNECT
    //  alters the state of the current endpoint (SetConnectionState/SignalFaultEvent).
    qBool IsExpired(const Time& currentTime) const;

    //  true if the time between Start() and +RTO is elapsed.
    //  E.g. - PING packets: the RTO is set as the keepalive timeout,
    //  meaning that the TimeoutManager schedules a ping at this interval.
    qBool IsAwaited(const Time& currentTime) const;

#ifdef NEX_TRACE_ENABLED
    void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS);
#endif

protected:
    TimeInterval    m_oRTO;
    Time            m_oAwaitedTime;
    Time            m_oExpirationTime;

    Time m_oTimeFromNow;

    Core *m_pCore;
};

END_NAMESPACE_NEX

