﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/EmulationDevice.h>
#include <OnlineCore/src/Transport/UDP/TimedQueue.h>

BEGIN_NAMESPACE_NEX

template <typename NodeType> class VirtualInternet: public TimedQueue<NodeType>
{
public:
    VirtualInternet(EmulationDevice *pDevice)
    {
        SYSTEMCHECK(pDevice != NULL);
        m_pDevice = pDevice;
    }
    virtual ~VirtualInternet() {}

    virtual void Queue(const NodeType& refNode, Time tNow = Time::GetTime())
    {
        qReal rPacketDropProbability = m_pDevice->GetPacketDropProbability();
        if (Platform::GetRealRandomNumber(QREAL(1.0)) >= rPacketDropProbability)
        {
            qUnsignedInt uiJitter = m_pDevice->GetJitter();
            qUnsignedInt uiLatency = m_pDevice->GetLatency();
            if (uiJitter > 0)
            {
                TimedQueue<NodeType>::Queue(refNode,tNow+
                                            TimeInterval(uiLatency)+TimeInterval(Platform::GetRandomNumber(uiJitter)));
            }
            else
            {
                TimedQueue<NodeType>::Queue(refNode,tNow+TimeInterval(uiLatency));
            }
        }
    }

protected:
    EmulationDevice             *m_pDevice;
};

qBool VirtualInternetTest();

END_NAMESPACE_NEX

