﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/EmulationDevice.h>
#include <OnlineCore/src/Transport/UDP/TimedQueue.h>

BEGIN_NAMESPACE_NEX

/*
 * NodeType must have the member:
 *
 * Buffer *GetBuffer() const;
 */
template <typename NodeType> class VirtualModem: public TimedQueue<NodeType>
{
public:
    VirtualModem(EmulationDevice *pDevice, Time tNow = Time::GetTime())
    {
        SYSTEMCHECK(pDevice != NULL);
        m_pDevice = pDevice;
        m_tiNextOutputTime = tNow;
    }
    virtual ~VirtualModem() {}

    virtual void Queue(const NodeType& refNode, Time tNow = Time::GetTime())
    {
        if (m_tiNextOutputTime < tNow)
        {
            m_tiNextOutputTime = tNow;
        }
        TimedQueue<NodeType>::Queue(refNode,m_tiNextOutputTime);
        if (m_pDevice->GetBandwidth() != INFINITE_BANDWIDTH)
        {
            m_tiNextOutputTime = ((refNode.GetBuffer()->GetContentSize()*1000*8)/m_pDevice->GetBandwidth())+m_tiNextOutputTime;
        }
    }

protected:
    EmulationDevice             *m_pDevice;
    Time                        m_tiNextOutputTime;
};

END_NAMESPACE_NEX

