﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/DebugString.h>


BEGIN_NAMESPACE_NEX

class ProtocolCallContext;


class OSDK_API AccountManagementCommand: public RootObject
{

public:
    AccountManagementCommand(const DebugString& strDescription,
                             const String&      strPassword );
    virtual ~AccountManagementCommand();

    qBool Invoke(ProtocolCallContext* pCC);

    const String& GetPassword() const
    {
        return m_strPassword;
    }

    //  Set by JobManageAccount for when password hash is computed
    void SetPartialKey(const String& strPartialKey)
    {
        m_strPartialKey = strPartialKey;
    }

    const qChar* GetDescription() const;



protected:

    //TO BE OVERRIDDEN
    virtual qBool InvokeImpl(ProtocolCallContext* pCC) = 0;

    const String& GetPartialKey() const
    {
        return m_strPartialKey;
    }


private:
    DebugString     m_strDescription;
    String          m_strPassword;  //  Clear text password
    // For most of the account mgmt commands, the password is required
    // JobManageAccount needs to access it for initial hashing
    String          m_strPartialKey;    //  One-way hash, as computed by Kerberos encryption

};

END_NAMESPACE_NEX

