﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Transport/Interface/EndPointEventHandler.h>
#include <RendezVous/Core/src/Common/Protocol.h>
#include <OnlineCore/src/Core/OperationManager.h>
#include <OnlineCore/src/Core/SecurityContextManager.h>
#include <OnlineCore/src/Platform/Core/BandwidthCounter.h>
#include <RendezVous/Core/src/Common/ProtocolRequestBrokerInterface.h>

BEGIN_NAMESPACE_NEX

class ConnectionOrientedStream;
class Message;

//==============================================================================
// Pack and unpack messages headers, and dispatch messages to the appropriate protocol.
//==============================================================================

// The ClientProtocolRequestBroker is a class that is used by the client.

// There is a lot of code duplication between the ClientProtocolRequestBroker and ProtocolRequestBroker (Server)
// which could be moved to a common base, but to minimize the changes to the server side for this refactor, we decided to
// just copy-paste the code.

class OSDK_API ClientProtocolRequestBroker : public ProtocolRequestBrokerInterface
{

public:
    ClientProtocolRequestBroker();
    virtual ~ClientProtocolRequestBroker();

    CallProtocolMethodOperation* GetCurrentOperation();
    EndPoint *GetCurrentEndPoint()
    {
        return m_pCurrentEndPoint;
    }

    // Called by the JobProcessProtocolMessage.
    virtual void ProcessMessage(CallProtocolMethodOperation* pOp, EndPoint *pEndPoint, Buffer *pBuffer);
    virtual void ProcessCallRequest(ProtocolID uiProtocolID, Message* pMessageReceived, Message* pmsgResponse, qBool *pbReturnsResponse, EndPoint *pEndPoint);
    virtual void ProcessCallResponse(ProtocolID uiProtocolID, Message* pMessageReceived, EndPoint *pEndPoint);

    virtual qBool RegisterServerProtocol(ProtocolID uiProtocolID, ServerProtocol* pServerProtocol);
    virtual qBool UnregisterServerProtocol(ProtocolID uiProtocolID);
    virtual ServerProtocol *GetServerProtocol(ProtocolID uiProtocolID);

public:
    virtual void PeerDisconnected(EndPoint *pEndPoint);
    virtual void FaultDetection(EndPoint *pEndPoint, qUnsignedInt32 uiFault);

    void ProcessLocalCallRequest(ServerProtocol* pProtocol, Message* pMessageReceived, Message* pmsgResponse, qBool *pbReturnsResponse, EndPoint *pEndPoint);
    static qBool SystemStateIsValid(qBool bBlockingCall);

private:
    void DispatchCallToProtocol(ServerProtocol* pProtocol, Message* pMessage, Message* pResponse,
                                qBool* pbReturnsMessage, EndPoint* pEndPoint);
    void ProcessMessageCore(CallProtocolMethodOperation* pOp, EndPoint *pEndPoint, Buffer *pBuffer);

private:
    // Block default copy construction and assignement
    ClientProtocolRequestBroker(const ClientProtocolRequestBroker &);
    ClientProtocolRequestBroker & operator=(const ClientProtocolRequestBroker &);

    // Server Broker interface, not implemented.
    virtual qBool RegisterProtocol(Protocol *pProtocol);
    virtual void UnregisterProtocol(Protocol *pProtocol);
    virtual Protocol *GetProtocol(ProtocolID uiProtocolID, Protocol::Type eType);
    virtual qBool ReloadProtocols();
    virtual qBool CloneProtocols();
    virtual qResult DispatchLocalRMC(Message* pmsgRequest, ClientProtocol *pCaller, ServerProtocol *pCallee, EndPoint *pSourceEndPoint);
    virtual qResult DispatchBlockingLocalRMC(Message* pmsgRequest, ClientProtocol *pCaller, ServerProtocol *pCallee, EndPoint *pSourceEndPoint);
    virtual qResult DispatchThreadSafeLocalRMC(Message* pmsgRequest, ClientProtocol *pCaller, ServerProtocol *pCallee, EndPoint *pSourceEndPoint);

private:
    typedef qMap<ProtocolID, ServerProtocol*> ProtocolMap;
    OSDK_DISABLE_EXPORT_WARNING(ProtocolMap m_mServerProtocols);

    EndPoint *m_pCurrentEndPoint;
    SingleThreadCallPolicy m_oCallPolicy;
};

END_NAMESPACE_NEX


