﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/NonCopyable.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Core/StepSequenceJob.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/NintendoHTTP/HttpClient.h>

BEGIN_NAMESPACE_NEX

class JobFileServer : public StepSequenceJob, public NonCopyable, public HttpEventListener
{
public:

    JobFileServer();
    virtual ~JobFileServer();

    virtual qResult Initialize(
        qUnsignedInt32 uiAssociatedCallID,
        const String& url,
        HttpConnection::Method method,
        HttpEventListener* pEventListener,
        size_t relayBufferSize
    );

    static const qUnsignedInt32 MAX_NUM_TRY = 3;
    static const qUnsignedInt32 MAX_NUM_REDIRECT_TRY = 5;
protected:

    qUnsignedInt32      m_AssociatedCallID;
    String              m_Url;
    HttpConnection::Method m_Method;
    HttpEventListener*  m_pEventListener;
    CallContext         m_CallContext;
    HttpClient          m_HttpClient;
    qUnsignedInt32      m_NumTry;

    qInt                m_StatusCode;
    qVector<qUnsignedInt8> m_ErrorBodyBuffer;
    qBool               m_IsErrorBufferOverflow;
    String              m_Location;
    size_t              m_RelayBufferSize;

#ifdef CTR_TARGET
    static const qUnsignedInt32 ROOTCA_NUM = 4;
    qVector<qByte>      m_RootCaCert[ROOTCA_NUM];
#endif

    void StepHttp();
    void StepWaitingForHttp();
    void CompleteJob(const qResult& result);
    qBool IsHttpStatusCodeSuccess();
    void ResetHttpResult();

    virtual void PrepareRequest(HttpConnection* httpConnection);
    virtual size_t PostChunkedBuffer(HttpConnection* connection, qUnsignedInt8* buffer, size_t bufferSize, qBool* completed);
    virtual void ProcessResponseHeader(HttpConnection* httpConnection, qInt statusCode);
    virtual qBool ProcessResponse(const qUnsignedInt8* buffer, size_t bufferSize);
};

END_NAMESPACE_NEX

