﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/EndPoint.h>

BEGIN_NAMESPACE_NEX

class OSDK_API SecureEndPoint: public EndPoint
{
public:
    SecureEndPoint(ConnectionOrientedStream *pTransport, const StationURL *pStationURL);
    virtual ~SecureEndPoint();

    EndPoint *GetAssociatedEndPoint();
    void SetAssociatedEndPoint(EndPoint *pEndPoint);

    virtual qUnsignedInt32  GetRTT() const;
    virtual qUnsignedInt32  GetRTTAverage() const;

    virtual qResult         RegisterEventHandler(EndPointEventHandler *pEndPointEventHandler);

    virtual qBool           IsNotConnected();
    virtual qBool           IsConnecting();
    virtual qBool           IsDisconnecting();
    virtual qBool           IsFaulty();
    virtual qBool           IsConnected();

    virtual qBool           PeerIsConnected();
    virtual qBool           PeerIsDisconnected();

    virtual void            SetPeerConnected();
    virtual void            SetPeerDisconnected();

    virtual ConnectionState GetConnectionState() const;
    virtual qBool           SetConnectionState(ConnectionState eState);

    virtual void            SetKeepAliveTimeout(qUnsignedInt32 uiKeepAliveTimeout);
    virtual void            SetMaxSilenceTime(qUnsignedInt32 uiMaxSilenceTime);
    virtual qUnsignedInt32  GetKeepAliveTimeout();
    virtual qUnsignedInt32  GetMaxSilenceTime();
    virtual qBool IsAnyPacketProcessed();
    virtual NetFaultReason  GetSignaledFaultError() const;

    /** Close the end point if it is faulty  */
    void    CompleteClose();
    void    CompleteDisconnect();



    /*
     * @brief       EndPointInfo のパラメータをリセットします。
     *
     * @see         EndPointInfo , LogPRUDPAnalyzeInfo() , GetEndPointInfo()
     */
    virtual void ResetEndPointInfo()
    {
        SYSTEMCHECK(false);
    };

    /*
     * @brief       EndPointInfo のパラメータを取得します。
     *
     * @param[out]   info コピー先のEndPointInfo構造体を設定します
     * @retval true 取得が成功した場合
     * @retval false 取得が失敗した場合 (infoがnull)
     * @see         EndPointInfo , ResetEndPointInfo() , GetEndPointInfo()
     */
    virtual qBool GetEndPointInfo(EndPointInfo* info) const
    {
        DUMMY_PARAM(info);
        SYSTEMCHECK(false);
        return false;
    }


    /*
     * @brief EndPointのreliable送信バッファの現在のパケット数を取得します。
     *
     * reliable送信バッファは通信経路の混雑時などに徐々に増加していきます。
     * reliable送信バッファのパケット数がStream::DOのMaxWindowMapSize(SetMaxWindowMapSizeで指定したサイズ)を超えた場合、送信バッファあふれによるエラーとなります。
     *
     * @return reliable送信バッファのパケット数。
     */
    virtual size_t GetReliableBufferPacketCount() const
    {
        SYSTEMCHECK(false);
        return 0;
    }

    virtual void GetReliableBufferPacketCounts(nn::nex::qVector<size_t>&) const
    {
        SYSTEMCHECK(false);
    }

    virtual qByte GetMaxSubStreamID() const
    {
        return 0;
    }

#if NEX_ENABLE_USE_SECURE_KEY
    virtual void SetEncryptionKey(const Key &oKey);
    virtual Key *GetEncryptionKey();
#endif

protected:

    //  TODO:CHECK - dead code
    virtual qResult         _Connect(Buffer *pInBuffer, Buffer *pOutBuffer,
                                     const SecureEndPoint::Callback pCallback, const UserContext &oContext,
                                     qUnsignedInt32 uiTimeout);

    //  TODO:CHECK - dead code
    virtual qResult         _Disconnect(const EndPoint::Callback pCallback, const UserContext &oContext,
                                        qUnsignedInt32 uiTimeout);

    //  TODO:CHECK - dead code
    virtual qResult         _Send(Buffer *pBuffer, qUnsignedInt32 uiFlags, qBool bPacketOutInternalBuffer=false, qByte substream=0, qUnsignedInt32 uiCallID=0);

    static void             DisconnectCallback(EndPoint* pEndPoint, qResult rResult,
            const UserContext* pContext);

    virtual qUnsignedInt64 GetSupportedFunctionsFlags() const;

private:

    void                    SignalFaultEvent(NetFaultReason uiError, qBool bProcessNow);

    EndPoint*                   m_pAssociatedEndPoint;

    SecureEndPoint::Callback    m_pOwnDisconnectCallback;
    UserContext                 m_oOwnDisconnectUserContext;
};


END_NAMESPACE_NEX

