﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

////////////////////////////////////////////////////////////////////////////////
// IMPORTANT NOTE: Values of each element of the following enumation must be
// specify so that translating this file for Python scripts works as expected.
// Also, the description of elements must use C-style /* */ comments where
// the opening and closing syntax must be on their own lines of code.
////////////////////////////////////////////////////////////////////////////////

BEGIN_NAMESPACE_NEX

/** \brief Namespace for enum ErrorCode. */
#ifdef SWIG
class PythonCoreErrorCode
{
public:
#else
/*!
 @brief    PythonCore モジュールのエラーコードを定義する名前空間です。
 */
namespace PythonCoreErrorCode
{
#endif
    /*
    \ingroup group_rdv_core_common
    \brief Enumerates the error codes that a call may return for the PythonCore facility.
    \remarks Details as to the cause of each ErrorCode are detailed in the documentation
    of each individual Rendez-Vous method that may return a result.
    */

    /*!
     @brief    PythonCore モジュールのエラーコード。サーバー側で発生したエラーを表します。
     */
    enum ErrorCode
    {
        START = 1,
        /**
        The operation raised an exception.
        */
        Exception           = 1,    //!<    例外発生
        /**
        The operation was applied to an object of inappropriate type.
        */
        TypeError           = 2,    //!<    型変換エラー
        /**
        A sequence subscript is out of range.
        */
        IndexError          = 3,    //!<    範囲外のindex
        /**
        The operation specifies or accesses an invalid reference.
        */
        InvalidReference    = 4,    //!<    不正な参照
        /**
        The call failed to complete.
        */
        CallFailure         = 5,    //!<    処理を中断した
        /**
        There was not enough memory available to complete the operation.
        */
        MemoryError         = 6,    //!<    メモリ不足
        /**
        The mapping (dictionary) key was not found in the set of existing keys.
        */
        KeyError            = 7,    //!<    キーが存在しない
        /**
        The operation raised an error.
        */
        OperationError      = 8,    //!<    処理を中断した
        /**
        Conversion of Python values to or from another language produced an error.
        */
        ConversionError     = 9,    //!<    python から 他の言語への変換でエラーが発生
        /**
        The caller of the operation does not have the necessary permission to perform the operation.
        */
        ValidationError     = 10     //!<    許可されていないオペレーション

    };
    //note: 新規result追加時には、OnlineCore/src/Platform/Core/ResulDefine.hのNEX_ERROR_RESULT_DEFINEの変更も必要
#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX

