﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <RendezVous/Core/src/Common/MessageRecipientDDL.h>
#include <RendezVous/Core/src/Common/Constants/IdentifierTypes.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringID.h>


BEGIN_NAMESPACE_NEX


class OSDK_API DDLCLASS(MessageRecipient);
class OSDK_API MessageRecipient: public DDLCLASS(MessageRecipient)
{

public:
    /** \brief Default constructor. */
    MessageRecipient()
    {
        m_uiRecipientType = static_cast<qUnsignedInt32>( IdentifierTypes::InvalidType );
        m_principalId = INVALID_PRINCIPALID;
        m_gatheringId = INVALID_GATHERINGID;
    }

    MessageRecipient(qUnsignedInt32 uiRecipientType, PrincipalID pid, GatheringID gid)
    {
        SYSTEMCHECK((uiRecipientType==IdentifierTypes::PIDType) || (uiRecipientType==IdentifierTypes::GIDType));
        SYSTEMCHECK((pid == INVALID_PRINCIPALID) || (gid == INVALID_GATHERINGID));
        m_uiRecipientType = uiRecipientType;
        m_principalId = pid;
        m_gatheringId = gid;
    }

    qUnsignedInt32 GetRecipientType() const
    {
        return (m_uiRecipientType);
    }

    void SetRecipientType(qUnsignedInt32 uiType)
    {
        m_uiRecipientType = uiType;
    }

    PrincipalID GetRecipientPrincipalID() const
    {
        return m_principalId;
    }

    void SetRecipientPrincipalID(PrincipalID pid)
    {
        m_principalId = pid;
    }

    GatheringID GetRecipientGatheringID() const
    {
        return m_gatheringId;
    }

    void SetRecipientGatheringID(GatheringID gid)
    {
        m_gatheringId = gid;
    }

    void Trace(qUnsignedInt64 uiTraceFlags)
    {
        DUMMY_PARAM(uiTraceFlags);
        QTRACE(uiTraceFlags, NEX_T("MessageRecipient (PID=") << m_principalId << NEX_T(", GID=") << m_gatheringId << NEX_T(", Type=") << m_uiRecipientType << NEX_T(")"));
    }

};


END_NAMESPACE_NEX

