﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <RendezVous/Core/src/Common/NintendoProtocolFoundation/NintendoNotificationEventDDL.h>
#include <RendezVous/Core/src/Common/DataHolder.h>

BEGIN_NAMESPACE_QUAZAL

//group=Rendez-Vous Core

#ifdef FOR_DOC_PURPOSE
/*
Description: Use this class to construct and retrieve information about NintendoNotificationEvents.
Remarks: This class is used to construct and retrieve information about NintendoNotificationEvents, which are
messages sent from a back-end service to another Process whenever an event occurs.
events.
See Also: NotificationEventHandler, NotificationEvents
*/
class NintendoNotificationEvent: public RootObject
{
#else
class NintendoNotificationEvent: public DDLCLASS(NintendoNotificationEvent)
{
#endif
    //group=Methods

public:
    //[TODO](masuda_takumi) write documents

    /*
    Parameters:
    */
    NintendoNotificationEvent()
    {
        m_uiType = 0;
    }
    NintendoNotificationEvent(qUnsignedInt32 uiType, PrincipalID pidSender, const DataHolder& customData)
    {
        m_uiType = uiType;
        m_pidSender = pidSender;
        m_dataholder = customData;
    }

    /*
    Description: Returns the type of this NintendoNotificationEvent.
    Returns: The type of the NintendoNotificationEvent as a qUnsignedInt32.
    Remarks: The source ID, type, subtype (if any), and parameters of a NintendoNotificationEvent are defined when
    the NintendoNotificationEvent is constructed.
    */
    qUnsignedInt32 GetType() const
    {
        return m_uiType;
    }

    PrincipalID GetSenderPrincipalID() const
    {
        return m_pidSender;
    }

    const DataHolder& GetData() const
    {
        return m_dataholder;
    }

};

// String streaming.
// {secret}
extern StringStream& operator<<(StringStream& oStream, const NintendoNotificationEvent& oEvent);

END_NAMESPACE_QUAZAL

