﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#define DDLPROTOCOL(CLASS) _Proto_##CLASS

#define PROTO_IMPLEMENTATION_NAME(CLASS)        _Proto_Impl_##CLASS

#if defined(WIN32_TARGET) || defined(LINUX_TARGET) || defined(NEX_DEBUG_VERSION)
#define SET_CONTEXT_RMCNAME(pCC, pName) pCC->SetRMCName(pName)
#else
#define SET_CONTEXT_RMCNAME(pCC, pName)
#endif


#define PROTO_RMC_INITREQUEST(CLASS,RMCNAME)    \
    ScopedCS oCS(Scheduler::GetInstance()->SystemLock());   \
    if (!RegisterCallContext(pMsg,pContext)) {return false;}    \
    SET_CONTEXT_RMCNAME(pContext,NEX_T(#CLASS) NEX_T("::") NEX_T(#RMCNAME));        \
    AddMethodID(pMsg,PROTO_RMCID(CLASS,RMCNAME));

#define PROTO_RMC_INITCALL(CLASS,RMCNAME)   \
    Message msgRequest; \
    {   \
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());   \
        Message* pMsg=&msgRequest;      \
        GetRequestBroker()->InitMessage(pMsg, GetProtocolID(), Protocol::Request);      \
        if (!RegisterCallContext(pMsg,pContext)) {return false;}    \
        SET_CONTEXT_RMCNAME(pContext,NEX_T(#CLASS) NEX_T("::") NEX_T(#RMCNAME));        \
        AddMethodID(pMsg,PROTO_RMCID(CLASS,RMCNAME));

#define PROTO_RMC_SENDMESSAGE() \
        if (!IsUsingLocalLoopback() || !FlagIsSet(ProtocolFlags::NonBlockingClient)) { return SendRMCMessage(pContext, &msgRequest); }        \
    }   \
    SYSTEMCHECK(msgRequest.GetSourceEndPoint() == NULL);    \
    return SendRMCMessage(pContext, &msgRequest);
// SYSTEMCHECK(msgRequest.GetSourceEndPoint() == NULL); について、
// msgRequest.GetSourceEndPoint()がNULLではないときはMessageのデストラクタでEndPointのCloseが行われる。
// その場合SystemLockをかけている必要がある。しかし、SystemLockをかけずにSendRMCMessageする機能があるのでここではSystemLockをかけていない。
// SourceEndPointのセットが行われるのはNetZだけだが念のためここではセットされていないことを確認する。


#define PROTO_RMCIMPL_STUB(CLASS, RMCNAME)      static void ProtoStub_##RMCNAME( Message *pMessage, Message *pReturnMessage, DDLPROTOCOL(CLASS##Server) *pProtocol )
#define PROTO_RMCNAMEL_RETURN(RMCNAME)  ProtoReturn_##RMCNAME
#define PROTO_RMCIMPL_RETURN(CLASS, RMCNAME)    static void PROTO_RMCNAMEL_RETURN(RMCNAME)( Message *pMsg, ProtocolCallContext* pContext )

#define PROTO_RMCID(CLASS,RMCNAME)          __##CLASS##__##RMCNAME##_RMCID
#define PROTO_RETURNRMCID(CLASS,RMCNAME)        (PROTO_RMCID(CLASS,RMCNAME) | 0x8000)
#define PROTO_DEFINE_RMCID(CLASS,RMCNAME,ID)    const qUnsignedInt16 PROTO_RMCID(CLASS,RMCNAME) = ID
#define PROTO_NUM_RMC(CLASS)    __##CLASS##__NUM_RMC
#define PROTO_DEFINE_NUM_RMC(CLASS,NUMRMC)  const qUnsignedInt32 PROTO_NUM_RMC(CLASS) = NUMRMC

#define PROTO_REGISTER_PROTO_RETURN_FUNCTION(CLASS,FUNCTION) \
    SYSTEMCHECK(s_protoReturnFunctions[PROTO_RMCID(CLASS,FUNCTION) - 1] == NULL \
        || s_protoReturnFunctions[PROTO_RMCID(CLASS,FUNCTION) - 1] == PROTO_RMCNAMEL_RETURN(FUNCTION)); \
    s_protoReturnFunctions[PROTO_RMCID(CLASS,FUNCTION) - 1] = PROTO_RMCNAMEL_RETURN(FUNCTION)

#define PROTO_GET_PROTO_RETURN_FUNCTION(RMCID) \
    s_protoReturnFunctions[RMCID - 1]

#define PROTO_DECLARE_PRIVATE_IMPLEMENTATION(CLASS) \
    DDLPROTOCOL(CLASS##Base)(Type eProtocolType=Protocol::Server): Protocol(eProtocolType) {SetName(NEX_T(#CLASS));}   \
    struct PROTO_IMPLEMENTATION_NAME(CLASS);\
    friend struct PROTO_IMPLEMENTATION_NAME(CLASS);

// uselsss?
#define PROTO_DECLARE_CLIENT(CLASS) CLASS##Client(): DDLPROTOCOL(CLASS##Base)(Protocol::Client) {}

#define PROTO_PREPARE_EXTRACTION()  DUMMY_PARAM(pReturnMessage); \
                                    Message *pMsg = pMessage;

#define PROTO_RETURNSTUB_IMPL()     DUMMY_PARAM(pMsg); DUMMY_PARAM(pContext); qInt iArgIndex=0

#define PROTO_RETURNSTUB_END()      iArgIndex = iArgIndex

#define PROTO_PREPARE_RETURN(CLASS,RMCNAME) \
    pMsg = pReturnMessage;\
    AddMethodID(pMsg, PROTO_RETURNRMCID(CLASS,RMCNAME));

#define PROTO_DISPATCHIMPL_START()  qUnsignedInt16 uiRMCID = static_cast<qUnsignedInt16>(ExtractMethodID(pMessage));\
                                    switch( uiRMCID ){

#define PROTO_DISPATCHIMPL_VOID_STUB(CLASS, RMCNAME)    case PROTO_RMCID(CLASS,RMCNAME): \
                                                                      PROTO_IMPLEMENTATION_NAME(CLASS)::ProtoStub_##RMCNAME( pMessage, pReturn, this );\
                                                                      *boReturnsMessage = false;\
                                                                      return;

#if defined(LIVE_VERSION)
#define PROTO_DISPATCHIMPL_RETURN_STUB(CLASS, RMCNAME) case PROTO_RMCID(CLASS,RMCNAME): \
                                                            {\
                                                                      ProfilingUnit::Scope oPS(m_ ## RMCNAME ## ProfilingUnit);\
                                                                      ProtoStub_##RMCNAME( pMessage, pReturn, this );\
                                                                      *boReturnsMessage = true;\
                                                                      return;\
                                                            }
#else // LIVE_VERSION
#define PROTO_DISPATCHIMPL_RETURN_STUB(CLASS, RMCNAME) case PROTO_RMCID(CLASS,RMCNAME): \
                                                                      ProtoStub_##RMCNAME( pMessage, pReturn, this );\
                                                                      *boReturnsMessage = true;\
                                                                      return;
#endif // LIVE_VERSION

#define PROTO_DISPATCHIMPL_PROCESSRESULT_STUB(CLASS, RMCNAME)       \
case PROTO_RETURNRMCID(CLASS,RMCNAME): ProtoReturn_##RMCNAME(pMessage, pContext); return;

#define PROTO_DISPATCHIMPL_END() default: SetCallError(QRESULT_ERROR(Core,NotImplemented));}

#define PROTO_DISPATCHIMPL_BY_USING_FUNCTION_TABLE(CLASS) \
    qUnsignedInt16 uiReturnRMCID = static_cast<qUnsignedInt16>(ExtractMethodID(pMessage));\
    qUnsignedInt16 uiRMCID = uiReturnRMCID & ~0x8000;\
    if(uiRMCID < 1 || uiRMCID > PROTO_NUM_RMC(CLASS))\
    {\
        SetCallError(QRESULT_ERROR(Core,NotImplemented));\
        return;\
    }\
    if(!PROTO_GET_PROTO_RETURN_FUNCTION(uiRMCID))\
    {\
        QLOG(EventLog::Warning, NEX_T("The RMC is not called Protocol:") << NEX_T(#CLASS) << NEX_T(", RMCID: ") << uiRMCID);\
        SYSTEMCHECK(false);\
        return;\
    }\
    PROTO_GET_PROTO_RETURN_FUNCTION(uiRMCID)(pMessage, pContext)

