﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

BEGIN_NAMESPACE_NEX

/**
\brief Namespace for enum ProtocolFlags.
\internal
*/
#ifdef SWIG
class ProtocolFlags
{
public:
#else
namespace ProtocolFlags
{
#endif

    /**
    \ingroup group_rdv_core_common
    \brief Enumerates the flags that may be associated to a protocol.
    \remarks By default custom protocols are assumed to be non thread-safe. If you wish to run \productname
    with multiple threads, you should ensure that the custom protocols you load are thread-safe and flag them
    as such when you load the protocol via LoadProtocolScript() of the SandboxConfiguration module. Nevertheless,
    if it is not possible to make a given protocol thread-safe then the protocol should not be flagged, meaning that
    it will only be run in a single thread. All calls for this protocol will be executed sequentially, though.
    \note Note that setting a protocol as thread-safe does not make it thread-safe... The purpose is to instruct the
    protocol server the way it will process calls.
    \sa SandboxConfiguration
    */
    enum ProtocolFlags
    {
        /** \internal */
        Unknown = 0,
        /** \internal */
        External = 1,
        /*
        The protocol is thread-safe.
        */
        ThreadSafe = 2,
        /** \internal */
        PatchReturnCode = 4,
        /** \internal
        The protocol is one way.  Before a protocol is declared one
        way, read the comments of bug 2600.
        */
        OneWay = 8,

        /** \internal
        This marks client protocols that are safe to use for non
        blocking operations, on the server.  See bug:3078.
        */
        NonBlockingClient = 16
    };

#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX


