﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Common/Protocol.h>
#include <OnlineCore/src/Platform/Core/PerfCounter.h>

BEGIN_NAMESPACE_NEX

//==============================================================================
// Abstract base class for Server-side protocol classes. The DDL compiler
// generates C++ classes that inherits from this class along with ClientProtocol
// The following link describes an overview of the related design:
// For NEX Developer http://jwiki/cgi-bin/wiki.cgi?RendezVousProtocol
//
// Note: ServerProtocol is conceptually an abstract base class
//==============================================================================
class OSDK_API ServerProtocol: public Protocol
{
public:
    NEX_DECLARE_SYSCOMPONENT_METHODS(ServerProtocol,Protocol);

    explicit ServerProtocol(qUnsignedInt32 uiFlags = ProtocolFlags::External);
    virtual ~ServerProtocol();

    virtual Protocol::Type GetProtocolType() const
    {
        return Protocol::Server;
    }

    void SetCallError(qResult rCallResult);

    void DispatchCallRequest(size_t attemptCount, Message *pMessage, Message * pResponse, qBool * pbReturnsMessage, TimeInterval* pPostponeDelay, EndPoint * pEndPoint);

protected:
    // I'd prefer to use DispatchMessage... but this is a macro
    // sometimes defined to DispatchMessageA by Microsoft...
    virtual void DispatchProtocolMessage(Message * /*pMsg*/, Message * /*pReturn*/, qBool * /*pReturnsMessage*/, EndPoint * /*pEndPoint*/)=0;

    /*下記メソッドが実装されている場合は、UseAttemptCountMethod()をtrueとする*/
    virtual void DispatchProtocolMessageWithAttemptCount(size_t /*attemptCount*/, Message * /*pMsg*/, Message * /*pReturn*/, qBool * /*pReturnsMessage*/, TimeInterval* /*pPostponeDelay*/, EndPoint * /*pEndPoint*/)
    {
        SYSTEMCHECK(false);
    }


#ifdef NEX_SYSTEMCHECK_ENABLED
    qBool LockStateIsValid() const;
#endif

    virtual qBool UseAttemptCountMethod() { return false; }

private:
#ifdef NEX_PERFCOUNTER_ENABLED
    /** Set the name of the perfcounter based on the protocol name/id. */
    void SetPerfCounterNames();

    PerfCounter* m_pProtocolCalls;
    PerfCounter* m_pRequestTraffic;
    PerfCounter* m_pResponseTraffic;
    PerfCounter* m_pMembershipRequiredErrorCount;

    static PerfCounter s_oNonBlockingRequests;
    static PerfCounter s_oBlockingRequests;

    static PerfCounter s_oSlowRequests_Total;
    static PerfCounter s_oSlowRequests_Execution;
#endif // NEX_PERFCOUNTER_ENABLED

#ifdef NEX_WATERMARK_ENABLED
    static WaterMark s_oNonInitializedServerProtocols;
#endif
};


END_NAMESPACE_NEX

