﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <OnlineCore/src/Core/Key.h>

BEGIN_NAMESPACE_NEX

class KerberosEncryption;

class OSDK_API Ticket: public RootObject
{
public:
    Ticket(PrincipalID idOwner, PrincipalID idTarget, const Key& oKey, DateTime dtCreatedAt,
        qUnsignedInt32 apiVersionGeneral, qUnsignedInt32 apiVersionCustom, const String& optionalString);
    Ticket(const Buffer& bufEncryptedTicket, PrincipalID idTarget);
    Ticket(const Buffer& bufEncryptedTicket, PrincipalID idOwner, PrincipalID idTarget, const Key& oKey, DateTime dtCreatedAt,
        qUnsignedInt32 apiVersionGeneral, qUnsignedInt32 apiVersionCustom, const String& optionalString);
    virtual ~Ticket();

    PrincipalID GetOwnerID() const
    {
        return m_idOwner;
    }
    PrincipalID GetTargetID() const
    {
        return m_idTarget;
    }
    DateTime GetCreatedAt() const
    {
        return m_dtCreatedAt;
    }
    const Buffer& GetBuffer() const
    {
        return m_bufEncryptedTicket;
    }
    const Key& GetKey() const
    {
        return m_key;
    }
    const String GetOptionalString() const
    {
        return m_optionalString;
    }

    void Encrypt(KerberosEncryption* pEncryption, const Key& oEncryptionKeyOfTarget);
    void Decrypt(KerberosEncryption* pEncryption, const Key& initializeVector);

    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const;

    qBool IsValid() const;

    /*!
        @internal
        @brief 標準APIバージョンを取得する。
        @return 標準APIバージョン。
     */
    qUnsignedInt32 GetApiVersionGeneral() const
    {
        return m_apiVersionGeneral;
    }

    /*!
        @internal
        @brief カスタムAPIバージョンを取得する。
        @return カスタムAPIバージョン。
     */
    qUnsignedInt32 GetApiVersionCustom() const
    {
        return m_apiVersionCustom;
    }

private:
    qBool m_bValid;
    // Real ticket (encrypted and signed)
    Buffer m_bufEncryptedTicket;

    // Ticket meaning
    PrincipalID m_idOwner;
    PrincipalID m_idTarget;
    Key m_key;
    DateTime m_dtCreatedAt;
    qUnsignedInt32 m_apiVersionGeneral;
    qUnsignedInt32 m_apiVersionCustom;
    String m_optionalString;
    // TODO - Should have timestamp/expiration info
};

END_NAMESPACE_NEX


