﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

////////////////////////////////////////////////////////////////////////////////
// IMPORTANT NOTE: Values of each element of the following enumation must be
// specify so that translating this file for Python scripts works as expected.
// Also, the description of elements must use C-style /* */ comments where
// the opening and closing syntax must be on their own lines of code.
////////////////////////////////////////////////////////////////////////////////

#pragma once

#ifndef SWIG
#include <OnlineCore/src/Platform/Core/Result.h>
#include <OnlineCore/src/Transport/Interface/TransportError.h>
#endif

BEGIN_NAMESPACE_NEX

//group=Error Code Enumerations
/*!
 @brief    Authentication モジュールのエラーコードを定義する名前空間です。
 */
#ifdef SWIG
class AuthenticationErrorCode
{
public:
#else
namespace AuthenticationErrorCode
{
#endif

    //group= Type Definitions

    /*
    Description: Enumerates the error codes for the StorageBase facility that a call may return.
    */

    /*!
     @brief    Authentication モジュールのエラーコード。
    */
    enum ErrorCode
    {
        START = 1,
        /*
         NAS returns error code
         */
        NASAuthenticateError = 1,    //!< 認証エラー
        /*
         can't parse NAS return content
         */
        TokenParseError = 2,         //!< 認証エラー
        /*
         Http Connection Error
         */
        HttpConnectionError = 3,     //!< HTTP 接続に失敗
        /*
         Http DNS Error
         */
        HttpDNSError = 4,            //!< DNS エラー
        /*
         Http Get Proxy Setting Error
         */
        HttpGetProxySetting = 5,     //!< プロキシ設定の取得に失敗
        /*
         RendezVous : Token Expired
         */
        TokenExpired = 6,            //!< トークンの有効期限切れ
        /*
         RendezVous : Validation fail
         */
        ValidationFailed = 7,        //!< 検証に失敗
        /*
         RendezVous : token has invalid param
         */
        InvalidParam = 8,            //!< 不正なパラメータ
        /*
         RendezVous : token pid is not user pid
         */
        PrincipalIdUnmatched = 9,    //!< PID が一致しない
        /*
         RendezVous : move count unmatch
         */
        MoveCountUnmatch = 10,       //!< 使用されません
        /*
         RendezVous : The servers are now under maintenance.
        */
        UnderMaintenance = 11,       //!< サーバーは現在メンテナンス中である
        /*
         RendezVous : This version of client is not supported.
        */
        UnsupportedVersion = 12,     //!< サポートされていないバージョンのクライアントを使用している
        /*
         RendezVous : NGS version is old for this client. Please update NGS.
        */
        ServerVersionIsOld = 13,     //!< NGSのバージョンがクライアントの要求バージョン未満のために使用できない。NGSの更新が必要。
        /*
         Unknown Error
         */
        Unknown = 14,                 //!< 不明なエラー
        /*
         RendezVous : NGS version is new for this client. Please update NEX.
         */
        ClientVersionIsOld = 15,      //!< NEXのバージョンがサーバーの要求バージョン未満のために使用できない。NEXの更新が必要。
        /*
         Account Library Error
         */
        AccountLibraryError = 16,     //!< アカウントライブラリのエラー

        ServiceNoLongerAvailable = 17, //!< サービス終了

        UnknownApplication = 18, //!< アプリケーションが登録されていないため、NGS にアプリケーションの登録が必要。

        ApplicationVersionIsOld = 19, //!< アプリケーションバージョンが NGS に登録されている必須バージョン未満のため、アプリケーションの更新が必要。

        OutOfService = 20, //!< サービス休止中

        NetworkServiceLicenseRequired = 21, //!< ネットワークサービスを利用する権利がない。

        NetworkServiceLicenseSystemError = 22, //!< ネットワークサービスの利用権に関するシステムエラー

        NetworkServiceLicenseError3 = 23,

        NetworkServiceLicenseError4 = 24
    };
    //note: 新規result追加時には、OnlineCore/src/Platform/Core/ResultDefine.hのNEX_ERROR_RESULT_DEFINEの変更も必要
    //Wiki エラーコード追加時に行う作業 を参照
    //アプリケーションがハンドリングするべきエラーを増やした場合には、commonmanualのoverall.rstの「アプリケーションがハンドリングするべきエラー一覧」に追加すること

#ifndef SWIG
    // LobbyApiの互換用
    inline qResult ConvertResultTransportToAuthentication(const qResult& result)
    {
        switch(result.GetReturnCode())
        {
        case QERROR(Transport, DnsError):
            return QRESULT_ERROR(Authentication, HttpDNSError);
        case QERROR(Transport, InvalidUrl):
            return QRESULT_ERROR(Authentication, HttpGetProxySetting);
        case QERROR(Transport, Unknown):
            return QRESULT_ERROR(Authentication, HttpConnectionError);
        default:
            return result;
        }
    }
#endif
#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX

