﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/Authentication/src/Common/AuthenticationInfoDDL.h>
#include <OnlineCore/src/version/nex_version.h>

BEGIN_NAMESPACE_NEX

class AuthenticationInfo: public DDLCLASS(AuthenticationInfo)
{
public:
    enum AuthTokenType
    {
        AUTH_TOKEN_TYPE_NAS = 0,    //!< NAS(CTR向け)
        AUTH_TOKEN_TYPE_NNAS = 1,   //!< NNAS(Cafe向け)
        AUTH_TOKEN_TYPE_AA = 2      //!< AA(NX向け)
        // ここを増やす場合はAuthenticationInfo.cppのダミートークンも増やす必要があります。
    };

#if defined(CTR_TARGET)
    static const qUnsignedInt32 DEFAULT_AUTH_TOKEN_TYPE = AUTH_TOKEN_TYPE_NAS;
#elif defined(CAFE_TARGET)
    static const qUnsignedInt32 DEFAULT_AUTH_TOKEN_TYPE = AUTH_TOKEN_TYPE_NNAS;
#elif defined(NEX_SIGLO_FAMILY)
    static const qUnsignedInt32 DEFAULT_AUTH_TOKEN_TYPE = AUTH_TOKEN_TYPE_AA;
#else
#ifdef NEX_USE_PRINCIPALID_64BIT
        // 64bitPIDのときはAAとみなす
    static const qUnsignedInt32 DEFAULT_AUTH_TOKEN_TYPE = AUTH_TOKEN_TYPE_AA;
#else
        // 32bitPIDのときはCTRと同じとみなす。
    static const qUnsignedInt32 DEFAULT_AUTH_TOKEN_TYPE = AUTH_TOKEN_TYPE_NAS;
#endif
#endif

    AuthenticationInfo()
    {
        Clear();
    }
    virtual ~AuthenticationInfo() {}

    void Clear()
    {
        m_authToken = NEX_T("");
        m_ngsVersion = NEX_VERSION_DDL; // 使用されない
        m_serverVersion = 0;            // 使用されない
        m_authTokenType = DEFAULT_AUTH_TOKEN_TYPE;
    }

    const String& GetAuthToken()
    {
        return m_authToken;
    }
    void SetAuthToken(const String& authToken)
    {
        m_authToken = authToken;
    }
    qByte GetAuthTokenType() const
    {
        return m_authTokenType;
    }
    void SetAuthTokenType(qByte authTokenType)
    {
        m_authTokenType = authTokenType;
    }

    // プラットフォームごとのダミートークンを取得する
    static const qChar* GetDefaultDummyAuthTokenForRvLogin();
private:
    static const qChar* s_dummyAuthTokenForRvLogin;
};

END_NAMESPACE_NEX

