﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

BEGIN_NAMESPACE_NEX

#ifdef SWIG
class DataStoreErrorCode
{
public:
#else
/*!
    @brief DataStore モジュールのエラーコードを定義する名前空間です。
*/
namespace DataStoreErrorCode
{
#endif
    /*!
        @brief    DataStore モジュールのエラーコード。
    */
    enum ErrorCode
    {
        START = 1,
        Unknown                     =  1,  //!<    不明なエラーです。
        InvalidArgument             =  2,  //!<    引数が不正です。
        PermissionDenied            =  3,  //!<    アクセス権限がありません。
        NotFound                    =  4,  //!<    指定したデータIDが見つかりません。
        AlreadyLocked               =  5,  //!<    すでにリードロックされているため参照できません。
        UnderReviewing              =  6,  //!<    審査中のためまだ参照できません。
        Expired                     =  7,  //!<    期限切れです。
        InvalidCheckToken           =  8,  //!<    指定したチェックトークンが不正です。
        SystemFileError             =  9,  //!<    システムファイルの読み込みに失敗しました。
        OverCapacity                = 10,  //!<    一括処理できる数を超えています。
        OperationNotAllowed         = 11,  //!<    要求された操作は許可されていません。
        InvalidPassword             = 12,  //!<    パスワードが間違っています。
        ValueNotEqual               = 13   //!<    値が等しくありません。
    };
    //70-99は、カスタムアプリケーション予約領域です。
    
    //Wiki エラーコード追加時に行う作業 を参照
    //アプリケーションがハンドリングするべきエラーを増やした場合には、commonmanualのoverall.rstの「アプリケーションがハンドリングするべきエラー一覧」に追加すること

#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX

