﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStorePasswordInfoDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      パスワード情報を表すクラスです。
*/
class DataStorePasswordInfo: public DDLCLASS(DataStorePasswordInfo)
{
public:
    /*!
        @brief      コンストラクタです。無効なパスワード情報を作成します。
    */
    DataStorePasswordInfo()
    {
        // ユーザーがこのクラスを生成したときにIsValid()がfalseを返すようにする
        dataId = DataStoreConstants::INVALID_DATAID;
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStorePasswordInfo() {}

    /*!
        @brief      データIDを取得します。

        @return     データID
    */
    qUnsignedInt64 GetDataId() const
    {
        return this->dataId;
    }

    /*!
        @brief      参照パスワードを取得します。

        @return     参照パスワード
    */
    qUnsignedInt64 GetAccessPassword() const
    {
        return this->accessPassword;
    }

    /*!
        @brief      更新パスワードを取得します。

        @return     更新パスワード
    */
    qUnsignedInt64 GetUpdatePassword() const
    {
        return this->updatePassword;
    }

    /*!
        @brief      このオブジェクトが有効かどうかを調べます。

        返値が false だった場合は内部パラメータは無効ですので値を取得しないでください。

        @return     このオブジェクトが有効なら true 、無効なら false 。
    */
    qBool IsValid() const
    {
        return dataId != DataStoreConstants::INVALID_DATAID;
    }

    /*!
        @brief      DataStorePasswordInfoオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, NEX_T("DataStorePasswordInfo: (")
               << NEX_T("dataId=") << dataId
               << NEX_T(", accessPassword=") << accessPassword
               << NEX_T(", updatePassword=") << updatePassword
               << NEX_T(")")
              );
    }
};

END_NAMESPACE_NEX

