﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStorePrepareGetParamV1DDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @internal
    @brief      データのダウンロードに関するパラメータを表すクラスです。
*/
class DataStorePrepareGetParamV1: public DDLCLASS(DataStorePrepareGetParamV1)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStorePrepareGetParamV1()
    {
        SetDataId(DataStoreConstants::INVALID_DATAID_V1);
        //        SetLockId(0);
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStorePrepareGetParamV1() {}

    /*!
        @brief      データIDをセットします。

        取得するデータのデータIDを指定します。本パラメータは必ず指定する必要があります。

        @param[in]  dataId_ 取得するデータのデータID。
    */
    void SetDataId(qUnsignedInt32 dataId_)
    {
        this->dataId = dataId_;
    }

    /*!
        @brief      データIDを取得します。

        @return     データID。
    */
    qUnsignedInt32 GetDataId() const
    {
        return this->dataId;
    }

    /*!
        @internal
        @brief      ロックIDをセットします。

        ダウンロードするデータに対するロックIDを指定します。
        データをアップロードするときに DataStoreConstants::DATA_FLAG_USE_READ_LOCK を指定した場合、
        初めて DataStoreClient::GetObject() で参照されたときにロックIDがサーバーで保存されます。以後はそのロックIDと一致する場合のみダウンロード可能となります。
        例えば誰か一人にデータを渡したいときに利用します。
        本パラメータの指定は必須ではありません。デフォルトは0です。

        @param[in]  lockId データにひも付けるロックID。
    */
    //    void SetLockId(qUnsignedInt32 lockId) { this->lockId = lockId; }

    /*!
        @internal
        @brief      ロックIDを取得します。

        @return     ロックID。
    */
    //    qUnsignedInt32 GetLockId() const { return this->lockId; }
};

END_NAMESPACE_NEX

