﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreRateObjectParamDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      評価に関するパラメータを表すクラスです。
*/
class DataStoreRateObjectParam: public DDLCLASS(DataStoreRateObjectParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStoreRateObjectParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreRateObjectParam() {}

    /*!
        @brief      評価値を指定します。
                    デフォルトは 1 です。

        @param[in]  ratingValue_ 評価値
    */
    void SetRatingValue(qInt32 ratingValue_)
    {
        this->ratingValue = ratingValue_;
    }

    /*!
        @brief      評価値を取得します。

        @return     評価値。
    */
    qInt32 GetRatingValue() const
    {
        return this->ratingValue;
    }

    /*!
        @brief      参照パスワードを設定します。

        参照権限を持たないデータを参照する場合に参照パスワードを指定します。
        データのオーナーから受け取った参照パスワードを指定してください。
        参照権限を持っている場合は参照パスワードは無視され、常に参照可能です。
        DataStoreConstants::INVALID_PASSWORD を指定した場合は参照パスワードでの認証を行いません。
        本関数で指定しない場合は DataStoreConstants::INVALID_PASSWORD が使用されます。

        @param[in]  accessPassword_ 参照パスワード
    */
    void SetAccessPassword(qUnsignedInt64 accessPassword_)
    {
        this->accessPassword = accessPassword_;
    }

    /*!
        @brief      参照パスワードを取得します。

        @return     参照パスワード。
    */
    qUnsignedInt64 GetAccessPassword() const
    {
        return this->accessPassword;
    }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetRatingValue(1);
        SetAccessPassword(DataStoreConstants::INVALID_PASSWORD);
    }

    /*!
        @brief      DataStoreRateObjectParamオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, NEX_T("DataStoreRateObjectParam: (")
               << NEX_T("ratingValue=") << ratingValue
               << NEX_T(", accessPassword=") << accessPassword
               << NEX_T(")")
              );
    }
};

END_NAMESPACE_NEX

