﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreRatingLogDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      評価ログを表すクラスです。
*/
class DataStoreRatingLog: public DDLCLASS(DataStoreRatingLog)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStoreRatingLog()
    {
        this->isRated = false;
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreRatingLog() {}

    /*!
        @brief      自分が過去に評価をしたかを取得します。

        @retval true 評価した。
        @retval false 評価していない。 DataStoreRatingLog::GetRatingValue() で得られる値は有効な値ではありません。
    */
    qBool IsRated() const
    {
        return this->isRated;
    }

    /*!
        @brief      評価ログ取得元のプリンシパル ID を取得します。(常に自分自身のプリンシパル ID となります)

        @return     自分自身のプリンシパル ID 。
    */
    PrincipalID GetPrincipalId() const
    {
        return this->pid;
    }

    /*!
        @brief      自分が前回評価をした際の評価値を取得します。

        DataStoreRatingLog::IsRated() で過去に評価したかどうかを確認後本関数を使用して下さい。

        @return     評価値。過去に評価をしていない場合は有効な値ではありません。
    */
    qInt32 GetRatingValue() const
    {
        return this->ratingValue;
    }

    /*!
        @brief      重複ロックが設定されていた場合に次に評価できるようになるまでの時間を取得します。

        @return     重複ロックが解除されるまでの時間。自分が過去に評価していない場合や重複ロックがされていない場合は DateTime(0) 。
    */
    const DateTime& GetLockExpirationTime() const
    {
        return this->lockExpirationTime;
    }
};

END_NAMESPACE_NEX

