﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreRatingTargetDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      評価対象を表すクラスです。
*/
class DataStoreRatingTarget: public DDLCLASS(DataStoreRatingTarget)
{
public:
    /*!
        @brief      コンストラクタです。

        データIDを DataStoreConstants::INVALID_DATAID 、評価スロットを 0 に初期化します
    */
    DataStoreRatingTarget()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreRatingTarget() {}

    /*!
        @brief      データIDをセットします。

        @param[in]  dataId_ データID
    */
    void SetDataId(qUnsignedInt64 dataId_)
    {
        this->dataId = dataId_;
    }

    /*!
        @brief      データIDを取得します。

        @return     データID。
    */
    qUnsignedInt64 GetDataId() const
    {
        return this->dataId;
    }

    /*!
        @brief      スロットをセットします。

        @param[in]  slot_ スロット番号
    */
    void SetSlot(qInt8 slot_)
    {
        this->slot = slot_;
    }

    /*!
        @brief      スロットを取得します。

        @return     スロット
    */
    qInt8 GetSlot() const
    {
        return this->slot;
    }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetDataId(DataStoreConstants::INVALID_DATAID);
        SetSlot(0);
    }

    /*!
        @brief      DataStoreRatingTargetオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, NEX_T("DataStoreRatingTarget: (")
               << NEX_T("dataId=") << dataId
               << NEX_T(", slot=") << slot
               << NEX_T(")")
              );
    }
};

END_NAMESPACE_NEX

