﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/DataStore/src/Common/DataStoreDDF.h>
#include <RendezVous/Services/Jugem/DataStore/src/Common/DataStoreProtocolDDL.h>
#include <RendezVous/Services/Jugem/DataStore/src/Common/DataStoreConstants.h>
#include <RendezVous/Services/Jugem/DataStore/src/Client/DataStoreLogicServerClientInterface.h>
#include "ServiceHttpClient.h"
#include "DataStoreHppClient.h"

BEGIN_NAMESPACE_NEX

/*!
    @internal
    @brief DataStoreサーバーとの通信を行うためのライブラリです。
*/
class DataStoreLogicServerHttpClient : public ServiceHttpClient, public DataStoreLogicServerClientInterface
{
public:
    /*!
        @internal
        @brief      コンストラクタです。
    */
    DataStoreLogicServerHttpClient();

    /*!
        @internal
        @brief      デストラクタです。
    */
    virtual ~DataStoreLogicServerHttpClient();

    /*!
        @internal
        @brief      ストレージのデータを取得するための情報を取得します。本関数を直接アプリケーションから呼び出さないでください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] pReqGetInfo データ取得用の情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool PrepareGetObject(ProtocolCallContext *pContext, const DataStorePrepareGetParam& param, DataStoreReqGetInfo* pReqGetInfo);

    /*!
        @internal
        @brief      ストレージのデータを取得するための情報を取得します。本関数を直接アプリケーションから呼び出さないでください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] pReqGetInfo データ取得用の情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool PrepareGetObject(ProtocolCallContext *pContext, const DataStorePrepareGetParamV1& param, DataStoreReqGetInfoV1* pReqGetInfo);

    /*!
        @internal
        @brief      ストレージのデータを取得するための情報を取得します。もしくはメタバイナリを取得します。本関数を直接アプリケーションから呼び出さないでください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] pReqGetInfo データ取得用の情報。
        @param[out] pReqGetAdditionalMeta 受信処理用の情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool PrepareGetObjectOrMetaBinary(ProtocolCallContext *pContext, const DataStorePrepareGetParam& param, DataStoreReqGetInfo* pReqGetInfo, DataStoreReqGetAdditionalMeta* pReqGetAdditionalMeta);

    /*!
        @internal
        @brief      ストレージのデータを取得するための情報を取得します。もしくはメタバイナリを取得します。本関数を直接アプリケーションから呼び出さないでください。未実装です。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] pReqGetInfo データ取得用の情報。
        @param[out] pReqGetAdditionalMeta 受信処理用の情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool PrepareGetObjectOrMetaBinary(ProtocolCallContext *pContext, const DataStorePrepareGetParamV1& param, DataStoreReqGetInfoV1* pReqGetInfo, DataStoreReqGetAdditionalMeta* pReqGetAdditionalMeta);

    /*!
        @internal
        @brief      ストレージにデータを保存するための情報を取得します。本関数を直接アプリケーションから呼び出さないでください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] pReqPostInfo データ保存用の情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool PreparePostObject(ProtocolCallContext *pContext, const DataStorePreparePostParam& param, DataStoreReqPostInfo* pReqPostInfo);

    /*!
        @internal
        @brief      ストレージにデータを保存するための情報を取得します。古いインタフェースの互換性維持用。本関数を直接アプリケーションから呼び出さないでください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] pReqPostInfo データ保存用の情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool PreparePostObject(ProtocolCallContext *pContext, const DataStorePreparePostParamV1& param, DataStoreReqPostInfoV1* pReqPostInfo);

    /*!
        @internal
        @brief      ストレージにデータを保存する処理が完了したことを通知します。本関数を直接アプリケーションから呼び出さないでください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool CompletePostObject(ProtocolCallContext *pContext, const DataStoreCompletePostParam& param);

    /*!
        @internal
        @brief      ストレージにデータを保存する処理が完了したことを通知します。古いインタフェースの互換性維持用。本関数を直接アプリケーションから呼び出さないでください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool CompletePostObject(ProtocolCallContext *pContext, const DataStoreCompletePostParamV1& param);

    /*!
        @internal
        @brief      メタ情報のみをストレージにデータを保存する。本関数を直接アプリケーションから呼び出さないでください。
        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] pDataId データID。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool PostMetaBinary(ProtocolCallContext *pContext, const DataStorePreparePostParam& param, qUnsignedInt64* pDataId);

    /*!
        @internal
        @brief      データID指定でメタ情報のみをストレージに保存します。本関数を直接アプリケーションから呼び出さないでください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  dataId データID。
        @param[in]  param 詳細情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool PostMetaBinaryWithDataId(ProtocolCallContext *pContext, const qUnsignedInt64 dataId, const DataStorePreparePostParam& param);

    /*!
        @internal
        @brief      データID指定でメタ情報のみをストレージに保存します。本関数を直接アプリケーションから呼び出さないでください。

        @param[in]   pContext 呼び出し情報。
        @param[in]   dataIds データ ID のベクタ。
        @param[in]   params 詳細情報のベクタ。
        @param[in]   transactional すべて成功する場合のみ処理を行うか (true)、できる限り処理を行うか。処理に失敗したデータについては処理が取り消されます。(false)。
        @param[out]  pResults 成功したかどうかの結果 (省略可)

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool PostMetaBinaryWithDataIds(ProtocolCallContext *pContext, const qVector< qUnsignedInt64 >& dataIds, const DataStorePreparePostParam& params, const qBool transactional = true, qVector<qResult>* pResults = NULL);

    /*!
        @internal
        @brief      データID指定でメタ情報のみをストレージに保存します。本関数を直接アプリケーションから呼び出さないでください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  params data_id と詳細情報のペアのベクタ。
        @param[in]   transactional すべて成功する場合のみ処理を行うか (true)、できる限り処理を行うか。処理に失敗したデータについては処理が取り消されます。(false)。
        @param[out]  pResults 成功したかどうかの結果 (省略可)

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool PostMetaBinariesWithDataId(ProtocolCallContext *pContext, const qVector< std::pair<qUnsignedInt64, DataStorePreparePostParam> >& params, const qBool transactional = true, qVector<qResult>* pResults = NULL);

    /*!
        @internal
        @brief      ストレージのデータを更新するための情報を取得します。本関数を直接アプリケーションから呼び出さないでください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] pReqUpdateInfo データ更新用の情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool PrepareUpdateObject(ProtocolCallContext *pContext, const DataStorePrepareUpdateParam& param, DataStoreReqUpdateInfo* pReqUpdateInfo);

    /*!
        @internal
        @brief      ストレージのデータの内容を変更する処理が完了したことを通知します。本関数を直接アプリケーションから呼び出さないでください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool CompleteUpdateObject(ProtocolCallContext *pContext, const DataStoreCompleteUpdateParam& param);

    /*!
        @brief      完了が保留されたアップロードに対してすべてのアップロード処理が完了したことをサーバーに伝え、そのデータを有効化します。

            DataStoreClient::PostObject() で DataStoreConstants::DATA_FLAG_NEED_COMPLETION を指定してアップロードされたデータに対してのみ本関数の利用が可能です。
            詳細については :ref:`#ref<datastore_atomic_post>` を参照して下さい。
            引数で与えたデータIDのデータをアトミックに有効化します。本関数呼び出し後は指定されたデータがすべて有効化されるか、すべて無効のままとなります。
            一つでも処理に失敗するとその時点で処理をロールバックします。CallContext::GetOutcome() では最初に発生したエラーが取得できます。
            関連するデータを複数アップロードしたい場合などに利用できます。
            DataStoreConstants::DATA_FLAG_NEED_COMPLETION を指定してアップロードされたデータは本関数で有効化されるまで誰からも参照・変更できません。
            もし永続化設定がなされていた場合は本非同期処理が成功したときに設定されます。
            DataStoreClient::PostObject() を開始してから 3 時間以内に本関数で有効化する必要があります。 3 時間以上経過すると自動的にサーバー上のデータが削除されます。
            DataStoreConstants::DATA_FLAG_NEED_COMPLETION を指定していない場合は DataStoreClient::PostObject() の完了と同時にサーバー上のデータが有効化されるため
            本関数の呼び出しは不要です。

        @param[in]  pContext 呼び出し情報。
        @param[in]  dataIds アップロードを完了させるデータIDのリスト。
                    指定できるのは DataStoreConstants::DATA_FLAG_NEED_COMPLETION を指定してアップロードされたデータに限ります。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool CompleteSuspendedPostObject(ProtocolCallContext* pContext, const qVector<qUnsignedInt64>& dataIds);

    /*!
        @brief      ストレージのデータを削除します。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool DeleteObject(ProtocolCallContext *pContext, const DataStoreDeleteParam& param);

    /*!
        @brief      ストレージのメタ情報を変更します。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool ChangeMeta(ProtocolCallContext *pContext, const DataStoreChangeMetaParam& param);

    /*!
       @brief      ストレージのメタ情報を取得します。

            対象のデータの参照権限を持っている人のみが取得できます。
            本関数でタグ、評価情報、メタ情報内のバイナリデータ、権限を持ったプリンシパルIDを取得したい場合は、 DataStoreGetMetaParam::SetResultOption を使用して取得したい情報をセットしてください。
            デフォルトではこれらの情報は取得しません。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] pMetaInfo 取得結果。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetMeta(ProtocolCallContext *pContext, const DataStoreGetMetaParam& param, DataStoreMetaInfo* pMetaInfo);

    /*!
        @brief      ストレージのメタ情報を一括取得します。

            対象のデータの参照権限を持っている人のみが取得できます。
            本関数でタグ、評価情報、メタ情報内のバイナリデータ、権限を持ったプリンシパルIDを取得したい場合は、 DataStoreGetMetaParam::SetResultOption を使用して取得したい情報をセットしてください。
            デフォルトではこれらの情報は取得しません。
            param に設定した条件を用いて dataIds で指定したデータのメタ情報を取得します。
            ただし、DataStoreGetMetaParam::SetDataId() 及び DataStoreGetMetaParam::SetPersistenceTarget() で設定した情報は無視されます。
            一括処理の上限数は100件です。
            pMetaInfos, pResult には dataIds で指定したデータIDに対する処理の結果が指定した数だけ同順で返ります。

        @param[in]  pContext 呼び出し情報。
        @param[in]  dataIds メタ情報を取得するデータのdataId
        @param[in]  param 詳細情報。
        @param[out] pMetaInfos 取得結果。
        @param[out] pResults 成功したかどうかの結果 (省略可)

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetMeta(ProtocolCallContext *pContext, const qVector<qUnsignedInt64>& dataIds, const DataStoreGetMetaParam& param, qVector<DataStoreMetaInfo>* pMetaInfos, qVector<qResult>* pResults = NULL);

    /*!
        @brief      ストレージのメタ情報を一括取得します。

            対象のデータの参照権限を持っている人のみが取得できます。
            本関数でタグ、評価情報、メタ情報内のバイナリデータ、権限を持ったプリンシパルIDを取得したい場合は、 DataStoreGetMetaParam::SetResultOption を使用して取得したい情報をセットしてください。
            デフォルトではこれらの情報は取得しません。
            一括処理の上限数は100件です。
            pMetaInfos, pResult には param に指定されたデータに対する処理の結果が指定した数だけ同順で返ります。

        @param[in]  pContext 呼び出し情報。
        @param[in]  params 詳細情報。
        @param[out] pMetaInfos 取得結果。
        @param[out] pResults 成功したかどうかの結果 (省略可)

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetMeta(ProtocolCallContext *pContext, const qVector<DataStoreGetMetaParam>& params, qVector<DataStoreMetaInfo>* pMetaInfos, qVector<qResult>* pResults = NULL);

    /*!
        @brief      ストレージの情報を検索します

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] pSearchResult 検索結果。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool SearchObject(ProtocolCallContext *pContext, const DataStoreSearchParam& param, DataStoreSearchResult* pSearchResult);

    /*!
        @brief      ストレージの情報を検索します

            本関数を使用する際は、タグ及び検索一致件数の取得を検索条件に指定することはできません。

            また本関数は DataStoreClient::SearchObject() よりも API 呼び出し頻度制限が緩和されています。
            詳細は、
            「CTRガイドライン インターネット通信編」のNEXの項目
            を参照してください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] pSearchResult 検索結果。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool SearchObjectLight(ProtocolCallContext *pContext, const DataStoreSearchParam& param, DataStoreSearchResult* pSearchResult);

    /*!
        @internal
        @brief      通知ファイルのURL情報を取得します。本関数を直接アプリケーションから呼び出さないでください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  info 取得結果。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetNotificationUrl(ProtocolCallContext *pContext, const DataStoreGetNotificationUrlParam& param, DataStoreReqGetNotificationUrlInfo* info);

    /*!
        @internal
        @brief      新着リストを取得します。同時に指定した通知ID以前の通知を削除します。本関数を直接アプリケーションから呼び出さないでください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 取得情報。
        @param[out] pResult 結果。
        @param[out] pHasNext 返却分より後にまだ新着があるか

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetNewArrivedNotifications(ProtocolCallContext *pContext, const DataStoreGetNewArrivedNotificationsParam& param, qVector<DataStoreNotification>* pResult, qBool* pHasNext);

    /*!
        @internal
        @brief      新着リストを取得します。同時に指定した通知ID以前の通知を削除します。古いインタフェースの互換性維持用。本関数を直接アプリケーションから呼び出さないでください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 取得情報。
        @param[out] pResult 結果。
        @param[out] pHasNext 返却分より後にまだ新着があるか

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetNewArrivedNotifications(ProtocolCallContext *pContext, const DataStoreGetNewArrivedNotificationsParam& param, qVector<DataStoreNotificationV1>* pResult, qBool* pHasNext);

    /*!
        @brief      データの評価を行います

            評価を行うには、予め該当のデータスロットが初期化されている必要があります。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] pRating 評価後の評価情報

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool RateObject(ProtocolCallContext* pContext, const DataStoreRatingTarget& target, const DataStoreRateObjectParam& param, DataStoreRatingInfo* pRating);

    /*!
        @brief      データの評価情報を取得します

            データIDとスロットを指定し、1個の評価情報を取得します。

        @param[in]  pContext 呼び出し情報。
        @param[in]  target 評価情報を取得する対象。
        @param[out] pRating 評価情報。
        @param[in]  accessPassword 参照パスワード。デフォルトは DataStoreConstants::INVALID_PASSWORD となっておりパスワード認証を利用しません。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetRating(ProtocolCallContext* pContext, const DataStoreRatingTarget& target, DataStoreRatingInfo* pRating, qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD);

    /*!
        @internal
        @brief      データの評価情報を取得します。本関数を直接アプリケーションから呼び出さないでください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  dataIds 評価情報を取得するデータIDのリスト
        @param[out] pRatings 評価情報
        @param[out] pResults 成功したかどうかの結果 (省略可)

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetRating(ProtocolCallContext* pContext, const qVector<qUnsignedInt64>& dataIds, qVector< qVector< DataStoreRatingInfoWithSlot > > *pRatings,
                            qVector<qResult>* pResults=NULL, qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD);

    /*!
        @brief      データの評価情報をリセットします

        @param[in]  pContext 呼び出し情報。
        @param[in]  target 評価情報をリセットする対象。
        @param[in]  updatePassword 更新パスワード。デフォルトは DataStoreConstants::INVALID_PASSWORD となっておりパスワード認証を利用しません。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool ResetRating(ProtocolCallContext* pContext, const DataStoreRatingTarget& target, qUnsignedInt64 updatePassword=DataStoreConstants::INVALID_PASSWORD);

    /*!
        @brief      データの評価情報と評価ログをリセットします

        @param[in]  pContext 呼び出し情報。
        @param[in]  target 評価情報と評価ログをリセットする対象。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool ResetRating(ProtocolCallContext* pContext, const qUnsignedInt64& dataId);

    /*!
        @brief      データの評価情報と評価ログをリセットします

        @param[in]  pContext 呼び出し情報。
        @param[in]  target 評価情報と評価ログをリセットするデータ ID

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool ResetRating(ProtocolCallContext* pContext, const qVector<qUnsignedInt64>& dataIds, const qBool transactional = true, qVector<qResult>* pResults = NULL);

    /*!
        @internal
        @brief      特定のメタデータを取得します

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[in]  pMetaInfos 取得結果。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetSpecificMeta(ProtocolCallContext* pContext, const DataStoreGetSpecificMetaParam& param, qVector<DataStoreSpecificMetaInfo>* pMetaInfos);

    /*!
        @internal
        @brief      特定のメタデータを取得します。古いインタフェースの互換性維持用。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[in]  pMetaInfos 取得結果。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetSpecificMeta(ProtocolCallContext* pContext, const DataStoreGetSpecificMetaParamV1& param, qVector<DataStoreSpecificMetaInfoV1>* pMetaInfos);

    /*!
        @brief      DataStore サーバーのデータの有効期限を更新します。

            データアップロード時に DataStoreConstants::DATA_FLAG_PERIOD_FROM_LAST_REFERRED を指定している場合、有効期限が更新されます。
            対象のデータの参照権限を持っている人のみが更新できます。

        @param[in]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool TouchObject(ProtocolCallContext *pContext, const DataStoreTouchObjectParam& param);

    /*!
        @internal
        @brief      データの評価ログを取得します。本関数を直接アプリケーションから呼び出さないでください。

            データIDとスロットを指定し、1個の評価情報と評価ログを取得します。

        @param[in]  pContext 呼び出し情報。
        @param[in]  target 評価情報と評価ログを取得する対象。
        @param[out] pRating 評価情報。
        @param[out] pRatingLog 評価ログ
        @param[in]  accessPassword 参照パスワード。デフォルトは DataStoreConstants::INVALID_PASSWORD となっておりパスワード認証を利用しません。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetRatingWithLog(ProtocolCallContext* pContext, const DataStoreRatingTarget& target, DataStoreRatingInfo* pRating, DataStoreRatingLog* pRatingLog, qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD);

    /*!
        @brief      永続化情報を取得します。

            オーナーのプリンシパルIDと永続化スロットIDを指定し、永続化情報を取得します。
            指定した永続化スロットが使用されていない場合やすでに非永続化されていた場合、対象のデータにアクセスする権限が無い場合は非同期処理に失敗します。
            ここで有効なデータIDを取得した場合でも、そのデータにアクセスした時にはすでにデータが削除されている可能性があることにご注意ください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  pid 永続化情報を取得したい対象のプリンシパルID。 INVALID_PRINCIPALID を指定すると自分の情報を取得します。
        @param[in]  persistenceSlotId 永続化スロットID。
        @param[out] pPersistenceInfo 永続化情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetPersistenceInfo(ProtocolCallContext* pContext, PrincipalID pid, qUnsignedInt16 persistenceSlotId, DataStorePersistenceInfo* pPersistenceInfo);

    /*!
        @brief      永続化情報を取得します。

            オーナーのプリンシパルIDと永続化スロットIDを指定し、永続化情報を取得します。
            非同期処理が成功すると、 persistenceSlotIds で指定した個数と順番通りに pPersistenceInfos と pResults に結果が格納されます。
            指定した永続化スロットが使用されていない場合やすでに非永続化されていた場合、対象のデータにアクセスする権限が無い場合は
            pPersistenceInfos に含まれるデータIDが DataStoreConstants::INVALID_DATAID となります。
            ここで有効なデータIDを取得した場合でも、そのデータにアクセスした時にはすでにデータが削除されている可能性があることにご注意ください。

        @param[in]  pContext 呼び出し情報。
        @param[in]  pid 永続化情報を取得したい対象のプリンシパルID。 INVALID_PRINCIPALID を指定すると自分の情報を取得します。
        @param[in]  persistenceSlotIds 永続化スロットIDのリスト。空を指定するとすべてのスロット情報を取得します。
        @param[out] pPersistenceInfos 永続化情報のリスト。
        @param[out] pResults 各スロットIDごとの取得結果。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetPersistenceInfo(ProtocolCallContext* pContext, PrincipalID pid, qVector<qUnsignedInt16> persistenceSlotIds, qVector<DataStorePersistenceInfo>* pPersistenceInfos, qVector<qResult>* pResults);

    /*!
        @brief      データを永続化します。

            データが有効期限により自動的に削除されないようにします。永続化スロットに登録されたデータは有効期限が DataStoreConstants::PERMANENT_DATE_TIME となります。
            もし以前に同じ永続化スロットIDを使っていた場合は、そのデータは非永続化されます。
            その際、 deleteLastObject でそのデータを削除するかどうかを設定します。
            自分がオーナーとなっているデータに対してのみ永続化可能です。
            以前設定した永続化スロットとデータIDが同じ場合は状態は変化せず成功します。
            同じデータIDを複数の永続化スロットに設定することはできません。
            永続化されたデータに対して DataStoreClient::DeleteObject() を呼び出して削除した場合は自動的に非永続化されます。

        @param[in]  pContext 呼び出し情報。
        @param[in]  persistenceSlotId 永続化に使用する永続化スロットID。
        @param[in]  dataId 永続化するデータのデータID。
        @param[in]  deleteLastObject 以前のデータを削除するかどうか。デフォルトでは削除します。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool PerpetuateObject(ProtocolCallContext* pContext, qUnsignedInt16 persistenceSlotId, qUnsignedInt64 dataId, qBool deleteLastObject = true);

    /*!
        @brief      データを非永続化します。

            データが自動的に削除されるように戻します。非永続化されたデータの有効期限が現在時刻プラス有効日数となります。
            deleteLastObject を true にした場合は非永続化すると同時にデータを削除します。

        @param[in]  pContext 呼び出し情報。
        @param[in]  persistenceSlotId 非永続化するデータが使用している永続化スロットID
        @param[in]  deleteLastObject 非永続化したデータを削除するかどうか。デフォルトでは削除しません。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool UnperpetuateObject(ProtocolCallContext* pContext, qUnsignedInt16 persistenceSlotId, qBool deleteLastObject = false);

    /*!
        @brief      パスワード情報を取得します。

            アップロードしたデータに設定されたパスワード情報を取得します。
            自分がオーナーとなっているデータのパスワードのみ取得可能です。

        @param[in]  pContext 呼び出し情報。
        @param[in]  dataId パスワード情報を取得する対象のデータID。
        @param[out]  pPasswordInfo パスワード情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetPasswordInfo(ProtocolCallContext* pContext, qUnsignedInt64 dataId, DataStorePasswordInfo* pPasswordInfo);

    /*!
        @brief      パスワード情報を複数取得します。

            アップロードしたデータに設定されたパスワード情報を取得します。
            自分がオーナーとなっているデータのパスワードのみ取得可能です。

        @param[in]  pContext 呼び出し情報。
        @param[in]  dataId パスワード情報を取得する対象のデータID。
        @param[out] pPasswordInfos パスワード情報。
        @param[out] pResults データIDごとの取得結果。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetPasswordInfo(ProtocolCallContext* pContext, const qVector<qUnsignedInt64>& dataIds, qVector<DataStorePasswordInfo>* pPasswordInfos, qVector<qResult>* pResults);

    /*!
        @brief      ストレージのデータを取得するための情報を一括で取得します。

        @param[in]  pContext 呼び出し情報。
        @param[in]  dataIds 情報を取得するデータIDのリスト
        @param[out] pInfos 取得した情報のリスト
        @param[out] pResults データIDごとの取得結果 (省略可)
        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetObjectInfo(ProtocolCallContext *pContext, const qVector<qUnsignedInt64>& dataIds, qVector<DataStoreReqGetInfo>* pInfos, qVector<qResult>* pResults = NULL);


protected:
    virtual PrincipalID GetMyPrincipalId() const
    {
        return m_PrincipalId;
    }
    // このClientにCredentialsのセットが必要かどうかを返す
    virtual qBool IsCredentialsNeeded() const
    {
        return false;
    };

private:
    DataStoreProtocolClient m_oProtocol;
    DataStoreHppClient m_oHppClient;

};

END_NAMESPACE_NEX

