﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/Debug/src/Common/ApiCallDataDDL.h>

BEGIN_NAMESPACE_NEX

/*!
    @brief      APIの呼び出し情報を格納したクラスです。
*/
#ifdef FOR_DOC_PURPOSE
class ApiCallData
#else
class ApiCallData: public DDLCLASS(ApiCallData)
#endif
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ApiCallData()
    {
        this->principalId = INVALID_PRINCIPALID;
        this->calledTime  = DateTime::Never;
        this->methodName  = "";
    };

    /*!
        @brief      デストラクタです。
    */
    virtual ~ApiCallData() {};

    /*!
        @brief      プリンシパルIDを取得します。

        @return     呼び出したプリンシパルID。
    */
    PrincipalID GetPrincipalId() const
    {
        return this->principalId;
    };

    /*!
        @brief      関数名を取得します。

        @return     呼び出し関数名。
    */
    const String& GetMethodName() const
    {
        return this->methodName;
    };

    /*!
        @brief      時刻を取得します。

        @return     呼び出し時刻。
    */
    DateTime GetCalledTime() const
    {
        return this->calledTime;
    };

};

END_NAMESPACE_NEX

