﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Core/Chrono.h>

#include <nn/account/account_Api.h>
#include <nn/account/account_ApiForApplications.h>

BEGIN_NAMESPACE_NEX

class JobAcquireNsaIdToken : public StepSequenceJob
{
public:
    // コンストラクタです。
    JobAcquireNsaIdToken();

    // デストラクタです。
    virtual ~JobAcquireNsaIdToken();

    qBool Initialize(qUnsignedInt32 uiAssociatedCallID, const nn::account::UserHandle* pUserHandle,
        String* pNsaIdToken, uint64_t* pNsaIdValue, nn::Result* pAccountResult);
private:
    // ゲームの認証処理を行います。
    void StepGetGameAuthentication();

    // ゲームの認証処理を待機します。
    void StepWaitingForGetGameAuthentication();

    // ジョブを完了させる
    void Complete(const qResult& rResult);
private:
    // 関連付いているCallContextのID
    qUnsignedInt32 m_AssociatedCallID;
    // 認証トークン取得用の非同期コンテキスト
    nn::account::AsyncContext m_accountAsyncContext;
    // 認証トークン取得用の非同期コンテキストが有効かを判別するフラグ
    qBool m_isAccountAsyncContextAvailable;
    // 非同期コンテキストの完了シグナルを受け取るシステムイベント
    nn::os::SystemEvent m_systemEvent;
    // nn::account関数呼び出し用のnn::Result
    nn::Result m_accountResult;
    // 処理時間計測用のタイマー
    Chrono m_timer;
    // NsaIdTokenを取得する対象のUserHandle
    const nn::account::UserHandle* m_pUserHandle;
    // 取得したNSA-IDトークンを書き込むポインタ
    String* m_pNsaIdToken;
    // 取得したNSA-IDの値を書き込むポインタ
    uint64_t* m_pNsaIdValue;
    // アカウントのエラー内容を書き込むポインタ
    nn::Result* m_pAccountResult;
};

END_NAMESPACE_NEX

