﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformImpl.h>
#include <OnlineCore/src/Core/Chrono.h>
#include <RendezVous/Services/Jugem/Login/src/Client/JobLogin.h>
#include <RendezVous/Services/Jugem/Login/src/Client/AcquireNsaIdTokenClient.h>

BEGIN_NAMESPACE_NEX

class JobSigloLogin : public JobLogin
{
public:
    // コンストラクタです。
    JobSigloLogin();

    // デストラクタです。
    virtual ~JobSigloLogin();

    void Initialize(qUnsignedInt32 contextId, qUnsignedInt32 gameId, NgsFacade* pClient, const void* pUserHandle, TimeInterval uiTimeout);
    void Initialize(qUnsignedInt32 contextId, qUnsignedInt32 gameId, NgsFacade* pClient, PrincipalID principalId, const void* pNexAuthResultArg, TimeInterval uiTimeout);
private:
    // ゲームの認証処理を行います。
    void StepGetGameAuthentication();

    // ゲームの認証処理を待機します。
    void StepWaitingForGetGameAuthentication();

    // ゲームサーバーにログインします。
    void StepGameLogin();
    // ゲームサーバーへのログイン処理を待機します。
    void StepWaitingForGameLogin();

    // 接続するゲームサーバのURIを生成する
    void CreateGamerServerUri(String* gameServerUri);

private:
    // 処理時間計測用のタイマー
    Chrono m_timer;
    // 取得したNSA-IDトークン
    String m_nsaIdToken;
    // 取得したNSA-IDに対応するプリンシパルID
    PrincipalID m_myPrincipalId;
    // NSA-IDトークンを取得するためのクライアント
    AcquireNsaIdTokenClient m_acquireNsaIdTokenClient;
    // アカウント関係のエラーを入れるResult
    nn::Result m_accountResult;
    // ユーザ情報
    const void* m_pUserHandle;
};

END_NAMESPACE_NEX

