﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "MatchmakeRefereePersonalRoundResult.h"
#include "MatchmakeRefereeEndRoundParamDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      統計データを表すクラスです。
*/
class MatchmakeRefereeEndRoundParam: public DDLCLASS(MatchmakeRefereeEndRoundParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    MatchmakeRefereeEndRoundParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~MatchmakeRefereeEndRoundParam() {}

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetRoundId(0);
        SetPersonalRoundResults(qVector<MatchmakeRefereePersonalRoundResult>());
    }

    /*!
        @brief      ラウンド ID をセットします。

        @param[in] roundId_ ラウンドID。
    */
    void SetRoundId(qUnsignedInt64 roundId_)
    {
        this->roundId = roundId_;
    }

    /*!
        @brief      ラウンド ID を取得します。

        @return ラウンドID。
    */
    qUnsignedInt64 GetRoundId() const
    {
        return this->roundId;
    }

    /*!
        @brief      ラウンド結果をセットします。

        @param[in] personalRoundResults_ ラウンド結果。
    */
    void SetPersonalRoundResults(const qVector<MatchmakeRefereePersonalRoundResult>& personalRoundResults_)
    {
        this->personalRoundResults = personalRoundResults_;
    }

    /*!
        @brief      ラウンド結果を取得します。

        @return ラウンド結果。
    */
    const qVector<MatchmakeRefereePersonalRoundResult>& GetPersonalRoundResults() const
    {
        return this->personalRoundResults;
    }

};

END_NAMESPACE_NEX

