﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringID.h>
#include "MatchmakeRefereeRoundDDL.h"
#include "MatchmakeRefereePersonalRoundResult.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief    ラウンドを表すクラスです。
*/
class MatchmakeRefereeRound: public DDLCLASS(MatchmakeRefereeRound)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    MatchmakeRefereeRound()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~MatchmakeRefereeRound() {}

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        this->roundId = 0;
        this->gid = INVALID_GATHERINGID;
        this->state = 0;
        this->personalDataCategory = 0;
        this->normalizedPersonalRoundResults.clear();
    }

    /*!
        @brief      ラウンド ID を取得します。

        @return     ラウンド ID。
    */
    qUnsignedInt64 GetRoundId() const
    {
        return this->roundId;
    }

    /*!
        @brief      ギャザリング ID を取得します。

        @return     ギャザリング ID。
    */
    GatheringID GetGatheringId() const
    {
        return this->gid;
    }

    /*!
        @brief      ラウンドの状態を取得します。

        @return     MatchmakeRefereeConstants::RoundState で表される状態。
    */
    qUnsignedInt32 GetState() const
    {
        return this->state;
    }

    /*!
        @brief      ラウンドのカテゴリを取得します。

        @return     カテゴリ番号。
    */
    qUnsignedInt32 GetPersonalDataCategory() const
    {
        return this->personalDataCategory;
    }

    /*!
        @brief      正規化済みのレポートを取得します。

        @return     正規化レポート、もしくは、未だ正規化されていない場合は空になります。
    */
    const qVector<MatchmakeRefereePersonalRoundResult>& GetNormalizedRoundResult() const
    {
        return this->normalizedPersonalRoundResults;
    }


    /*!
        @brief      MatchmakeRefereeRoundオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
        @param[in]  bShowBufferContents MatchmakeRefereePersonalRoundResult の buffer の中身も表示するか。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, qBool bShowBufferContents=false) const
    {
        QTRACE(uiTraceFlags, NEX_T("MatchmakeRefereeRound: (")
                            << NEX_T("roundId=") << this->roundId
                            << NEX_T(", gid=") << this->gid
                            << NEX_T(", state=") << state
                            << NEX_T(", personalDataCategory=") << this->personalDataCategory
                            << NEX_T(", normalizedPersonalRoundResults size=") << this->normalizedPersonalRoundResults.size()
                            << NEX_T(", normalizedPersonalRoundResults contents="));
        for (qVector<MatchmakeRefereePersonalRoundResult>::const_iterator it = this->normalizedPersonalRoundResults.begin(); 
             it != this->normalizedPersonalRoundResults.end(); ++it)
        {
            it->Trace(uiTraceFlags, bShowBufferContents);
        }
    }
};

END_NAMESPACE_NEX

