﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringID.h>
#include "MatchmakeRefereeStartRoundParamDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      MatchmakeReferee::StartRound() で指定するラウンドの初期パラメータを表すクラスです。
*/
class MatchmakeRefereeStartRoundParam: public DDLCLASS(MatchmakeRefereeStartRoundParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    MatchmakeRefereeStartRoundParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~MatchmakeRefereeStartRoundParam() {}

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetPersonalDataCategory(0);
        SetGatheringId(INVALID_GATHERINGID);
        SetPrincipalIds(qVector<PrincipalID>());
    }

    /*!
        @brief      カテゴリ番号をセットします。

        個人統計の分類に使用されるカテゴリ番号を指定します。

        @param[in]  personalDataCategory_ 個人統計の分類に使用されるカテゴリ番号。
    */
    void SetPersonalDataCategory(qUnsignedInt32 personalDataCategory_)
    {
        this->personalDataCategory = personalDataCategory_;
    }

    /*!
        @brief      カテゴリ番号を取得します。

        @return 個人統計の分類に使用されるカテゴリ番号。
    */
    qUnsignedInt32 GetPersonalDataCategory() const
    {
        return this->personalDataCategory;
    }

    /*!
        @brief      ギャザリング ID をセットします。

        @param[in]  gid_ ラウンドの参加者が参加しているギャザリングのギャザリング ID
    */
    void SetGatheringId(GatheringID gid_)
    {
        this->gid = gid_;
    }

    /*!
        @brief      ギャザリング ID を取得します。

        @return ギャザリング ID。
    */
    GatheringID GetGatheringId() const
    {
        return this->gid;
    }

    /*!
        @brief      ラウンド参加者をセットします。

        @param[in]  pids_ ラウンドの参加者に指定するプリンシパル ID
    */
    void SetPrincipalIds(const qVector<PrincipalID>& pids_)
    {
        this->pids = pids_;
    }

    /*!
        @brief      ラウンド参加者を取得します。

        @return プリンシパル ID。
    */
    const qVector<PrincipalID>& GetPrincipalIds() const
    {
        return this->pids;
    }

    /*!
        @brief      MatchmakeRefereeStartRoundParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        StringStream ss;
        ss << NEX_T("MatchmakeRefereeStartRoundParam: (")
           << NEX_T("personalDataCategory=") << this->personalDataCategory
           << NEX_T(", gid=") << this->gid;
        ss << NEX_T(", pids=(");
        for (qVector<PrincipalID>::const_iterator it = this->pids.begin(); it != this->pids.end(); ++it)
        {
            ss << *it << NEX_T(", ");
        }
        ss << NEX_T("))");
        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX

