﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

/*!
  @brief Ranking に関する定数が定義された名前空間です。
 */
namespace RankingConstants
{
/*!
    @brief ランキング共通データの最大サイズです。
*/
static const qByte MAX_COMMON_DATA_SIZE = 255;

/*!
    @brief 範囲ランキングで取得可能な順位の範囲の最大値です。
*/
static const qUnsignedInt32 MAX_RANGE_RANKING_ORDER = 1000;

/*!
    @brief 正確な順位計算が行われる順位の最大値です。

    RankingClient::GetRanking() （周辺ランキング、個人のランキングデータ取得時）や
    RankingClient::GetApproxOrder() で得られる順位がこの値よりも小さい範囲においては、
    リアルタイムで正確な順位計算が行われます。@n

    この値よりも大きい範囲では、取得タイミングによって微妙な誤差が生じる可能性があります。
*/
static const qUnsignedInt32 MAX_ACCURATE_ORDER = 5000;

/*!
    @brief ランキングの取得モードです。
*/
enum RankingMode
{
    RANKING_MODE_RANGE        = 0, //!< 範囲ランキング。
    RANKING_MODE_NEAR         = 1, //!< 周辺ランキング。
    RANKING_MODE_FRIEND_RANGE = 2, //!< フレンドランキング。
    RANKING_MODE_FRIEND_NEAR  = 3, //!< フレンド周辺ランキング。
    RANKING_MODE_USER         = 4  //!< 個人のランキングデータ取得。
};

/*!
    @brief フィルタリングするグループのインデックスを指定する定数です。
*/
enum FilterGroupIndex
{
    FILTER_GROUP_INDEX_0 = 0,     //!< グループ0の値を指定してフィルタします。
    FILTER_GROUP_INDEX_1 = 1,     //!< グループ1の値を指定してフィルタします。
    FILTER_GROUP_INDEX_2 = 2,     //!< 使用しません。
    FILTER_GROUP_INDEX_3 = 3,     //!< 使用しません。
    FILTER_GROUP_INDEX_NONE = 255 //!< グループの値によるフィルタを行いません。
};

/*!
    @brief ランキングのソート順です。
*/
enum OrderBy
{
    ORDER_BY_ASC  = 0, //!< 昇順。
    ORDER_BY_DESC = 1  //!< 降順。
};

/*!
    @brief スコアアップロード時に指定するスコアの更新モードです。
*/
enum UpdateMode
{
    UPDATE_MODE_NORMAL  = 0,    //!< 対象カテゴリの過去のスコアよりも良いスコアの場合にスコアを登録します。
    UPDATE_MODE_DELETE_OLD = 1  //!< 対象カテゴリの過去のスコアを削除した上で新しいスコアを登録します。スコアの良し悪しに関わらず常に最新のスコアを利用する場合に使用します。
};

/*!
    @brief 順位の計算方法です。
*/
enum OrderCalculation
{
    ORDER_CALCULATION_113 = 0, //!< 同点が複数ある場合に、同じ順位とします（例： 1 位、 1 位、 3 位）。
    ORDER_CALCULATION_123 = 1  //!< 同点が複数ある場合に、更新時刻（秒単位）の古い方を上位とします。更新時刻も同じ場合は、プリンシパル ID の小さい方を上位とします。
};

/*!
    @brief ランキングの時間範囲を指定する定数です。
*/
enum TimeScope
{
    TIME_SCOPE_CUSTOM_0 = 0,  //!< NMAS上で設定した範囲。（デフォルトは 無効 です）
    TIME_SCOPE_CUSTOM_1 = 1,  //!< NMAS上で設定した範囲。（デフォルトは 無効 です）
    TIME_SCOPE_ALL = 2        //!< 全ての範囲。
};

/*!
    @brief 変更するスコアの属性を設定するフラグです。
*/
enum ModificationFlag
{
    MODIFICATION_FLAG_NONE   = 0,      //!< 使用しません。
    MODIFICATION_FLAG_GROUP0 = 1 << 0, //!< グループ0を変更します。
    MODIFICATION_FLAG_GROUP1 = 1 << 1, //!< グループ1を変更します。
    MODIFICATION_FLAG_GROUP2 = 1 << 2, //!< 使用しません。
    MODIFICATION_FLAG_GROUP3 = 1 << 3, //!< 使用しません。
    MODIFICATION_FLAG_PARAM  = 1 << 4  //!< パラメータを変更します。
};

/*!
    @brief 取得する統計値を指定するフラグです。
*/
enum StatsFlag
{
    STATS_FLAG_TOTAL   = 1 << 0, //!< スコアの総数
    STATS_FLAG_SUM     = 1 << 1, //!< スコアの合計値
    STATS_FLAG_MIN     = 1 << 2, //!< スコアの最小値
    STATS_FLAG_MAX     = 1 << 3, //!< スコアの最大値
    STATS_FLAG_AVERAGE = 1 << 4  //!< スコアの平均値
};
}

END_NAMESPACE_NEX

