﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

BEGIN_NAMESPACE_NEX

//group=Error Code Enumerations

/*
Description: This enumeration defines the error codes that may be returned for the Ranking facility.
*/
namespace RankingSuccessCode
{
//group= Type Definitions

/*
Description: Enumerates the error codes for the Ranking facility that a call may return.
*/
enum SuccessCode
{
    /*
    */
    Success = 1
};
}

//group=Error Code Enumerations

/*!
    @brief    Ranking モジュールのエラーコードを定義する名前空間です。
*/
#ifdef SWIG
class RankingErrorCode
{
public:
#else
namespace RankingErrorCode
{
#endif
    //group= Type Definitions

    /*
    Description: Enumerates the error codes for the Ranking facility that a call may return.
    */

    /*!
        @brief ランキングのエラーコード

        以下のエラーは、サーバー側に問題のあるエラーですので、エラーを表示して下さい。

        @li QERROR(Ranking, RegistrationError)
        @li QERROR(Ranking, Unknown)

        なお、以下のエラーはライブラリ開発用に使用されており、実際には発生しません。

        @li QERROR(Ranking, NotImplemented)

        残りのエラーについては、アプリケーション側で適切なハンドリングを行い、エラーコードおよびエラーコードに対応したメッセージ表示はしないで下さい。
    */
    enum ErrorCode
    {
        START = 1,
        /*
        */
        NotInitialized              = 1,  //!<    初期化されていません。

        /*
        */
        InvalidArgument             = 2,  //!<    引数が不正です。

        /*
        */
        RegistrationError           = 3,  //!<    登録処理が失敗しました。

        /*
        */
        //NotActivated                = 4,  //!<    アクティベートされていません。

        /*
        */
        NotFound                    = 5,  //!<    データは存在しません。

        /*
        */
        InvalidScore                = 6,  //!<    スコアが不正です。

        /*
        */
        InvalidDataSize             = 7,  //!<    データサイズが不正です。

        /*
        */
        //AccessLimitReached          = 8,  //!<    アクセス制限に引っ掛かりました。

        /*
        */
        PermissionDenied            = 9,  //!<    アクセス権がありません。

        /*
        */
        Unknown                     = 10, //!<    不明なエラーです。

        /*
        */
        NotImplemented              = 11  //!<    実装されていない機能を呼び出そうとしました。
    };

    //note: 新規result追加時には、OnlineCore/src/Platform/Core/ResulDefine.hのNEX_ERROR_RESULT_DEFINEの変更も必要
    //Wiki エラーコード追加時に行う作業 を参照
    //アプリケーションがハンドリングするべきエラーを増やした場合には、commonmanualのoverall.rstの「アプリケーションがハンドリングするべきエラー一覧」に追加すること

#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX

