﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "Ranking2CategorySettingDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      カテゴリの設定値を表すクラスです。
*/
class Ranking2CategorySetting: public DDLCLASS(Ranking2CategorySetting)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    Ranking2CategorySetting()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~Ranking2CategorySetting() {}

    /*!
        @brief      有効スコアの最小値を取得します。

        @return     有効スコアの最小値。
    */
    qUnsignedInt32 GetMinScore() const
    {
        return minScore;
    };

    /*!
        @brief      有効スコアの最大値を取得します。

        @return     有効スコアの最大値。
    */
    qUnsignedInt32 GetMaxScore() const
    {
        return maxScore;
    };

    /*!
        @brief      スコアの並び順を取得します。

        @return     スコアの並び順。
                    Ranking2Constants::SCORE_ORDER_ASC または
                    Ranking2Constants::SCORE_ORDER_DESC 。
    */
    qBool GetScoreOrder() const
    {
        return scoreOrder;
    };

    /*!
        @brief      ランキングの最低順位を取得します。

        @return     ランキングの最低順位。
    */
    qUnsignedInt32 GetLowestRank() const
    {
        return lowestRank;
    };

    /*!
        @brief      過去シーズンの保存数を取得します。

        @return     過去シーズン保存数。
    */
    qByte GetMaxSeasonsToGoBack() const
    {
        return maxSeasonsToGoBack;
    };

    /*!
        @brief      定期リセットのモードを取得します。

        @return     定期リセットのモード。Ranking2ResetMode の値。

        @see        Ranking2ResetMode
    */
    qByte GetResetMode() const
    {
        return resetMode;
    };

    /*!
        @brief      定期リセットの時刻（時）を取得します。

        @return     定期リセットの時刻（時）。UTC。
    */
    qByte GetResetHour() const
    {
        return resetHour;
    };

    /*!
        @brief      定期リセットの日にちまたは曜日を取得します。

        @return     定期リセットの日にちまたは曜日。UTC。
    */
    qByte GetResetDay() const
    {
        return resetDay;
    };

    /*!
        @brief      定期リセットの月を取得します。

        @return     各月を 1 ビットで表し、下位から 1 月, 2 月, …とする全 12 ビットの値。UTC。
                    ビットの立っている月で定期リセットを実行します。
    */
    qUnsignedInt16 GetResetMonth() const
    {
        return resetMonth;
    };

    /*!
        @brief      Ranking2CategorySetting オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const Ranking2CategorySetting& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const Ranking2CategorySetting& rhs)
{
    oStr << NEX_T("Ranking2CategorySetting: (")
               << NEX_T("minScore=") << rhs.minScore
               << NEX_T(", maxScore=") << rhs.maxScore
               << NEX_T(", lowestRank=") << rhs.lowestRank
               << NEX_T(", resetMonth=") << rhs.resetMonth
               << NEX_T(", resetDay=") << rhs.resetDay
               << NEX_T(", resetHour=") << rhs.resetHour
               << NEX_T(", resetMode=") << rhs.resetMode
               << NEX_T(", maxSeasonsToGoBack=") << rhs.maxSeasonsToGoBack
               << NEX_T(", scoreOrder=") << rhs.scoreOrder;
    oStr << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

