﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include "Ranking2ChartInfoInputDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      取得対象のランキングチャートデータを設定するクラスです。
*/
class Ranking2ChartInfoInput: public DDLCLASS(Ranking2ChartInfoInput)
{
public:
    /*!
        @brief      コンストラクタです。

        すべて 0 で初期化されます。
    */
    Ranking2ChartInfoInput()
    {
        chartIndex = 0;
        numSeasonsToGoBack = 0;
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~Ranking2ChartInfoInput() {}

    /*!
        @brief ランキングチャートデータのインデックス番号を設定します。

        @param c ランキングチャートデータのインデックス番号
     */
    void SetChartIndex(qUnsignedInt32 c)
    {
        chartIndex = c;
    }
    /*!
        @brief ランキングチャートデータのインデックス番号を取得します。

        初期値は、0です。

        @return ランキングチャートデータのインデックス番号
     */
    qUnsignedInt32 GetChartIndex() const
    {
        return chartIndex;
    }

    /*!
        @brief さかのぼるシーズンの数を設定します。

        @param n さかのぼるシーズンの数
     */
    void SetNumSeasonsToGoBack(qByte n)
    {
        numSeasonsToGoBack = n;
    }
    /*!
        @brief さかのぼるシーズンの数を取得します。

        初期値は、0です。

        @return さかのぼるシーズンの数
     */
    qByte GetNumSeasonsToGoBack() const
    {
        return numSeasonsToGoBack;
    }

    
    /*!
        @brief      Ranking2ChartInfoInput オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const Ranking2ChartInfoInput& rch);
};

inline StringStream& operator<<(StringStream& oStr, const Ranking2ChartInfoInput& rcii)
{
    oStr << NEX_T("Ranking2ChartInfoInput: (")
         << NEX_T("chartIndex=") << rcii.chartIndex
         << NEX_T(" numSeasonsToGoBack=") << rcii.numSeasonsToGoBack
         << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

