﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "Ranking2EstimateScoreRankOutputDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      スコアのランク推定用の出力値クラスです。
*/
class Ranking2EstimateScoreRankOutput: public DDLCLASS(Ranking2EstimateScoreRankOutput)
{
public:
    /*!
        @brief 順位を取得します。

        @return 順位
    */
    qUnsignedInt32 GetRank() const
    {
        return rank;
    }

    /*!
        @brief スコアを取得します。

        @return スコア
    */
    qUnsignedInt32 GetScore() const
    {
        return score;
    }

    /*!
        @brief カテゴリを取得します。

        @return カテゴリ
    */
    qUnsignedInt32 GetCategory() const
    {
        return category;
    }

    /*!
        @brief シーズンを取得します。

        @return シーズン
    */
    qUnsignedInt GetSeason() const
    {
        return season;
    }
    /*!
        @brief 標本比率を取得します。

        実際の標本比率は、(1/ (1 << GetSamplingRate()) となり、
        母集団 * (1 / (1 << GetSamplingRate())) の分の標本データが推定に用いられていることを示します。
        0 が返された場合は、全てのスコアが標本データとなっている事を示します。
        1 が返された場合は、約 50% のスコアが標本データとなっている事を示します。
        2 が返された場合は、約 25% のスコアが標本データとなっている事を示します。

        @return 標本比率の逆数のビットシフト前の値
    */
    qUnsignedInt GetSamplingRate() const
    {
        return static_cast<qUnsignedInt>(samplingRate);
    }

    /*!
        @brief 推定スコア総数を取得します。

        アップロードされたスコアの推定総数です。

        @return 推定スコア総数
    */
    qUnsignedInt32 GetLength() const
    {
        return length;
    }


    /*!
        @brief      Ranking2CategorySetting オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
    @internal
    @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const Ranking2EstimateScoreRankOutput& resro);

    /*!
        @brief デバッグ用にオブジェクトの内容を文字列として出力します。

        @return デバッグ用の出力文字列
    */
    String ToString() const
    {
        StringStream ss;
        ss << *this;
        return String(ss.CStr());
    }
};



inline StringStream& operator<<(StringStream& oStr, const Ranking2EstimateScoreRankOutput& resro)
{
    oStr << NEX_T("Ranking2EstimateScoreRankOutput: (")
        << NEX_T("category=") << resro.category
        << NEX_T(", season=") << resro.season
        << NEX_T(", length=") << resro.length
        << NEX_T(", score=") << resro.score
        << NEX_T(", rank=") << resro.rank
        << NEX_T(", samplingRate=") << static_cast<qInt>(resro.samplingRate)
        << NEX_T(")");
    return oStr;
}


END_NAMESPACE_NEX

