﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "Ranking2InfoDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      ランキングの取得結果を表すクラスです。
*/
class Ranking2Info: public DDLCLASS(Ranking2Info)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    Ranking2Info()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~Ranking2Info() {}

    /*!
        @brief      ランクデータのリストを取得します。

        @return     ランクデータのリスト。
    */
    const qVector<Ranking2RankData>& GetRankDataList() const
    {
        return rankDataList;
    };

    /*!
        @brief      設定上の最低順位を取得します。

        @return     設定上の最低順位。同順位が並んでいると、最下位がこの順位にならないことがあります。
    */
    qUnsignedInt32 GetLowestRank() const
    {
        return lowestRank;
    };

    /*!
        @brief      ランク内人数を取得します。

        @return     ランク内人数の値。
    */
    qUnsignedInt32 GetNumRankedIn() const
    {
        return numRankedIn;
    };

    /*!
        @brief      シーズンを取得します。

        @return     シーズン番号。0 から始まり、0x7FFFFFFF で止まります。
    */
    qInt32 GetSeason() const
    {
        return season;
    };

    /*!
        @brief      Ranking2Info オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const Ranking2Info& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const Ranking2Info& rhs)
{
    oStr << NEX_T("Ranking2Info: (")
         << NEX_T("lowestRank=") << rhs.lowestRank
         << NEX_T(", numRankedIn=") << rhs.numRankedIn
         << NEX_T(", season=") << rhs.season
         << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

