﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/ServiceItem/src/Client/ServiceItemInternalClient.h>

BEGIN_NAMESPACE_NEX

/*!
    @internal
    @brief      非同期RMCの種類です。
*/
enum SERVICEITEM_ASYNCHRONOUS_RMC
{
    SERVICEITEM_ASYNCHRONOUS_RMC_HTTP_GET,
    SERVICEITEM_ASYNCHRONOUS_RMC_PURCHASE_SERVICE_ITEM,
    SERVICEITEM_ASYNCHRONOUS_RMC_LIST_SERVICE_ITEM,
    SERVICEITEM_ASYNCHRONOUS_RMC_GET_BALANCE,
    SERVICEITEM_ASYNCHRONOUS_RMC_GET_PREPURCHASE_INFO,
    SERVICEITEM_ASYNCHRONOUS_RMC_GET_SERVICE_ITEM_RIGHT,
    SERVICEITEM_ASYNCHRONOUS_RMC_GET_SERVICE_ITEM_RIGHT_STATEMENT,
    SERVICEITEM_ASYNCHRONOUS_RMC_GET_PURCHASE_HISTORY,
    SERVICEITEM_ASYNCHRONOUS_RMC_GET_LAW_MESSAGE,
    SERVICEITEM_ASYNCHRONOUS_RMC_LIST_AUTO_BILLING_SERVICE_ITEM,
    SERVICEITEM_ASYNCHRONOUS_RMC_GET_AUTO_BILLING_CONTRACT,
    SERVICEITEM_ASYNCHRONOUS_RMC_CANCEL_AUTO_BILLING,
    //SERVICEITEM_ASYNCHRONOUS_RMC_POST_RIGHT_BINARY_BY_ACCOUNT,
    SERVICEITEM_ASYNCHRONOUS_RMC_USE_SERVICE_ITEM_BY_ACCOUNT,
    SERVICEITEM_ASYNCHRONOUS_RMC_LIST_REDEMPTION_SERVICE_ITEM,
    SERVICEITEM_ASYNCHRONOUS_RMC_REDEEM_SERVICE_ITEM
};

/*!
    @internal
    @brief      非同期RMCの特性を表すクラスです。
*/
template<SERVICEITEM_ASYNCHRONOUS_RMC>
struct ServiceItemAsynchronousRmcTraits;

// SERVICEITEM_ASYNCHRONOUS_RMC_HTTP_GETで利用
class ServiceItemHttpGetParam;
class ServiceItemHttpGetResponse;

// SERVICEITEM_ASYNCHRONOUS_RMC_PURCHASE_SERVICE_ITEMで利用
class ServiceItemPurchaseServiceItemParam;
class ServiceItemPurchaseServiceItemResponse;

// SERVICEITEM_ASYNCHRONOUS_RMC_LIST_SERVICE_ITEMで利用
class ServiceItemListServiceItemParam;
class ServiceItemListServiceItemResponse;

// SERVICEITEM_ASYNCHRONOUS_RMC_GET_BALANCEで利用
class ServiceItemGetBalanceParam;
class ServiceItemGetBalanceResponse;

// SERVICEITEM_ASYNCHRONOUS_RMC_GET_PREPURCHASE_INFOで利用
class ServiceItemGetPrepurchaseInfoParam;
class ServiceItemGetPrepurchaseInfoResponse;

// SERVICEITEM_ASYNCHRONOUS_RMC_GET_SERVICE_ITEM_RIGHT, SERVICEITEM_ASYNCHRONOUS_RMC_GET_SERVICE_ITEM_RIGHT_STATEMENTで利用
class ServiceItemGetServiceItemRightParam;
class ServiceItemGetServiceItemRightResponse;

// SERVICEITEM_ASYNCHRONOUS_RMC_GET_PURCHASE_HISTORYで利用
class ServiceItemGetPurchaseHistoryParam;
class ServiceItemGetPurchaseHistoryResponse;

// SERVICEITEM_ASYNCHRONOUS_RMC_GET_LAW_MESSAGEで利用
class ServiceItemGetLawMesssageParam;
class ServiceItemGetLawMesssageResponse;

// SERVICEITEM_ASYNCHRONOUS_RMC_LIST_AUTO_BILLING_SERVICE_ITEM で利用
class ServiceItemListAutoBillingServiceItemParam;
class ServiceItemListAutoBillingServiceItemResponse;

// SERVICEITEM_ASYNCHRONOUS_RMC_GET_AUTO_BILLING_SERVICE_ITEM_RIGHT で利用
class ServiceItemGetAutoBillingContractParam;
class ServiceItemGetAutoBillingContractResponse;

// SERVICEITEM_ASYNCHRONOUS_RMC_CANCEL_AUTO_BILLING で利用
class ServiceItemCancelAutoBillingParam;
class ServiceItemCancelAutoBillingResponse;

// SERVICEITEM_ASYNCHRONOUS_RMC_POST_RIGHT_BINARY_BY_ACCOUNTで使用
//class ServiceItemPostRightBinaryByAccountParam;
//class ServiceItemPostRightBinaryResponse;

// SERVICEITEM_ASYNCHRONOUS_RMC_USE_SERVICE_ITEM_BY_ACCOUNTで使用
class ServiceItemUseServiceItemByAccountParam;
class ServiceItemUseServiceItemResponse;

// SERVICEITEM_ASYNCHRONOUS_RMC_LIST_REDEMPTION_SERVICE_ITEMで使用
class ServiceItemListRedemptionServiceItemParam;
class ServiceItemListRedemptionServiceItemResponse;

// SERVICEITEM_ASYNCHRONOUS_RMC_REDEEM_SERVICE_ITEMで使用
class ServiceItemRedeemServiceItemParam;
class ServiceItemRedeemServiceItemResponse;

/*!
    @internal
    @brief      非同期RMCの特性を表すクラスです。SERVICEITEM_ASYNCHRONOUS_RMC_HTTP_GETの特殊化です。
*/
template<>
struct ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_HTTP_GET>
{
    // リクエストパラメータの型
    typedef ServiceItemHttpGetParam TRequest;

    // レスポンスパラメータの型
    typedef ServiceItemHttpGetResponse TResponse;

    // 非同期RMCで使用するリクエストとレスポンスの関数型
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousRequest)(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const TRequest& request);
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousResponse)(ProtocolCallContext *pContext, qUnsignedInt32 requestId, TResponse* response);

    // 非同期RMCで使用するリクエストとレスポンスの関数ポインタ
    static AsynchronousRequest pAsynchronousRequest;
    static AsynchronousResponse pAsynchronousResponse;
};

/*!
    @internal
    @brief      非同期RMCの特性を表すクラスです。SERVICEITEM_ASYNCHRONOUS_RMC_PURCHASE_SERVICE_ITEMの特殊化です。
*/
template<>
struct ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_PURCHASE_SERVICE_ITEM>
{
    // リクエストパラメータの型
    typedef ServiceItemPurchaseServiceItemParam TRequest;

    // レスポンスパラメータの型
    typedef ServiceItemPurchaseServiceItemResponse TResponse;

    // 非同期RMCで使用するリクエストとレスポンスの関数型
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousRequest)(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const TRequest& request);
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousResponse)(ProtocolCallContext *pContext, qUnsignedInt32 requestId, TResponse* response);

    // 非同期RMCで使用するリクエストとレスポンスの関数ポインタ
    static AsynchronousRequest pAsynchronousRequest;
    static AsynchronousResponse pAsynchronousResponse;
};

/*!
    @internal
    @brief      非同期RMCの特性を表すクラスです。SERVICEITEM_ASYNCHRONOUS_RMC_LIST_SERVICE_ITEMの特殊化です。
*/
template<>
struct ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_LIST_SERVICE_ITEM>
{
    // リクエストパラメータの型
    typedef ServiceItemListServiceItemParam TRequest;

    // レスポンスパラメータの型
    typedef ServiceItemListServiceItemResponse TResponse;

    // 非同期RMCで使用するリクエストとレスポンスの関数型
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousRequest)(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const TRequest& request);
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousResponse)(ProtocolCallContext *pContext, qUnsignedInt32 requestId, TResponse* response);

    // 非同期RMCで使用するリクエストとレスポンスの関数ポインタ
    static AsynchronousRequest pAsynchronousRequest;
    static AsynchronousResponse pAsynchronousResponse;
};

/*!
    @internal
    @brief      非同期RMCの特性を表すクラスです。SERVICEITEM_ASYNCHRONOUS_RMC_GET_BALANCEの特殊化です。
*/
template<>
struct ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_GET_BALANCE>
{
    // リクエストパラメータの型
    typedef ServiceItemGetBalanceParam TRequest;

    // レスポンスパラメータの型
    typedef ServiceItemGetBalanceResponse TResponse;

    // 非同期RMCで使用するリクエストとレスポンスの関数型
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousRequest)(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const TRequest& request);
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousResponse)(ProtocolCallContext *pContext, qUnsignedInt32 requestId, TResponse* response);

    // 非同期RMCで使用するリクエストとレスポンスの関数ポインタ
    static AsynchronousRequest pAsynchronousRequest;
    static AsynchronousResponse pAsynchronousResponse;
};

/*!
    @internal
    @brief      非同期RMCの特性を表すクラスです。SERVICEITEM_ASYNCHRONOUS_RMC_GET_PREPURCHASE_INFOの特殊化です。
*/
template<>
struct ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_GET_PREPURCHASE_INFO>
{
    // リクエストパラメータの型
    typedef ServiceItemGetPrepurchaseInfoParam TRequest;

    // レスポンスパラメータの型
    typedef ServiceItemGetPrepurchaseInfoResponse TResponse;

    // 非同期RMCで使用するリクエストとレスポンスの関数型
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousRequest)(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const TRequest& request);
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousResponse)(ProtocolCallContext *pContext, qUnsignedInt32 requestId, TResponse* response);

    // 非同期RMCで使用するリクエストとレスポンスの関数ポインタ
    static AsynchronousRequest pAsynchronousRequest;
    static AsynchronousResponse pAsynchronousResponse;
};

/*!
    @internal
    @brief      非同期RMCの特性を表すクラスです。SERVICEITEM_ASYNCHRONOUS_RMC_GET_SERVICE_ITEM_RIGHTの特殊化です。
*/
template<>
struct ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_GET_SERVICE_ITEM_RIGHT>
{
    // リクエストパラメータの型
    typedef ServiceItemGetServiceItemRightParam TRequest;

    // レスポンスパラメータの型
    typedef ServiceItemGetServiceItemRightResponse TResponse;

    // 非同期RMCで使用するリクエストとレスポンスの関数型
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousRequest)(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const TRequest& request);
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousResponse)(ProtocolCallContext *pContext, qUnsignedInt32 requestId, TResponse* response);

    // 非同期RMCで使用するリクエストとレスポンスの関数ポインタ
    static AsynchronousRequest pAsynchronousRequest;
    static AsynchronousResponse pAsynchronousResponse;
};

/*!
  @internal
  @brief      非同期RMCの特性を表すクラスです。SERVICEITEM_ASYNCHRONOUS_RMC_GET_SERVICE_ITEM_RIGHT_STATEMENTの特殊化です。
*/
template<>
struct ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_GET_SERVICE_ITEM_RIGHT_STATEMENT>
{
    // リクエストパラメータの型
    typedef ServiceItemGetServiceItemRightParam TRequest;

    // レスポンスパラメータの型
    typedef ServiceItemGetServiceItemRightResponse TResponse;

    // 非同期RMCで使用するリクエストとレスポンスの関数型
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousRequest)(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const TRequest& request);
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousResponse)(ProtocolCallContext *pContext, qUnsignedInt32 requestId, TResponse* response);

    // 非同期RMCで使用するリクエストとレスポンスの関数ポインタ
    static AsynchronousRequest pAsynchronousRequest;
    static AsynchronousResponse pAsynchronousResponse;
};

/*!
    @internal
    @brief      非同期RMCの特性を表すクラスです。SERVICEITEM_ASYNCHRONOUS_RMC_GET_PURCHASE_HISTORYの特殊化です。
*/
template<>
struct ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_GET_PURCHASE_HISTORY>
{
    // リクエストパラメータの型
    typedef ServiceItemGetPurchaseHistoryParam TRequest;

    // レスポンスパラメータの型
    typedef ServiceItemGetPurchaseHistoryResponse TResponse;

    // 非同期RMCで使用するリクエストとレスポンスの関数型
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousRequest)(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const TRequest& request);
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousResponse)(ProtocolCallContext *pContext, qUnsignedInt32 requestId, TResponse* response);

    // 非同期RMCで使用するリクエストとレスポンスの関数ポインタ
    static AsynchronousRequest pAsynchronousRequest;
    static AsynchronousResponse pAsynchronousResponse;
};

/*!
    @internal
    @brief      非同期RMCの特性を表すクラスです。SERVICEITEM_ASYNCHRONOUS_RMC_GET_LAW_MESSAGEの特殊化です。
*/
template<>
struct ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_GET_LAW_MESSAGE>
{
    // リクエストパラメータの型
    typedef ServiceItemGetLawMessageParam TRequest;

    // レスポンスパラメータの型
    typedef ServiceItemGetLawMessageResponse TResponse;

    // 非同期RMCで使用するリクエストとレスポンスの関数型
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousRequest)(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const TRequest& request);
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousResponse)(ProtocolCallContext *pContext, qUnsignedInt32 requestId, TResponse* response);

    // 非同期RMCで使用するリクエストとレスポンスの関数ポインタ
    static AsynchronousRequest pAsynchronousRequest;
    static AsynchronousResponse pAsynchronousResponse;
};

/*!
    @internal
    @brief      非同期RMCの特性を表すクラスです。SERVICEITEM_ASYNCHRONOUS_RMC_LIST_AUTO_BILLING_SERVICE_ITEMの特殊化です。
*/
template<>
struct ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_LIST_AUTO_BILLING_SERVICE_ITEM>
{
    // リクエストパラメータの型
    typedef ServiceItemListAutoBillingServiceItemParam TRequest;

    // レスポンスパラメータの型
    typedef ServiceItemListAutoBillingServiceItemResponse TResponse;

    // 非同期RMCで使用するリクエストとレスポンスの関数型
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousRequest)(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const TRequest& request);
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousResponse)(ProtocolCallContext *pContext, qUnsignedInt32 requestId, TResponse* response);

    // 非同期RMCで使用するリクエストとレスポンスの関数ポインタ
    static AsynchronousRequest pAsynchronousRequest;
    static AsynchronousResponse pAsynchronousResponse;
};

/*!
    @internal
    @brief      非同期RMCの特性を表すクラスです。SERVICEITEM_ASYNCHRONOUS_RMC_GET_AUTO_BILLING_CONTRACTの特殊化です。
*/
template<>
struct ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_GET_AUTO_BILLING_CONTRACT>
{
    // リクエストパラメータの型
    typedef ServiceItemGetAutoBillingContractParam TRequest;

    // レスポンスパラメータの型
    typedef ServiceItemGetAutoBillingContractResponse TResponse;

    // 非同期RMCで使用するリクエストとレスポンスの関数型
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousRequest)(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const TRequest& request);
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousResponse)(ProtocolCallContext *pContext, qUnsignedInt32 requestId, TResponse* response);

    // 非同期RMCで使用するリクエストとレスポンスの関数ポインタ
    static AsynchronousRequest pAsynchronousRequest;
    static AsynchronousResponse pAsynchronousResponse;
};

/*!
    @internal
    @brief      非同期RMCの特性を表すクラスです。SERVICEITEM_ASYNCHRONOUS_RMC_CANCEL_AUTO_BILLINGの特殊化です。
*/
template<>
struct ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_CANCEL_AUTO_BILLING>
{
    // リクエストパラメータの型
    typedef ServiceItemCancelAutoBillingParam TRequest;

    // レスポンスパラメータの型
    typedef ServiceItemCancelAutoBillingResponse TResponse;

    // 非同期RMCで使用するリクエストとレスポンスの関数型
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousRequest)(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const TRequest& request);
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousResponse)(ProtocolCallContext *pContext, qUnsignedInt32 requestId, TResponse* response);

    // 非同期RMCで使用するリクエストとレスポンスの関数ポインタ
    static AsynchronousRequest pAsynchronousRequest;
    static AsynchronousResponse pAsynchronousResponse;
};

/*!
    @internal
    @brief      非同期RMCの特性を表すクラスです。SERVICEITEM_ASYNCHRONOUS_RMC_LIST_REDEMPTION_SERVICE_ITEMの特殊化です。
*/
template<>
struct ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_LIST_REDEMPTION_SERVICE_ITEM>
{
    // リクエストパラメータの型
    typedef ServiceItemListRedemptionServiceItemParam TRequest;

    // レスポンスパラメータの型
    typedef ServiceItemListRedemptionServiceItemResponse TResponse;

    // 非同期RMCで使用するリクエストとレスポンスの関数型
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousRequest)(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const TRequest& request);
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousResponse)(ProtocolCallContext *pContext, qUnsignedInt32 requestId, TResponse* response);

    // 非同期RMCで使用するリクエストとレスポンスの関数ポインタ
    static AsynchronousRequest pAsynchronousRequest;
    static AsynchronousResponse pAsynchronousResponse;
};

/*!
    @internal
    @brief      非同期RMCの特性を表すクラスです。SERVICEITEM_ASYNCHRONOUS_RMC_REDEEM_SERVICE_ITEMの特殊化です。
*/
template<>
struct ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_REDEEM_SERVICE_ITEM>
{
    // リクエストパラメータの型
    typedef ServiceItemRedeemServiceItemParam TRequest;

    // レスポンスパラメータの型
    typedef ServiceItemRedeemServiceItemResponse TResponse;

    // 非同期RMCで使用するリクエストとレスポンスの関数型
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousRequest)(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const TRequest& request);
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousResponse)(ProtocolCallContext *pContext, qUnsignedInt32 requestId, TResponse* response);

    // 非同期RMCで使用するリクエストとレスポンスの関数ポインタ
    static AsynchronousRequest pAsynchronousRequest;
    static AsynchronousResponse pAsynchronousResponse;
};

/*!
    @internal
    @brief      非同期RMCの特性を表すクラスです。SERVICEITEM_ASYNCHRONOUS_RMC_POST_RIGHT_BINARY_BY_ACCOUNTの特殊化です。
*/
//template<>
//struct ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_POST_RIGHT_BINARY_BY_ACCOUNT>
//{
//    // リクエストパラメータの型
//    typedef ServiceItemPostRightBinaryByAccountParam TRequest;
//
//    // レスポンスパラメータの型
//    typedef ServiceItemPostRightBinaryResponse TResponse;
//
//    // 非同期RMCで使用するリクエストとレスポンスの関数型
//    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousRequest)(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const TRequest& request);
//    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousResponse)(ProtocolCallContext *pContext, qUnsignedInt32 requestId, TResponse* response);
//
//    // 非同期RMCで使用するリクエストとレスポンスの関数ポインタ
//    static AsynchronousRequest pAsynchronousRequest;
//    static AsynchronousResponse pAsynchronousResponse;
//};

/*!
    @internal
    @brief      非同期RMCの特性を表すクラスです。SERVICEITEM_ASYNCHRONOUS_RMC_USE_SERVICE_ITEM_BY_ACCOUNTの特殊化です。
*/
template<>
struct ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_USE_SERVICE_ITEM_BY_ACCOUNT>
{
    // リクエストパラメータの型
    typedef ServiceItemUseServiceItemByAccountParam TRequest;

    // レスポンスパラメータの型
    typedef ServiceItemUseServiceItemResponse TResponse;

    // 非同期RMCで使用するリクエストとレスポンスの関数型
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousRequest)(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const TRequest& request);
    typedef qBool (ServiceItemInternalClientInterface::*AsynchronousResponse)(ProtocolCallContext *pContext, qUnsignedInt32 requestId, TResponse* response);

    // 非同期RMCで使用するリクエストとレスポンスの関数ポインタ
    static AsynchronousRequest pAsynchronousRequest;
    static AsynchronousResponse pAsynchronousResponse;
};

END_NAMESPACE_NEX

