﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Services/Jugem/ServiceItem/src/Common/ServiceItemDDF.h>
#include <RendezVous/Services/Jugem/ServiceItem/src/Common/ServiceItemProtocolDDL.h>

BEGIN_NAMESPACE_NEX

class Credentials;

/*!
    @brief ServiceItem サーバーとの通信を行うためのインタフェースです。
*/
class ServiceItemInternalClientInterface
{
public:
    ServiceItemInternalClientInterface() {};
    virtual ~ServiceItemInternalClientInterface() {};
    
    virtual Credentials* GetCredentials() const = 0;

    virtual qBool HttpGetRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemHttpGetParam& request) = 0;
    virtual qBool HttpGetResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemHttpGetResponse* response) = 0;

    virtual qBool PurchaseServiceItemRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemPurchaseServiceItemParam& request) = 0;
    virtual qBool PurchaseServiceItemResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemPurchaseServiceItemResponse* response) = 0;

    virtual qBool ListServiceItemRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemListServiceItemParam& request) = 0;
    virtual qBool ListServiceItemResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemListServiceItemResponse* response) = 0;

    virtual qBool GetBalanceRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemGetBalanceParam& request) = 0;
    virtual qBool GetBalanceResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemGetBalanceResponse* response) = 0;

    virtual qBool GetPrepurchaseInfoRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemGetPrepurchaseInfoParam& request) = 0;
    virtual qBool GetPrepurchaseInfoResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemGetPrepurchaseInfoResponse* response) = 0;

    virtual qBool GetServiceItemRightRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemGetServiceItemRightParam& request) = 0;
    virtual qBool GetServiceItemRightResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemGetServiceItemRightResponse* response) = 0;

    virtual qBool GetServiceItemRightStatementRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemGetServiceItemRightParam& request) = 0;
    virtual qBool GetServiceItemRightStatementResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemGetServiceItemRightResponse* response) = 0;

    virtual qBool GetPurchaseHistoryRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemGetPurchaseHistoryParam& request) = 0;
    virtual qBool GetPurchaseHistoryResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemGetPurchaseHistoryResponse* response) = 0;

    virtual qBool GetLawMessageRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemGetLawMessageParam& request) = 0;
    virtual qBool GetLawMessageResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemGetLawMessageResponse* response) = 0;

    virtual qBool ListAutoBillingServiceItemRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemListAutoBillingServiceItemParam& request) = 0;
    virtual qBool ListAutoBillingServiceItemResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemListAutoBillingServiceItemResponse* response) = 0;

    virtual qBool GetAutoBillingContractRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemGetAutoBillingContractParam& request) = 0;
    virtual qBool GetAutoBillingContractResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemGetAutoBillingContractResponse* response) = 0;

    virtual qBool CancelAutoBillingRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemCancelAutoBillingParam& request) = 0;
    virtual qBool CancelAutoBillingResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemCancelAutoBillingResponse* response) = 0;

    virtual qBool ListRedemptionServiceItemRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemListRedemptionServiceItemParam& request) = 0;
    virtual qBool ListRedemptionServiceItemResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemListRedemptionServiceItemResponse* response) = 0;

    virtual qBool RedeemServiceItemRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemRedeemServiceItemParam& request) = 0;
    virtual qBool RedeemServiceItemResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemRedeemServiceItemResponse* response) = 0;

//    virtual qBool PostRightBinaryByAccountRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemPostRightBinaryByAccountParam& request) = 0;
//    virtual qBool PostRightBinaryByAccountResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemPostRightBinaryResponse* response) = 0;

    virtual qBool UseServiceItemByAccountRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemUseServiceItemByAccountParam& request) = 0;
    virtual qBool UseServiceItemByAccountResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemUseServiceItemResponse* response) = 0;
};

END_NAMESPACE_NEX

