﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemAccountRightDDL.h"
#include "ServiceItemConstants.h"
#include "ServiceItemRightBinary.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      一つのプリンシパル ID に対する権利情報を表すクラスです。
*/
class ServiceItemAccountRight: public DDLCLASS(ServiceItemAccountRight)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemAccountRight()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemAccountRight() {}

    /*!
        @brief      プリンシパル ID を取得します。

        本体単位で利用する権利の場合、取得する値は権利を保持しているアカウントのプリンシパル ID になります。

        @return     プリンシパル ID。
    */
    PrincipalID GetPid() const { return pid; }

    /*!
        @brief      権利情報を取得します。

        @return     権利情報。
    */
    const ServiceItemLimitation& GetLimitation() const { return limitation; }

    /*!
        @brief      権利用バイナリデータのリストを取得します。

        @return     権利用バイナリデータのリスト。
    */
    const qVector<ServiceItemRightBinary>& GetRightBinaries() const { return rightBinaries; }

    /*!
        @brief      ServiceItemAccountRight オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        {
            StringStream ss;
            ss << title << NEX_T("ServiceItemAccountRight(");
            ss << NEX_T("PID:") << GetPid() << NEX_T(")");
            QTRACE(uiTraceFlags, ss.CStr());
        }

        GetLimitation().Trace(uiTraceFlags, title + NEX_T("    "));

        qVector<ServiceItemRightBinary>::const_iterator it = GetRightBinaries().begin();
        for (qInt i = 0; it != GetRightBinaries().end(); ++it, ++i)
        {
            StringStream ss;
            ss << title << NEX_T("    [") << i << NEX_T("]");
            it->Trace(uiTraceFlags, ss.CStr());
        }
    }
};

END_NAMESPACE_NEX

