﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include "ServiceItemAccountRightTimeDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @extends    ServiceItemAccountRight
    @brief      一つのプリンシパル ID に対する永久利用券または期間券の権利情報を表すクラスです。
*/
class ServiceItemAccountRightTime: public DDLCLASS(ServiceItemAccountRightTime)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemAccountRightTime()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemAccountRightTime() {}

    /*!
        @brief      有効期限 (1970 年 1 月 1 日 00:00:00 UTC からの経過秒数) を取得します。

        永久利用券の場合は使用しないでください。

        @return     有効期限。
    */
    qUnsignedInt32 GetExpiryDate() const { return limitation.limitationValue; }

    /*!
        @brief      継続課金契約が有効かどうかを表す真偽値を取得します。

        @return     継続課金契約が有効かどうかを表す真偽値。
    */
    qBool GetAutoBillingApplied() const { return autoBillingApplied; }

    /*!
        @brief      ServiceItemAccountRightTime オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        {
            StringStream ss;
            ss << title << NEX_T("ServiceItemAccountRightTime(");
            ss << NEX_T("PID:") << dec << GetPid();
            ss << NEX_T(" ExpiryDate(Epoch):") << dec << GetExpiryDate();
            DateTime d = DateTime::FromUnixEpochTime(GetExpiryDate());
            ss << NEX_T(" ExpiryDate(DateTime):")
                << d.GetYear() << NEX_T("/") << d.GetMonth() << NEX_T("/") << d.GetDay()
                << NEX_T(" ") << d.GetHour() << NEX_T(":") << d.GetMinute() << NEX_T(":") << d.GetSecond();
            ss << " AutoBillingApplied:" << (GetAutoBillingApplied() ? "true" : "false");
            ss << NEX_T(")");
            QTRACE(uiTraceFlags, ss.CStr());
        }
        
        GetLimitation().Trace(uiTraceFlags, title + NEX_T("    "));

        qVector<ServiceItemRightBinary>::const_iterator it = GetRightBinaries().begin();
        for (qInt i = 0; it != GetRightBinaries().end(); ++it, ++i)
        {
            StringStream ss;
            ss << title << NEX_T("    [") << i << NEX_T("]");
            it->Trace(uiTraceFlags, ss.CStr());
        }
    }
};

END_NAMESPACE_NEX

