﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemAmountDDL.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      金額情報を表すクラスです。
*/
class ServiceItemAmount: public DDLCLASS(ServiceItemAmount)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemAmount()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemAmount() {}

    /*!
        @brief      通貨表記でフォーマット済みの金額 (e.g. $10.00) を取得します。

        @return     通貨表記でフォーマット済みの金額。
    */
    const String& GetFormattedAmount() const { return formattedAmount; }

    /*!
        @brief      ISO4217 で定義された通貨コード (e.g. USD) を取得します。

        @return     ISO4217 で定義された通貨コード。
    */
    const String& GetCurrency() const { return currency; }

    /*!
        @brief      未フォーマットの金額 (e.g. 10) を取得します。

        @return     未フォーマットの金額。
    */
    const String& GetRawValue() const { return rawValue; }

    /*!
        @brief      ServiceItemAmount オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title << NEX_T("ServiceItemAmount(");
        ss << NEX_T("RawValue:") << GetRawValue();
        ss << NEX_T(" Currency:") << GetCurrency();
        ss << NEX_T(" FormattedAmount:") << GetFormattedAmount();
        ss << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX

