﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemAutoBillingContractDDL.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      継続課金アイテムの契約情報を表すクラスです。
*/
class ServiceItemAutoBillingContract: public DDLCLASS(ServiceItemAutoBillingContract)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemAutoBillingContract()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemAutoBillingContract() {}

    /*!
        @brief      アイテムコードを取得します。

        @return     アイテムコード。
    */
    const String& GetItemCode() const { return itemCode; }

    /*!
        @brief      アイテムのリファレンス IDを取得します。

        @return     アイテムのリファレンス ID。
    */
    const String& GetReferenceId() const { return referenceId; }

    /*!
        @brief      アイテムの説明文を取得します。

        @return     アイテムの説明文。
    */
    const String& GetDescription() const { return description; }

    /*!
        @brief      アイテムのアイテム名を取得します。

        @return     アイテムのアイテム名。
    */
    const String& GetName() const { return name; }

    /*!
        @brief      次回決済予定日を取得します。

        @return     次回決済予定日。
    */
    const DateTime& GetNextPurchaseDate() const { return nextPurchaseDate; }

    /*!
        @brief      ServiceItemAutoBillingContract オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title << NEX_T("ServiceItemAutoBillingContract(");
        ss << NEX_T("itemCode:") << GetItemCode();
        ss << NEX_T(" referenceId:") << GetReferenceId();
        ss << NEX_T(" description:") << GetDescription();
        ss << NEX_T(" name:") << GetName();
        ss << NEX_T(" nextPurchaseDate:") << GetNextPurchaseDate();
        ss << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX

