﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemAutoBillingPlanDDL.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      継続課金アイテムの情報を表すクラスです。
*/
class ServiceItemAutoBillingPlan: public DDLCLASS(ServiceItemAutoBillingPlan)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemAutoBillingPlan()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemAutoBillingPlan() {}

    /*!
        @brief      アイテムコードを取得します。

        @return     アイテムコード。
    */
    const String& GetItemCode() const { return itemCode; }

    /*!
        @brief      アイテムのリファレンス IDを取得します。

        @return     アイテムのリファレンス ID。
    */
    const String& GetReferenceId() const { return referenceId; }

    /*!
        @brief      アイテムの説明文を取得します。

        @return     アイテムの説明文。
    */
    const String& GetDescription() const { return description; }

    /*!
        @brief      アイテムの購入前注意文言を取得します。

        @return     アイテムの購入前注意文言。
    */
    const String& GetDisclaimer() const { return description; }

    /*!
        @brief      アイテムのアイテム名を取得します。

        @return     アイテムのアイテム名。
    */
    const String& GetName() const { return name; }

    /*!
        @brief      アイテムの販売価格を取得します。

        @return     アイテムの販売価格。
    */
    const ServiceItemAmount& GetRegularPrice() const { return regularPrice; }

    /*!
        @brief      ServiceItemAutoBillingPlan オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title << NEX_T("ServiceItemAutoBillingPlan(");
        ss << NEX_T("itemCode:") << GetItemCode();
        ss << NEX_T(" referenceId:") << GetReferenceId();
        ss << NEX_T(" description:") << GetDescription();
        ss << NEX_T(" disclaimer:") << GetDisclaimer();
        ss << NEX_T(" name:") << GetName();
        ss << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());

        GetRegularPrice().Trace(uiTraceFlags, title + NEX_T("    RegularPrice"));
    }
};

END_NAMESPACE_NEX

