﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

/*!
    @brief ServiceItem に関する定数が定義された名前空間です。
*/
namespace ServiceItemConstants
{

/// eShop サーバーへのアクセスが成功したときに返されるエラーコードです。
static const qUnsignedInt32 ESHOP_ERRORCODE_SUCCESS = 0;

// eShop サーバーへのアクセス時に返される HTTP ステータスコードの無効値です。
static const qUnsignedInt32 ESHOP_HTTP_STATUS_UNKNOWN = 0;

/// 権利用バイナリデータの利用タイプの無効値です。
static const qByte INVALID_USE_TYPE = 0;

/// ユニーク ID の無効値です。
static const qUnsignedInt32 INVALID_UNIQUE_ID = 0;

/// 言語の無効値です。
extern const qChar* INVALID_LANGUAGE;

/// リファレンス ID の不正値です。
extern const qChar* INVALID_REFERENCE_ID;

/// ログメッセージの無効値です。
extern const qChar* INVALID_LOG_MESSAGE;

/// 本体固有 ID の無効値です。
extern const qChar* INVALID_DEVICE_ID;

/// 無効なアイテムコードです。
extern const qChar* INVALID_ITEM_CODE;

/// 無効な EC サービストークンです。
extern const qChar* INVALID_EC_SERVICE_TOKEN;

/// 権利用バイナリの最大バイト数です。
static const qUnsignedInt32 RIGHT_BINARY_MAX_SIZE = 64;

/// ユーザサポート向けの管理ツールに表示されるログメッセージの最大バイト数です。
static const qUnsignedInt32 SUPPORT_LOG_MESSAGE_MAX_SIZE = 32;

/*!
    @brief 権利種別です。
*/
enum LimitationType
{
    LIMITATION_TYPE_INVALID         = 0,    //!< 無効値です。
    LIMITATION_TYPE_ADDITIONAL_TIME = 1,    //!< サービスアイテムの有効期限をエポック秒 (1970 年 1 月 1 日 00:00:00 UTC からの経過秒数) で表します。
    LIMITATION_TYPE_CONSUMPTION     = 2,    //!< サービスアイテムの消費可能回数を表します。
    LIMITATION_TYPE_PERMANENT       = 3     //!< サービスアイテムを恒久的に使用する権利があることを表します。
};

/*!
    @brief リファレンス ID 種別です。
*/
enum ReferenceIdType
{
    REFERENCE_ID_TYPE_PURCHASED = 0,    //!< 有料の権利のリファレンス ID です。
    REFERENCE_ID_TYPE_FREE      = 1,    //!< 無料の権利のリファレンス ID です。
    REFERENCE_ID_TYPE_MIXED     = 2     //!< 無料と有料の権利が混在したリファレンス ID です。
};

/*!
    @brief トランザクション種別です。
*/
enum TransactionType
{
    TRANSACTION_TYPE_INVALID        = 0,    //!< 無効値です。
    TRANSACTION_TYPE_PURCHASE       = 1,    //!< 購入を示します。
    TRANSACTION_TYPE_REDEEM         = 2,    //!< 引き換えを示します。
    TRANSACTION_TYPE_DELETE         = 3     //!< 削除を表します。
};

/*!
    @internal
    @brief ServiceItemClient::GetServiceItemRight() で使用する、アイテムグループのデフォルト値です。
*/
static const qByte DEFAULT_ITEM_GROUP = 0xFF;

}

END_NAMESPACE_NEX

