﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemGetAutoBillingContractParamDDL.h"
#include "ServiceItemConstantsInternal.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      現在契約中の継続課金アイテムを取得します。
*/
class ServiceItemGetAutoBillingContractParam: public DDLCLASS(ServiceItemGetAutoBillingContractParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemGetAutoBillingContractParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemGetAutoBillingContractParam() {}

    /*!
        @brief      ユニーク ID を取得します。

        @return     ユニーク ID。
    */
    qUnsignedInt32 GetUniqueId() const { return uniqueId; }

    /*!
        @brief      [必須パラメタ] ユニーク ID をセットします。

        NEX サーバーから接続する eShop サーバーを決定するためのパラメーターです。
        通常はタイトルのユニーク ID を指定します。

        @param[in]  uniqueId ユニーク ID。
    */
    void SetUniqueId(qUnsignedInt32 uniqueId) { this->uniqueId = uniqueId; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetUniqueId(ServiceItemConstants::INVALID_UNIQUE_ID);
        platform = NEX_SERVICEITEM_GET_PLATFORM;
    }

    /*!
        @brief      ServiceItemGetAutoBillingContractParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const ServiceItemGetAutoBillingContractParam& rhs);

    /*!
        @internal
        @brief      RMCリクエスト前にパラメタをチェックする関数です。
    */
    qBool RequestPreCheck() const
    {
        qBool success = true;
        if(GetUniqueId() == ServiceItemConstants::INVALID_UNIQUE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid uniqueId. Please set uniqueId with ServiceItemGetAutoBillingContractParam::SetUniqueId()."));
            success = false;
        }
        return success;
    }
};

inline StringStream& operator<<(StringStream& oStr, const ServiceItemGetAutoBillingContractParam& rhs)
{
    (void)rhs;
    oStr << NEX_T("ServiceItemGetAutoBillingContractParam: (")
               << NEX_T(")");

    return oStr;
}

END_NAMESPACE_NEX
