﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemLawMessageDDL.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      法令に基づく表示のメッセージを表すクラスです。
*/
class ServiceItemLawMessage: public DDLCLASS(ServiceItemLawMessage)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemLawMessage()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemLawMessage() {}

    /*!
        @brief      法令に基づく表示のメッセージを取得します。

        @return     法令に基づく表示のメッセージ。
    */
    const String& GetLawMessage() const { return lawMessage; }

    /*!
        @brief      法令に基づく表示のメッセージが必要かどうかを返します。

        @return     法令に基づく表示のメッセージが必要かどうか。
    */
    qBool IsMessageRequired() const { return isMessageRequired; }

    /*!
        @brief      ServiceItemLawMessage オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title << NEX_T("ServiceItemLawMessage(");
        ss << NEX_T("Message:") << GetLawMessage();
        ss << NEX_T(" IsMessageRequired:") << IsMessageRequired();
        ss << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX
