﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemLimitationDDL.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      権利情報を表すクラスです。
*/
class ServiceItemLimitation: public DDLCLASS(ServiceItemLimitation)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemLimitation()
    {
    }

    /*!
        @internal
        @brief      コンストラクタです。

        デバッグ用です。

        @param[in]  limitationType 権利種別です。
        @param[in]  limitationValue 権利種別に応じた値です。
    */
    ServiceItemLimitation(ServiceItemConstants::LimitationType limitationType, qUnsignedInt32 limitationValue)
    {
        this->limitationType = static_cast<qUnsignedInt32>(limitationType);
        this->limitationValue = limitationValue;
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemLimitation() {}

    /*!
        @brief      権利種別を取得します。

        @return     権利種別。
    */
    ServiceItemConstants::LimitationType GetLimitationType() const { return static_cast<ServiceItemConstants::LimitationType>(limitationType); }

    /*!
        @brief      権利種別に応じた値を取得します。

        権利種別が ServiceItemConstants::LIMITATION_TYPE_PERMANENT の場合は使用しません。
        ServiceItemConstants::LIMITATION_TYPE_ADDITIONAL_TIME の場合は有効期限 (1970 年 1 月 1 日 00:00:00 UTC からの経過秒数) 、
        もしくは有効期間 (秒数) を表します。
        ServiceItemConstants::LIMITATION_TYPE_CONSUMPTION の場合は個数を表します。

        @return     権利種別に応じた値。
    */
    qUnsignedInt32 GetLimitationValue() const { return limitationValue; }

    /*!
        @brief      ServiceItemLimitation オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title << NEX_T("ServiceItemLimitation(");
        ss << NEX_T("Type:");
        switch (GetLimitationType())
        {
        case ServiceItemConstants::LIMITATION_TYPE_CONSUMPTION:
            ss << NEX_T("CONSUMPTION");
            break;
        case ServiceItemConstants::LIMITATION_TYPE_ADDITIONAL_TIME:
            ss << NEX_T("ADDITIONAL_TIME");
            break;
        default:
            ss << NEX_T("PERMANENT");
            break;
        }
        ss << NEX_T(" Value:") << GetLimitationValue();
        ss << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());
    }
};



END_NAMESPACE_NEX

