﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemPurchaseHistoryDDL.h"
#include "ServiceItemTransaction.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      購入履歴情報を表すクラスです。
*/
class ServiceItemPurchaseHistory: public DDLCLASS(ServiceItemPurchaseHistory)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemPurchaseHistory()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemPurchaseHistory() {}

    /*!
        @brief      検索にマッチしたトランザクション情報の総数を取得します。

        @return     検索にマッチしたトランザクション情報の総数。
    */
    qUnsignedInt32 GetTotalSize() const { return totalSize; }

    /*!
        @brief      オフセットを取得します。

        @return     オフセット。
    */
    qUnsignedInt32 GetOffset() const { return offset; }

    /*!
        @brief      トランザクション情報のリストを取得します。

        @return     トランザクション情報のリスト。
    */
    const qVector<ServiceItemTransaction>& GetTransactions() const { return transactions; }

    /*!
        @brief      ServiceItemPurchaseHistory オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title << NEX_T("ServiceItemPurchaseHistory(");
        ss << NEX_T("totalSize=") << GetTotalSize();
        ss << NEX_T(" offset=") << GetOffset();
        ss << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());

        qVector<ServiceItemTransaction>::const_iterator itr = GetTransactions().begin();
        for (qInt i = 0; itr != GetTransactions().end(); ++i, ++itr)
        {
            ss.Clear();
            ss << title << NEX_T("    transactions[") << i << NEX_T("]");
            itr->Trace(uiTraceFlags, ss.CStr());
        }
    }
};

END_NAMESPACE_NEX

