﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _SERVICEITEM_REDEEM_INFO_H
#define _SERVICEITEM_REDEEM_INFO_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemRedeemInfoDDL.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      引換結果を表すクラスです。
*/
class ServiceItemRedeemInfo: public DDLCLASS(ServiceItemRedeemInfo)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemRedeemInfo()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemRedeemInfo() {}

    /*!
        @brief      eShop サーバで発行されたトランザクション ID を取得します。

        @return     eShop サーバで発行されたトランザクション ID。
    */
    const String& GetTransactionId() const { return transactionId; }

    /*!
        @brief      NGS で発行されたトランザクション ID を取得します。

        @return     NGS で発行されたトランザクション ID。
    */
    const String& GetExtTransactionId() const { return extTransactionId; }

    /*!
        @brief      引き換えたサービスアイテムのアイテムコードを取得します。

        @return     引き換えたサービスアイテムのアイテムコード。
    */
    const String& GetItemCode() const { return itemCode; }

    /*!
        @brief      ServiceItemRedeemInfo オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title << NEX_T("ServiceItemRedeemInfo(");
        ss << NEX_T("TransactionId:") << GetTransactionId();
        ss << NEX_T(" ExtTransactionId:") << GetExtTransactionId();
        ss << NEX_T(" ItemCode:") << GetItemCode();
        ss << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX

#endif
