﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Core/qBuffer.h>
#include "ServiceItemRightBinaryDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @defgroup   DescriptionOfRightBinary 権利用バイナリデータの管理方法
    @{
    権利用バイナリデータは、以下のすべてをキーにしてサーバで管理されます。
    そのため、例えばリファレンス ID が同じであっても利用タイプを使い分けることで、
    複数の権利用バイナリデータを登録することができます。

    @li プリンシパル ID
    @li リファレンス ID
    @li 利用タイプ

    キーに該当する権利用バイナリが未登録の場合は追加、登録済の場合は更新されます。

    登録した権利用バイナリデータは、
    @ref nn::nex::ServiceItemClient::GetServiceItemRight(nn::nex::ProtocolCallContext*,const nn::nex::ServiceItemGetServiceItemRightParam&,nn::nex::ServiceItemGetServiceItemRightResponse*) "ServiceItemClient::GetServiceItemRight()"
    で取得できます。
    @}
*/

/*!
    @brief      権利用バイナリデータ情報を表すクラスです。
*/
class ServiceItemRightBinary: public DDLCLASS(ServiceItemRightBinary)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemRightBinary()
    {
        useType = ServiceItemConstants::INVALID_USE_TYPE;
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemRightBinary() {}

    /*!
        @brief      利用タイプを取得します。

        @return     利用タイプ。
    */
    qByte GetUseType() const { return useType; }

    /*!
        @brief      権利用バイナリデータを取得します。

        @param[out]  rightBinary_ 権利用バイナリデータ。
    */
    void GetRightBinary(qVector<qByte>& rightBinary_) const
    {
        // qBuffer -> qVector<qByte>への変換
        rightBinary_.assign(this->rightBinary.begin(), this->rightBinary.end());
    }

    /*!
        @brief      ServiceItemRightBinary オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        {
            StringStream ss;
            ss  << title << NEX_T("ServiceItemRightBinary(")
                << NEX_T("useType:") << GetUseType()
                << NEX_T(")");
            QTRACE(uiTraceFlags, ss.CStr());
        }

        {
            // rightBinaryは大きい可能性があるので別でログ出力する
            StringStream ss;
            ss << title << NEX_T("    rightBinary=<");
            if(!rightBinary.empty())
            {
                ss.BytesDump(&rightBinary[0], rightBinary.size());
            }
            ss << NEX_T(">");
            QTRACE(uiTraceFlags, ss.CStr());
        }

    }
};


END_NAMESPACE_NEX

