﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "SubscriberGetFollowerResultDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @internal
    @brief      フォロワーの取得結果に関するパラメータを表すクラスです。
*/
class SubscriberGetFollowerResult: public DDLCLASS(SubscriberGetFollowerResult)
{
public:
    /*!
        @brief      コンストラクタです。無効なフォロワーの取得結果を生成します。
    */
    SubscriberGetFollowerResult()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~SubscriberGetFollowerResult() {}

    /*!
    @brief      フォロワーを取得したトピックを取得します。

    @return フォロワーを取得したトピックを取得します。
    */
    const String& GetTopic() const { return topic; }

    /*!
    @brief      フォロワーの人数を取得します。

    取得したフォロワーの人数ではなく、実際にフォローしている人数を表します。

    @return フォロワーの人数を取得します。
    */
    qUnsignedInt32 GetTotalSize() const { return totalSize; }

    /*!
    @brief      フォロワーのプリンシパル ID を取得します。

    @return フォロワーのプリンシパル ID を取得します。昇順に並びます。
    */
    const qVector<PrincipalID>& GetFollowerPrincipalId() const { return pids; }


    /*!
        @brief      SubscriberGetFollowerResult オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const SubscriberGetFollowerResult& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const SubscriberGetFollowerResult& rhs)
{
    oStr << NEX_T("SubscriberGetContentParam: (")
               << NEX_T("topic=") << rhs.topic
               << NEX_T(", totalSize=") << rhs.totalSize;

    oStr << NEX_T(", pids=(");
    for (qVector<PrincipalID>::const_iterator it = rhs.pids.begin(); it != rhs.pids.end(); ++it)
    {
        oStr << *it << NEX_T(", ");
    }

    oStr << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

