﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "UniqueIdInfoDDL.h"
#include "UniqueId.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      NEX ユニーク ID、パスワード情報を表すクラスです。
*/
class UniqueIdInfo: public DDLCLASS(UniqueIdInfo)
{
public:
    /*!
        @brief      コンストラクタです。無効な NEX ユニーク ID を持つオブジェクトを作成します。
    */
    UniqueIdInfo()
    {
        nexUniqueId = INVALID_UNIQUEID;
    }

    /*!
        @brief      コンストラクタです。NEX ユニーク ID とパスワードを指定して作成します。

        @param[in] nexUniqueId_ NEX ユニーク ID
        @param[in] nexUniqueIdPassword_ パスワード
    */
    UniqueIdInfo(qUnsignedInt64 nexUniqueId_, qUnsignedInt64 nexUniqueIdPassword_)
    {
        nexUniqueId = nexUniqueId_;
        nexUniqueIdPassword = nexUniqueIdPassword_;
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~UniqueIdInfo() {}

    /*!
        @brief      NEX ユニーク ID を設定します。

        @param[in] nexUniqueId_ NEX ユニーク ID
    */
    void SetUniqueId(qUnsignedInt64 nexUniqueId_)
    {
        nexUniqueId = nexUniqueId_;
    }

    /*!
        @brief      NEX ユニーク ID を取得します。

        @return     NEX ユニーク ID
    */
    qUnsignedInt64 GetUniqueId() const
    {
        return nexUniqueId;
    }

    /*!
        @brief      パスワードを設定します。

        @param[in] nexUniqueIdPassword_ パスワード
    */
    void SetPassword(qUnsignedInt64 nexUniqueIdPassword_)
    {
        nexUniqueIdPassword = nexUniqueIdPassword_;
    }

    /*!
        @brief      パスワードを取得します。

        @return     パスワード
    */
    qUnsignedInt64 GetPassword() const
    {
        return nexUniqueIdPassword;
    }

    /*!
        @brief      このオブジェクトが有効かどうかを調べます。

        返値が false だった場合は内部パラメータは無効ですので値を取得しないでください。

        @return     このオブジェクトが有効なら true 、無効なら false 。
    */
    qBool IsValid() const
    {
        return nexUniqueId != INVALID_UNIQUEID;
    }

    qBool operator==(const UniqueIdInfo& rhs) const
    {
        return nexUniqueId == rhs.nexUniqueId
               && nexUniqueIdPassword == rhs.nexUniqueIdPassword;
    }
    qBool operator!=(const UniqueIdInfo& rhs) const
    {
        return !(*this == rhs);
    }

    /*!
        @brief      UniqueIdInfo オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, NEX_T("UniqueIdInfo: (")
               << NEX_T("nexUniqueId=") << hex << nexUniqueId
               << NEX_T(", nexUniqueIdPassword=") << nexUniqueIdPassword
               << NEX_T(")")
              );
    }
};

END_NAMESPACE_NEX

