﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

BEGIN_NAMESPACE_NEX

/**
   \ingroup group_rdv_services_gathering_client
   \brief   This class contains the enumeration of states that can be set for a Gathering.
*/
class GatheringStates
{
public:
    /**
        \ingroup group_rdv_services_gathering_client
        \brief Enumerates the various states that may be set for a Gathering.

        The state of a Gathering is set via MatchMakingClient::SetState and returned via
        MatchMakingClient::GetState. A Gathering may have the state HasFinished without first having the
        state HasStarted. The exact significance of each state and the calls that may or may not be made in each state depend
        on the specific Participation policy set for the Gathering. Refer to BuiltInParticipationPolicies for more information.

        \note In order to access these types you must include the RendezVous.h file.
    */

    enum States
    {
        /**
            Participation in this Gathering is locked and therefore, no more participants may
            join or leave the Gathering.
        */
        ParticipationIsLocked = 1,
        /**
            The Gathering has started. This state must be set before the actual game play of a Gathering commences.
        */
        HasStarted = 2,
        /**
            The Gathering has finished. This state will be set if a Gathering is cancelled. If a Gathering
            is non-persistent then when this state is set the Gathering will automatically be deleted by Rendez-Vous.
            If the Gathering is persistent, then this state must be set before the gathering is deleted.
        */
        HasFinished = 4
    };
};

END_NAMESPACE_NEX

