﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <RendezVous/Services/Messaging/src/Common/TextMessageDDL.h>

BEGIN_NAMESPACE_NEX

// Group=Messaging Service

#ifdef FOR_DOC_PURPOSE
/*
Description: Construct a message containing essentially a text body.
See Also: Messaging Service
*/

/*!
 @brief    テキストメッセージデータを格納するクラスです

 インスタントメッセージング機能を利用する場合は本クラス、@ref UserMessage 、@ref BinaryMessage のいずれかのクラスを利用してください。
 */
class TextMessage: public UserMessage
{

#else
class TextMessage: public DDLCLASS(TextMessage)
{

#endif

public:

    /*
     Description: Default constructor.
     */

    /*!
     @brief コンストラクタです
     */
    TextMessage()
    {
    }

    /*
     Description: Assignment constructor.
     Parameters:
     strTextBody: Text body to attach to message.
     */

    /*!
     @brief コンストラクタです

     @param[in]    strTextBody    テキストメッセージの本文
     */
    TextMessage(const String& strTextBody)
    {
        m_strTextBody = strTextBody;
    }

    /*
     Description: Accessor to set text body.
     Parameters:
     strTextBody: Text body to attach to message.
     */

    /*!
     @brief テキストメッセージをセットします。
            MESSAGING_STRING_MAX_LENGTH 文字( NULL終端含まず ) を超えるテキストは、
            インスタントメッセージング機能では使用できません。サーバー送信時にエラーとなります。

     @param[in]    strTextBody    テキストメッセージの本文
     */
    void SetTextBody(const String& strTextBody)
    {
        m_strTextBody = strTextBody;
    }

    /*
     Description: Accessor to get text body.
     Returns: Message text body
     */

    /*!
     @brief テキストメッセージを取得します。

     @return    テキストメッセージの本文
     */
    const String& GetTextBody() const
    {
        return m_strTextBody;
    }

    /*
     for debug
     */
    virtual void Trace(qUnsignedInt64 traceFlag)
    {
        UserMessage::Trace(traceFlag);
        QTRACE(traceFlag, NEX_T("TextMessage:") << this)
        QTRACE(traceFlag, NEX_T("  TextBody:") << GetTextBody());
    }
};


END_NAMESPACE_NEX

