﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/NATRelayInterface.h>
#include <OnlineCore/src/Transport/Interface/NATTraversalEngine.h>

#include <RendezVous/Services/NATTraversal/src/Client/RVNATEcho.h>
#include <RendezVous/Services/NATTraversal/src/Client/RVConnectivityTester.h>

BEGIN_NAMESPACE_NEX

class NATTraversalRelayClient;
class RelaySignatureKey;

class RVNATRelay: public NATRelayInterface
{
public:
    RVNATRelay();
    virtual ~RVNATRelay();

    virtual void RegisterRelayClient(NATTraversalRelayClient *pClient);
    virtual void UnregisterRelayClient();

    virtual void RelayProbeRequest(const qList<StationURL> &lstURLRequestTargets);
    virtual void RelayProbeRequest(const qList<StationURL> &lstURLRequestTargets,
                                   const StationURL &oURLRequester);

    /* JobCheckAndRelay/ConnectivetyManagerから呼び出す */
    virtual void CheckCurrentPublicPort(CallContext *pCallContext, const InetAddress &rInterfaceAddress, qBool bPublicPortIncremental);

    /* ConnectivetyManagerから呼び出す */
    virtual void DetectSimplifiedNATProperties(CallContext *pCallContext, const InetAddress &rInterfaceAddress);

    virtual void RequestProbe(const StationURL &oURLStationToProbe);

    /* ConnectivetyManagerから呼び出す */
    virtual void ReportNATTraversalResult(const ConnectionID &cid, const qBool &result, const qUnsignedInt32 &rtt);

    /* ConnectivetyManagerから呼び出す */
    virtual void ReportNATProperties(qUnsignedInt32 natmapping, qUnsignedInt32 natfiltering, qUnsignedInt32 rtt);

    virtual qBool GetRelaySignatureKey(CallContext* pCallcontext, RelaySignatureKey* pRelaySignatureKey);

    virtual void ReportNATTraversalResultDetail(const ConnectionID &cid, const qBool &result, const NATTraversalResult &detail, qUnsignedInt32& rtt);

    virtual void UpdateConnectionState(const ConnectionID cid, const StationURL& oURL);

    // NATRelayInterface と NATTraversalEngine が依存しないように継承クラスで用意
    virtual void AssociateEngine(NATTraversalEngine *pEngine);


private:
    NATTraversalRelayClient *m_pClient;
    NATTraversalEngine      *m_pEngine;
    RVNATEcho                m_oEcho;
    RVConnectivityTester     m_oTester;
};

END_NAMESPACE_NEX

