﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

//#include <ObjDup/ObjDupImpl.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <RendezVous/Services/NATTraversal/src/Common/NATTraversalProtocolDDL.h>
#include <RendezVous/Services/NATTraversal/src/Common/NATTraversalReportInternalProtocolDDL.h>
#include <OnlineCore/src/Transport/Interface/RelaySignatureKey.h>
#include <OnlineCore/src/Transport/Interface/NATTraversalResult.h>

BEGIN_NAMESPACE_NEX

class EndPoint;
class ScriptEngine;
class DateTime;
class qBuffer;
class NATTraversalRelayClient;
#ifdef SN_TARGET_NON_SUPPORT_A
#pragma control %push diag
#pragma diag_suppress=999
#endif

class NATTraversalRelayProtocol: public DDLPROTOCOL(NATTraversalProtocolServer)
{
public:

    NATTraversalRelayProtocol();
    virtual ~NATTraversalRelayProtocol();

    void CloneRelayProtocol(NATTraversalRelayProtocol *pProtocol) const;

    void RegisterProtocols(ProtocolRequestBrokerInterface* pBroker);
    void UnregisterProtocols(ProtocolRequestBrokerInterface* pBroker);

    NATTraversalProtocolClient* GetClient()
    {
        SYSTEMCHECK(m_pClient != NULL);
        return m_pClient;
    }

    void SetClientConnection(EndPoint *pConnection);
    void SetReportServer(ScriptEngine *pServer)
    {
        m_pReportServer = pServer;
    }
    void SetRelayClient(NATTraversalRelayClient* pRelayClient) 
    {
        m_pRelayClient = pRelayClient;
    }

    // {secret}
    virtual void SetDefaultCredentials(Credentials* pCredentials);

private:
    virtual void InitiateProbe(const DDLTYPE(stationurl)::CPPType& /*urlStationToProbe*/)
    {
        SYSTEMCHECK(false);
    }
    virtual void RequestProbeInitiation(const qList<StationURL>& /*lstTarget*/)
    {
        SYSTEMCHECK(false);
    }
    virtual void RequestProbeInitiationExt(const qList<StationURL>& /*lstTarget*/, const StationURL& /*urlStationToProbe*/)
    {
        SYSTEMCHECK(false);
    }
    virtual void ReportNATTraversalResult(const ConnectionID & /*cid*/, const qBool & /*result*/, const qUnsignedInt32& /*rtt*/)
    {
        SYSTEMCHECK(false);
    }
    virtual void ReportNATProperties(const qUnsignedInt32& /*natmapping*/, const qUnsignedInt32& /*natfiltering*/, const qUnsignedInt32& /*rtt*/)
    {
        SYSTEMCHECK(false);
    }

    virtual void GetRelaySignatureKey(qInt32* /*relayType*/, DateTime* /*currentUTCTime*/, String* /*address*/, qUnsignedInt16* /*port*/, qInt32* /*relayAddressType*/, qUnsignedInt32* /*gameServerID*/)
    {
        SYSTEMCHECK(false);
    }

    virtual void ReportNATTraversalResultDetail(
            const ConnectionID &/*cid*/, const qBool &/*result*/,
            const qInt32 &/*detail*/, const qUnsignedInt32& /*rtt*/)
    {
        SYSTEMCHECK(false);
    }

protected:
    NATTraversalRelayClient* m_pRelayClient;
    NATTraversalProtocolClient  *m_pClient;
    NATTraversalReportInternalProtocolClient m_oReportClient;
    ScriptEngine *m_pReportServer;
};
#ifdef SN_TARGET_NON_SUPPORT_A
#pragma control %pop diag
#endif

END_NAMESPACE_NEX

