﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

////////////////////////////////////////////////////////////////////////////////
// IMPORTANT NOTE: Values of each element of the following enumation must be
// specified so that translating this file for Python scripts works as expected.
// Also, the description of elements must use C-style /* */ comments where
// the opening and closing syntax must be on their own lines of code.
////////////////////////////////////////////////////////////////////////////////

#pragma once

BEGIN_NAMESPACE_NEX

/*
    \ingroup group_rdv_errors
    \brief This enumeration defines the error codes that may be returned for the RendezVous facility.
*/
#ifdef SWIG
class RendezVousErrorCode
{
public:
#else
/*!
 @brief    RendezVous モジュールのエラーコードを定義する名前空間です。
 */
namespace RendezVousErrorCode
{
#endif


    /*
        \brief Enumerates the error codes for the RendezVous facility that a call may return.
        \remark Details as to the cause of each ErrorCode are detailed in the documentation of each individual Rendez-Vous method that may return a result.
    */

    /*!
     @brief    RendezVous モジュールのエラーコード
    */
    enum ErrorCode
    {
        START = 1,
        /**
            Connection was unable to be established, either with the Rendez-Vous back end or a Peer.
        */
        ConnectionFailure                   = 1,    //!<    NGS との接続に失敗

        /**
            The Principal could not be authenticated by the Authentication Service.
        */
        NotAuthenticated                    = 2,    //!<    NGS 認証からの認証処理が完了していない

        /**
            The Principal tried to log in with an invalid user name, /i.e./ the user name does not exist in the database.
        */
        InvalidUsername                     = 100,    //!<    不正なユーザー名

        /**
            The Principal either tried to log in with an invalid password for the provided user name or tried to join a Gathering with an invalid password.
        */
        InvalidPassword                     = 101,    //!<    不正なパスワード

        /**
            The provided user name already exists in the database. All usernames must be unique.
        */
        UsernameAlreadyExists               = 102,    //!<    同じユーザー名が登録されている

        /**
            The Principal's account still exists in the database but the account has been disabled.
        */
        AccountDisabled                     = 103,    //!<    アカウントが使用できない

        /**
            The Principal's account still exists in the database but the account has expired.
        */
        AccountExpired                      = 104,    //!<    アカウントの有効期限が切れている

        /**
            The Principal does not have the Capabilities to perform concurrent log ins, /i.e./ at any given time only one log-in may be maintained.
            It may occur also if a client application holding the same user exited previously without logging out properly (e.g. crash).
        */
        ConcurrentLoginDenied               = 105,    //!<    すでに同じユーザーがログインしている。(NEXでは後でログインしたものが優先されるので発生しません)

        /**
            Data encryption failed.
        */
        EncryptionFailure                   = 106,    //!<    暗号化エラー

        /**
            The operation specifies or accesses an invalid PrincipalID.
        */
        InvalidPID                          = 107,    //!<    不正な PrincipalID

        /**
            Maximum connnection number is reached.
        */
        MaxConnectionsReached               = 108,    //!<    NGS 側のコネクション数に達した

        /**
            The operation specifies or accesses an invalid GatheringID.
        */
        InvalidGID                          = 109,    //!<    不正な GatheringID（存在しないなど）。

        /**
            The Control Script ID that was given is not a user control script id, but the ControlScript to be scheduled is a user ControlScript or the Control Script ID that was given is not a system control script id, but the ControlScript to be scheduled is a system ControlScript.
        */
        InvalidControlScriptID              = 110,    //!<    不正なスレッド ID

        /**
            The operation is not allowed to be run in a Live environment.
        */
        InvalidOperationInLiveEnvironment   = 111,    //!<    許可されていない関数を実行しようとした

        /**
            The operation would have caused a duplicate to be created, which is an error: operation aborted.
        */
        DuplicateEntry =                    112,    //!<    同一のエントリを登録しようとした

        /**
            An error occured while processing a control script: missing file, missing ControlScript class or syntax error in script.
        */
        ControlScriptFailure =              113,    //!<    サーバーの初期化に失敗（クライアントでは発生しません）

        /**
            The Python class was not found.
        */
        ClassNotFound =                     114,    //!<    クライアントとサーバーのプロトコルが一致していない

        /**
            The session gathering to join was no longer available.
        */
        SessionVoid =                       115,    //!<    ギャザリングが存在しない。
        /**
            The server encountered an unrecognized or mismatched user defined protocol or class. Please verify that the latest BPT files are uploaded to the server and loaded from the UserConfiguration.py and that clients compiled with older incompatible versions of the DDL are not connecting to the server.
        */
        DDLMismatch =                       117,    //!<    クライアントとサーバーのプロトコルが一致していない

        /**
            The server encountered an error. Please verify that the server is well configured.
        */
        InvalidConfiguration =              118,    //!<    ゲームサーバーの設定に不備がある。製品環境では発生しません。

        //
        // Nintendo defined under here.
        //

        /*
        The session gathering to join was full.
        */
        SessionFull                 = 200,    //!<    ギャザリングの最大人数に達している

        /*
        Invalid gathering password.
        */
        InvalidGatheringPassword    = 201,    //!<    不正なギャザリングのパスワード

        WithoutParticipationPeriod  = 202,    //!<    参加可能期間外

        PersistentGatheringCreationMax  = 203,          //!<    永続ギャザリング登録数の上限に達した

        PersistentGatheringParticipationMax  = 204,     //!<    永続ギャザリング参加数の上限に達した

        DeniedByParticipants  = 205,          //!<    参加者に拒否された(ブロックリストに登録されている)

        ParticipantInBlockList  = 206,         //!<    参加者の中に自分がブロックリストに登録したユーザーが存在する

        GameServerMaintenance  = 207,         //!<    ゲームサーバーのメンテナンスが開始される

        OperationPostpone  = 208,     //    オペレーションの実行が延期された(内部使用)

        OutOfRatingRange = 209,    //!< レーティングマッチの範囲外となるレーティング値が指定された

        ConnectionDisconnected = 210, //!< ゲームサーバーとの接続が切れた

        InvalidOperation   = 211,    //!<    不正な操作をしようとした

        NotParticipatedGathering = 212, //!< 指定したユーザーがギャザリングに参加していない

        MatchmakeSessionUserPasswordUnmatch = 213, //!< マッチメイクセッションユーザーパスワードが一致しない

        MatchmakeSessionSystemPasswordUnmatch = 214, //!< マッチメイクセッションシステムパスワードが一致しない

        UserIsOffline = 215,   //!< 指定したユーザーがオフライン

        AlreadyParticipatedGathering = 216, //!< ギャザリングにすでに参加している

        PermissionDenied = 217, //!< 権限がない操作をしようとした

        NotFriend = 218, //!< フレンド関係がない

        SessionClosed = 219, //!< ギャザリングが締め切られている

        DatabaseTemporarilyUnavailable = 220, //!< 一時的にデータベースにアクセスできない

        InvalidUniqueId = 221, //!< 不正な NEX ユニーク ID

        MatchmakingWithdrawn = 222, //!< マッチメイクリクエストが取り下げられた

        LimitExceeded = 223, //!< 制限を超過した

        AccountTemporarilyDisabled =     224,    //!< 一時的にアカウントが使用できない
        
        PartiallyServiceClosed = 225, //!< 一部サービスが終了して利用できない
        
        ConnectionDisconnectedForConcurrentLogin = 226  //!< 同一のプリンシパル ID を持つユーザーがログインしたためゲームサーバーから切断された
    };
    //note: 新規result追加時には、OnlineCore/src/Platform/Core/ResulDefine.hのNEX_ERROR_RESULT_DEFINE_WITHINDEXの変更も必要
    //Wiki エラーコード追加時に行う作業 を参照
    //アプリケーションがハンドリングするべきエラーを増やした場合には、commonmanualのoverall.rstの「アプリケーションがハンドリングするべきエラー一覧」に追加すること

#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX

